package com.mortals.xhx.busiz.web;

import cn.hutool.core.util.IdUtil;
import cn.hutool.core.util.RandomUtil;
import com.alibaba.fastjson.JSONObject;
import com.mortals.framework.annotation.UnAuth;
import com.mortals.framework.common.Rest;
import com.mortals.framework.exception.AppException;
import com.mortals.framework.service.IAuthTokenService;
import com.mortals.framework.service.ICacheService;
import com.mortals.framework.service.IUser;
import com.mortals.framework.util.DateUtils;
import com.mortals.framework.util.SecurityUtil;
import com.mortals.framework.util.StringUtils;
import com.mortals.framework.web.BaseCRUDJsonBodyMappingController;
import com.mortals.xhx.base.system.resource.service.ResourceService;
import com.mortals.xhx.base.system.role.model.RoleUserQuery;
import com.mortals.xhx.base.system.role.service.RoleUserService;
import com.mortals.xhx.base.system.user.model.UserEntity;
import com.mortals.xhx.base.system.user.model.UserQuery;
import com.mortals.xhx.base.system.user.service.UserService;
import com.mortals.xhx.busiz.req.H5Req;
import com.mortals.xhx.common.utils.WeChatUtils;
import com.mortals.xhx.module.company.model.CompanyEntity;
import com.mortals.xhx.module.company.service.CompanyService;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.util.ObjectUtils;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RestController;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.util.Arrays;
import java.util.HashMap;
import java.util.Map;

import static com.mortals.xhx.common.key.ErrorCode.ERROR_TOKEN_EXPIRED;
import static com.mortals.xhx.common.key.ErrorCode.ERROR_TOKEN_EXPIRED_CONTENT;


/**
 * h5授权登录
 *
 * @author:
 * @date: 2023/7/15 10:26
 */
@RestController
@Slf4j
@RequestMapping("h5")
public class H5LoginController extends BaseCRUDJsonBodyMappingController<UserService, UserEntity, Long> {

    @Autowired
    private UserService userService;
    @Autowired
    private IAuthTokenService authTokenService;
    @Autowired
    private CompanyService companyService;

    @Autowired
    private ResourceService resourceService;
    @Autowired
    private RoleUserService roleUserService;

    /**
     * 微信appId
     */
    @Value("${WeChat.pc.appID}")
    private String APP_ID;

    /**
     * 微信appSecret
     */
    @Value("${WeChat.pc.appSecret}")
    private String APP_SECRET;

    @RequestMapping(value = {"bind"},method = {RequestMethod.POST, RequestMethod.GET})
    @UnAuth
    public String bind(HttpServletRequest request, HttpServletResponse response){
        JSONObject ret = new JSONObject();
        Map<String, Object> data = new HashMap<>();
        try {
            // 第三方发起微信授权登录请求，微信用户允许授权第三方应用后，微信会拉起应用或重定向到第三方网站，并且带上授权临时票据code参数；
            String loginCode = request.getParameter("code");
            //用户拒绝授权时code为空，微信也不会请求该接口。为了防止非法请求，若code为空则返回
            if (StringUtils.isEmpty(loginCode)) {
                throw new AppException("登入请求被拒绝");
            }
            Map<String,String> accessTokenMap = WeChatUtils.getAccessToken(loginCode,APP_ID,APP_SECRET);
            //String access_token = accessTokenMap.get("access_token");//接口调用凭证，登录后右上角展示数据需要该值去获取
            String openid = accessTokenMap.get("openid");//授权用户唯一标识
            if(StringUtils.isEmpty(openid)){
                log.error("bind error ", "调用微信接口失败:"+JSONObject.toJSONString(accessTokenMap));
                ret.put(KEY_RESULT_CODE, VALUE_RESULT_FAILURE);
                ret.put(KEY_RESULT_MSG, "调用微信接口失败:"+JSONObject.toJSONString(accessTokenMap));
                return ret.toJSONString();
            }
            data.put("openId", openid);
            ret.put(KEY_RESULT_DATA, data);
            ret.put(KEY_RESULT_CODE, VALUE_RESULT_SUCCESS);
            ret.put(KEY_RESULT_MSG, "微信扫码成功!");
            return ret.toJSONString();
        } catch (Exception e) {
            log.error("bind error ", e);
            ret.put(KEY_RESULT_CODE, VALUE_RESULT_FAILURE);
            ret.put(KEY_RESULT_MSG, super.convertException(e));
            return ret.toJSONString();
        }
    }

    @RequestMapping("login")
    @UnAuth
    public String login(@RequestBody H5Req req) throws Exception {
        JSONObject ret = new JSONObject();
        JSONObject data = new JSONObject();
        UserEntity userEntity = null;
        try {
            if (ObjectUtils.isEmpty(req.getOpenId())) {
                throw new AppException("openId不能为空！");
            }
            //根据钉钉的用户查询系统绑定的用户,获取
            UserQuery userQuery = new UserQuery();
            userQuery.setOpenId(req.getOpenId());
            userEntity = userService.selectOne(userQuery);
            if (ObjectUtils.isEmpty(userEntity)) {
                //throw new AppException("不存在当前匹配的关联用户！openId:" + req.getOpenId());
                userEntity = new UserEntity();
                userEntity.initAttrValue();
                userEntity.setOpenId(req.getOpenId());
                userEntity.setRealName(req.getUserName());
                userEntity.setLoginName(req.getOpenId());
//                userEntity.setPhone(req.getPhone());
//                userEntity.setMobile(req.getPhone());
                userEntity.setLoginPwd(SecurityUtil.md5DoubleEncoding("123"));
                userEntity.setUserType(2);
                userEntity.setRoleIds("2");
                userEntity = userService.save(userEntity);
            }
            if(userEntity.getCompanyId()!=null){
                CompanyEntity companyEntity= companyService.get(userEntity.getCompanyId());
                data.put("parkId",companyEntity.getParkId());
                data.put("parkName",companyEntity.getParkName());
            }
            data.put("id", userEntity.getId());

            data.put("staffId", userEntity.getCustomerId());
            data.put("userType", userEntity.getUserType());
            userEntity.setLoginTime(System.currentTimeMillis());
            userEntity.setToken(IdUtil.fastSimpleUUID());
            userEntity.setExpireTime(DateUtils.addCurrDate(1).getTime());
            String token = authTokenService.createToken(userEntity);
            data.put("token", token);
            ret.put(KEY_RESULT_DATA, data);
            ret.put(KEY_RESULT_CODE, VALUE_RESULT_SUCCESS);
            ret.put(KEY_RESULT_MSG, "用户登录系统成功!");
            recordSysLog(request, userEntity, "h5用户登录系统成功!");
            return ret.toJSONString();
        } catch (AppException e) {
            log.error("h5 login error ", e);
            ret.put(KEY_RESULT_CODE, e.getCode());
            ret.put(KEY_RESULT_MSG, super.convertException(e));
            return ret.toJSONString();
        } catch (Exception e) {
            log.error("系统异常 error ", e);
            ret.put(KEY_RESULT_CODE, VALUE_RESULT_FAILURE);
            ret.put(KEY_RESULT_MSG, super.convertException(e));
            return ret.toJSONString();
        }
    }


    @RequestMapping("register")
    @UnAuth
    public String resgister(@RequestBody H5Req req) {
        JSONObject ret = new JSONObject();

        UserEntity userEntity = null;
        try {
            if (ObjectUtils.isEmpty(req.getUserName()))
                throw new AppException("用户姓名不能为空！");

            if (ObjectUtils.isEmpty(req.getPhone()))
                throw new AppException("手机号码不能为空！");

            if (ObjectUtils.isEmpty(req.getOpenId()))
                throw new AppException("openId不能为空！");

            //查询用户是否已经绑定公司，如果已经绑定则不做后续

            UserQuery userQuery = new UserQuery();
            userQuery.setOpenId(req.getOpenId());
            userEntity = userService.selectOne(userQuery);

            if (ObjectUtils.isEmpty(userEntity)) {
                //todo h5登录的用户不存在，新增用户，用户名用openId，密码用123
                userEntity = new UserEntity();
                userEntity.initAttrValue();
                userEntity.setOpenId(req.getOpenId());
                userEntity.setRealName(req.getUserName());
                userEntity.setLoginName(req.getOpenId());
                userEntity.setPhone(req.getPhone());
                userEntity.setMobile(req.getPhone());
                userEntity.setLoginPwd(SecurityUtil.md5DoubleEncoding("123"));
                userEntity.setUserType(2);
                CompanyEntity companyEntity = companyService.getCompanyByH5(req);
                userEntity.setCompanyId(companyEntity.getId());
                userEntity.setRoleIds("2");
                userEntity = userService.save(userEntity);

                //更新默认用户角色
                RoleUserQuery roleUserQuery = new RoleUserQuery();
                roleUserQuery.setUserId(userEntity.getId());
                roleUserQuery.setRoleIdList(Arrays.asList(2L));
                roleUserService.doDistributionRole(roleUserQuery);

            }else {
                userEntity.setPhone(req.getPhone());
                userEntity.setMobile(req.getPhone());
            }

            if (!ObjectUtils.isEmpty(userEntity)
                    && !ObjectUtils.isEmpty(userEntity.getCompanyId()))
                throw new AppException("当前用户已经绑定公司，请勿重复绑定！");

            if (!ObjectUtils.isEmpty(userEntity)
                    && ObjectUtils.isEmpty(userEntity.getCompanyId())) {
                //todo 当前用户存在 但是未绑定公司 可绑定
                CompanyEntity companyEntity = companyService.getCompanyByH5(req);
                userEntity.setCompanyId(companyEntity.getId());
                userEntity.setRoleIds("2");
                userService.update(userEntity);
            }
            //当前用户绑定公司  用户名可与phone绑定
            JSONObject data = new JSONObject();
            if(userEntity.getCompanyId()!=null){
                CompanyEntity companyEntity= companyService.get(userEntity.getCompanyId());
                data.put("parkId",companyEntity.getParkId());
                data.put("parkName",companyEntity.getParkName());
            }
            data.put("id", userEntity.getId());
            data.put("userType", userEntity.getUserType());
            userEntity.setLoginTime(System.currentTimeMillis());
            userEntity.setToken(IdUtil.fastSimpleUUID());
            userEntity.setExpireTime(DateUtils.addCurrDate(7).getTime());
            String token = authTokenService.createToken(userEntity);
            data.put("token", token);
            ret.put(KEY_RESULT_DATA, data);
            ret.put(KEY_RESULT_CODE, VALUE_RESULT_SUCCESS);
            ret.put(KEY_RESULT_MSG, "h5用户登录系统成功!");
            recordSysLog(request, userEntity, "钉钉用户登录系统成功!");
            return ret.toJSONString();
        } catch (AppException e) {
            log.error("dinding register error ", e);
            ret.put(KEY_RESULT_CODE, e.getCode());
            ret.put(KEY_RESULT_MSG, super.convertException(e));
            return ret.toJSONString();
        } catch (Exception e) {
            log.error("系统异常 error ", e);
            ret.put(KEY_RESULT_CODE, VALUE_RESULT_FAILURE);
            ret.put(KEY_RESULT_MSG, super.convertException(e));
            return ret.toJSONString();
        }
    }


    @RequestMapping("index")
    public String index() throws Exception {
        JSONObject ret = new JSONObject();
        IUser user = this.getCurUser();
        if (user == null) {
            return JSONObject.toJSONString(Rest.fail(ERROR_TOKEN_EXPIRED, ERROR_TOKEN_EXPIRED_CONTENT));
        }
        JSONObject data = new JSONObject();
        UserEntity userEntity = userService.get(user.getId());
        String token = authTokenService.getToken(request);
        data.put("id", user.getId());
        data.put("token", token);
        data.put("staffId", userEntity.getCustomerId());
        data.put("userType", userEntity.getUserType());
        ret.put(KEY_RESULT_DATA, data);
        ret.put(KEY_RESULT_CODE, VALUE_RESULT_SUCCESS);
        ret.put(KEY_RESULT_MSG, "用户登录系统成功!");
        return ret.toJSONString();
    }

}
