package com.mortals.xhx.module.declare.service.impl;

import cn.hutool.core.date.DateUnit;
import cn.hutool.core.date.DateUtil;
import com.mortals.framework.common.Rest;
import com.mortals.framework.model.PageInfo;
import com.mortals.xhx.common.code.DeclareStatusEnum;
import com.mortals.xhx.module.category.model.CategoryEntity;
import com.mortals.xhx.module.category.service.CategoryService;
import com.mortals.xhx.module.company.model.CompanyEntity;
import com.mortals.xhx.module.company.service.CompanyService;
import com.mortals.xhx.module.declare.model.*;
import com.mortals.xhx.module.declare.service.DeclareFinImagesService;
import com.mortals.xhx.module.park.model.ParkEntity;
import com.mortals.xhx.module.park.service.ParkService;
import org.springframework.beans.BeanUtils;

import java.util.Map;
import java.util.function.Function;

import org.springframework.stereotype.Service;
import com.mortals.framework.service.impl.AbstractCRUDServiceImpl;
import com.mortals.framework.exception.AppException;
import com.mortals.framework.model.Context;
import com.mortals.xhx.module.declare.dao.DeclareDao;
import com.mortals.xhx.module.declare.service.DeclareService;
import org.springframework.beans.factory.annotation.Autowired;
import com.mortals.xhx.module.declare.service.DeclareImagesService;
import org.springframework.util.ObjectUtils;

import java.util.Date;
import java.util.Arrays;
import java.util.List;
import java.util.stream.Collectors;

import lombok.extern.slf4j.Slf4j;

/**
 * DeclareService
 * 企业代办申报 service实现
 *
 * @author zxfei
 * @date 2025-04-27
 */
@Service("declareService")
@Slf4j
public class DeclareServiceImpl extends AbstractCRUDServiceImpl<DeclareDao, DeclareEntity, Long> implements DeclareService {
    @Autowired
    private DeclareImagesService declareImagesService;

    @Autowired
    private DeclareFinImagesService declareFinImagesService;
    @Autowired
    private CategoryService categoryService;
    @Autowired
    private ParkService parkService;
    @Autowired
    private CompanyService companyService;


    @Override
    protected void findAfter(DeclareEntity params, PageInfo pageInfo, Context context, List<DeclareEntity> list) throws AppException {
        fillSubData(list);
        super.findAfter(params, pageInfo, context, list);
    }

    @Override
    protected void findAfter(DeclareEntity params, Context context, List<DeclareEntity> list) throws AppException {
        fillSubData(list);
        super.findAfter(params, context, list);
    }

    private void fillSubData(List<DeclareEntity> list) {
        List<Long> idList = list.stream().map(i -> i.getId()).collect(Collectors.toList());
        if (ObjectUtils.isEmpty(idList)) return;
        Map<Long, List<DeclareImagesEntity>> declareImagesListMap = declareImagesService
                .find(new DeclareImagesQuery().declareIdList(idList)).parallelStream()
                .collect(Collectors.groupingBy(DeclareImagesEntity::getDeclareId));
        list.forEach(item -> item.setDeclareImagesList(declareImagesListMap.get(item.getId())));


        Map<Long, List<DeclareFinImagesEntity>> declareFinImagesListMap = declareFinImagesService
                .find(new DeclareFinImagesQuery().declareIdList(idList)).parallelStream()
                .collect(Collectors.groupingBy(DeclareFinImagesEntity::getDeclareId));
        list.forEach(item -> item.setDeclareFinImagesList(declareFinImagesListMap.get(item.getId())));
    }


    @Override
    protected void saveBefore(DeclareEntity entity, Context context) throws AppException {
        if(!ObjectUtils.isEmpty(entity.getCategoryId())){
            CategoryEntity categoryEntity = categoryService.get(entity.getCategoryId());
            entity.setCategoryName(categoryEntity==null?"":categoryEntity.getCategoryName());
            entity.setCategoryCode(categoryEntity==null?"":categoryEntity.getCategoryCode());
        }
        if(!ObjectUtils.isEmpty(entity.getParkId())){
            ParkEntity parkEntity = parkService.get(entity.getParkId());
            entity.setParkName(parkEntity==null?"":parkEntity.getName());
        }
        if(!ObjectUtils.isEmpty(entity.getCompanyId())){
            CompanyEntity companyEntity = companyService.get(entity.getCompanyId());
            entity.setCompanyName(companyEntity==null?"":companyEntity.getName());
        }


        super.saveBefore(entity, context);
    }

    @Override
    protected void saveAfter(DeclareEntity entity, Context context) throws AppException {
        if (!ObjectUtils.isEmpty(entity.getDeclareImagesList())) {
            entity.getDeclareImagesList().stream().peek(item -> {
                item.setDeclareId(entity.getId());
                item.setCreateUserId(this.getContextUserId(context));
                item.setCreateTime(new Date());
            }).count();
            declareImagesService.save(entity.getDeclareImagesList());
        }

        if (!ObjectUtils.isEmpty(entity.getDeclareFinImagesList())) {
            entity.getDeclareFinImagesList().stream().peek(item -> {
                item.setDeclareId(entity.getId());
                item.setCreateUserId(this.getContextUserId(context));
                item.setCreateTime(new Date());
            }).count();
            declareFinImagesService.save(entity.getDeclareFinImagesList());
        }
        super.saveAfter(entity, context);
    }

    @Override
    protected void updateAfter(DeclareEntity entity, Context context) throws AppException {
        if (!ObjectUtils.isEmpty(entity.getDeclareImagesList())) {
            Long[] declareImagesIds = declareImagesService.find(new DeclareImagesQuery().declareId(entity.getId())).stream().map(DeclareImagesEntity::getId).toArray(Long[]::new);
            declareImagesService.remove(declareImagesIds, context);
            entity.getDeclareImagesList().stream().peek(item -> {
                item.setDeclareId(entity.getId());
                item.setCreateUserId(this.getContextUserId(context));
                item.setCreateTime(new Date());
                item.setUpdateUserId(this.getContextUserId(context));
                item.setUpdateTime(new Date());
            }).count();
            declareImagesService.save(entity.getDeclareImagesList());
        }

        if (!ObjectUtils.isEmpty(entity.getDeclareFinImagesList())) {
            Long[] declareFinImagesIds = declareFinImagesService.find(new DeclareFinImagesQuery().declareId(entity.getId())).stream().map(DeclareFinImagesEntity::getId).toArray(Long[]::new);
            declareFinImagesService.remove(declareFinImagesIds, context);
            entity.getDeclareFinImagesList().stream().peek(item -> {
                item.setDeclareId(entity.getId());
                item.setCreateUserId(this.getContextUserId(context));
                item.setCreateTime(new Date());
                item.setUpdateUserId(this.getContextUserId(context));
                item.setUpdateTime(new Date());
            }).count();
            declareFinImagesService.save(entity.getDeclareFinImagesList());
        }
        super.updateAfter(entity, context);
    }

    @Override
    protected void removeAfter(Long[] ids, Context context, int result) throws AppException {
        List<DeclareImagesEntity> declareImageslist = declareImagesService.find(new DeclareImagesQuery().declareIdList(Arrays.asList(ids)));
        declareImagesService.removeList(declareImageslist, context);

        List<DeclareFinImagesEntity> declareFinImageslist = declareFinImagesService.find(new DeclareFinImagesQuery().declareIdList(Arrays.asList(ids)));
        declareFinImagesService.removeList(declareFinImageslist, context);
        super.removeAfter(ids, context, result);
    }

    @Override
    public Rest<String> declareCancel(DeclareQuery query, Context context) {
        //校验取消申报时间
        DeclareEntity declareEntity = this.get(query.getId(), context);
        if(ObjectUtils.isEmpty(declareEntity)) throw new AppException("申报信息不存在！id:"+query.getId());
        long between = DateUtil.between(declareEntity.getCreateTime(), new Date(), DateUnit.MINUTE);
        if(between > 30) throw new AppException("超过30分钟无法取消申报！");
        if(declareEntity.getDeclareStatus() != DeclareStatusEnum.草稿.getValue()) throw new AppException("当前申报不是草稿状态，不能撤销！");

        declareEntity.setDeclareStatus(DeclareStatusEnum.撤销.getValue());
        declareEntity.setUpdateTime(new Date());
        declareEntity.setUpdateUserId(this.getContextUserId(context));
        this.update(declareEntity, context);
        return Rest.ok("取消申报成功！");
    }
}