package com.mortals.xhx.module.declare.web;

import com.alibaba.fastjson.JSONObject;
import com.mortals.framework.annotation.UnAuth;
import com.mortals.framework.common.Rest;
import com.mortals.framework.exception.AppException;
import com.mortals.framework.model.Context;
import com.mortals.framework.web.BaseCRUDJsonBodyMappingController;
import com.mortals.xhx.common.code.DeclareStatusEnum;
import com.mortals.xhx.common.code.YesNoEnum;
import com.mortals.xhx.module.declare.model.*;
import com.mortals.xhx.module.declare.service.DeclareFinImagesService;
import com.mortals.xhx.module.declare.service.DeclareImagesService;
import com.mortals.xhx.module.declare.service.DeclareService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import java.util.List;
import java.util.Map;

/**
 * 企业代办申报
 *
 * @author zxfei
 * @date 2025-04-27
 */
@RestController
@RequestMapping("declare")
public class DeclareController extends BaseCRUDJsonBodyMappingController<DeclareService, DeclareEntity, Long> {

    @Autowired
    private DeclareImagesService declareImagesService;

    @Autowired
    private DeclareFinImagesService declareFinImagesService;

    public DeclareController() {
        super.setModuleDesc("企业代办申报");
    }

    @Override
    protected void init(Map<String, Object> model, Context context) {
        this.addDict(model, "declareStatus", DeclareStatusEnum.getEnumMap());
        super.init(model, context);
    }


    @Override
    protected int infoAfter(Long id, Map<String, Object> model, DeclareEntity entity, Context context) throws AppException {
        List<DeclareImagesEntity> declareImagesEntities = declareImagesService.find(new DeclareImagesQuery().declareId(id));
        entity.setDeclareImagesList(declareImagesEntities);
        List<DeclareFinImagesEntity> declareFinImagesEntities = declareFinImagesService.find(new DeclareFinImagesQuery().declareId(id));
        entity.setDeclareFinImagesList(declareFinImagesEntities);
        return super.infoAfter(id, model, entity, context);
    }


    /**
     * 撤销
     */
    @PostMapping(value = "cancel")
    @UnAuth
    public String cancel(@RequestBody DeclareQuery query) {
        JSONObject jsonObject = new JSONObject();
        String busiDesc = "申报撤销" + this.getModuleDesc();
        try {
            Rest<String> rest = this.service.declareCancel(query, getContext());
            if(YesNoEnum.NO.getValue()==rest.getCode()){
                throw new AppException(rest.getMsg());
            }
            jsonObject.put(KEY_RESULT_CODE, VALUE_RESULT_SUCCESS);
            jsonObject.put(KEY_RESULT_MSG, "申报撤销成功！");
        } catch (Exception e) {
            log.error("获取异常", e);
            jsonObject.put(KEY_RESULT_CODE, VALUE_RESULT_FAILURE);
            jsonObject.put(KEY_RESULT_MSG, super.convertException(e));
        }
        return jsonObject.toJSONString();
    }
}