import api from "@/services/api"
import {request} from "@/utils/request"
import moment from "moment"
export default{
    data(){
        return{
            tablename:"",//接口名称
            tablequery:{},//表格查询条件
            tableLoading:false,//表格加载动画
            tablecolumns:[],//表头数据
            tabledataSource:[],//表格数据
            selectedRows:[],//选中数据对象
            selectedId:[],//选中id
            columnsSearch:[],//表格搜索
            pagination:{
                current:1,
                pageSize:10,
                total:0,
                showSizeChanger:true,
                onShowSizeChange:(current, pageSize)=>this.onShowSize(current, pageSize)
            }
        }
    },
    created(){
        this.getTableInfo()
    },
    computed:{
        tableurl:function(vm){
            return api[vm.tablename]?api[vm.tablename]:{}
        }
    },
    methods:{
        /**
         * 监听页码选择
         * @param {int} current 
         * @param {int} pageSize 
         */
        onShowSize(current, pageSize){
            this.pagination.current = current
            this.pagination.pageSize = pageSize
            this.getTableInfo()
        },
        /**
         * 分页
         * @param {*} pagination 
         */
        handleTableChange(pagination){
            const pager = { ...this.pagination };
            pager.current = pagination.current;
            this.pagination = pager;
            this.getTableInfo()
        },
        /**
         * 复选框选中
         * @param {Number} id 选中id
         * @param {Object} selected 一条数据
         */
        selectedRowChange(id, selected){
            this.selectedId = id
            this.selectedRows = selected
        },
        /**
         * 接口返回数据之后
         * @param {Object} data 接口返回数据 
         * @returns 
         */
        tableAfterData(data){
            return data
        },
        /**
         * 接口提交之前数据
         * @param {Object} data 
         * @returns 
         */
        tableBeforeData(data){
            return data
        },
        /** 统一表格查询 */
        getTableInfo(){
            this.tableLoading = true
            const {current,pageSize} = this.pagination
            const query = this.tableBeforeData(Object.assign({},{size:pageSize,page:current},this.tablequery))
            request(this.tableurl.list,'GET',query).then(res=>{
                this.stopLoading()
                const pagination = { ...this.pagination };
                const {code,data} = this.tableAfterData(res.data)
                if(code == 0){
                    this.tabledataSource = data.data?data.data:data

                    pagination.total = data.total
                    this.pagination = pagination
                }
            }).catch(error=>{
                this.stopLoading()
                this.$message.error("接口请求错误")
                console.log(error)
            })
        },
        /** 从新加载 */
        tableReload(){
            this.pagination.current = 1
            this.getTableInfo()
        },
        /** 结束加载loading */
        stopLoading(){
            setTimeout(() => {
                this.tableLoading = false
            }, 300);
        },
        /** 时间格式化 */
        RenderMomentTimes(dates=new Date(),format='YYYY-MM-DD HH:mm:ss'){
            return moment(dates).format(format)
        },
        /**
         * 序号格式化
         * @param {*} text 
         * @param {*} record 
         * @param {*} index 
         * @returns 
         */
        RenderIndex(text, record, index){
            return index+1
        },
        /**
         * 字典格式化
         * @param {*} v 查询字典中对应key
         * @param {Array} dictionaries 字典数据
         */
        RenderCustomRender(v,dictionaries){
            const obj = dictionaries.find(e=>e.value == v)
            // 如果字典中有颜色传递颜色参数
            const style = {color:obj.color}
            return obj?(<span style={style}>{obj.label}</span>):'--'
        },
        /**
         * 表格数据删除
         * @param {Object|String} row 一行数据
         */
        OnTableDel(row){
            let id = null
            if(typeof row === 'string'){
                id = row //批量删除1,2...
            }else{
                id = row.id
            }
            request(this.tableurl.delete,'get',{id:id}).then(res=>{
                const {code,msg} = res.data
                if(code == 0){
                    this.$message.success('删除成功')
                    this.getTableInfo()
                }else{
                    this.$message.error(msg)
                }
            }).catch(error=>{
                this.$message.error("接口请求错误")
                console.log(error)
            })
        },
        /**
         * 批量删除
         * @param {Array} arr 
         */
        OnTableAllDel(arr){
            if(arr && arr.length>0){
                let newarr = []
                arr.forEach(v=>{
                    newarr.push(v.id)
                })
                this.OnTableDel(newarr.join(','))
            }
        },
        /**
         * 表格查询
         * @param {Object} obj 查询表单数据
         */
        onSearch(obj){
            this.tablequery = Object.assign({},this.tablequery,obj)
            this.tableReload()
        },
        /**
         * 编辑数据
         * @param {Object} row 一行数据
         */
        OnTableEdit(row){
            this.$refs.pageshow && this.$refs.pageshow.onEdit(row)
        },
        /**
         * 新增数据
         */
        OnTableAdd(){
            this.$refs.pageshow && this.$refs.pageshow.onAdd()
        }
    }
}