package com.mortals.xhx.busiz.web;

import cn.hutool.core.codec.Base64;
import cn.hutool.http.HttpUtil;
import com.alibaba.fastjson.JSON;
import com.mortals.framework.annotation.UnAuth;
import com.mortals.framework.common.Rest;
import com.mortals.framework.model.PageInfo;
import com.mortals.framework.model.ParamDto;
import com.mortals.framework.model.Result;
import com.mortals.xhx.common.code.SourceEnum;
import com.mortals.xhx.module.app.model.AppDatasetEntity;
import com.mortals.xhx.module.app.model.AppDatasetQuery;
import com.mortals.xhx.module.app.service.AppDatasetService;
import com.mortals.xhx.module.app.service.AppInfoFieldService;
import com.mortals.xhx.module.business.model.BusinessMatterEntity;
import com.mortals.xhx.module.business.model.BusinessMatterQuery;
import com.mortals.xhx.module.business.service.BusinessMatterService;
import com.mortals.xhx.module.dept.model.DeptEntity;
import com.mortals.xhx.module.dept.model.DeptQuery;
import com.mortals.xhx.module.dept.service.DeptService;
import com.mortals.xhx.module.matter.model.MatterEntity;
import com.mortals.xhx.module.matter.model.MatterQuery;
import com.mortals.xhx.module.matter.service.MatterExtService;
import com.mortals.xhx.module.matter.service.MatterService;
import com.mortals.xhx.module.site.model.*;
import com.mortals.xhx.module.site.service.SiteMatterService;
import com.mortals.xhx.module.site.service.SiteService;
import com.mortals.xhx.module.site.service.SiteThemeMatterService;
import com.mortals.xhx.module.site.service.SiteThemeService;
import com.mortals.xhx.module.window.model.WindowEntity;
import com.mortals.xhx.module.window.model.WindowMatterEntity;
import com.mortals.xhx.module.window.model.WindowMatterQuery;
import com.mortals.xhx.module.window.model.WindowQuery;
import com.mortals.xhx.module.window.service.WindowMatterService;
import com.mortals.xhx.module.window.service.WindowService;
import lombok.extern.slf4j.Slf4j;
import org.apache.http.HttpResponse;
import org.apache.http.client.CookieStore;
import org.apache.http.client.HttpClient;
import org.apache.http.client.methods.HttpGet;
import org.apache.http.impl.client.BasicCookieStore;
import org.apache.http.impl.client.HttpClientBuilder;
import org.apache.http.util.EntityUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.util.ObjectUtils;
import org.springframework.util.StopWatch;
import org.springframework.web.bind.annotation.*;

import java.util.*;
import java.util.stream.Collectors;

/**
 * 测试接口
 *
 * @author:
 * @date: 2022/12/14 9:18
 */
@RestController
@RequestMapping("test")
@Slf4j
public class DemoWebApiController {

    @Autowired
    private WindowService windowService;
    @Autowired
    private SiteMatterService siteMatterService;
    @Autowired
    private WindowMatterService windowMatterService;
    @Autowired
    private DeptService deptService;
    @Autowired
    private MatterService matterService;
    @Autowired
    private SiteThemeMatterService siteThemeMatterService;
    @Autowired
    private BusinessMatterService businessMatterService;
    @Autowired
    private AppDatasetService appDatasetService;
    @Autowired
    private AppInfoFieldService appInfoFieldService;
    @Autowired
    private MatterExtService matterExtService;
    @Autowired
    private SiteService siteService;

    @Autowired
    private SiteThemeService siteThemeService;


    @PostMapping(value = "testGov")
    @UnAuth
    public Rest<String> testGov(@RequestBody MatterQuery query) {
        String resp = HttpUtil.get(query.getUrl());
        return Rest.ok(resp);
    }

    @PostMapping(value = "testCookie")
    @UnAuth
    public Rest<String> testCookie(@RequestBody MatterQuery query) {
        log.info("测试cookie");
        HttpClient http = null;
        CookieStore httpCookieStore = new BasicCookieStore();
        http = HttpClientBuilder.create().setDefaultCookieStore(httpCookieStore).build();

        /* do stuff */
        HttpGet httpRequest = new HttpGet(query.getUrl());
        HttpResponse httpResponse = null;
        try {
            httpResponse = http.execute(httpRequest);
            byte[] data = EntityUtils.toByteArray(httpResponse.getEntity());
            String encode = Base64.encode(data);

            //String content = EntityUtils.toString(entity, charset);
            //httpResponse.getEntity()
            //httpResponse.toString()
        } catch (Exception e) {
            log.error("异常", e);
        }
        /* check cookies */
        String cookieStr = Arrays.asList(httpCookieStore.getCookies()).stream().map(item -> JSON.toJSONString(item)).collect(Collectors.joining("|"));
        log.info("cookies:{}", cookieStr);
        return Rest.ok(cookieStr);
    }

    @PostMapping(value = "reEventShow")
    @UnAuth
    public Rest<String> reEventShow() {
        List<SiteMatterEntity> siteMatterEntities = siteMatterService.find(new SiteMatterQuery());
        for (SiteMatterEntity siteMatterEntity : siteMatterEntities) {
            if (ObjectUtils.isEmpty(siteMatterEntity.getEventTypeShow())) {
                MatterEntity matterEntity = matterService.get(siteMatterEntity.getMatterId());
                if (!ObjectUtils.isEmpty(matterEntity) && !ObjectUtils.isEmpty(matterEntity.getEventTypeShow())) {
                    siteMatterEntity.setEventTypeShow(matterEntity.getEventTypeShow());
                    siteMatterService.update(siteMatterEntity);
                }
            }
        }

        return Rest.ok();
    }


    @PostMapping(value = "reDepts")
    @UnAuth
    public Rest<String> reDepts() {
        log.info("更新部门与设备关联");
        //更新窗口部门
        List<WindowEntity> windowEntities = windowService.find(new WindowQuery());
        for (WindowEntity windowEntity : windowEntities) {
            String deptName = windowEntity.getDeptName();
            //根据部门名称查询部门
            DeptEntity deptEntity = deptService.selectOne(new DeptQuery().name(deptName + "%"));
            if (!ObjectUtils.isEmpty(deptEntity)) {
                if (deptEntity.getId() != windowEntity.getDeptId() && deptEntity.getName().trim().equals(windowEntity.getDeptName().trim())) {
                    log.info("部门:{},更新部门id:orgin deptId:{} ,updateDeptId:{}", deptName, windowEntity.getDeptId(), deptEntity.getId());
                    WindowEntity temp = new WindowEntity();
                    temp.setId(windowEntity.getId());
                    temp.setDeptId(deptEntity.getId());
                    windowService.getDao().update(temp);

                    // windowService.update(windowEntity);
                }
            }
        }
        //更新窗口事项中的部门id
        List<WindowMatterEntity> windowMatterEntities = windowMatterService.find(new WindowMatterQuery());
        for (WindowMatterEntity windowMatterEntity : windowMatterEntities) {
            DeptEntity deptEntity = deptService.selectOne(new DeptQuery().name(windowMatterEntity.getDeptName() + "%"));
            if (!ObjectUtils.isEmpty(deptEntity)) {
                if (deptEntity.getId() != windowMatterEntity.getDeptId() && windowMatterEntity.getDeptName().trim().equals(deptEntity.getName())) {
                    WindowMatterEntity temp = new WindowMatterEntity();
                    temp.setId(windowMatterEntity.getId());
                    temp.setDeptId(deptEntity.getId());
                    temp.setDeptCode(deptEntity.getDeptNumber());
                    windowMatterService.getDao().update(temp);
                    // windowMatterService.update(windowMatterEntity);
                }
            }
        }

        List<SiteMatterEntity> siteMatterEntities = siteMatterService.find(new SiteMatterQuery());
        for (SiteMatterEntity siteMatterEntity : siteMatterEntities) {
            DeptEntity deptEntity = deptService.selectOne(new DeptQuery().name(siteMatterEntity.getDeptName() + "%"));
            if (!ObjectUtils.isEmpty(deptEntity)) {
                if (deptEntity.getId() != siteMatterEntity.getDeptId() && siteMatterEntity.getDeptName().trim().equals(deptEntity.getName().trim())) {
                    SiteMatterEntity temp = new SiteMatterEntity();
                    temp.setId(siteMatterEntity.getId());
                    temp.setDeptId(deptEntity.getId());
                    temp.setDeptCode(deptEntity.getDeptNumber());
                    siteMatterService.getDao().update(temp);

                    //siteMatterService.update(siteMatterEntity);
                }
            }
        }

        return Rest.ok();
    }


    @PostMapping(value = "syncThemeMatter")
    @UnAuth
    public Rest<String> syncThemeMatter(@RequestBody SiteEntity siteEntity) {
        siteThemeMatterService.deleteGovBySiteId(siteEntity.getId(), null);
        Rest<String> grRest = siteThemeMatterService.syncThemeMatterBySiteId(siteEntity.getId(), "2", null);
        log.info("同步站点主题个人事项:" + JSON.toJSONString(grRest));
        log.info("同步站点法人主题事项开始.....");
        Rest<String> frRest = siteThemeMatterService.syncThemeMatterBySiteId(siteEntity.getId(), "3", null);
        log.info("同步站点主题法人事项:" + JSON.toJSONString(frRest));

        return Rest.ok();
    }

    @PostMapping(value = "testAllMatter")
    @UnAuth
    public Rest<String> testAllMatter(@RequestBody SiteEntity siteEntity) {
        StopWatch stopWatch = new StopWatch("stopwatch syncmatter");
        log.info("开始查询事项==");
        stopWatch.start("开始同步事项方法");
        MatterQuery matterQuery = new MatterQuery();
        matterQuery.setAreaCode(siteEntity.getAreaCode());
        matterQuery.setSource(SourceEnum.政务网.getValue());
        List<MatterEntity> matterList = matterService.getDao().getMatterListByAreaCode(matterQuery);
        stopWatch.stop();

        log.info("本地事项总数：{},{}s", matterList.size(), stopWatch.getLastTaskTimeMillis() / 1000);
        return Rest.ok();
    }

    @PostMapping(value = "testAllMatterPage")
    @UnAuth
    public Rest<String> testAllMatterPage(@RequestBody SiteEntity siteEntity) {
        StopWatch stopWatch = new StopWatch("stopwatch syncmatter");
        log.info("开始分页查询事项==");
        MatterQuery matterQuery = new MatterQuery();
        matterQuery.setAreaCode(siteEntity.getAreaCode());
        matterQuery.setSource(SourceEnum.政务网.getValue());

        ParamDto param = new ParamDto();
        param.getCondition().put("areaCode", matterQuery.getAreaCode());

        param.getCondition().put("source", matterQuery.getSource());

        int count = matterService.getDao().getMatterListByAreaCodeCount(param);
        log.info("本地事项总数：{}", count);
        ArrayList<MatterEntity> allList = new ArrayList<>();
        if (count > 0) {
            //分页获取
            int pages = count / 500 + 1;
            for (int i = 1; i <= pages; i++) {
                stopWatch.start("开始同步事项方法");
                PageInfo pageInfo = new PageInfo();
                pageInfo.setCurrPage(i);
                pageInfo.setPrePageResult(500);
                Result<MatterEntity> rest = matterService.getDao().getMatterListByAreaCodePage(matterQuery, pageInfo);
                allList.addAll(rest.getList());
                stopWatch.stop();
                log.info("本地事项总数：{},{}s", allList.size(), stopWatch.getLastTaskTimeMillis() / 1000);
            }
            // stopWatch.stop();
            log.info("本地事项总数：{},{}s", allList.size(), stopWatch.getLastTaskTimeMillis() / 1000);

        }


        return Rest.ok();
    }


    @PostMapping(value = "testAllMatterCount")
    @UnAuth
    public Rest<String> testAllMatterCount(@RequestBody SiteEntity siteEntity) {
        StopWatch stopWatch = new StopWatch("stopwatch syncmatter");
        log.info("开始分页查询事项==");
        MatterQuery matterQuery = new MatterQuery();
        matterQuery.setAreaCode(siteEntity.getAreaCode());
        matterQuery.setSource(SourceEnum.政务网.getValue());

        ParamDto param = new ParamDto();
        param.getCondition().put("areaCode", matterQuery.getAreaCode());
        if (!ObjectUtils.isEmpty(matterQuery.getSource())) {
            param.getCondition().put("source", matterQuery.getSource());
        }
        int count = matterService.getDao().getMatterListByAreaCodeCount(param);
        log.info("本地事项总数：{}", count);

        return Rest.ok();
    }


    @PostMapping(value = "saveWinMatters")
    @UnAuth
    public Rest<String> saveWinMatters(@RequestBody WindowMatterQuery windowMatterQuery) {

        log.info("保存窗口站点==");
        BusinessMatterQuery businessMatterQuery = new BusinessMatterQuery();
        businessMatterQuery.setSiteBusinessId(17L);
        List<BusinessMatterEntity> businessMatterEntities = businessMatterService.find(businessMatterQuery);


        List<Long> windowIdList = windowMatterQuery.getWindowIdList();
        for (Long windowId : windowIdList) {
            List<WindowMatterEntity> windowMatterEntities = businessMatterEntities.stream().map(item -> {

                WindowMatterEntity windowMatterEntity = new WindowMatterEntity();
                WindowEntity cache = windowService.getCache(windowId.toString());
                windowMatterEntity.setWindowId(cache.getId());
                windowMatterEntity.setWindowName(cache.getName());
                windowMatterEntity.setSiteMatterId(item.getMatterId());
                windowMatterEntity.setMatterName(item.getMatterName());
                windowMatterEntity.setDeptId(50L);
                windowMatterEntity.setDeptCode("3890499118653034496");
                windowMatterEntity.setDeptName("县行政审批局");

                windowMatterEntity.setCreateTime(new Date());
                windowMatterEntity.setCreateUserId(1L);
                return windowMatterEntity;

            }).collect(Collectors.toList());
            windowMatterService.getDao().insertBatch(windowMatterEntities);
        }


        return Rest.ok();
    }


    @PostMapping(value = "saveSiteMatters")
    @UnAuth
    public Rest<String> saveSiteMatters(@RequestBody SiteQuery siteQuery) {
        log.info("保存站点事项相关==");
        //复制部门
        Long copySiteId = siteQuery.getCopySiteId();
        Long targetSiteId = siteQuery.getTargetSiteId();



        List<DeptEntity> deptEntityList = deptService.find(new DeptQuery().siteId(copySiteId).source(SourceEnum.政务网.getValue()));
        deptEntityList.stream().forEach(item -> {
            item.setSiteId(targetSiteId);
            item.setId(null);

            item.setDeptNumber(item.getDeptNumber() + "_" + targetSiteId);

        });
        deptService.save(deptEntityList);

        SiteEntity siteEntity = siteService.get(targetSiteId);

        //保存站点事项
        List<MatterEntity> matterList = matterService.getDao().getMatterListByAreaCode(new MatterQuery().areaCode(siteQuery.getAreaCode()).source(SourceEnum.政务网.getValue()));
        List<SiteMatterEntity> targetSiteMatterEntities = new ArrayList<>();
        for (MatterEntity matterEntity : matterList) {
            DeptEntity deptEntity =deptService.selectOne(new DeptQuery().deptNumber(matterEntity.getDeptCode()));

            //DeptEntity deptEntity = deptService.getExtCache(matterEntity.getDeptCode());
            SiteMatterEntity siteMatterEntity = new SiteMatterEntity();
            siteMatterEntity.initAttrValue();
            //SiteEntity siteEntity = siteService.get(siteId);
            siteMatterEntity.setSiteId(siteEntity.getId());
            siteMatterEntity.setSiteName(siteEntity.getSiteName());
            siteMatterEntity.setMatterId(matterEntity.getId());
            siteMatterEntity.setMatterCode(matterEntity.getMatterNo());
            siteMatterEntity.setMatterName(matterEntity.getMatterName());
            siteMatterEntity.setSource(matterEntity.getSource());
            siteMatterEntity.setEventTypeShow(matterEntity.getEventTypeShow());
            siteMatterEntity.setDeptName(deptEntity == null ? "" : deptEntity.getName());
            siteMatterEntity.setDeptId(deptEntity == null ? -1L : deptEntity.getId());
            siteMatterEntity.setDeptCode(deptEntity == null ? "" : deptEntity.getDeptNumber());
            siteMatterEntity.setCreateUserId(1L);
            siteMatterEntity.setCreateTime(new Date());
            targetSiteMatterEntities.add(siteMatterEntity);
        }
        siteMatterService.save(targetSiteMatterEntities);

        //同步主题
        List<SiteThemeEntity> siteThemeEntities = siteThemeService.find(new SiteThemeQuery().siteId(copySiteId));
        for (SiteThemeEntity siteThemeEntity : siteThemeEntities) {
            siteThemeEntity.setId(null);
            siteThemeEntity.setSiteId(targetSiteId);
        }
        siteThemeService.save(siteThemeEntities);

        List<SiteThemeMatterEntity> siteThemeMatterEntities = siteThemeMatterService.find(new SiteThemeMatterQuery().siteId(copySiteId));
        for (SiteThemeMatterEntity siteThemeMatterEntity : siteThemeMatterEntities) {
            siteThemeMatterEntity.setId(null);
            siteThemeMatterEntity.setSiteId(targetSiteId);
            siteThemeMatterEntity.setSiteName(siteEntity.getSiteName());
        }
        siteThemeMatterService.save(siteThemeMatterEntities);


        return Rest.ok();
    }


    @PostMapping(value = "saveSiteMattersTwo")
    @UnAuth
    public Rest<String> saveSiteMattersTwo(@RequestBody SiteQuery siteQuery) {
        log.info("保存站点事项相关222==");
        //复制部门
        Long copySiteId = siteQuery.getCopySiteId();
        Long targetSiteId = siteQuery.getTargetSiteId();

        SiteEntity siteEntity = siteService.get(targetSiteId);

/*        Long[] ids = siteMatterService.find(new SiteMatterQuery().siteId(targetSiteId)).stream().map(SiteMatterEntity::getId).toArray(Long[]::new);
        siteMatterService.remove(ids,null);*/

        Map<String, Long> targetDeptMap = deptService.find(new DeptQuery().siteId(targetSiteId)).stream().collect(Collectors.toMap(x -> x.getDeptNumber(), y -> y.getId(), (o, n) -> n));

        SiteMatterQuery siteMatterQuery = new SiteMatterQuery();
        siteMatterQuery.setSiteId(targetSiteId);
        List<SiteMatterEntity> siteMatterEntities = siteMatterService.find(siteMatterQuery);
        List<SiteMatterEntity> targetSiteMatterEntities = new ArrayList<>();
        for (SiteMatterEntity siteMatterEntity : siteMatterEntities) {
            siteMatterEntity.getDeptId();
           String targetDeptCode= siteMatterEntity.getDeptCode()+ "_50";


            Long targetDeptId = targetDeptMap.get(targetDeptCode);
            if(!ObjectUtils.isEmpty(targetDeptId)){

                siteMatterEntity.setDeptId(targetDeptId);
                siteMatterEntity.setDeptCode(targetDeptCode);
                targetSiteMatterEntities.add(siteMatterEntity);
            }
        }

        if(!ObjectUtils.isEmpty(targetSiteMatterEntities)){
            siteMatterService.update(targetSiteMatterEntities);
        }



        return Rest.ok();
    }


    @PostMapping(value = "limit")
    @UnAuth
    //@TokenBucketLimit
    public Rest<String> limit() {
        log.info("limit in");
        try {
            String tableName = "mortals_sys_area";
            //CacheDataUtil.getInstance().getBaseData()

            return Rest.ok();
        } catch (Exception e) {
            log.error("异常：", e.getMessage());
            return Rest.fail(e.getMessage());
        }
    }


    @GetMapping(value = "genApp")
    @UnAuth
    public Rest<String> genApp() {
        try {

            AppDatasetQuery appDatasetQuery = new AppDatasetQuery();
            appDatasetQuery.setAppId(304L);
            List<AppDatasetEntity> appDatasetEntities = appDatasetService.find(appDatasetQuery);
            StringBuilder sb = new StringBuilder();
            for (AppDatasetEntity appDatasetEntity : appDatasetEntities) {


                sb.append(String.format("INSERT INTO `mortals_sys_app_info_field`(`id`, `datasetId`, `fieldCode`, `fieldName`, `fieldType`, `fieldTypeValue`, `dataType`, `fieldValue`, `defaultValue`, `fieldLen`, `fieldNull`, `isList`, `fieldOrderNo`, `remark`, `createTime`, `createUserId`, `updateTime`, `updateUserId`, `serviceApi`, `serviceApiParams`) VALUES (null, %s, 'name', '政策全称', 'input', '', 'string', '', '', 128, 0, 1, 99, '', '2024-05-29 17:55:38', 1, '2024-05-29 17:55:38', 1, '', '');", appDatasetEntity.getId()));
                sb.append("\n");
            }
            log.info("sql:{}", sb.toString());

            //INSERT INTO `mortals_sys_app_info_field`(`id`, `datasetId`, `fieldCode`, `fieldName`, `fieldType`, `fieldTypeValue`, `dataType`, `fieldValue`, `defaultValue`, `fieldLen`, `fieldNull`, `isList`, `fieldOrderNo`, `remark`, `createTime`, `createUserId`, `updateTime`, `updateUserId`, `serviceApi`, `serviceApiParams`) VALUES (null, 755, 'name', '政策全称', 'input', '', 'string', '', '', 128, 0, 1, 99, '', '2024-05-29 17:55:38', 1, '2024-05-29 17:55:38', 1, '', '');

            return Rest.ok();
        } catch (Exception e) {
            log.error("异常：", e.getMessage());
            return Rest.fail(e.getMessage());
        }
    }


    public static void main(String[] args) {
        System.out.println(1001 / 500);

       /* HttpClient http = null;
        CookieStore httpCookieStore = new BasicCookieStore();
        http = HttpClientBuilder.create().setDefaultCookieStore(httpCookieStore).build();

        *//* do stuff *//*
        HttpGet httpRequest = new HttpGet("http://zxbl.sczwfw.gov.cn/app/account/imageCode");
        HttpResponse httpResponse = null;
        try {
            httpResponse = http.execute(httpRequest);
            byte[] data = EntityUtils.toByteArray(httpResponse.getEntity());
            String encode = Base64.encode(data);

            log.info("encode64:{}", encode);
            // String resp = com.mortals.framework.util.HttpUtil.processMultipartResponse(httpResponse);
            // log.info("resp:{}", resp);
            //String content = EntityUtils.toString(entity, charset);
            //httpResponse.getEntity()
            //httpResponse.toString()
        } catch (Exception e) {
            log.error("异常", e);
        }
        *//* check cookies *//*
        String cookieStr = Arrays.asList(httpCookieStore.getCookies()).stream().map(item -> JSON.toJSONString(item)).collect(Collectors.joining("|"));
        // log.info("cookies:{}", cookieStr);*/
    }


}
