package com.mortals.xhx.busiz.web;

import cn.hutool.core.util.RandomUtil;
import cn.hutool.http.HttpUtil;
import cn.hutool.http.Method;
import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.mortals.framework.annotation.UnAuth;
import com.mortals.framework.common.Rest;
import com.mortals.xhx.busiz.req.MidReq;
import com.mortals.xhx.busiz.rsp.SignResp;
import com.mortals.xhx.common.utils.EncryptionUtils;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.codec.binary.Base64;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.util.ObjectUtils;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import java.util.HashMap;
import java.util.Map;

/**
 * 微中台签名接口
 *
 * @author:
 * @date: 2023/3/2 18:07
 */
@RestController
@RequestMapping("mid")
@Slf4j
public class MidSignApiController {

    @Value("${mid.midUrl:http://112.19.80.237:9000/masm_agent}")
    private String midUrl;

    @Value("${mid.appId:01C67D56D0630517}")
    private String appId;

    @Value("${mid.appKey:a0b946c858aa8d06}")
    private String appKey;

    @Value("${mid.secretKey:ad80c59e575a78ab}")
    private String secretKey;

    @Value("${mid.publickey:MFwwDQYJKoZIhvcNAQEBBQADSwAwSAJBAIwYDUWp+QkuihFJxWJmAQ0ZmdTVbOg7yETtckUZkUQ6JUKEzKz9UIbmjrKS8kLY39QDjyzCuOzBR2eF2ZrS89MCAwEAAQ==}")
    private String publickey = "";
    @Value("${mid.user:yibinTFTB}")
    private String user;
    @Value("${mid.pwd:f82c10dea5fc4936853ff90e576ac216}")
    private String pwd;


    @PostMapping(value = "sign")
    @UnAuth
    public Rest<SignResp> midSign(@RequestBody MidReq midReq) {
        SignResp signResp = new SignResp();
        try {
            Map<String, String> headerMap = new HashMap<>();

            if ("post".equalsIgnoreCase(midReq.getMethod())) {
                headerMap.put("appId", appId);
                headerMap.put("appKey", appKey);
                JSONObject object1 = JSONObject.parseObject(midReq.getBody());
                headerMap.put("body", object1.toJSONString());
                String timeStamp = System.currentTimeMillis() + "";
                headerMap.put("timeStamp", timeStamp);
                String nonce = RandomUtil.randomNumbers(6);
                headerMap.put("nonce", nonce);
                headerMap.put("secretKey", secretKey);

                signResp.setAppId(appId);
                signResp.setAppKey(appKey);
                signResp.setTimeStamp(timeStamp);
                signResp.setNonce(nonce);
                signResp.setSecretKey(secretKey);
            } else if ("get".equalsIgnoreCase(midReq.getMethod())) {
                HashMap<String, String> paramsMap = JSON.parseObject(midReq.getBody(), HashMap.class);
                if (!paramsMap.isEmpty()) {
                    for (Map.Entry<String, String> entry : paramsMap.entrySet()) {
                        headerMap.put(entry.getKey(), entry.getValue());
                    }
                }
            } else {

            }

            StringBuilder signSb = new StringBuilder();

            signSb.append("appId").append("=").append(headerMap.get("appId")).append("&");
            signSb.append("appKey").append("=").append(headerMap.get("appKey")).append("&");
            signSb.append("body").append("=").append(headerMap.get("body")).append("&");
            signSb.append("nonce").append("=").append(headerMap.get("nonce")).append("&");
            signSb.append("secretKey").append("=").append(headerMap.get("secretKey")).append("&");
            signSb.append("timeStamp").append("=").append(headerMap.get("timeStamp")).append("&");
/*            for (Map.Entry<String, String> params : headerMap.entrySet()) {
                signSb.append(params.getKey()).append("=").append(params.getValue() + "").append("&");
            }*/
            String signStr = signSb.substring(0, signSb.length() - 1);
            log.info("签名源字符串: " + signStr);
            String sign = EncryptionUtils.SHA256(signStr);
            log.info("签名计算结果: " + sign);
            signResp.setSign(sign);
            return Rest.ok(signResp);
        } catch (Exception e) {
            log.error("签名异常", e);
            return Rest.fail("签名异常！");
        }
    }

    /**
     * 透传请求
     *
     * @param midReq
     * @return
     */
    @PostMapping(value = "trans")
    @UnAuth
    public Rest<String> trans(@RequestBody MidReq midReq) {
        try {
            Map<String, String> headerMap = new HashMap<>();
            StringBuilder signSb = new StringBuilder();
            headerMap.put("appId", appId);
            headerMap.put("appKey", appKey);
            String timeStamp = System.currentTimeMillis() + "";
            headerMap.put("timeStamp", timeStamp);
            String nonce = RandomUtil.randomNumbers(6);
            headerMap.put("nonce", nonce);
            headerMap.put("secretKey", secretKey);
            JSONObject object = JSONObject.parseObject(midReq.getBody());
            headerMap.put("body", object.toJSONString());

            signSb.append("appId").append("=").append(headerMap.get("appId")).append("&");
            signSb.append("appKey").append("=").append(headerMap.get("appKey")).append("&");
            signSb.append("body").append("=").append(headerMap.get("body")).append("&");
            signSb.append("nonce").append("=").append(headerMap.get("nonce")).append("&");
            signSb.append("secretKey").append("=").append(headerMap.get("secretKey")).append("&");
            signSb.append("timeStamp").append("=").append(headerMap.get("timeStamp")).append("&");
            String signStr = signSb.substring(0, signSb.length() - 1);
            log.info("签名源字符串: " + signStr);
            String sign = EncryptionUtils.SHA256(signStr);
            log.info("签名计算结果: " + sign);
            headerMap.put("sign", sign);
            if (ObjectUtils.isEmpty(midReq.getType()) || midReq.getType() == 2) {
                //rsa提交
                //todo 添加rsa非对称加密
                String pwdRsa = pwd + "#" + timeStamp;
                pwdRsa = EncryptionUtils.encrypt(pwdRsa, publickey);
                String param = "{\"USER\":\"" + user + "\",\"PWD\":\"" + pwdRsa + "\"}";
                param = Base64.encodeBase64String(param.getBytes("UTF-8"));
                headerMap.put("param", param);
                String request = Base64.encodeBase64String(object.toJSONString().getBytes("UTF-8"));
                JSONObject reqBody = new JSONObject();
                reqBody.put("request", request);
                String fullUrl = midUrl + midReq.getPath();
                log.info("url:{} \n body:{} \n", fullUrl, reqBody.toJSONString());
                String body = HttpUtil.createRequest(Method.POST, fullUrl).headerMap(headerMap, true).body(reqBody.toJSONString()).execute().body();
                JSONObject bodyJson = JSONObject.parseObject(body);
                log.info("mid resp:" + bodyJson.toJSONString());
                return Rest.ok(bodyJson.toJSONString());
            } else {
                //sha加密提交
                String fullUrl = midUrl + midReq.getPath();

                if ("post".equalsIgnoreCase(midReq.getMethod())) {
                    log.info("mid post url:{},body:{}", fullUrl, headerMap.get("body"));
                    String body = HttpUtil.createRequest(Method.POST, fullUrl).headerMap(headerMap, true).body(headerMap.get("body")).execute().body();
                    JSONObject bodyJson = JSONObject.parseObject(body);
                    log.info("mid resp:" + bodyJson.toJSONString());
                    return Rest.ok(bodyJson.toJSONString());
                } else if ("get".equalsIgnoreCase(midReq.getMethod())) {
                    Map<String, Object> paramsMap = JSON.parseObject(midReq.getBody(), HashMap.class);
                    log.info("mid get url:{},body:{}", fullUrl, JSON.toJSONString(paramsMap));
//                    if (!paramsMap.isEmpty()) {
//                        for (Map.Entry<String, Object> entry : paramsMap.entrySet()) {
//                            headerMap.put(entry.getKey(), entry.getValue());
//                        }
//                    }
                    String body = HttpUtil.createRequest(Method.GET, fullUrl).headerMap(headerMap, true).form(paramsMap).execute().body();
                    JSONObject bodyJson = JSONObject.parseObject(body);
                    log.info("mid resp:" + bodyJson.toJSONString());
                    return Rest.ok(bodyJson.toJSONString());
                }else{
                    log.info("mid post url:{},body:{}", fullUrl, headerMap.get("body"));
                    String body = HttpUtil.createRequest(Method.POST, fullUrl).headerMap(headerMap, true).body(headerMap.get("body")).execute().body();
                    JSONObject bodyJson = JSONObject.parseObject(body);
                    log.info("mid resp:" + bodyJson.toJSONString());
                    return Rest.ok(bodyJson.toJSONString());
                }
            }
        } catch (Exception e) {
            log.error("透传请求异常", e);
            return Rest.fail("透传请求异常！");
        }
    }

}
