package com.mortals.xhx.common.utils;

import com.arcsoft.face.*;
import com.arcsoft.face.enums.DetectMode;
import com.arcsoft.face.enums.DetectOrient;
import com.arcsoft.face.enums.ErrorInfo;
import com.arcsoft.face.toolkit.ImageInfo;
import com.mortals.framework.exception.AppException;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Component;

import java.util.ArrayList;
import java.util.List;

import static com.arcsoft.face.toolkit.ImageFactory.getRGBData;

@Slf4j
@Component
public class FaceUtil {

    public FaceEngine initFace(String appId, String sdkKey) {
        FaceEngine faceEngine = new FaceEngine(getClass().getResource(getOsName()).getPath());
        //激活引擎
        int errorCode = faceEngine.activeOnline(appId, sdkKey);
        isTrue(!(errorCode != ErrorInfo.MOK.getValue() && errorCode != ErrorInfo.MERR_ASF_ALREADY_ACTIVATED.getValue()), "引擎激活失败");
        ActiveFileInfo activeFileInfo = new ActiveFileInfo();
        errorCode = faceEngine.getActiveFileInfo(activeFileInfo);
        isTrue(!(errorCode != ErrorInfo.MOK.getValue() && errorCode != ErrorInfo.MERR_ASF_ALREADY_ACTIVATED.getValue()), "获取激活文件信息失败");
        //引擎配置
        EngineConfiguration engineConfiguration = new EngineConfiguration();
        engineConfiguration.setDetectMode(DetectMode.ASF_DETECT_MODE_IMAGE);
        engineConfiguration.setDetectFaceOrientPriority(DetectOrient.ASF_OP_ALL_OUT);
        engineConfiguration.setDetectFaceMaxNum(10);
        engineConfiguration.setDetectFaceScaleVal(16);
        //功能配置
        FunctionConfiguration functionConfiguration = new FunctionConfiguration();
        functionConfiguration.setSupportAge(true);
        functionConfiguration.setSupportFace3dAngle(true);
        functionConfiguration.setSupportFaceDetect(true);
        functionConfiguration.setSupportFaceRecognition(true);
        functionConfiguration.setSupportGender(true);
        functionConfiguration.setSupportLiveness(true);
        functionConfiguration.setSupportIRLiveness(true);
        engineConfiguration.setFunctionConfiguration(functionConfiguration);
        //初始化引擎
        errorCode = faceEngine.init(engineConfiguration);
        isTrue(errorCode == ErrorInfo.MOK.getValue(), "初始化引擎失败");
        return faceEngine;
    }


    /**
     * 人脸检测、特征提取
     *
     * @param faceEngine
     * @param bytes
     * @return
     */
    public byte[] featureData(FaceEngine faceEngine, byte[] bytes) {
        //人脸检测
        ImageInfo imageInfo = getRGBData(bytes);
        List<FaceInfo> faceInfoList = new ArrayList<FaceInfo>();
        faceEngine.detectFaces(imageInfo.getImageData(), imageInfo.getWidth(), imageInfo.getHeight(), imageInfo.getImageFormat(), faceInfoList);
        //特征提取
        FaceFeature faceFeature = new FaceFeature();
        faceEngine.extractFaceFeature(imageInfo.getImageData(), imageInfo.getWidth(), imageInfo.getHeight(), imageInfo.getImageFormat(), faceInfoList.get(0), faceFeature);
        return faceFeature.getFeatureData();
    }


    /**
     * 特征比对
     *
     * @param targeFace
     * @param sourceFace
     * @return
     */
    public boolean featureComparison(FaceEngine faceEngine, byte[] targeFace, byte[] sourceFace) {
        //特征比对
        FaceFeature targetFaceFeature = new FaceFeature();
        targetFaceFeature.setFeatureData(targeFace);
        FaceFeature sourceFaceFeature = new FaceFeature();
        sourceFaceFeature.setFeatureData(sourceFace);
        FaceSimilar faceSimilar = new FaceSimilar();
        faceEngine.compareFaceFeature(targetFaceFeature, sourceFaceFeature, faceSimilar);
        float score = faceSimilar.getScore();
        if (score > 0.8) {
            return true;
        }
        return false;
    }

    public void unInit(FaceEngine faceEngine) {
        faceEngine.unInit();
    }

    public String getOsName() {
        String os = System.getProperty("os.name");
        String osName = os.toLowerCase().startsWith("win") ? "/face_lib/win64" : "/face_lib/linux";
        return osName;
    }


    private void isTrue(boolean b, String errorMsg) {
        if (!b) {
            throw new AppException(errorMsg);
        }
    }

}
