package com.mortals.xhx.common.utils;


import lombok.extern.slf4j.Slf4j;
import org.bouncycastle.jce.provider.BouncyCastleProvider;
import org.springframework.security.crypto.bcrypt.BCryptPasswordEncoder;

import javax.crypto.Cipher;
import javax.crypto.NoSuchPaddingException;
import javax.crypto.spec.SecretKeySpec;
import java.nio.charset.StandardCharsets;
import java.security.AlgorithmParameters;
import java.security.NoSuchAlgorithmException;

/**
 * 安全服务工具类
 *
 * @author zxfei
 */
@Slf4j
public class SecurityUtils {

    private static Cipher SM4Cipher = null;

    private static Cipher SM4DecryptCipher;

    static {
        try {
            SM4DecryptCipher = Cipher.getInstance("SM4/ECB/PKCS7Padding", new BouncyCastleProvider());

        } catch (NoSuchAlgorithmException e) {
            throw new RuntimeException(e);
        } catch (NoSuchPaddingException e) {
            throw new RuntimeException(e);
        }
    }

    private static BouncyCastleProvider bouncyCastleProvider = new BouncyCastleProvider();


    public  static BouncyCastleProvider geBouncyCastleProvider() {
        return bouncyCastleProvider;
    }

    public synchronized static Cipher getSM4Cipher(String key) {

        try {
            if (SM4Cipher == null) {
                //创建实例之前可能会有一些准备性的耗时工作
                Thread.sleep(300);
                synchronized (SecurityUtils.class) {
                    if (SM4Cipher == null) {//二次检查
                        SM4Cipher = Cipher.getInstance("SM4/ECB/PKCS7Padding", new BouncyCastleProvider());

                    }
                }

            }
        } catch (Exception e) {
            log.error("SM4Cipher init error", e);
        }
        return SM4Cipher;

    }


    public synchronized static Cipher getSM4DecryptCipher(String key) {

        try {
            if (SM4DecryptCipher == null) {
                //创建实例之前可能会有一些准备性的耗时工作
                Thread.sleep(300);
                synchronized (SecurityUtils.class) {
                    if (SM4DecryptCipher == null) {//二次检查
                        //SM4DecryptCipher = Cipher.getInstance("SM4/ECB/PKCS7Padding", new BouncyCastleProvider());
                        SM4DecryptCipher.init(Cipher.ENCRYPT_MODE, new SecretKeySpec(key.getBytes(StandardCharsets.UTF_8), "SM4"), (AlgorithmParameters) null);

                    }
                }

            }

            SM4DecryptCipher.init(Cipher.ENCRYPT_MODE, new SecretKeySpec(key.getBytes(StandardCharsets.UTF_8), "SM4"), (AlgorithmParameters) null);

        } catch (Exception e) {
            log.error("SM4Cipher init error", e);
        }

        return SM4DecryptCipher;

    }


    /**
     * 获取Authentication
     */
    //public static Authentication getAuthentication() {
//        return SecurityContextHolder.getContext().getAuthentication();
//    }

    /**
     * 生成BCryptPasswordEncoder密码
     *
     * @param password 密码
     * @return 加密字符串
     */
    public static String encryptPassword(String password) {
        BCryptPasswordEncoder passwordEncoder = new BCryptPasswordEncoder();
        return passwordEncoder.encode(password);
    }

    /**
     * 判断密码是否相同
     *
     * @param rawPassword     真实密码
     * @param encodedPassword 加密后字符
     * @return 结果
     */
    public static boolean matchesPassword(String rawPassword, String encodedPassword) {
        BCryptPasswordEncoder passwordEncoder = new BCryptPasswordEncoder();
        return passwordEncoder.matches(rawPassword, encodedPassword);
    }

    /**
     * 是否为管理员
     *
     * @param userId 用户ID
     * @return 结果
     */
    public static boolean isAdmin(Long userId) {
        return userId != null && 1L == userId;
    }
}
