package com.mortals.xhx.daemon.task;

import cn.hutool.core.util.StrUtil;
import com.alibaba.fastjson.JSONObject;
import com.mortals.framework.exception.AppException;
import com.mortals.framework.service.ICacheService;
import com.mortals.framework.service.ITask;
import com.mortals.framework.service.ITaskExcuteService;
import com.mortals.xhx.common.code.SourceEnum;
import com.mortals.xhx.common.code.YesNoEnum;
import com.mortals.xhx.common.keys.RedisCacheKeys;
import com.mortals.xhx.module.dept.model.DeptEntity;
import com.mortals.xhx.module.dept.model.DeptQuery;
import com.mortals.xhx.module.dept.service.DeptService;
import com.mortals.xhx.module.matter.model.MatterEntity;
import com.mortals.xhx.module.matter.model.MatterQuery;
import com.mortals.xhx.module.matter.service.MatterService;
import com.mortals.xhx.module.matters.service.MattersService;
import com.mortals.xhx.module.site.model.*;
import com.mortals.xhx.module.site.service.SiteMatterService;
import com.mortals.xhx.module.site.service.SiteService;
import com.mortals.xhx.module.site.service.SiteThemeService;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.*;
import java.util.stream.Collectors;

/**
 * 统计站点部门事项
 */
@Slf4j
@Service("StatSiteDeptMatterTask")
public class StatSiteDeptMatterTaskImpl implements ITaskExcuteService {
    @Autowired
    private DeptService deptService;
    @Autowired
    private MatterService matterService;
    @Autowired
    private SiteService siteService;

    @Autowired
    private ICacheService cacheService;
    @Autowired
    private SiteMatterService siteMatterService;

    @Autowired
    private SiteThemeService siteThemeService;


    @Override
    public void excuteTask(ITask task) throws AppException {
        log.info("开始同步事项列表！");

        List<SiteEntity> siteEntities = siteService.find(new SiteQuery());
        for (SiteEntity siteEntity : siteEntities) {
            DeptQuery query = new DeptQuery();
            query.setSiteId(siteEntity.getId());
            List<DeptEntity> deptEntities = deptService.find(query);
            for (DeptEntity deptEntity : deptEntities) {

                MatterQuery matterQuery = new MatterQuery();
                matterQuery.setEventTypeShowNotList(Arrays.asList("行政处罚"));
                int total = 0;
                if (StrUtil.contains(deptEntity.getDeptNumber(), "_")) {

                    String deptCode = StrUtil.subBefore(deptEntity.getDeptNumber(), "_", true);

                    total = matterService.count(matterQuery.source(SourceEnum.政务网.getValue()).deptCode(deptCode), null);

                } else {
                    total = matterService.count(matterQuery.source(SourceEnum.政务网.getValue()).deptCode(deptEntity.getDeptNumber()), null);

                }

                DeptEntity deptQuery = new DeptEntity();
                deptQuery.setTotal(total);
                deptQuery.setUpdateTime(new Date());

                //统计入驻事项
                SiteMatterQuery siteMatterQuery = new SiteMatterQuery();
                //siteMatterQuery.setDeptCode(deptEntity.getDeptNumber());
                siteMatterQuery.setDeptId(deptEntity.getId());
                siteMatterQuery.setSource(SourceEnum.政务网.getValue());
                siteMatterQuery.setHallCheckIn(YesNoEnum.YES.getValue());
                int incount = siteMatterService.count(siteMatterQuery, null);
                deptQuery.setInNum(incount);

                DeptEntity condition = new DeptEntity();
                condition.setId(deptEntity.getId());
                deptService.getDao().update(deptQuery, condition);
            }

        }
        //分组暂存 站点 个人企业  主题
        Map<String, List<SiteThemeEntity>> collect = siteThemeService.find(new SiteThemeQuery())
                .parallelStream()
                .collect(Collectors.groupingBy(x -> x.getSiteId() + "_" + x.getUserType() + "_" + x.getThemeCode()
                ));
        Iterator<Map.Entry<String, List<SiteThemeEntity>>> iterator = collect.entrySet().iterator();
        while (iterator.hasNext()) {
            Map.Entry<String, List<SiteThemeEntity>> next = iterator.next();
            if (next.getValue().size() == 0) {
                iterator.remove();
            }
        }
        //重新  构建redis siteId+userType= >themeCodelist
        List<SiteThemeEntity> themeEntities = collect.values().stream().flatMap(i -> i.stream()).collect(Collectors.toList());

        themeEntities.stream()
                .collect(Collectors.groupingBy(x -> x.getSiteId() + "_" + x.getUserType()))
                .entrySet()
                .stream()
                .forEach(item -> {
                    String key = item.getKey();
                    List<SiteThemeEntity> values = item.getValue();
                    String[] split = key.split("_");
                    String siteId = split[0];
                    String userType = split[1];
                    cacheService.hset(RedisCacheKeys.getSiteUserTypeThemeCodeKey() + siteId, userType, JSONObject.toJSONString(values));
                });
    }

    @Override
    public void stopTask(ITask task) throws AppException {

    }


    public static void main(String[] args) {
        String deptCode = StrUtil.subBefore("4166885406123696128_50", "_", true);


        System.out.println(deptCode);
    }
}
