package com.mortals.xhx.daemon.task;

import cn.hutool.core.date.DateUtil;
import cn.hutool.http.HttpUtil;
import com.alibaba.fastjson.JSON;
import com.fasterxml.jackson.core.JsonProcessingException;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.mortals.framework.common.Rest;
import com.mortals.framework.exception.AppException;
import com.mortals.framework.service.ITask;
import com.mortals.framework.service.ITaskExcuteService;
import com.mortals.xhx.common.code.YesNoEnum;
import com.mortals.xhx.common.pdu.RespData;
import com.mortals.xhx.common.pdu.device.DevicePdu;
import com.mortals.xhx.common.utils.BeanUtil;
import com.mortals.xhx.feign.device.IDeviceFeign;
import com.mortals.xhx.module.device.model.DeviceEntity;
import com.mortals.xhx.module.device.model.DeviceQuery;
import com.mortals.xhx.module.device.service.DeviceService;
import com.mortals.xhx.module.holiday.model.HolidayEntity;
import com.mortals.xhx.module.holiday.model.HolidayQuery;
import com.mortals.xhx.module.holiday.model.vo.HolidayVo;
import com.mortals.xhx.module.holiday.service.HolidayService;
import com.mortals.xhx.module.site.model.SiteEntity;
import com.mortals.xhx.module.site.model.SiteQuery;
import com.mortals.xhx.module.site.service.SiteService;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;
import org.springframework.util.ObjectUtils;

import java.io.IOException;
import java.util.Date;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * 同步假期与周末需要补班的
 *
 * @author: zxfei
 * @date: 2023/10/8 16:31
 */
@Slf4j
@Service("SyncHolidayTask")
public class SyncHolidayTaskImpl implements ITaskExcuteService {
    @Autowired
    private HolidayService holidayService;

    @Value("${holidayUrl:https://timor.tech/api/holiday/year/2024}")
    private String holidayUrl;


    @Override
    public void excuteTask(ITask task) throws AppException {
        log.info("开始同步当年假期列表！");
        syncHoliday();
        log.info("结束同步当年假期列表！");
    }

    private void syncHoliday() throws AppException {
        holidayUrl += DateUtil.year(new Date());
        log.info("holidayUrl:{}", holidayUrl);
        String holidayJson = HttpUtil.get(holidayUrl);
        ObjectMapper mapper = new ObjectMapper();

        //查询全年节假日、调休
        Map holidayMap = null;
        try {
            holidayMap = mapper.readValue(holidayJson, Map.class);
            LinkedHashMap holidayList = (LinkedHashMap) holidayMap.get("holiday");
            holidayList.forEach((key, value) -> {
                Map value1 = (Map) value;
                String date = value1.get("date").toString();
                //判断当前日期的节假日是否存在
                HolidayQuery holidayQuery = new HolidayQuery();
                holidayQuery.setStartTimeStart(date);
                holidayQuery.setStartTimeEnd(date);
                HolidayEntity holidayEntity = holidayService.selectOne(holidayQuery);

                if (ObjectUtils.isEmpty(holidayEntity)) {
                    //新增
                    holidayEntity = new HolidayEntity();
                    holidayEntity.initAttrValue();

                    holidayEntity.setSiteId(1L);
                    holidayEntity.setStartTime(DateUtil.parseDate(date));
                    holidayEntity.setEndTime(DateUtil.parseDate(date));
                    holidayEntity.setYear(DateUtil.year(new Date()));
                    String name = "法定节假日(" + value1.get("name").toString() + ")";

                    Boolean holiday = (Boolean) value1.get("holiday");
                    if (!holiday) {
                        name = "节假日调休补班(" + value1.get("target").toString() + ")";
                        holidayEntity.setWorkorholiday(1);
                    } else {
                        holidayEntity.setWorkorholiday(0);
                    }
                    holidayEntity.setName(name);

                    holidayEntity.setCreateTime(new Date());
                    holidayEntity.setCreateUserId(1L);

                    holidayService.save(holidayEntity);
                }
            });
        } catch (JsonProcessingException e) {
            throw new AppException(e.getMessage());
        }


    }


    @Override
    public void stopTask(ITask task) throws AppException {

    }
}
