package com.mortals.xhx.daemon.task;

import cn.hutool.http.HttpUtil;
import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.mortals.framework.exception.AppException;
import com.mortals.framework.model.PageInfo;
import com.mortals.framework.service.ITask;
import com.mortals.framework.service.ITaskExcuteService;
import com.mortals.framework.utils.ServletUtils;
import com.mortals.xhx.base.system.upload.service.UploadService;
import com.mortals.xhx.module.matter.model.MatterDatumFileEntity;
import com.mortals.xhx.module.matter.model.MatterDatumFileQuery;
import com.mortals.xhx.module.matter.service.MatterDatumFileService;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.ObjectUtils;
import org.springframework.util.StopWatch;
import org.springframework.web.multipart.MultipartFile;

import java.io.ByteArrayInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.net.ConnectException;
import java.net.SocketTimeoutException;
import java.util.ArrayList;
import java.util.List;

import static com.mortals.xhx.common.key.Constant.MAX_RETRY_COUNT;

/**
 * 同步全部站点事项材料附件
 *
 * @author: zxfei
 * @date: 2024/2/27 15:34
 */
@Slf4j
//@Service("SyncMatterDaumFileTask")
public class SyncMatterDaumFileTaskImpl implements ITaskExcuteService {

    @Autowired
    private MatterDatumFileService matterDatumFileService;
    @Autowired
    private UploadService uploadService;


    @Override
    public void excuteTask(ITask task) throws AppException {
        log.info("开始同步材料列表附件！ 分页获取");
        int total = matterDatumFileService.count(new MatterDatumFileQuery(), null);
        int page = total / 1000 + 1;
        StopWatch stopWatch = new StopWatch("watch daum file");
        for (int i = 1; i <= page; i++) {
            try {
                stopWatch.start("同步附件");
                PageInfo pageInfo = new PageInfo();
                pageInfo.setCurrPage(i);
                pageInfo.setPrePageResult(1000);
                List<MatterDatumFileEntity> datumFileEntities = matterDatumFileService.find(new MatterDatumFileQuery(), pageInfo, null).getList();
                List<MatterDatumFileEntity> updateList = new ArrayList<>();

                for (MatterDatumFileEntity datumFileEntity : datumFileEntities) {
                    String localFileUrl = datumFileEntity.getLocalFileUrl();
                    if (ObjectUtils.isEmpty(localFileUrl) && !ObjectUtils.isEmpty(datumFileEntity.getFileUrl())) {
                        String encryUrl = "http://www.sczwfw.gov.cn/jiq/interface/item/annex/encryptUrl?id=" + datumFileEntity.getFileUrl();
                        // Thread.sleep(300);
                        String resp = HttpUtil.get(encryUrl, 60000);
                        JSONObject obj = JSON.parseObject(resp);
                        String code = obj.getString("code");
                        String data = obj.getString("data");
                        if ("0".equals(code)) {
                            //下载文件
                            int reCount = 0;
                            while (reCount < 3) {
                                try {
                                    byte[] fileData = HttpUtil.downloadBytes(data);
                                    InputStream inputStream = new ByteArrayInputStream(fileData);
                                    MultipartFile file = ServletUtils.getMultipartFile(inputStream, datumFileEntity.getFileName());
                                    String filePath = uploadService.saveFileUpload(file, "file/fileupload/daumfile", null);

                                    if (!ObjectUtils.isEmpty(filePath)) {
                                        datumFileEntity.setLocalFileUrl("/" + filePath);
                                        MatterDatumFileEntity updateEntity = new MatterDatumFileEntity();
                                        updateEntity.setId(datumFileEntity.getId());
                                        updateEntity.setLocalFileUrl(datumFileEntity.getLocalFileUrl());
                                        updateList.add(updateEntity);
                                    }
                                    break;
                                } catch (Exception e) {
                                    log.error("下载附件异常，id:{},url:{}",datumFileEntity.getFileUrl(),data, e);
                                    reCount++;
                                    try {
                                        Thread.sleep(2000);
                                    } catch (InterruptedException ie) {
                                    }
                                }
                            }
                        }
                    }
                }
                if (!ObjectUtils.isEmpty(updateList)) {
                    log.info("开始更新附件本地地址==》{}", updateList.size());
                    matterDatumFileService.update(updateList);
                }

                log.info("同步材料附件{}条完成,耗时：{}s", updateList.size(), stopWatch.getLastTaskTimeMillis() / 1000);
            } catch (Exception e) {
                log.error("同步材料附件异常", e);
            }

            stopWatch.stop();
        }

    }


    @Override
    public void stopTask(ITask task) throws AppException {

    }


    public static void main(String[] args) {
        int total = 1001;
        int page = total / 1000;
        System.out.println(page);
    }
}
