package com.mortals.xhx.daemon.task;

import com.mortals.framework.exception.AppException;
import com.mortals.framework.service.ITask;
import com.mortals.framework.service.ITaskExcuteService;
import com.mortals.xhx.module.dept.service.DeptService;
import com.mortals.xhx.module.matter.service.MatterExtService;
import com.mortals.xhx.module.site.model.SiteEntity;
import com.mortals.xhx.module.site.service.SiteMatterService;
import com.mortals.xhx.module.site.service.SiteService;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.StopWatch;

import java.util.List;

/**
 * 同步全部站点事项
 *
 * @author: zxfei
 * @date: 2024/2/27 15:34
 */
@Slf4j
@Service("SyncMatterTask")
public class SyncMatterTaskImpl implements ITaskExcuteService {

    @Autowired
    private SiteService siteService;

    @Autowired
    private SiteMatterService siteMatterService;

    @Autowired
    private MatterExtService matterExtService;
    @Autowired
    private DeptService deptService;


    @Override
    public void excuteTask(ITask task) throws AppException {
        log.info("开始同步事项！");
        List<SiteEntity> siteEntities = siteService.find(new SiteEntity());
        StopWatch stopWatch = new StopWatch("stopwatch syncmatter");
        try {
            for (SiteEntity siteEntity : siteEntities) {
                log.info("开始同步站点部门==》{}", siteEntity.getSiteName());
                stopWatch.start("开始同步站点部门");
                deptService.syncDeptBySiteId(siteEntity, null);
                stopWatch.stop();
                log.info("同步站点部门完成,耗时：{}s", stopWatch.getLastTaskTimeMillis() / 1000);
                log.info("开始同步事项==》{}", siteEntity.getSiteName());
                stopWatch.start("同步删除本地多余的政务事项");
                log.info("同步删除本地多余的政务事项");
                matterExtService.syncDelMatterBySiteId(siteEntity, null);
                stopWatch.stop();
                log.info("同步删除本地多余的政务事项完成,耗时：{}s", stopWatch.getLastTaskTimeMillis() / 1000);

                stopWatch.start("开始同步事项方法");
                matterExtService.syncMatterBySiteId(siteEntity, null);
                stopWatch.stop();
                log.info("同步事项完成,耗时：{}s", stopWatch.getLastTaskTimeMillis() / 1000);

                log.info("开始添加站点事项==》{}", siteEntity.getSiteName());
                stopWatch.start("开始添加站点事项");
                siteMatterService.doSyncSiteMatter(siteEntity, null);
                stopWatch.stop();
                log.info("开始添加站点事项,耗时：{} s", stopWatch.getLastTaskTimeMillis() / 1000);

                //同步主题
                stopWatch.start("开始同步主题事项");
                matterExtService.syncMatterThemeBySiteId(siteEntity, null);
                stopWatch.stop();
                log.info("同步主题事项完成,耗时：{}s", stopWatch.getLastTaskTimeMillis() / 1000);
            }
        } catch (Exception e) {
            log.error("同步站点事项异常，站点:{}", siteEntities.get(0).getSiteName(), e);
        }
        log.info("结束同步事项！,耗时：{}秒", stopWatch.getLastTaskTimeMillis() / 1000);
    }


    @Override
    public void stopTask(ITask task) throws AppException {

    }
}
