package com.mortals.xhx.module.app.web;

import cn.hutool.core.net.url.UrlBuilder;
import cn.hutool.core.util.ZipUtil;
import com.mortals.framework.annotation.UnAuth;
import com.mortals.framework.ap.GlobalSysInfo;
import com.mortals.framework.common.Rest;
import com.mortals.framework.exception.AppException;
import com.mortals.framework.model.Context;
import com.mortals.framework.model.OrderCol;
import com.mortals.framework.util.DataUtil;
import com.mortals.framework.web.BaseCRUDJsonBodyMappingController;
import com.mortals.xhx.base.system.param.service.ParamService;
import com.mortals.xhx.common.code.AppTypeEnum;
import com.mortals.xhx.common.code.YesNoEnum;
import com.mortals.xhx.common.key.Constant;
import com.mortals.xhx.module.app.model.AppCategoryEntity;
import com.mortals.xhx.module.app.model.AppCategoryQuery;
import com.mortals.xhx.module.app.model.AppEntity;
import com.mortals.xhx.module.app.model.AppQuery;
import com.mortals.xhx.module.app.service.AppCategoryService;
import com.mortals.xhx.module.app.service.AppService;
import com.mortals.xhx.module.site.model.SiteEntity;
import com.mortals.xhx.module.site.model.SiteQuery;
import com.mortals.xhx.module.site.service.SiteService;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.ObjectUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import java.util.*;
import java.util.stream.Collectors;

import static com.mortals.xhx.common.key.Constant.CUSTAPP_ROOT_PATH;

/**
 * 自助终端应用
 *
 * @author zxfei
 * @date 2022-11-28
 */
@RestController
@RequestMapping("app")
@Slf4j
public class AppController extends BaseCRUDJsonBodyMappingController<AppService, AppEntity, Long> {

    @Autowired
    private ParamService paramService;
    @Autowired
    private SiteService siteService;
    @Autowired
    private AppCategoryService appCategoryService;


    public AppController() {
        super.setModuleDesc("应用");
    }

    @Override
    protected void init(Map<String, Object> model, Context context) {
        this.addDict(model, "type", AppTypeEnum.getEnumMap());
        this.addDict(model, "shelves", paramService.getParamBySecondOrganize("App", "shelves"));
        //this.addDict(model, "appThemeName", paramService.getParamBySecondOrganize("App", "appThemeName"));
        this.addDict(model, "distribute", YesNoEnum.getEnumMap());
        this.addDict(model, "dateUpdate", YesNoEnum.getEnumMap());
        super.init(model, context);
    }

    /**
     * @param query
     * @return
     */
    @Override
    @UnAuth
    public Rest<Object> list(@RequestBody AppEntity query) {
        return super.list(query);
    }

    /**
     * @param query
     * @param model
     * @param context
     * @throws AppException
     */
    @Override
    protected void doListBefore(AppEntity query, Map<String, Object> model, Context context) throws AppException {
        String serverName = request.getHeader("server-name");
        Integer serverPort = DataUtil.converStr2Int(request.getHeader("server-port"), 0);
        //log.info("【应用请求】【请求体】-->serverName:{} ,port:{}", serverName, serverPort);
        query.setServerName(serverName);
        query.setServerPort(serverPort);
        super.doListBefore(query, model, context);
    }

    /**
     * @param query
     * @param model
     * @param context
     * @return
     * @throws AppException
     */
    @Override
    protected int doListAfter(AppEntity query, Map<String, Object> model, Context context) throws AppException {
        model.put("appCategory", new ArrayList<>());
        if (!ObjectUtils.isEmpty(query.getSiteId())) {
            AppCategoryQuery appCategoryQuery = new AppCategoryQuery();
            appCategoryQuery.setSiteId(query.getSiteId());
            appCategoryQuery.setOrderColList(Arrays.asList(new OrderCol("sort")));
            List<AppCategoryEntity> appCategoryEntities = appCategoryService.find(appCategoryQuery);
            if (!ObjectUtils.isEmpty(appCategoryEntities)) {
                model.put("appCategory", appCategoryEntities);
                Map<String,Object> appThemeName = new HashMap<>();
                for (AppCategoryEntity appCategory:appCategoryEntities){
                    appThemeName.put(appCategory.getId().toString(),appCategory.getCategoryName());
                }
                this.addDict(model, "appThemeName", appThemeName);
            }
        }
        return super.doListAfter(query, model, context);
    }

    @Override
    protected int infoAfter(Long id, Map<String, Object> model, AppEntity entity, Context context) throws AppException {
        List<AppEntity> appEntityList = this.service.find(new AppQuery().appCode(entity.getAppCode()), context);
        entity.setApplianceSiteScope(appEntityList.size());
        entity.setSiteIdList(appEntityList.stream().map(AppEntity::getSiteId).collect(Collectors.toList()));
        SiteEntity siteEntity = siteService.getCache(entity.getSiteId().toString());
        if (!ObjectUtils.isEmpty(siteEntity)) {
            //请求地址 http://domian/app/siteCode/appcode/html 应用终端访问服务器地址，可能是政务网，也可能是内网
            String domainUrl = GlobalSysInfo.getParamValue(Constant.PARAM_SERVER_HTTP_URL, "http://192.168.0.98:11078");
            entity.setCustUrl(UrlBuilder.of(domainUrl)
                    .addPath(CUSTAPP_ROOT_PATH)
                    .addPath(siteEntity.getSiteCode())
                    .addPath(entity.getAppCode())
                    .addPath(entity.getVersion().toString())
                    .toString());
        } else {
            entity.setCustUrl("");
        }
        return super.infoAfter(id, model, entity, context);
    }

    /**
     * app应用部署
     */
    @PostMapping(value = "appDistribute")
    public Rest<String> appDistribute(@RequestBody AppEntity appEntity) {
        String busiDesc = this.getModuleDesc() + "自助服务应用部署";
        Rest<String> rest = Rest.ok(busiDesc + " 【成功】");
        try {
            if (ObjectUtils.isEmpty(appEntity.getId())) {
                throw new AppException("应用Id不能为空！");
            }
            appEntity = this.service.get(appEntity.getId(), getContext());
            if (ObjectUtils.isEmpty(appEntity)) {
                throw new AppException("当前应用不存在！appId:" + appEntity.getId());
            }
            rest = this.service.appDistribute(appEntity, getContext());
            recordSysLog(request, busiDesc + " 【成功】");
        } catch (Exception e) {
            log.error("自助服务应用部署", e);
            rest = Rest.fail(super.convertException(e));
        }
        return rest;
    }

    /**
     * app基础应用部署
     */
    @PostMapping(value = "appCommonDistribute")
    public Rest<Void> appCommonDistribute(@RequestBody AppEntity appEntity) {
        String busiDesc = this.getModuleDesc() + "自助服务应用部署";
        Rest<Void> rest = Rest.ok(busiDesc + " 【成功】");
        try {
            rest = this.service.appCommonDistribute(appEntity, getContext());
            recordSysLog(request, busiDesc + " 【成功】");
        } catch (Exception e) {
            log.error("基础服务应用部署", e);
            rest = Rest.fail(super.convertException(e));
        }
        return rest;
    }


    /**
     * app应用克隆给其它站点
     */
    @PostMapping(value = "cloneAppsBySites")
    public Rest<Void> cloneAppsBySites(@RequestBody AppQuery appQuery) {
        String busiDesc = this.getModuleDesc() + "自助服务应用部署克隆";
        Rest<Void> rest = Rest.ok(busiDesc + " 【成功】");
        try {
            if (ObjectUtils.isEmpty(appQuery.getIdList())) {
                throw new AppException("应用appId列表不能为空！");
            }
            if (ObjectUtils.isEmpty(appQuery.getSiteIdList())) {
                throw new AppException("克隆站点Id列表不能为空！");
            }
            List<AppEntity> appEntityList = this.service.find(new AppQuery().idList(appQuery.getIdList()), getContext());
            List<SiteEntity> siteEntityList = siteService.find(new SiteQuery().idList(appQuery.getSiteIdList()), getContext());
             this.service.cloneAppsBySites(appEntityList, siteEntityList, getContext());
            recordSysLog(request, busiDesc + " 【成功】");
        } catch (Exception e) {
            log.error("自助服务应用部署", e);
            rest = Rest.fail(super.convertException(e));
        }
        return rest;
    }

    /**
     * app应用克隆给本站点
     */
    @PostMapping(value = "cloneAppBySameSite")
    @UnAuth
    public Rest<Void> cloneAppBySameSite(@RequestBody AppEntity appQuery) {
        String busiDesc = this.getModuleDesc() + "自助服务应用部署克隆";
        Rest<Void> rest = Rest.ok(busiDesc + " 【成功】");
        try {
            if (ObjectUtils.isEmpty(appQuery.getSourceAppId())) {
                throw new AppException("源appId不能为空！");
            }
            if (ObjectUtils.isEmpty(appQuery.getTargetAppId())) {
                throw new AppException("目标appId不能为空！");
            }

            if(appQuery.getSourceAppId()==appQuery.getTargetAppId()){
                throw new AppException("源appId不能等于目标appId！");
            }

            Rest<String> cloneRest = this.service.cloneAppBySameSite(appQuery, getContext());
            if (YesNoEnum.NO.getValue() == cloneRest.getCode()) {
                return Rest.fail(cloneRest.getMsg());
            }
            recordSysLog(request, busiDesc + " 【成功】");
        } catch (Exception e) {
            log.error("应用克隆异常", e);
            rest = Rest.fail(super.convertException(e));
        }
        return rest;
    }


    public static void main(String[] args) {
/*        String homeUrl = "app/1/2";
        String domainUrl = "http://192.168.0.98:11078";
        String s = UrlBuilder.of(domainUrl).addPath("app").addPath("1").addPath("2").toString();
        System.out.println(s);*/

        ZipUtil.unzip("F://1672973316144.zip", "E://abc");

    }


}