package com.mortals.xhx.module.app.web;

import cn.hutool.core.collection.ListUtil;
import cn.hutool.core.util.StrUtil;
import com.alibaba.fastjson.JSON;
import com.mortals.framework.annotation.UnAuth;
import com.mortals.framework.common.Rest;
import com.mortals.framework.exception.AppException;
import com.mortals.framework.model.PageInfo;
import com.mortals.framework.model.Result;
import com.mortals.framework.web.BaseCRUDJsonBodyMappingController;
import com.mortals.xhx.base.system.param.service.ParamService;
import com.mortals.xhx.module.app.model.AppDatasetQuery;
import com.mortals.xhx.module.app.model.AppInfoFieldEntity;
import org.checkerframework.checker.units.qual.A;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.util.ObjectUtils;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;
import org.springframework.web.bind.annotation.*;
import com.mortals.framework.model.Context;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import com.mortals.framework.web.BaseCRUDJsonBodyMappingController;
import com.mortals.xhx.module.app.model.AppDatasetEntity;
import com.mortals.xhx.module.app.service.AppDatasetService;
import org.apache.commons.lang3.ArrayUtils;
import com.mortals.framework.util.StringUtils;

import java.util.*;
import java.util.stream.Collectors;

import com.alibaba.fastjson.JSONObject;

import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

import static com.mortals.framework.ap.SysConstains.*;

/**
 * 自助终端应用数据集
 *
 * @author zxfei
 * @date 2022-11-28
 */
@RestController
@RequestMapping("app/dataset")
public class AppDatasetController extends BaseCRUDJsonBodyMappingController<AppDatasetService, AppDatasetEntity, Long> {


    public AppDatasetController() {
        super.setModuleDesc("自助终端应用数据集");
    }

    /**
     * @param query
     * @return
     */
    @Override
    @UnAuth
    public Rest<Object> list(@RequestBody AppDatasetEntity query) {
        return super.list(query);
    }

    @Override
    protected void init(Map<String, Object> model, Context context) {
        super.init(model, context);
    }


    @Override
    protected void doListBefore(AppDatasetEntity query, Map<String, Object> model, Context context) throws AppException {
        if (!ObjectUtils.isEmpty(query.getOrConditionList()) || !ObjectUtils.isEmpty(query.getAndConditionList())) {
            AppDatasetQuery appDatasetQuery = new AppDatasetQuery();

            if (!ObjectUtils.isEmpty(query.getAndConditionList())) {
                appDatasetQuery.setOrConditionList(query.getAndConditionList());
            }

            if (!ObjectUtils.isEmpty(query.getOrConditionList())) {
                appDatasetQuery.setOrConditionList(query.getOrConditionList());
                appDatasetQuery.setAppId(query.getAppId());
                appDatasetQuery.setOrderCols(query.getOrderCols());
                PageInfo pageInfo = this.buildPageInfo(query);
                Result<AppInfoFieldEntity> customResult = this.service.findCustomList(appDatasetQuery, pageInfo, context);
                //根据查询条件的数量去除数量不足的结果
                int count = query.getOrConditionList().size();
                List<Long> datasetIdList = customResult.getList().stream().map(item -> item.getDatasetId()).collect(Collectors.toList());
                Map<Long, Long> collect = datasetIdList.stream().collect(Collectors.groupingBy(x -> x, Collectors.counting()));
                List<Long> dataSets = collect.entrySet().stream().map(entrySet -> {
                    if (entrySet.getValue() == count) {
                        return entrySet.getKey();
                    } else {
                        return null;
                    }
                }).filter(f -> f != null).collect(Collectors.toList());


                if (!ObjectUtils.isEmpty(dataSets)) {
                    //排序-
                    List<Long> orderList = new ArrayList<>();
                    for (Long dataId : datasetIdList) {
                        if(dataSets.contains(dataId)){
                            orderList.add(dataId);
                        }
                    }
                    query.setIdList(orderList);
                } else {
                    query.setIdList(ListUtil.toList(0L));
                }
                query.setOrConditionList(null);
            }
        }

        if (!ObjectUtils.isEmpty(query.getFieldCode()) || !ObjectUtils.isEmpty(query.getFieldName())) {
            AppDatasetQuery appDatasetQuery = new AppDatasetQuery();
            appDatasetQuery.setFieldCode(query.getFieldCode());
            appDatasetQuery.setFieldName(query.getFieldName());
            appDatasetQuery.setAppId(query.getAppId());
            appDatasetQuery.setFieldValue(query.getFieldValue());
            PageInfo pageInfo = this.buildPageInfo(query);
            Result<AppInfoFieldEntity> customResult = this.service.findCustomList(appDatasetQuery, pageInfo, context);
            List<Long> datasetIdList = customResult.getList().stream().map(item -> item.getDatasetId()).collect(Collectors.toList());
            if (!ObjectUtils.isEmpty(datasetIdList)) {
                query.setIdList(datasetIdList);
                //query.setOrderColList();
            } else {
                query.setIdList(ListUtil.toList(0L));
            }
        }
        super.doListBefore(query, model, context);
    }


    public static void main(String[] args) {
        ArrayList<AppDatasetQuery> list = new ArrayList<>();

        AppDatasetQuery condition = new AppDatasetQuery();
        condition.setFieldName("制定机关");
        condition.setFieldValue("%四川省人民政府%");
        list.add(condition);

        condition = new AppDatasetQuery();
        condition.setFieldName("标题");
        condition.setFieldValue("%范性文件%");

        list.add(condition);

        AppDatasetEntity appDatasetEntity = new AppDatasetEntity();
        //  appDatasetEntity.setAndConditionList(list);

        System.out.println(JSON.toJSONString(appDatasetEntity));
    }
}