package com.mortals.xhx.module.app.web;

import cn.hutool.core.net.url.UrlBuilder;
import com.mortals.framework.annotation.UnAuth;
import com.mortals.framework.ap.GlobalSysInfo;
import com.mortals.framework.common.Rest;
import com.mortals.framework.exception.AppException;
import com.mortals.framework.model.OrderCol;
import com.mortals.framework.web.BaseCRUDJsonBodyMappingController;
import com.mortals.xhx.base.system.param.service.ParamService;
import com.mortals.xhx.common.code.YesNoEnum;
import com.mortals.xhx.common.key.Constant;
import com.mortals.xhx.module.app.model.AppEntity;
import com.mortals.xhx.module.app.service.AppService;
import com.mortals.xhx.module.site.model.SiteEntity;
import com.mortals.xhx.module.site.service.SiteService;
import org.apache.commons.lang3.ObjectUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;
import org.springframework.web.bind.annotation.*;
import com.mortals.framework.model.Context;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import com.mortals.framework.web.BaseCRUDJsonBodyMappingController;
import com.mortals.xhx.module.app.model.AppVersionEntity;
import com.mortals.xhx.module.app.service.AppVersionService;
import org.apache.commons.lang3.ArrayUtils;
import com.mortals.framework.util.StringUtils;

import java.util.*;
import java.util.stream.Collectors;

import com.alibaba.fastjson.JSONObject;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

import static com.mortals.framework.ap.SysConstains.*;
import static com.mortals.xhx.common.key.Constant.CUSTAPP_ROOT_PATH;

/**
 * 自助终端应用版本历史
 *
 * @author zxfei
 * @date 2022-11-28
 */
@RestController
@RequestMapping("app/version")
public class AppVersionController extends BaseCRUDJsonBodyMappingController<AppVersionService, AppVersionEntity, Long> {

    @Autowired
    private SiteService siteService;
    @Autowired
    private AppService appService;

    public AppVersionController() {
        super.setModuleDesc("自助终端应用版本历史");
    }

    @Override
    protected void init(Map<String, Object> model, Context context) {
        super.init(model, context);
    }


    /**
     * @param query
     * @return
     */
    @Override
    @UnAuth
    public Rest<Object> list(@RequestBody AppVersionEntity query) {
        return super.list(query);
    }

    @Override
    protected void doListBefore(AppVersionEntity query, Map<String, Object> model, Context context) throws AppException {
        List<OrderCol> orderColList = new ArrayList<>();
        orderColList.add(new OrderCol("version", OrderCol.DESCENDING));
        query.setOrderColList(orderColList);
        super.doListBefore(query, model, context);
    }


    /**
     * app历史版本使用
     */
    @GetMapping(value = "used")
    public Rest<String> used(Long appVersionId) {
        String busiDesc = this.getModuleDesc() + "自助服务应用使用";
        Rest<String> rest = Rest.ok(busiDesc + " 【成功】");
        try {
            if (ObjectUtils.isEmpty(appVersionId)) {
                throw new AppException("应用版本Id不能为空！");
            }
            AppVersionEntity appVersionEntity = this.service.get(appVersionId, getContext());
            if (ObjectUtils.isEmpty(appVersionEntity)) {
                throw new AppException("当前应用版本不存在！appVersionId:" + appVersionId);
            }

            if (appVersionEntity.getUsed() == YesNoEnum.YES.getValue()) {
                throw new AppException("当前应用使用中！");
            }
             this.service.appUsed(appVersionEntity, getContext());
            recordSysLog(request, busiDesc + " 【成功】");

        } catch (Exception e) {
            log.error("自助服务应用预览", e);
            return Rest.fail(super.convertException(e));
        }
        return rest;
    }


    /**
     * app应用预览
     */
    @GetMapping(value = "preview")
    public Rest<String> preview(Long appVersionId) {
        String busiDesc = this.getModuleDesc() + "自助服务应用预览";
        Rest<String> rest = Rest.ok(busiDesc + " 【成功】");
        try {
            if (ObjectUtils.isEmpty(appVersionId)) {
                throw new AppException("应用版本Id不能为空！");
            }
            AppVersionEntity appVersionEntity = this.service.get(appVersionId, getContext());
            if (ObjectUtils.isEmpty(appVersionEntity)) {
                throw new AppException("应用版本未找到！");
            }
            AppEntity appEntity = appService.get(appVersionEntity.getAppId(), getContext());
            if (ObjectUtils.isEmpty(appEntity)) {
                throw new AppException("当前应用不存在！appId:" + appVersionEntity.getAppId());
            }
           // Rest<String> disRest = this.service.appDistribute(appVersionEntity, getContext());
            SiteEntity siteEntity = siteService.get(appEntity.getSiteId(), getContext());
            //请求地址 http://domian/app/siteCode/appcode/html
            String domainUrl = GlobalSysInfo.getParamValue(Constant.PARAM_SERVER_HTTP_URL, "http://192.168.0.98:11078");
            rest.setData(UrlBuilder.of(domainUrl)
                    .addPath(CUSTAPP_ROOT_PATH)
                    .addPath(siteEntity.getSiteCode())
                    .addPath(appEntity.getAppCode())
                    .addPath(appEntity.getVersion().toString())
                    .toString());
            recordSysLog(request, busiDesc + " 【成功】");
        } catch (Exception e) {
            log.error("自助服务应用预览", e);
            return Rest.fail(super.convertException(e));
        }
        return rest;
    }
}