package com.mortals.xhx.module.dept.service.impl;

import com.mortals.framework.ap.GlobalSysInfo;
import com.mortals.framework.common.Rest;
import com.mortals.framework.exception.AppException;
import com.mortals.framework.model.Context;
import com.mortals.framework.model.PageInfo;
import com.mortals.framework.service.impl.AbstractCRUDCacheServiceImpl;
import com.mortals.framework.util.ThreadPool;
import com.mortals.xhx.common.code.SourceEnum;
import com.mortals.xhx.common.code.YesNoEnum;
import com.mortals.xhx.common.key.Constant;
import com.mortals.xhx.common.utils.MatterHtmlParseUtil;
import com.mortals.xhx.module.business.model.BusinessEntity;
import com.mortals.xhx.module.business.model.BusinessQuery;
import com.mortals.xhx.module.business.service.BusinessService;
import com.mortals.xhx.module.dept.dao.DeptDao;
import com.mortals.xhx.module.dept.model.DeptEntity;
import com.mortals.xhx.module.dept.model.DeptQuery;
import com.mortals.xhx.module.dept.model.vo.DeptVo;
import com.mortals.xhx.module.dept.service.DeptService;
import com.mortals.xhx.module.matter.model.MatterQuery;
import com.mortals.xhx.module.matter.service.MatterService;
import com.mortals.xhx.module.matters.model.MattersDeptEntity;
import com.mortals.xhx.module.matters.model.MattersDeptQuery;
import com.mortals.xhx.module.matters.service.MattersDeptService;
import com.mortals.xhx.module.site.model.*;
import com.mortals.xhx.module.site.service.SiteBusinessService;
import com.mortals.xhx.module.site.service.SiteMatterService;
import com.mortals.xhx.module.site.service.SiteService;
import com.mortals.xhx.module.window.model.*;
import com.mortals.xhx.module.window.service.WindowBusinessService;
import com.mortals.xhx.module.window.service.WindowMatterService;
import com.mortals.xhx.module.window.service.WindowService;
import com.mortals.xhx.module.workman.model.WorkmanEntity;
import com.mortals.xhx.module.workman.model.WorkmanQuery;
import com.mortals.xhx.module.workman.service.WorkmanService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.ObjectUtils;

import java.util.*;
import java.util.stream.Collectors;

/**
 * DeptService
 * 部门 service实现
 *
 * @author zxfei
 * @date 2022-01-12
 */
@Service("deptService")
public class DeptServiceImpl extends AbstractCRUDCacheServiceImpl<DeptDao, DeptEntity, Long> implements DeptService {

    @Autowired
    private SiteService siteService;
    @Autowired
    private DeptService deptService;
    @Autowired
    private MattersDeptService mattersDeptService;
    @Autowired
    private WindowService windowService;
    @Autowired
    private WorkmanService workmanService;
    @Autowired
    private WindowMatterService windowMatterService;
    @Autowired
    private SiteMatterService siteMatterService;
    @Autowired
    private MatterService matterService;


    @Override
    protected String getExtKey(DeptEntity data) {
        return data.getDeptNumber() + "_" + data.getSiteId();
    }

    /**
     * @param entity
     * @param context
     * @throws AppException
     */
    @Override
    protected void saveBefore(DeptEntity entity, Context context) throws AppException {
        super.saveBefore(entity, context);
        //新增校验部门编码与站点是否重复
        DeptEntity extCache = this.getExtCache(entity.getDeptNumber() + "_" + entity.getSiteId());
        if (!ObjectUtils.isEmpty(extCache)) {
            throw new AppException("部门编码重复!deptCode:" + extCache.getDeptNumber());
        }
    }

    /**
     * @param entity
     * @param context
     * @throws AppException
     */
    @Override
    protected void updateBefore(DeptEntity entity, Context context) throws AppException {
        super.updateBefore(entity, context);
        DeptEntity beforeEntity = this.get(entity.getId(), context);
        if (!(beforeEntity.getDeptNumber() + "_" + beforeEntity.getSiteId()).equals(entity.getDeptNumber() + "_" + entity.getSiteId())) {
            DeptEntity extCache = this.getExtCache(entity.getDeptNumber() + "_" + entity.getSiteId());
            if (!ObjectUtils.isEmpty(extCache)) {
                throw new AppException("部门编码重复!deptCode:" + extCache.getDeptNumber());
            }
        }

        //todo 判断部门名称是否修改，如果一体化的部门 修改名称则同步修改对应事项的部门名称
        if (!ObjectUtils.isEmpty(entity.getSiteId()) &&
                !ObjectUtils.isEmpty(entity.getName()) &&
                !ObjectUtils.isEmpty(beforeEntity) &&
                !entity.getName().equals(beforeEntity.getName())) {

            SiteEntity siteCache = siteService.getCache(entity.getSiteId().toString());
            //matterService.
            MatterQuery condition = new MatterQuery();
            condition.setAreaCode(siteCache.getAreaCode());
            condition.setDeptName(beforeEntity.getName());

            MatterQuery updateQuery = new MatterQuery();
            updateQuery.setDeptName(entity.getName());
            matterService.getDao().update(updateQuery, condition);
        }

    }

    @Override
    public void syncDept(String areaCode, Context context) {
        List<MattersDeptEntity> deptList = mattersDeptService.find(new MattersDeptQuery());
        deptList.forEach(dept -> {
            //根据区域查询当前区域下存在的站点
            List<SiteEntity> siteEntityList = siteService.getFlatSitesByAreaCode(dept.getAreaCode(), context);
            siteEntityList.forEach(site -> {
                //根据站点添加部门信息，部门编号存在时候不添加
                DeptEntity deptExistEntity = deptService.selectOne(new DeptQuery().siteId(site.getId()).deptNumber(dept.getDeptCode()), context);
                if (ObjectUtils.isEmpty(deptExistEntity)) {
                    DeptEntity deptEntity = new DeptEntity();
                    deptEntity.initAttrValue();
                    deptEntity.setDeptNumber(dept.getDeptCode());
                    deptEntity.setSiteId(site.getId());
                    deptEntity.setName(dept.getName());
                    deptEntity.setCreateTime(new Date());
                    deptEntity.setCreateUserId(1L);
                    deptService.save(deptEntity, context);
                }
            });
        });
    }

    @Override
    public Rest<String> syncDeptBySiteId(SiteEntity siteEntity, Context context) {
        String url = GlobalSysInfo.getParamValue(Constant.GOV_DEPT_URL, "http://www.sczwfw.gov.cn/jiq/front/channel/deptSwitch");
        Long siteId = siteEntity.getId();
        String areaCode = siteEntity.getAreaCode();
        Map<String, String> params = new HashMap<>();
        params.put("areaCode", areaCode);
        Rest<Map<String, String>> rest = MatterHtmlParseUtil.syncDeptBySiteId(params, url);
        if (rest.getCode() == YesNoEnum.YES.getValue() && rest.getData().keySet().size() > 0) {
            //删除部门中属于政务部门的
            // deptService.deleteGovBySiteId(siteId, context);
            int sortN = 1;
            Map<String, String> data = rest.getData();
            for (Map.Entry<String, String> item : data.entrySet()) {
                String deptCode = item.getKey();
                String deptName = item.getValue();
                DeptEntity deptEntity = deptService.selectOne(new DeptQuery().siteId(siteEntity.getId()).deptNumber(deptCode));
                if (ObjectUtils.isEmpty(deptEntity)) {
                    deptEntity = new DeptEntity();
                    deptEntity.initAttrValue();
                    deptEntity.setDeptNumber(deptCode);
                    deptEntity.setSiteId(siteId);
                    deptEntity.setName(deptName);
                    deptEntity.setSort(sortN);
                    deptEntity.setSource(SourceEnum.政务网.getValue());
                    deptEntity.setCreateTime(new Date());
                    deptEntity.setCreateUserId(1L);
                    deptService.save(deptEntity, context);
                }
                sortN++;
            }
        } else {
            return Rest.fail(rest.getMsg());
        }
        return Rest.ok("当前站点同步添加部门成功！");
    }

    /**
     * @param areaCode
     * @param context
     * @return
     */
    @Override
    public Rest<List<DeptEntity>> getDeptByWeb(String areaCode, Context context) {
        String url = GlobalSysInfo.getParamValue(Constant.GOV_DEPT_URL, "http://www.sczwfw.gov.cn/jiq/front/channel/deptSwitch");
        Map<String, String> params = new HashMap<>();
        params.put("areaCode", areaCode);
        Rest<Map<String, String>> rest = MatterHtmlParseUtil.syncDeptBySiteId(params, url);
        ArrayList<DeptEntity> deptEntities = new ArrayList<>();
        if (rest.getCode() == YesNoEnum.YES.getValue() && rest.getData().keySet().size() > 0) {
            int sortN = 1;
            Map<String, String> data = rest.getData();
            for (Map.Entry<String, String> item : data.entrySet()) {
                String deptCode = item.getKey();
                String deptName = item.getValue();
                DeptEntity deptEntity = new DeptEntity();
                deptEntity.initAttrValue();
                deptEntity.setDeptNumber(deptCode);
                deptEntity.setName(deptName);
                deptEntity.setSort(sortN);
                deptEntity.setSource(SourceEnum.政务网.getValue());
                deptEntities.add(deptEntity);
                sortN++;
            }
        } else {
            return Rest.fail(deptEntities);
        }
        return Rest.ok(deptEntities);
    }

    @Override
    public Rest<Map<String, List<DeptVo>>> getBusinessByDept(DeptQuery deptQuery, Context context) {
        List<DeptVo> businessDeptList = this.dao.getBusinessByDept(deptQuery);
        Map<String, List<DeptVo>> collect = businessDeptList.parallelStream().collect(Collectors.groupingBy(x -> x.getId().toString()));
        return Rest.ok(collect);
    }

    @Override
    public void deleteGovBySiteId(Long siteId, Context context) {
        Map<String, Object> condition = new HashMap<>();
        condition.put("siteId", siteId);
        condition.put("source", SourceEnum.政务网.getValue());
        this.dao.delete(condition);
    }

    /**
     * @param deptQuery
     * @param context
     * @return
     */
    @Override
    public Rest<Map<String, List<DeptVo>>> getDeptListByBusiness(DeptQuery deptQuery, Context context) {
        List<DeptVo> deptListByBusiness = this.dao.getDeptListByBusiness(deptQuery);
        Map<String, List<DeptVo>> collect = deptListByBusiness.parallelStream().collect(Collectors.groupingBy(x -> x.getSiteBusinessId().toString()));
        return Rest.ok(collect);
    }

    /**
     * @return
     */
    @Override
    public List<DeptEntity> getDeptListByExistBusiness(DeptQuery deptQuery, Context context) {
        return this.dao.getDeptListByExistBusiness(deptQuery);
    }


    /**
     * @param entity
     * @param context
     * @throws AppException
     */
    @Override
    protected void updateAfter(DeptEntity entity, Context context) throws AppException {

        //异步更新
        Runnable runnable = new Runnable() {
            /**
             *
             */
            @Override
            public void run() {
                List<WindowEntity> windowEntityList = windowService.find(new WindowQuery().deptId(entity.getId())).stream().map(item -> {
                    item.setDeptName(entity.getName());
                    return item;
                }).collect(Collectors.toList());
                if (!ObjectUtils.isEmpty(windowEntityList)) {
                    windowService.update(windowEntityList, context);
                }

                List<WorkmanEntity> workmanEntityList = workmanService.find(new WorkmanQuery().deptId(entity.getId())).stream().map(item -> {
                    item.setDeptName(entity.getName());
                    return item;
                }).collect(Collectors.toList());
                if (!ObjectUtils.isEmpty(workmanEntityList)) {
                    workmanService.update(workmanEntityList, context);
                }

                List<WindowMatterEntity> windowMatterEntityList = windowMatterService.find(new WindowMatterQuery().deptId(entity.getId())).stream().map(item -> {
                    item.setDeptName(entity.getName());
                    return item;
                }).collect(Collectors.toList());

                if (!ObjectUtils.isEmpty(windowMatterEntityList)) {
                    windowMatterService.update(windowMatterEntityList, context);
                }

                List<SiteMatterEntity> siteMatterEntityList = siteMatterService.find(new SiteMatterQuery().deptId(entity.getId())).stream().map(item -> {
                    item.setDeptName(entity.getName());
                    return item;
                }).collect(Collectors.toList());

                if (!ObjectUtils.isEmpty(siteMatterEntityList)) {
                    siteMatterService.update(siteMatterEntityList, context);
                }
            }
        };
        ThreadPool.getInstance().execute(runnable);
        super.updateAfter(entity, context);
    }
}