package com.mortals.xhx.module.dept.web;

import com.alibaba.fastjson.JSONObject;
import com.mortals.framework.annotation.UnAuth;
import com.mortals.framework.common.Rest;
import com.mortals.framework.exception.AppException;
import com.mortals.framework.model.Context;
import com.mortals.framework.model.OrderCol;
import com.mortals.framework.util.DataUtil;
import com.mortals.framework.web.BaseCRUDJsonBodyMappingController;
import com.mortals.xhx.base.system.param.service.ParamService;
import com.mortals.xhx.common.code.SourceEnum;
import com.mortals.xhx.common.code.YesNoEnum;
import com.mortals.xhx.module.business.model.BusinessEntity;
import com.mortals.xhx.module.dept.model.DeptEntity;
import com.mortals.xhx.module.dept.model.DeptQuery;
import com.mortals.xhx.module.dept.model.vo.DeptVo;
import com.mortals.xhx.module.dept.service.DeptService;
import com.mortals.xhx.module.site.model.SiteEntity;
import com.mortals.xhx.module.site.model.SiteQuery;
import com.mortals.xhx.module.site.model.vo.SiteAreaVo;
import com.mortals.xhx.module.site.service.SiteService;
import org.checkerframework.checker.units.qual.A;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.util.ObjectUtils;
import org.springframework.web.bind.annotation.*;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * 部门
 *
 * @author zxfei
 * @date 2022-01-20
 */
@RestController
@RequestMapping("dept")
public class DeptController extends BaseCRUDJsonBodyMappingController<DeptService, DeptEntity, Long> {

    @Autowired
    private ParamService paramService;
    @Autowired
    private SiteService siteService;

    public DeptController() {
        super.setModuleDesc("部门");
    }

    @Override
    protected void doListBefore(DeptEntity query, Map<String, Object> model, Context context) throws AppException {

        if (ObjectUtils.isEmpty(query.getIdList())) {
            if (ObjectUtils.isEmpty(query.getOrderColList())) {
                query.setOrderColList(new ArrayList<OrderCol>() {
                    {
                        add(new OrderCol("sort", OrderCol.ASCENDING));
                        add(new OrderCol("createTime", OrderCol.DESCENDING));
                    }
                });
            } else {
                query.getOrderColList().add(new OrderCol("createTime", OrderCol.DESCENDING));
            }
        } else {

        }
        if (!ObjectUtils.isEmpty(query.getFilter()) && YesNoEnum.YES.getValue() == query.getFilter()) {
            //过滤部门事项数据为0的部门
            query.setTotalStart(1);
        }

        if (!ObjectUtils.isEmpty(query.getHallCheckIn()) && YesNoEnum.YES.getValue() == query.getHallCheckIn()) {
            //过滤部门事项数据为0的部门
            query.setInNumStart(1);
        }
        super.doListBefore(query, model, context);
    }

    @Override
    protected void init(Map<String, Object> model, Context context) {
        this.addDict(model, "isAutotable", paramService.getParamBySecondOrganize("Dept", "isAutotable"));
        this.addDict(model, "isOrder", paramService.getParamBySecondOrganize("Dept", "isOrder"));
        this.addDict(model, "isBkb", paramService.getParamBySecondOrganize("Dept", "isBkb"));
        this.addDict(model, "isWorkGuide", paramService.getParamBySecondOrganize("Dept", "isWorkGuide"));
        this.addDict(model, "usValid", paramService.getParamBySecondOrganize("Dept", "usValid"));
        this.addDict(model, "isSecphone", paramService.getParamBySecondOrganize("Dept", "isSecphone"));
        this.addDict(model, "isEnglish", paramService.getParamBySecondOrganize("Dept", "isEnglish"));
        super.init(model, context);
    }


    /**
     * 根据站点同步部门
     */
    @PostMapping(value = "syncDeptBySiteId")
    public String syncDeptBySiteId(Long siteId) {
        JSONObject jsonObject = new JSONObject();
        Map<String, Object> model = new HashMap<>();
        SiteEntity siteEntity = siteService.get(siteId, getContext());
        try {
            Rest<String> rest = this.service.syncDeptBySiteId(siteEntity, getContext());
            jsonObject.put(KEY_RESULT_MSG, rest.getMsg());
            jsonObject.put(KEY_RESULT_DATA, model);
            jsonObject.put(KEY_RESULT_CODE, VALUE_RESULT_SUCCESS);
            this.recordSysLog(this.request, rest.getMsg());
        } catch (Exception e) {
            log.error("获取异常", e);
            jsonObject.put(KEY_RESULT_CODE, VALUE_RESULT_FAILURE);
            jsonObject.put(KEY_RESULT_MSG, super.convertException(e));
        }
        return jsonObject.toJSONString();
    }


    /**
     * 根据部门查询业务
     */
    @PostMapping(value = "getBusinessByDept")
    @UnAuth
    public String getBusinessByDept(@RequestBody DeptQuery deptQuery) {
        JSONObject jsonObject = new JSONObject();
        String busiDesc = "根据部门查询业务" + this.getModuleDesc();
        try {
            if (ObjectUtils.isEmpty(deptQuery.getIdList())) {
                throw new AppException("参数部门ids不能为空！");
            }
            Rest<Map<String, List<DeptVo>>> rest = this.service.getBusinessByDept(deptQuery, getContext());
            if (!ObjectUtils.isEmpty(getContext()) && !ObjectUtils.isEmpty(getContext().getUser())) {
                recordSysLog(request, busiDesc + " 【成功】");
            }
            jsonObject.put(KEY_RESULT_DATA, rest.getData());
            jsonObject.put(KEY_RESULT_CODE, VALUE_RESULT_SUCCESS);
            jsonObject.put(KEY_RESULT_MSG, busiDesc + "成功！");
        } catch (Exception e) {
            log.error("获取异常", e);
            jsonObject.put(KEY_RESULT_CODE, VALUE_RESULT_FAILURE);
            jsonObject.put(KEY_RESULT_MSG, super.convertException(e));
        }
        return jsonObject.toJSONString();
    }


    /**
     * 根据部门查询业务
     */
    @GetMapping(value = "getDeptListByExistBusiness")
    @UnAuth
    public String getDeptListByExistBusiness(@RequestParam("siteId") Long siteId) {
        JSONObject jsonObject = new JSONObject();
        String busiDesc = "根据业务查询存在的部门列表" + this.getModuleDesc();
        try {
            if (ObjectUtils.isEmpty(siteId)) {
                throw new AppException("站点id不能为空！");
            }
            DeptQuery query = new DeptQuery();
            query.setSiteId(siteId);
            List<DeptEntity> deptList = this.service.getDeptListByExistBusiness(query, getContext());
            if (!ObjectUtils.isEmpty(getContext()) && !ObjectUtils.isEmpty(getContext().getUser())) {
                recordSysLog(request, busiDesc + " 【成功】");
            }
            jsonObject.put(KEY_RESULT_DATA, deptList);
            jsonObject.put(KEY_RESULT_CODE, VALUE_RESULT_SUCCESS);
            jsonObject.put(KEY_RESULT_MSG, busiDesc + "成功！");
        } catch (Exception e) {
            log.error("获取异常", e);
            jsonObject.put(KEY_RESULT_CODE, VALUE_RESULT_FAILURE);
            jsonObject.put(KEY_RESULT_MSG, super.convertException(e));
        }
        return jsonObject.toJSONString();
    }


    /**
     * 根据部门查询业务
     */
    @PostMapping(value = "getDeptListByBusiness")
    @UnAuth
    public String getDeptListByBusiness(@RequestBody DeptQuery deptQuery) {
        JSONObject jsonObject = new JSONObject();
        String busiDesc = "根据业务查询部门列表" + this.getModuleDesc();
        try {
            if (ObjectUtils.isEmpty(deptQuery.getSiteBusinessIdList())) {
                throw new AppException("业务id不能为空！");
            }
            Rest<Map<String, List<DeptVo>>> rest = this.service.getDeptListByBusiness(deptQuery, getContext());
            if (!ObjectUtils.isEmpty(getContext()) && !ObjectUtils.isEmpty(getContext().getUser())) {
                recordSysLog(request, busiDesc + " 【成功】");
            }
            jsonObject.put(KEY_RESULT_DATA, rest.getData());
            jsonObject.put(KEY_RESULT_CODE, VALUE_RESULT_SUCCESS);
            jsonObject.put(KEY_RESULT_MSG, busiDesc + "成功！");
        } catch (Exception e) {
            log.error("获取异常", e);
            jsonObject.put(KEY_RESULT_CODE, VALUE_RESULT_FAILURE);
            jsonObject.put(KEY_RESULT_MSG, super.convertException(e));
        }
        return jsonObject.toJSONString();
    }


    /**
     * @param entity
     * @param model
     * @param context
     * @throws AppException
     */
    @Override
    protected void saveBefore(DeptEntity entity, Map<String, Object> model, Context context) throws AppException {

        if (entity.newEntity()) {
            entity.setSource(SourceEnum.自定义.getValue());

        }
        super.saveBefore(entity, model, context);
    }
}