package com.mortals.xhx.module.matter.service;

import com.mortals.framework.common.Rest;
import com.mortals.framework.exception.AppException;
import com.mortals.framework.model.Context;
import com.mortals.framework.model.PageInfo;
import com.mortals.framework.model.Result;
import com.mortals.framework.service.ICRUDService;
import com.mortals.xhx.module.matter.dao.MatterDao;
import com.mortals.xhx.module.matter.model.MatterEntity;
import com.mortals.xhx.module.matter.model.MatterQuery;
import com.mortals.xhx.module.matter.model.vo.MatterCustomImportEntity;
import com.mortals.xhx.module.matter.model.vo.MatterInfo;
import com.mortals.xhx.module.site.model.SiteEntity;
import com.mortals.xhx.module.site.model.SiteMatterEntity;
import org.springframework.web.bind.annotation.RequestBody;

import java.io.IOException;
import java.util.List;
import java.util.Map;

/**
 * MatterService
 * <p>
 * 基础事项 service接口
 *
 * @author zxfei
 * @date 2022-01-12
 */
public interface MatterService extends ICRUDService<MatterEntity, Long> {


    MatterDao getDao();

    /**
     * 添加业务到站点
     *
     * @param matterIds
     * @param siteId
     * @param context
     */
    Rest<String> addMatterToSite(String matterIds, Long siteId, Context context);

    /**
     * 同步政务网事项数据
     *
     * @param areaCode
     * @param context
     */
    void syncMatter(String areaCode, Context context);

    Map<String, String> getBaseInfoMap(Context context);

    Map<String, String> getSqclInfoMap(Context context);

    /**
     * 构建事项详细相关信息
     *
     * @param matterEntity
     * @param context
     */
    Rest<String> buildMatterDetail(MatterEntity matterEntity, Context context);

    /**
     * 添加事项到站点事项
     *
     * @param matterEntity
     * @param context
     */
    Rest<SiteMatterEntity> switchMatterToSiteMatter(MatterEntity matterEntity, SiteEntity siteEntity, Context context);

    /**
     * 查找差集子项
     *
     * @param matterQuery
     * @param pageInfo
     * @param context
     * @return
     * @throws AppException
     */
    Result<MatterEntity> findSubList(MatterEntity matterQuery, PageInfo pageInfo, Context context) throws AppException;

    Result<MatterInfo> findMicroList(MatterQuery matterQuery, PageInfo pageInfo, Context context) throws AppException;

    /**
     * 生成事项文本
     * @param context
     * @return
     */
    Rest<Void> genMatterCsv(MatterQuery matterQuery, Context context);

    Rest<Void> genMatterMarkdown(MatterQuery matterQuery,Context context);

    Rest<Void> genMatterTXT(MatterQuery matterQuery, Context context);


    Rest<String> doImportCustomData(List<MatterCustomImportEntity> list, Context context);


    Rest<byte[]> genQrCode(@RequestBody MatterQuery query) throws IOException;

}