package com.mortals.xhx.module.matter.service.impl;

import com.alibaba.fastjson.JSON;
import com.google.common.collect.Lists;
import com.mortals.framework.ap.GlobalSysInfo;
import com.mortals.framework.common.Rest;
import com.mortals.framework.model.Context;
import com.mortals.framework.service.impl.AbstractCRUDServiceImpl;
import com.mortals.framework.util.ThreadPool;
import com.mortals.xhx.common.code.DxTypeEnum;
import com.mortals.xhx.common.code.SourceEnum;
import com.mortals.xhx.common.code.YesNoEnum;
import com.mortals.xhx.common.key.Constant;
import com.mortals.xhx.common.utils.MatterHtmlParseUtil;
import com.mortals.xhx.module.area.model.AreaEntity;
import com.mortals.xhx.module.area.service.AreaService;
import com.mortals.xhx.module.dept.model.DeptEntity;
import com.mortals.xhx.module.dept.model.DeptQuery;
import com.mortals.xhx.module.dept.service.DeptService;
import com.mortals.xhx.module.matter.dao.MatterExtDao;
import com.mortals.xhx.module.matter.model.MatterEntity;
import com.mortals.xhx.module.matter.model.MatterExtEntity;
import com.mortals.xhx.module.matter.model.MatterQuery;
import com.mortals.xhx.module.matter.service.MatterExtService;
import com.mortals.xhx.module.matter.service.MatterService;
import com.mortals.xhx.module.site.model.*;
import com.mortals.xhx.module.site.service.SiteMatterService;
import com.mortals.xhx.module.site.service.SiteThemeMatterService;
import com.mortals.xhx.module.site.service.SiteThemeService;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;
import org.springframework.util.ObjectUtils;

import java.util.*;
import java.util.stream.Collectors;

/**
 * MatterExtService
 * 事项扩展 service实现
 *
 * @author zxfei
 * @date 2022-11-23
 */
@Service("matterExtService")
@Slf4j
public class MatterExtServiceImpl extends AbstractCRUDServiceImpl<MatterExtDao, MatterExtEntity, Long> implements MatterExtService {

    @Autowired
    private AreaService areaService;
    @Autowired
    private DeptService deptService;
    @Autowired
    private MatterService matterService;
    @Autowired
    private SiteThemeMatterService siteThemeMatterService;
    @Autowired
    private SiteThemeService siteThemeService;
    @Autowired
    private SiteMatterService siteMatterService;
    @Value("${sync.matter.deleted:false}")
    private Boolean deleted;


    /**
     * 根据站点 同步事项
     *
     * @param siteEntity
     * @param context
     * @return
     */
    @Override
    public Rest<String> syncMatterBySiteId(SiteEntity siteEntity, Context context) {
        List<MatterEntity> govMatterList = getMatterEntities(siteEntity, context);
        //当前本地区事项全部事项
        log.info("计算差集");
        MatterQuery matterQuery = new MatterQuery();
        matterQuery.setAreaCode(siteEntity.getAreaCode());
        matterQuery.setSource(SourceEnum.政务网.getValue());
        List<MatterEntity> matterList = matterService.getDao().getMatterListByAreaCode(matterQuery);
        log.info("本地事项总数：{}", matterList.size());
        Set<String> matterNoSet = matterList.parallelStream().map(i -> i.getMatterNo()).collect(Collectors.toSet());
        List<MatterEntity> subList = this.subListSet(govMatterList, matterNoSet);
        subList = subList.parallelStream().distinct().collect(Collectors.toList());
        log.info("抓取事项总数：{} 需要添加事项数量：{}", govMatterList.size(), subList.size());

        //差集进行插入
        if (!ObjectUtils.isEmpty(subList)) {
            for (MatterEntity matterEntity : subList) {
                DeptEntity deptCache = deptService.selectOne(new DeptQuery().deptNumber(matterEntity.getDeptCode()));

                //  DeptEntity deptCache = deptService.getExtCache(matterEntity.getDeptCode());
                matterEntity.setDeptName(deptCache == null ? "" : deptCache.getName());
                matterEntity.setCreateTime(new Date());
                matterEntity.setMatterEdition("0");
            }
            List<List<MatterEntity>> partition = Lists.partition(subList, 100);
            for (List<MatterEntity> partitionList : partition) {
                matterService.save(partitionList, context);
            }
            matterList = matterService.getDao().getMatterListByAreaCode(matterQuery.areaCode(siteEntity.getAreaCode()));
        }
        log.info("开始更新事项详细！");
        for (MatterEntity matterEntity : matterList) {
            Runnable runnable = new Runnable() {
                @Override
                public void run() {
                    Rest<String> rest = matterService.buildMatterDetail(matterEntity, null);
                    if (rest.getCode() == YesNoEnum.YES.getValue()) {
                        matterEntity.setUpdateTime(new Date());
                        matterService.update(matterEntity, null);
                    }
                }
            };
            ThreadPool.getInstance().execute(runnable);
        }
        return Rest.ok();
    }

    private List<MatterEntity> getMatterEntities(SiteEntity siteEntity, Context context) {
        AreaEntity areaEntity = areaService.getCache(siteEntity.getAreaCode());
        List<DeptEntity> deptEntities = deptService.find(new DeptQuery().siteId(siteEntity.getId()).source(SourceEnum.政务网.getValue()));
        List<MatterEntity> govMatterList = deptEntities.stream().flatMap(deptEntity -> {
            HashMap<String, String> params = new HashMap<>();
            params.put("areaCode", areaEntity.getAreaCode());
            params.put("dxType", "6");
            params.put("deptCode", deptEntity.getDeptNumber());
            params.put("searchtext", "");
            params.put("taskType", "");
            List<MatterEntity> deptMatterList = this.getMatters(params, context);
            return deptMatterList.stream();
        }).collect(Collectors.toList());
        return govMatterList;
    }

    /**
     * 同步站点主题事项
     *
     * @param siteEntity
     * @param context
     * @return
     */
    @Override
    public Rest<String> syncMatterThemeBySiteId(SiteEntity siteEntity, Context context) {
        AreaEntity areaEntity = areaService.getCache(siteEntity.getAreaCode());
        //判断站点区域乡镇情况
        if (areaEntity.getAreaLevel() <= 3) {
            //省，市，区
            Rest<String> themeRest = siteThemeService.syncThemeBySiteId(siteEntity, null);
            log.info("同步站点主题:" + JSON.toJSONString(themeRest));
            log.info("同步站点个人主题事项开始.....");
            siteThemeMatterService.deleteGovBySiteId(siteEntity.getId(), null);
            Rest<String> grRest = siteThemeMatterService.syncThemeMatterBySiteId(siteEntity.getId(), "2", null);
            log.info("同步站点主题个人事项:" + JSON.toJSONString(grRest));
            log.info("同步站点法人主题事项开始.....");
            Rest<String> frRest = siteThemeMatterService.syncThemeMatterBySiteId(siteEntity.getId(), "3", null);
            log.info("同步站点主题法人事项:" + JSON.toJSONString(frRest));
        } else if (areaEntity.getAreaLevel() > 3) {
            //街道，镇，乡
            Rest<String> themeTownRest = siteThemeService.syncTownThemeBySiteId(siteEntity, null);
            log.info("同步乡镇站点主题:" + JSON.toJSONString(themeTownRest));
            if (themeTownRest.getCode() == YesNoEnum.YES.getValue()) {
                siteThemeMatterService.deleteGovBySiteId(siteEntity.getId(), null);
                String dxType = DxTypeEnum.街道镇服务.getValue();
                if (areaEntity.getAreaLevel() == 5) {
                    dxType = DxTypeEnum.乡村服务.getValue();
                }
                Rest<String> townThemeRest = siteThemeMatterService.syncTownThemeMatterBySiteId(siteEntity, dxType, null);
                log.info("同步站点乡镇主题事项:" + JSON.toJSONString(townThemeRest));
            } else {
                try {
                    //  cacheService.del(RedisCacheKeys.getSyncMatterLockKey() + siteEntity.getAreaCode(), siteEntity.getAreaCode());
                    log.info("同步站点乡镇主题事项失败:" + themeTownRest.getData());
                } catch (Exception e) {
                    log.error("同步站点乡镇主题事项失败", e);
                }

            }
        }
        return Rest.ok();
    }


    public List<MatterEntity> subListSet(List<MatterEntity> firstList, Set<String> secondSet) {
        return firstList.parallelStream().filter(item -> !secondSet.contains(item.getMatterNo())).collect(Collectors.toList());
    }

    private List<MatterEntity> getMatters(HashMap<String, String> params, Context context) {
        String url = GlobalSysInfo.getParamValue(Constant.GOV_MATTER_PAGELIST_URL, "http://www.sczwfw.gov.cn/jiq/interface/item/tags");
        Rest<Map<String, Integer>> restStat = MatterHtmlParseUtil.statSiteMatterCount(params, url);
        if (restStat.getCode() == YesNoEnum.YES.getValue()) {
            Integer pageNum = restStat.getData().getOrDefault("pageNum", 0);
            Integer total = restStat.getData().getOrDefault("total", 0);
            //获取事项全列表
            Rest<List<MatterEntity>> matterAllRest = this.getMatterAllListByGOV(params, pageNum, context);
            if (matterAllRest.getCode() == YesNoEnum.YES.getValue()) {
                return matterAllRest.getData();
            }
        }
        return new ArrayList<>();

    }

    public Rest<List<MatterEntity>> getMatterAllListByGOV(Map<String, String> params, Integer pageNum, Context context) {
        //分页获取所有事项列表
        List<MatterEntity> allList = new ArrayList<>();
        String url = GlobalSysInfo.getParamValue(Constant.GOV_MATTER_PAGELIST_URL, "http://www.sczwfw.gov.cn/jiq/interface/item/tags");
        for (int i = 1; i <= pageNum; i++) {
            params.put("pageno", String.valueOf(i));
            Rest<List<MatterEntity>> rest = MatterHtmlParseUtil.getMatterList(params, url);
            if (rest.getCode() == YesNoEnum.YES.getValue() && !ObjectUtils.isEmpty(rest.getData())) {
                allList.addAll(rest.getData());
            }
        }
        return Rest.ok(allList);
    }


    /**
     * @param matterId
     * @param context
     */
    @Override
    public void deleteByMatterId(Long matterId, Context context) {
        Map<String, Object> condition = new HashMap<>();
        condition.put("matterId", matterId);
        this.dao.delete(condition);
    }

    /**
     * @param siteEntity
     * @param context
     * @return
     */
    @Override
    public Rest<String> syncDelMatterBySiteId(SiteEntity siteEntity, Context context) {
        List<MatterEntity> govMatterList = getMatterEntities(siteEntity, context);
        if (!deleted) return Rest.ok("配置不允许删除！");
        if (ObjectUtils.isEmpty(govMatterList)) return Rest.ok("爬取事项列表为空！");

        log.info("计算差集");
        MatterQuery matterQuery = new MatterQuery();
        matterQuery.setAreaCode(siteEntity.getAreaCode());
        matterQuery.setSource(SourceEnum.政务网.getValue());
        //本地事项
        List<MatterEntity> matterList = matterService.getDao().getMatterListByAreaCode(matterQuery);
        log.info("本地事项总数：{}", matterList.size());
        //政务网存在set
        Set<String> matterExistSet = govMatterList.parallelStream().map(i -> i.getMatterNo()).collect(Collectors.toSet());
        List<MatterEntity> delList = this.subListSet(matterList, matterExistSet);
        log.info("抓取事项总数：{} ,需要删除事项数量：{}", govMatterList.size(), delList.size());
        //删除matter 和siteMatter 开启了自动删除才启用
        if (!ObjectUtils.isEmpty(delList)) {
            Long[] delMatterIds = delList.parallelStream().map(i -> i.getId()).toArray(Long[]::new);
            matterService.remove(delMatterIds, context);
            SiteMatterQuery siteMatterQuery = new SiteMatterQuery();
            siteMatterQuery.setMatterIdList(Arrays.asList(delMatterIds));
            List<SiteMatterEntity> delSiteMatterList = siteMatterService.find(siteMatterQuery);
            if (!ObjectUtils.isEmpty(delSiteMatterList)) {
                log.info("删除站点事项数量：{}", delSiteMatterList.size());
                siteMatterService.removeList(delSiteMatterList, context);
            }
            SiteThemeMatterQuery siteThemeMatterQuery = new SiteThemeMatterQuery();
            siteThemeMatterQuery.setMatterIdList(Arrays.asList(delMatterIds));
            List<SiteThemeMatterEntity> delSiteThemeMatterList = siteThemeMatterService.find(siteThemeMatterQuery);
            if (!ObjectUtils.isEmpty(delSiteThemeMatterList)) {
                log.info("删除站点主题事项数量：{}", delSiteThemeMatterList.size());
                siteThemeMatterService.removeList(delSiteThemeMatterList, context);
            }
        }
        return Rest.ok("同步删除成功！");
    }

    /**
     * @param context
     * @return
     */
    @Override
    public Rest<List<MatterEntity>> getWebMatterListByAreaCode(String areaCode, Context context) {

        List<DeptEntity> deptEntities = deptService.getDeptByWeb(areaCode, context).getData();
        List<MatterEntity> govMatterList = deptEntities.parallelStream().flatMap(deptEntity -> {
            HashMap<String, String> params = new HashMap<>();
            params.put("areaCode", areaCode);
            params.put("dxType", "6");
            params.put("deptCode", deptEntity.getDeptNumber());
            params.put("searchtext", "");
            params.put("taskType", "");
            List<MatterEntity> deptMatterList = this.getMatters(params, context);
            return deptMatterList.stream();
        }).collect(Collectors.toList());

        return Rest.ok(govMatterList);
    }


}