package com.mortals.xhx.module.matter.service.impl;

import cn.hutool.core.io.FileUtil;
import cn.hutool.core.io.IoUtil;
import cn.hutool.core.util.*;
import cn.hutool.extra.qrcode.QrCodeUtil;
import cn.hutool.setting.Setting;
import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.mortals.framework.common.Rest;
import com.mortals.framework.exception.AppException;
import com.mortals.framework.model.Context;
import com.mortals.framework.model.PageInfo;
import com.mortals.framework.model.Result;
import com.mortals.framework.service.impl.AbstractCRUDCacheServiceImpl;
import com.mortals.framework.service.impl.AbstractCRUDServiceImpl;
import com.mortals.framework.util.DataUtil;
import com.mortals.framework.util.ThreadPool;
import com.mortals.xhx.base.framework.config.InterceptorConfig;
import com.mortals.xhx.base.system.upload.service.UploadService;
import com.mortals.xhx.common.code.AppTypeEnum;
import com.mortals.xhx.common.code.FiletypeEnum;
import com.mortals.xhx.common.code.SourceEnum;
import com.mortals.xhx.common.code.YesNoEnum;
import com.mortals.xhx.common.utils.BeanUtil;
import com.mortals.xhx.common.utils.MatterDetailHtmlParseUtil;
import com.mortals.xhx.module.area.model.AreaEntity;
import com.mortals.xhx.module.area.service.AreaService;
import com.mortals.xhx.module.base.model.BaseAreaEntity;
import com.mortals.xhx.module.base.model.BaseAreaQuery;
import com.mortals.xhx.module.base.service.BaseAreaService;
import com.mortals.xhx.module.business.model.BusinessMatterEntity;
import com.mortals.xhx.module.business.model.BusinessMatterQuery;
import com.mortals.xhx.module.business.service.BusinessMatterService;
import com.mortals.xhx.module.dept.model.DeptEntity;
import com.mortals.xhx.module.dept.model.DeptQuery;
import com.mortals.xhx.module.dept.service.DeptService;
import com.mortals.xhx.module.matter.dao.MatterDao;
import com.mortals.xhx.module.matter.model.*;
import com.mortals.xhx.module.matter.model.vo.MatterCustomImportEntity;
import com.mortals.xhx.module.matter.model.vo.MatterInfo;
import com.mortals.xhx.module.matter.service.*;
import com.mortals.xhx.module.matters.model.MattersDetailEntity;
import com.mortals.xhx.module.matters.model.MattersDetailQuery;
import com.mortals.xhx.module.matters.model.MattersEntity;
import com.mortals.xhx.module.matters.model.MattersQuery;
import com.mortals.xhx.module.matters.service.MattersDetailService;
import com.mortals.xhx.module.matters.service.MattersService;
import com.mortals.xhx.module.site.model.*;
import com.mortals.xhx.module.site.service.SiteMatterService;
import com.mortals.xhx.module.site.service.SiteService;
import com.mortals.xhx.module.site.service.SiteThemeMatterService;
import com.mortals.xhx.module.site.service.SiteThemeService;
import com.mortals.xhx.module.window.model.WindowMatterEntity;
import com.mortals.xhx.module.window.model.WindowMatterQuery;
import com.mortals.xhx.module.window.service.WindowMatterService;
import io.netty.handler.timeout.ReadTimeoutException;
import lombok.Getter;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.io.IOUtils;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.jsoup.Jsoup;
import org.jsoup.nodes.Document;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;
import org.springframework.util.ObjectUtils;

import javax.imageio.ImageIO;
import java.awt.image.BufferedImage;
import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.net.ConnectException;
import java.net.SocketTimeoutException;
import java.util.*;
import java.util.stream.Collectors;
import java.util.zip.ZipEntry;
import java.util.zip.ZipOutputStream;

import static com.mortals.xhx.common.key.Constant.MAX_RETRY_COUNT;
import static com.mortals.xhx.common.key.Constant.QR_CODE;

/**
 * MatterService
 * 基础事项 service实现
 *
 * @author zxfei
 * @date 2022-01-12
 */
@Service("matterService")
@Slf4j
public class MatterServiceImpl extends AbstractCRUDServiceImpl<MatterDao, MatterEntity, Long> implements MatterService {
    @Autowired
    private SiteMatterService siteMatterService;
    @Autowired
    private SiteService siteService;
    @Autowired
    private MattersService mattersService;
    @Autowired
    private InterceptorConfig interceptorConfig;
    @Autowired
    private MattersDetailService mattersDetailService;
    @Autowired
    private MatterFlowlimitService matterFlowlimitService;
    @Autowired
    private MatterAcceptService matterAcceptService;
    @Autowired
    private MatterSetbaseService matterSetbaseService;
    @Autowired
    private MatterQuestionService matterQuestionService;
    @Autowired
    private MatterIntermediaryService matterIntermediaryService;
    @Autowired
    private MatterDatumService matterDatumService;
    @Autowired
    private MatterDatumFileService matterDatumFileService;
    @Autowired
    private MatterChargesService matterChargesService;
    @Autowired
    private DeptService deptService;
    @Autowired
    private MatterExtService matterExtService;
    @Autowired
    private SiteThemeMatterService siteThemeMatterService;
    @Autowired
    private SiteThemeService siteThemeService;
    @Autowired
    private WindowMatterService windowMatterService;
    @Autowired
    private BaseAreaService baseAreaService;
    @Autowired
    private BusinessMatterService businessMatterService;
    @Autowired
    private AreaService areaService;
    @Autowired
    private MatterChannelService matterChannelService;
    @Autowired
    private UploadService uploadService;


    @Override
    protected void saveBefore(MatterEntity entity, Context context) throws AppException {
        //判断如果是自定义事项，需要更新区域编码
        if (SourceEnum.自定义.getValue().equals(entity.getSource())) {
            if (!ObjectUtils.isEmpty(entity.getSiteId())) {
                SiteEntity siteEntity = siteService.get(entity.getSiteId(), context);
                entity.setAreaCode(siteEntity.getAreaCode());
                DeptEntity extCache = deptService.getExtCache(entity.getDeptCode() + "_" + entity.getSiteId());
                entity.setDeptName(extCache == null ? "" : extCache.getName());
            }

        }

        updateAreaInfo(entity);
        super.saveBefore(entity, context);
    }

    private void updateAreaInfo(MatterEntity entity) {
        if (!ObjectUtils.isEmpty(entity.getAreaCode())) {
            AreaEntity areCache = areaService.getExtCache(entity.getAreaCode());
            if (!ObjectUtils.isEmpty(areCache)) {
                entity.setAreaLevel(areCache.getAreaLevel());
                entity.setAreaName(areCache.getName());
            }
        }
    }

    /**
     * @param entity
     * @param context
     * @throws AppException
     */
    @Override
    protected void saveAfter(MatterEntity entity, Context context) throws AppException {
        //主题添加自定义关联
        if (!ObjectUtils.isEmpty(entity.getThemeCode())) {
            SiteThemeEntity siteThemeEntity = siteThemeService.selectOne(new SiteThemeQuery().siteId(entity.getSiteId()).themeCode(entity.getThemeCode()));
            SiteThemeMatterEntity siteThemeMatterEntity = new SiteThemeMatterEntity();
            siteThemeMatterEntity.initAttrValue();
            siteThemeMatterEntity.setThemeCode(entity.getThemeCode());
            siteThemeMatterEntity.setSiteId(entity.getSiteId());
            siteThemeMatterEntity.setSource(entity.getSource());
            siteThemeMatterEntity.setMatterId(entity.getId());
            siteThemeMatterEntity.setMatterCode(entity.getMatterNo());
            siteThemeMatterEntity.setMatterName(entity.getMatterName());
            siteThemeMatterEntity.setUserType(siteThemeEntity == null ? "2" : siteThemeEntity.getUserType());
            siteThemeMatterEntity.setCreateTime(new Date());
            siteThemeMatterEntity.setCreateUserId(this.getContextUserId(context));
            siteThemeMatterService.save(siteThemeMatterEntity, context);
        }

        if (!ObjectUtils.isEmpty(entity.getMatterAcceptList())) {
            entity.getMatterAcceptList().stream().peek(item -> {
                item.setMatterId(entity.getId());
                item.setMatterCode(entity.getMatterNo());
                item.setCreateUserId(this.getContextUserId(context));
                item.setCreateTime(new Date());
            }).count();
            matterAcceptService.save(entity.getMatterAcceptList());
        }

        if (!ObjectUtils.isEmpty(entity.getMatterDatumList())) {
            entity.getMatterDatumList().stream().peek(item -> {
                item.setMatterId(entity.getId());
                item.setCreateUserId(this.getContextUserId(context));
                item.setCreateTime(new Date());
            }).count();

            for (MatterDatumEntity matterDatumEntity : entity.getMatterDatumList()) {
                matterDatumService.save(matterDatumEntity);
            }
        }

        if (!ObjectUtils.isEmpty(entity.getMatterChannelList())) {
            entity.getMatterChannelList().stream().peek(item -> {
                item.setMatterId(entity.getId());
                item.setCreateUserId(this.getContextUserId(context));
                item.setCreateTime(new Date());
            }).count();
            matterChannelService.save(entity.getMatterChannelList());
        }

        if (!ObjectUtils.isEmpty(entity.getMatterChargesList())) {
            entity.getMatterChargesList().stream().peek(item -> {
                item.setMatterId(entity.getId());
                item.setCreateUserId(this.getContextUserId(context));
                item.setCreateTime(new Date());
            }).count();
            matterChargesService.save(entity.getMatterChargesList());
        }

        if (!ObjectUtils.isEmpty(entity.getMatterFlowlimitList())) {
            entity.getMatterFlowlimitList().stream().peek(item -> {
                item.setMatterId(entity.getId());
                item.setCreateUserId(this.getContextUserId(context));
                item.setCreateTime(new Date());
            }).count();
            matterFlowlimitService.save(entity.getMatterFlowlimitList());
        }

        if (!ObjectUtils.isEmpty(entity.getMatterIntermediaryList())) {
            entity.getMatterIntermediaryList().stream().peek(item -> {
                item.setMatterId(entity.getId());
                item.setCreateUserId(this.getContextUserId(context));
                item.setCreateTime(new Date());
            }).count();
            matterIntermediaryService.save(entity.getMatterIntermediaryList());
        }

        if (!ObjectUtils.isEmpty(entity.getMatterQuestionList())) {
            entity.getMatterQuestionList().stream().peek(item -> {
                item.setMatterId(entity.getId());
                item.setCreateUserId(this.getContextUserId(context));
                item.setCreateTime(new Date());
            }).count();
            matterQuestionService.save(entity.getMatterQuestionList());
        }

        if (!ObjectUtils.isEmpty(entity.getMatterSetbaseList())) {
            entity.getMatterSetbaseList().stream().peek(item -> {
                item.setMatterId(entity.getId());
                item.setCreateUserId(this.getContextUserId(context));
                item.setCreateTime(new Date());
            }).count();
            matterSetbaseService.save(entity.getMatterSetbaseList());
        }


        super.saveAfter(entity, context);
    }

    @Override
    protected void updateBefore(MatterEntity entity, Context context) throws AppException {
        updateAreaInfo(entity);
        if (SourceEnum.自定义.getValue().equals(entity.getSource())) {
            if (!ObjectUtils.isEmpty(entity.getSiteId())) {
                SiteEntity siteEntity = siteService.get(entity.getSiteId(), context);
                entity.setAreaCode(siteEntity.getAreaCode());
                DeptEntity extCache = deptService.getExtCache(entity.getDeptCode() + "_" + entity.getSiteId());
                entity.setDeptName(extCache == null ? "" : extCache.getName());
            }
            MatterEntity beforeMatterEntity = this.get(entity.getId(), context);
            //如果部门切换了 级联修改关联的其它项
            if (!ObjectUtils.isEmpty(entity.getDeptCode())) {
                if (!entity.getDeptCode().equals(beforeMatterEntity.getDeptCode())) {
                    //更新站点事项关联项
                    SiteMatterQuery condition = new SiteMatterQuery();
                    condition.setMatterId(entity.getId());
                    SiteMatterQuery siteMatterQuery = new SiteMatterQuery();
                    DeptEntity extCache = deptService.getExtCache(entity.getDeptCode() + "_" + entity.getSiteId());

                    siteMatterQuery.setDeptId(extCache == null ? 0L : extCache.getId());
                    siteMatterQuery.setDeptCode(entity.getDeptCode());
                    siteMatterQuery.setDeptName(entity.getDeptName());
                    siteMatterService.updateBatch(siteMatterQuery, condition, context);

                    //更新站点窗口事项关联项
                    WindowMatterQuery winCondition = new WindowMatterQuery();
                    winCondition.setSiteMatterId(entity.getId());
                    WindowMatterQuery windowMatterQuery = new WindowMatterQuery();
                    windowMatterQuery.setDeptId(extCache == null ? 0L : extCache.getId());
                    windowMatterQuery.setDeptCode(entity.getDeptCode());
                    windowMatterQuery.setDeptName(entity.getDeptName());
                    windowMatterService.updateBatch(windowMatterQuery, winCondition, context);
                }
            }

        }
        super.updateBefore(entity, context);
    }


    @Override
    protected void updateAfter(MatterEntity entity, Context context) throws AppException {

        Runnable runnable = new Runnable() {
            /**
             *
             */
            @Override
            public void run() {
                //同步更新所有站点
                List<SiteMatterEntity> siteMatterEntityList = siteMatterService.find(new SiteMatterQuery().matterId(entity.getId())).stream()
                        .map(item -> {
                            item.setMatterName(entity.getMatterName());
                            return item;
                        }).collect(Collectors.toList());
                if (!ObjectUtils.isEmpty(siteMatterEntityList)) {
                    siteMatterService.update(siteMatterEntityList, context);
                }

                List<BusinessMatterEntity> businessMatterEntityList = businessMatterService.find(new BusinessMatterQuery().matterId(entity.getId())).stream().map(item -> {
                    item.setMatterName(entity.getMatterName());
                    return item;
                }).collect(Collectors.toList());
                if (!ObjectUtils.isEmpty(businessMatterEntityList)) {
                    businessMatterService.update(businessMatterEntityList, context);
                }

                for (SiteMatterEntity siteMatterEntity : siteMatterEntityList) {
                    List<WindowMatterEntity> windowMatterEntities = windowMatterService.find(new WindowMatterQuery()
                            .siteMatterId(siteMatterEntity.getId())).stream().map(item -> {
                        item.setMatterName(entity.getMatterName());
                        return item;
                    }).collect(Collectors.toList());

                    if (!ObjectUtils.isEmpty(windowMatterEntities)) {
                        windowMatterService.update(windowMatterEntities, context);
                    }

                }

            }
        };
        ThreadPool.getInstance().execute(runnable);

        if (!ObjectUtils.isEmpty(entity.getMatterAcceptList())) {
            Long[] ids = matterAcceptService.find(new MatterAcceptQuery().matterId(entity.getId())).stream().map(MatterAcceptEntity::getId).toArray(Long[]::new);
            matterAcceptService.remove(ids, context);
            entity.getMatterAcceptList().stream().peek(item -> {
                item.setMatterId(entity.getId());
                item.setCreateUserId(this.getContextUserId(context));
                item.setCreateTime(new Date());
                item.setUpdateUserId(this.getContextUserId(context));
                item.setUpdateTime(new Date());
            }).count();
            matterAcceptService.save(entity.getMatterAcceptList());
        }


        if (!ObjectUtils.isEmpty(entity.getMatterDatumList())) {
            Long[] ids = matterDatumService.find(new MatterDatumQuery().matterId(entity.getId())).stream().map(MatterDatumEntity::getId).toArray(Long[]::new);
            matterDatumService.remove(ids, context);
            entity.getMatterDatumList().stream().peek(item -> {
                item.setMatterId(entity.getId());
                item.setCreateUserId(this.getContextUserId(context));
                item.setCreateTime(new Date());
                item.setUpdateUserId(this.getContextUserId(context));
                item.setUpdateTime(new Date());
            }).count();
            for (MatterDatumEntity matterDatumEntity : entity.getMatterDatumList()) {
                matterDatumService.save(matterDatumEntity);
            }
        }


        if (!ObjectUtils.isEmpty(entity.getMatterChannelList())) {
            Long[] ids = matterChannelService.find(new MatterChannelQuery().matterId(entity.getId())).stream().map(MatterChannelEntity::getId).toArray(Long[]::new);
            matterChannelService.remove(ids, context);
            entity.getMatterChannelList().stream().peek(item -> {
                item.setMatterId(entity.getId());
                item.setCreateUserId(this.getContextUserId(context));
                item.setCreateTime(new Date());
                item.setUpdateUserId(this.getContextUserId(context));
                item.setUpdateTime(new Date());
            }).count();
            matterChannelService.save(entity.getMatterChannelList());
        }

        if (!ObjectUtils.isEmpty(entity.getMatterChargesList())) {
            Long[] ids = matterChargesService.find(new MatterChargesQuery().matterId(entity.getId())).stream().map(MatterChargesEntity::getId).toArray(Long[]::new);
            matterChargesService.remove(ids, context);
            entity.getMatterChargesList().stream().peek(item -> {
                item.setMatterId(entity.getId());
                item.setCreateUserId(this.getContextUserId(context));
                item.setCreateTime(new Date());
                item.setUpdateUserId(this.getContextUserId(context));
                item.setUpdateTime(new Date());
            }).count();
            matterChargesService.save(entity.getMatterChargesList());
        }

        if (!ObjectUtils.isEmpty(entity.getMatterFlowlimitList())) {
            Long[] ids = matterFlowlimitService.find(new MatterFlowlimitQuery().matterId(entity.getId())).stream().map(MatterFlowlimitEntity::getId).toArray(Long[]::new);
            matterFlowlimitService.remove(ids, context);
            entity.getMatterFlowlimitList().stream().peek(item -> {
                item.setMatterId(entity.getId());
                item.setCreateUserId(this.getContextUserId(context));
                item.setCreateTime(new Date());
                item.setUpdateUserId(this.getContextUserId(context));
                item.setUpdateTime(new Date());
            }).count();
            matterFlowlimitService.save(entity.getMatterFlowlimitList());
        }

        if (!ObjectUtils.isEmpty(entity.getMatterIntermediaryList())) {
            Long[] ids = matterIntermediaryService.find(new MatterIntermediaryQuery().matterId(entity.getId())).stream().map(MatterIntermediaryEntity::getId).toArray(Long[]::new);
            matterIntermediaryService.remove(ids, context);
            entity.getMatterIntermediaryList().stream().peek(item -> {
                item.setMatterId(entity.getId());
                item.setCreateUserId(this.getContextUserId(context));
                item.setCreateTime(new Date());
                item.setUpdateUserId(this.getContextUserId(context));
                item.setUpdateTime(new Date());
            }).count();
            matterIntermediaryService.save(entity.getMatterIntermediaryList());
        }


        if (!ObjectUtils.isEmpty(entity.getMatterQuestionList())) {
            Long[] ids = matterQuestionService.find(new MatterQuestionQuery().matterId(entity.getId())).stream().map(MatterQuestionEntity::getId).toArray(Long[]::new);
            matterQuestionService.remove(ids, context);
            entity.getMatterQuestionList().stream().peek(item -> {
                item.setMatterId(entity.getId());
                item.setCreateUserId(this.getContextUserId(context));
                item.setCreateTime(new Date());
                item.setUpdateUserId(this.getContextUserId(context));
                item.setUpdateTime(new Date());
            }).count();
            matterQuestionService.save(entity.getMatterQuestionList());
        }

        if (!ObjectUtils.isEmpty(entity.getMatterSetbaseList())) {
            Long[] ids = matterSetbaseService.find(new MatterSetbaseQuery().matterId(entity.getId())).stream().map(MatterSetbaseEntity::getId).toArray(Long[]::new);
            matterSetbaseService.remove(ids, context);
            entity.getMatterSetbaseList().stream().peek(item -> {
                item.setMatterId(entity.getId());
                item.setCreateUserId(this.getContextUserId(context));
                item.setCreateTime(new Date());
                item.setUpdateUserId(this.getContextUserId(context));
                item.setUpdateTime(new Date());
            }).count();
            matterSetbaseService.save(entity.getMatterSetbaseList());
        }

        super.updateAfter(entity, context);
    }


    @Override
    protected void removeBefore(Long[] ids, Context context) throws AppException {
        //有子节点 禁止删除
        super.removeBefore(ids, context);
    }

    @Override
    public Rest<String> addMatterToSite(String matterIds, Long siteId, Context context) {
        if (ObjectUtils.isEmpty(siteId)) {
            throw new AppException("请选择对应站点");
        }
        List<Long> matterIdList = Arrays.asList(matterIds.split(",")).stream().map(Long::parseLong).collect(Collectors.toList());
        MatterQuery matterQuery = new MatterQuery();
        matterQuery.setIdList(matterIdList);
        List<MatterEntity> matterEntities = this.find(matterQuery);
        int success = 0;
        int fail = 0;
        List<SiteMatterEntity> siteMatterEntities = new ArrayList<>();

        for (MatterEntity matterEntity : matterEntities) {
            SiteMatterEntity siteMatterEntity = updateOrSave(matterEntity, siteId, context);
            if (!ObjectUtils.isEmpty(siteMatterEntity)) {
                siteMatterEntities.add(siteMatterEntity);
                success++;
            } else {
                fail++;
            }
        }
        if (!ObjectUtils.isEmpty(siteMatterEntities)) {
            siteMatterService.save(siteMatterEntities, context);
        }
        String msg = "当前加入事项已存在！";
        if (matterIdList.size() == 1) {
            if (success > 0) {
                msg = "加入事项成功！";
            }
        } else if (matterIdList.size() > 1) {
            if (success > 0 && fail == 0) {
                msg = String.format("加入事项成功%d条!", success);
            } else if (success > 0 && fail > 0) {
                msg = String.format("加入事项成功%d条，重复加入事项%d条！", success, fail);
            } else if (success == 0 && fail > 0) {
                msg = String.format("重复加入事项%d条！", fail);
            }
        }
        return Rest.ok(msg);
    }

    private SiteMatterEntity updateOrSave(MatterEntity item, Long siteId, Context context) {
        SiteMatterEntity siteMatterEntity = siteMatterService.selectOne(new SiteMatterQuery().siteId(siteId).matterId(item.getId()));
        if (ObjectUtils.isEmpty(siteMatterEntity)) {
            SiteEntity siteEntity = siteService.getCache(siteId.toString());
            Rest<SiteMatterEntity> rest = switchMatterToSiteMatter(item, siteEntity, context);
            if (rest.getCode() == YesNoEnum.YES.getValue()) {
                return rest.getData();
            }
        }
        return null;
    }

    @Override
    public Rest<SiteMatterEntity> switchMatterToSiteMatter(MatterEntity item, SiteEntity siteEntity, Context context) {
        try {
            DeptEntity deptEntity = deptService.getExtCache(item.getDeptCode() + "_" + item.getSiteId());
            SiteMatterEntity siteMatterEntity = new SiteMatterEntity();
            siteMatterEntity.initAttrValue();
            //SiteEntity siteEntity = siteService.get(siteId);
            siteMatterEntity.setSiteId(siteEntity.getId());
            siteMatterEntity.setSiteName(siteEntity.getSiteName());
            siteMatterEntity.setMatterId(item.getId());
            siteMatterEntity.setMatterCode(item.getMatterNo());
            siteMatterEntity.setMatterName(item.getMatterName());
            siteMatterEntity.setSource(item.getSource());
            siteMatterEntity.setEventTypeShow(item.getEventTypeShow());
            siteMatterEntity.setDeptName(deptEntity == null ? "" : deptEntity.getName());
            siteMatterEntity.setDeptId(deptEntity == null ? -1L : deptEntity.getId());
            siteMatterEntity.setDeptCode(deptEntity == null ? "" : deptEntity.getDeptNumber());
            siteMatterEntity.setCreateUserId(context == null ? 1L : context.getUser() == null ? 1L : context.getUser().getId());
            siteMatterEntity.setCreateTime(new Date());
            return Rest.ok(siteMatterEntity);

        } catch (Exception e) {
            log.error("转换异常", e);
            return Rest.fail(e.getMessage());
        }
    }

    @Override
    public Result<MatterEntity> findSubList(MatterEntity matterQuery, PageInfo pageInfo, Context context) throws AppException {
        //  SiteEntity siteCache = siteService.getCache(matterQuery.getSiteId().toString());
        SiteEntity siteCache = siteService.get(matterQuery.getSiteId());
        if (ObjectUtils.isEmpty(siteCache))
            throw new AppException("查询站点id不能为空！siteId:" + matterQuery.getSiteId());
        if (ObjectUtils.isEmpty(matterQuery.getAreaCode())) {
            matterQuery.setAreaCode(siteCache == null ? null : siteCache.getAreaCode());
        }
        return this.dao.getSubList(matterQuery, pageInfo);
    }

    @Override
    public Result<MatterInfo> findMicroList(MatterQuery matterQuery, PageInfo pageInfo, Context context) throws AppException {
        Result<MatterInfo> matterInfoResult = new Result<>();
        Result<MatterEntity> matterEntityResult = this.find(matterQuery, pageInfo, context);
        List<MatterInfo> matterInfoList = matterEntityResult.getList().stream().map(item -> {
            MatterInfo matterInfo = new MatterInfo();
            BeanUtils.copyProperties(item, matterInfo, BeanUtil.getNullPropertyNames(item));
            return matterInfo;
        }).collect(Collectors.toList());
        matterInfoResult.setList(matterInfoList);
        matterInfoResult.setPageInfo(pageInfo);

        return matterInfoResult;
    }

    /**
     * @param context
     * @return
     */


    @Override
    public void syncMatter(String areaCode, Context context) {
        List<MatterEntity> list = this.find(new MatterQuery().source(SourceEnum.政务网.getValue()));
        if (!ObjectUtils.isEmpty(list)) {
            // TODO: 2022/3/30  查询本地数据，进行交叉更新对比
            //暂时以删除后重新添加为准
            //this.getDao().delete(new MatterQuery().source(SourceEnum.政务网.getValue()));
        }
        //读取区域数据
        //mattersAreaService.find()
        //读取事项基础数据

        MattersQuery mattersQuery = new MattersQuery();
        if (!ObjectUtils.isEmpty(areaCode)) {
            //查询区域与当前区域
        }

        Integer total = mattersService.count(mattersQuery, null);
        int totalPage = PageUtil.totalPage(total, 1000);
        for (int i = 0; i < totalPage; i++) {
            PageInfo pageInfo = new PageInfo();
            pageInfo.setCurrPage(i);
            pageInfo.setPrePageResult(1000);
            List<MattersEntity> mattersEntities = mattersService.find(mattersQuery, pageInfo, null).getList();
            for (MattersEntity item : mattersEntities) {
                if ("true".equals(item.getHaveGetMatterInfo())) {
                    MattersDetailEntity itemDetail = mattersDetailService.selectOne(new MattersDetailQuery().matterCode(item.getMatterCode()));
                    if (!ObjectUtils.isEmpty(itemDetail)) {
                        //解析事项详细数据
                        MatterEntity matterEntity = new MatterEntity();
                        matterEntity.initAttrValue();
                        matterEntity.setMatterNo(item.getMatterCode());
                        matterEntity.setTcode(item.getMatterCode());
                        matterEntity.setMatterName(item.getName());
                        matterEntity.setTname(item.getName());
                        matterEntity.setSource(SourceEnum.政务网.getValue());
                        matterEntity.setCreateTime(new Date());
                        matterEntity.setCreateUserName("system");
                        matterEntity.setCreateUserId(1L);
                        String basicinfoJbxx = itemDetail.getBasicinfoJbxx();
                        //解析基础数据
                        //todo 设置基础值
                        //基本信息属性与字段映射表
                        Setting baseInfoSetting = interceptorConfig.getBaseInfoSetting();
                        Setting sqclInfoSetting = interceptorConfig.getSqclInfoSetting();
                        createBaseMatterJson(basicinfoJbxx, baseInfoSetting, matterEntity);

                        matterEntity.setAreaCode(item.getAreaCode());
                        matterEntity.setDeptCode(item.getDeptCode());
                        matterEntity.setThemeCode(item.getThemeCode());
                        matterEntity.setPowerCode(item.getPowerCode());
                        matterEntity.setGroupName(item.getGroupName());
                        matterEntity.setHaveGetMatterInfo(item.getHaveGetMatterInfo());
                        matterEntity.setUsertypeCode(item.getUsertypeCode());
                        matterEntity.setUrl(item.getUrl());
                        this.save(matterEntity);
                        //todo 材料
                        String sqclJson = itemDetail.getMaterialsSqcl();
                        if (!ObjectUtils.isEmpty(sqclJson) || !"{}".equals(sqclJson)) {
                            createSqclJson(sqclJson, sqclInfoSetting, matterEntity);
                        }
                        //todo 受理条件
                        String sltjJson = itemDetail.getConditionsSltj();
                        if (!ObjectUtils.isEmpty(sltjJson) || !"{}".equals(sltjJson)) {
                            createSltjJson(sltjJson, matterEntity);
                        }

                        //todo 收费标准
                        String sfbzJson = itemDetail.getChargingSfbz();
                        if (!ObjectUtils.isEmpty(sfbzJson) || !"{}".equals(sfbzJson)) {
                            createSfbzJson(sfbzJson, matterEntity);
                        }
                        //todo 办理流程
                        String bllcJson = itemDetail.getProcessBllc();
                        if (!ObjectUtils.isEmpty(bllcJson) || !"{}".equals(bllcJson)) {
                            createBllcJson(bllcJson, matterEntity);
                        }
                        //todo 常见问题
                        String cjwtJson = itemDetail.getQuestionCjwt();
                        if (!ObjectUtils.isEmpty(cjwtJson) || !"{}".equals(cjwtJson)) {
                            createCjwtJson(cjwtJson, matterEntity);
                        }
                        //todo 设定依据
                        String sdyjJson = itemDetail.getBasisSdyj();
                        if (!ObjectUtils.isEmpty(sdyjJson) || !"{}".equals(sdyjJson)) {
                            createSdyjJson(sdyjJson, matterEntity);
                        }
                        //todo 中介服务
                        String zjfwJson = itemDetail.getIntermediaryZjfw();
                        if (!ObjectUtils.isEmpty(zjfwJson) || !"{}".equals(zjfwJson)) {
                            createZjfwJson(zjfwJson, matterEntity);
                        }
                    }
                }
            }
            //根据事项查询事项详细等，并更新
        }

    }

    public void createBaseMatterJson(String baseJson, Setting baseInfoSetting, MatterEntity matterEntity) {
        JSONObject root = JSON.parseObject(baseJson);
        JSONArray jsonArray = root.getJSONArray("基本信息");
        if (!ObjectUtils.isEmpty(jsonArray)) {
            for (int i = 0; i < jsonArray.size(); i++) {
                JSONArray subArray = jsonArray.getJSONArray(i);
                if (!ObjectUtils.isEmpty(subArray)) {
                    for (int j = 0; j < subArray.size(); j++) {
                        subArray.getJSONObject(j).entrySet().stream()
                                .peek(m -> {
                                    String value = baseInfoSetting.getOrDefault(m.getKey(), "");
                                    if (!ObjectUtils.isEmpty(value) && !"无".equals(m.getValue())) {
                                        //value 为对象配置的属性值，反射设置属性值
                                        try {
                                            ReflectUtil.setFieldValue(matterEntity, value, m.getValue().toString().replaceAll("、", ","));
                                        } catch (Exception e) {
                                            log.error(String.format("设置值异常！key:%s,value:%s", value, m.getValue()));
                                        }
                                    }
                                }).count();
                    }
                }
            }
        }
        //反射设置事项属性
    }

    public void createSqclJson(String sqclJson, Setting sqclInfoSetting, MatterEntity matterEntity) {
        JSONObject root = null;
        try {
            root = JSON.parseObject(sqclJson);
        } catch (Exception e) {
            sqclJson = sqclJson.replaceAll("\\\\", "\\\\\\\\");
            try {
                root = JSON.parseObject(sqclJson);
            } catch (Exception e1) {
                //log.error("error json:" + sqclJson);
                return;
            }
        }

        if (root.isEmpty()) {
            //log.info("root is null");
            return;
        }
        JSONArray jsonArray = null;
        try {
            jsonArray = root.getJSONArray("申请材料");
        } catch (Exception e) {
            log.error("error 无申请材料");
        }

        if (ObjectUtils.isEmpty(jsonArray)) {
            log.info("申请材料 is null");
            return;
        }

        for (int i = 0; i < jsonArray.size(); i++) {
            JSONObject secondJsonObject = jsonArray.getJSONObject(i);
            MatterDatumEntity datumEntity = new MatterDatumEntity();
            datumEntity.initAttrValue();
            datumEntity.setMatterId(matterEntity.getId());
            datumEntity.setMaterialName(matterEntity.getMatterName());
            datumEntity.setSource(SourceEnum.政务网.getValue());
            datumEntity.setCreateTime(new Date());
            datumEntity.setCreateUser("system");
            datumEntity.setCreateUserId(1L);

            //材料详情
            JSONArray jsonArrayThree = secondJsonObject.getJSONArray("材料详情");
            if (!ObjectUtils.isEmpty(jsonArrayThree)) {
                for (int j = 0; j < jsonArrayThree.size(); j++) {
                    jsonArrayThree.getJSONObject(j).entrySet().stream().peek(m -> {
                        String value = sqclInfoSetting.getOrDefault(m.getKey(), "");
                        if (!ObjectUtils.isEmpty(value)) {
                            //value 为对象配置的属性值，反射设置属性值
                            try {
                                ReflectUtil.setFieldValue(datumEntity, value, m.getValue().toString().replaceAll("、", ","));
                            } catch (Exception e) {
                                log.error(String.format("设置值异常！key:%s,value:%s", value, m.getValue()));
                            }
                        }
                    }).count();
                }
            }
            matterDatumService.save(datumEntity);
            //示例样表
            JSONArray jsonArrayone = secondJsonObject.getJSONArray("示例样表");
            if (!ObjectUtils.isEmpty(jsonArrayone)) {
                ArrayList<MatterDatumFileEntity> fileArrayList = new ArrayList<>();
                for (int j = 0; j < jsonArrayone.size(); j++) {
                    MatterDatumFileEntity fileEntity = new MatterDatumFileEntity();
                    fileEntity.setCreateTime(new Date());
                    fileEntity.setCreateUser("system");
                    fileEntity.setCreateUserId(1L);
                    fileEntity.setDatumId(datumEntity.getId());
                    fileEntity.setMaterialName(datumEntity.getMaterialName());
                    fileEntity.setSource(SourceEnum.政务网.getValue());
                    fileEntity.setFiletype(FiletypeEnum.空白表格.getValue());
                    jsonArrayone.getJSONObject(j).entrySet().stream().peek(m -> {
                        fileEntity.setFileName(m.getKey());
                        fileEntity.setFileUrl(m.getValue().toString());
                    }).count();
                    fileArrayList.add(fileEntity);
                }
                matterDatumFileService.save(fileArrayList);

            }
            //空白表格
            JSONArray jsonArraytwo = secondJsonObject.getJSONArray("空白表格");
            if (!ObjectUtils.isEmpty(jsonArraytwo)) {
                ArrayList<MatterDatumFileEntity> fileArrayList = new ArrayList<>();
                for (int j = 0; j < jsonArraytwo.size(); j++) {
                    MatterDatumFileEntity fileEntity = new MatterDatumFileEntity();
                    fileEntity.setCreateTime(new Date());
                    fileEntity.setCreateUser("system");
                    fileEntity.setDatumId(datumEntity.getId());
                    fileEntity.setMaterialName(datumEntity.getMaterialName());
                    fileEntity.setSource(SourceEnum.政务网.getValue());
                    fileEntity.setFiletype(FiletypeEnum.空白表格.getValue());
                    jsonArraytwo.getJSONObject(j).entrySet().stream().peek(m -> {
                        fileEntity.setFileName(m.getKey());
                        fileEntity.setFileUrl(m.getValue().toString());
                    }).count();
                    fileArrayList.add(fileEntity);
                }
                matterDatumFileService.save(fileArrayList);
            }
        }

    }

    public void createSltjJson(String sltjJson, MatterEntity matterEntity) {
        if (ObjectUtils.isEmpty(sltjJson) || "{}".equals(sltjJson)) {
            return;
        }
        JSONObject root = null;
        try {
            root = JSON.parseObject(sltjJson);
        } catch (Exception e) {
            sltjJson = sltjJson.replaceAll("\\\\", "\\\\\\\\");
            try {
                root = JSON.parseObject(sltjJson);
            } catch (Exception e1) {
                // log.error("error json:" + sltjJson);
                return;
            }
        }

        if (root.isEmpty()) {
            //log.info("root is null");
            return;
        }

        String value = null;
        try {
            value = root.getString("受理条件");
            MatterAcceptEntity matterAcceptEntity = new MatterAcceptEntity();
            matterAcceptEntity.initAttrValue();
            matterAcceptEntity.setContent(value);
            matterAcceptEntity.setMatterId(matterEntity.getId());
            matterAcceptEntity.setMatterName(matterEntity.getMatterName());
            matterAcceptEntity.setSource(SourceEnum.政务网.getValue());
            matterAcceptEntity.setCreateTime(new Date());
            matterAcceptEntity.setCreateUser("system");
            matterAcceptEntity.setCreateUserId(1L);
            matterAcceptService.save(matterAcceptEntity);
        } catch (Exception e) {
            log.error("error 受理条件", e);
        }
    }

    public void createSfbzJson(String sfbzJson, MatterEntity matterEntity) {
        if (ObjectUtils.isEmpty(sfbzJson) || "{}".equals(sfbzJson)) {
            return;
        }
        JSONObject root = null;
        try {
            root = JSON.parseObject(sfbzJson);
        } catch (Exception e) {
            sfbzJson = sfbzJson.replaceAll("\\\\", "\\\\\\\\");
            try {
                root = JSON.parseObject(sfbzJson);
            } catch (Exception e1) {
                //log.error("error json:" + sfbzJson);
                return;
            }
        }

        if (root.isEmpty()) {
            //log.info("root is null");
            return;
        }

        String value = null;
        try {
            value = root.getString("收费标准");
            MatterChargesEntity matterChargesEntity = new MatterChargesEntity();
            matterChargesEntity.initAttrValue();
            matterChargesEntity.setContent(value);
            matterChargesEntity.setMatterId(matterEntity.getId());
            matterChargesEntity.setMatterName(matterEntity.getMatterName());
            matterChargesEntity.setSource(SourceEnum.政务网.getValue());
            matterChargesEntity.setCreateTime(new Date());
            matterChargesEntity.setCreateUser("system");
            matterChargesEntity.setCreateUserId(1L);
            matterChargesService.save(matterChargesEntity);
        } catch (Exception e) {
            log.error("error 收费标准", e);
        }
    }

    public void createCjwtJson(String cjwtJson, MatterEntity matterEntity) {
        if (ObjectUtils.isEmpty(cjwtJson) || "{}".equals(cjwtJson)) {
            //log.info("json:" + cjwtJson);
            return;
        }
        JSONObject root = null;
        try {
            root = JSON.parseObject(cjwtJson);
        } catch (Exception e) {
            cjwtJson = cjwtJson.replaceAll("\\\\", "\\\\\\\\");
            try {
                root = JSON.parseObject(cjwtJson);
            } catch (Exception e1) {
                //log.error("error json:" + cjwtJson);
                return;
            }
        }

        if (root.isEmpty()) {
            //log.info("root is null");
            return;
        }
        JSONArray jsonArray = null;
        try {
            jsonArray = root.getJSONArray("常见问题");
        } catch (Exception e) {
            //log.error("error 无常见问题");
        }

        if (ObjectUtils.isEmpty(jsonArray)) {
            //log.info("常见问题 is null");
            return;
        }

        for (int i = 0; i < jsonArray.size(); i++) {
            JSONObject jsonObject = jsonArray.getJSONObject(i);
            List<MatterQuestionEntity> questionEntityList = jsonObject.entrySet().stream().map(question -> {
                MatterQuestionEntity questionEntity = new MatterQuestionEntity();
                questionEntity.initAttrValue();
                questionEntity.setQuestion(question.getKey());
                questionEntity.setAnswer(question.getValue().toString());
                questionEntity.setMatterId(matterEntity.getId());
                questionEntity.setMatterName(matterEntity.getMatterName());
                questionEntity.setSource(SourceEnum.政务网.getValue());
                questionEntity.setCreateTime(new Date());
                questionEntity.setCreateUser("system");
                return questionEntity;
            }).collect(Collectors.toList());
            matterQuestionService.save(questionEntityList);
        }
    }

    public void createBllcJson(String bllcJson, MatterEntity matterEntity) {
        if (ObjectUtils.isEmpty(bllcJson) || "{}".equals(bllcJson)) {
            //log.info("json:" + cjwtJson);
            return;
        }
        JSONObject root = null;
        try {
            root = JSON.parseObject(bllcJson);
        } catch (Exception e) {
            bllcJson = bllcJson.replaceAll("\\\\", "\\\\\\\\");
            try {
                root = JSON.parseObject(bllcJson);
            } catch (Exception e1) {
                log.error("error json:" + bllcJson);
                return;
            }
        }

        if (root == null || root.isEmpty()) {
            //log.info("root is null");
            return;
        }
        JSONArray jsonArray = null;
        try {
            jsonArray = root.getJSONArray("办理流程");
        } catch (Exception e) {
            log.error("error 无办理流程");
        }

        if (ObjectUtils.isEmpty(jsonArray)) {
            //log.info("办理流程 is null");
            return;
        }

        List<MatterFlowlimitEntity> flowlimitEntityArrayList = new ArrayList<>();

        for (int i = 0; i < jsonArray.size(); i++) {
            MatterFlowlimitEntity matterFlowlimitEntity = new MatterFlowlimitEntity();
            matterFlowlimitEntity.initAttrValue();
            matterFlowlimitEntity.setMatterId(matterEntity.getId());
            matterFlowlimitEntity.setMatterName(matterEntity.getMatterName());
            matterFlowlimitEntity.setSource(SourceEnum.政务网.getValue());
            matterFlowlimitEntity.setCreateTime(new Date());
            matterFlowlimitEntity.setCreateUser("system");
            matterFlowlimitEntity.setCreateUserId(1L);
            JSONArray innerJsonArray = jsonArray.getJSONArray(i);
            for (int j = 0; j < innerJsonArray.size(); j++) {
                innerJsonArray.getJSONObject(j).entrySet().stream().peek(m -> {
                    Boolean bool = false;
                    if ("办理流程".equals(m.getKey().trim())) {
                        matterFlowlimitEntity.setFlowName(m.getValue().toString());
                        bool = true;
                    }

                    if ("办理时长".equals(m.getKey().trim())) {
                        matterFlowlimitEntity.setFlowTime(m.getValue().toString());
                        bool = true;
                    }

                    if ("转外时限".equals(m.getKey().trim())) {
                        matterFlowlimitEntity.setFlowLimit(m.getValue().toString());
                        bool = true;
                    }

                    if ("办理流程说明".equals(m.getKey().trim())) {
                        matterFlowlimitEntity.setFlowDesc(m.getValue().toString());
                        bool = true;
                    }
                }).count();
            }
            flowlimitEntityArrayList.add(matterFlowlimitEntity);
        }
        matterFlowlimitService.save(flowlimitEntityArrayList);

    }

    public void createSdyjJson(String sdyjJson, MatterEntity matterEntity) {
        if (ObjectUtils.isEmpty(sdyjJson) || "{}".equals(sdyjJson)) {
            //log.info("json:" + cjwtJson);
            return;
        }
        JSONObject root = null;
        try {
            root = JSON.parseObject(sdyjJson);
        } catch (Exception e) {
            sdyjJson = sdyjJson.replaceAll("\\\\", "\\\\\\\\");
            try {
                root = JSON.parseObject(sdyjJson);
            } catch (Exception e1) {
                log.error("error json:" + sdyjJson);
                return;
            }
        }

        if (root.isEmpty()) {
            log.info("root is null");
            return;
        }
        JSONArray jsonArray = null;
        try {
            jsonArray = root.getJSONArray("设定依据");
        } catch (Exception e) {
            log.error("error 无设定依据");
        }

        if (ObjectUtils.isEmpty(jsonArray)) {
            //log.info("设定依据 is null");
            return;
        }

        List<MatterSetbaseEntity> setbaseEntityArrayList = new ArrayList<>();
        for (int i = 0; i < jsonArray.size(); i++) {
            MatterSetbaseEntity matterSetbaseEntity = new MatterSetbaseEntity();
            matterSetbaseEntity.setMatterId(matterEntity.getId());
            matterSetbaseEntity.setMatterName(matterEntity.getMatterName());
            matterSetbaseEntity.setSource(SourceEnum.政务网.getValue());
            matterSetbaseEntity.setCreateTime(new Date());
            matterSetbaseEntity.setCreateUser("system");
            matterSetbaseEntity.setCreateUserId(1L);
            JSONArray innerJsonArray = jsonArray.getJSONArray(i);
            for (int j = 0; j < innerJsonArray.size(); j++) {
                innerJsonArray.getJSONObject(j).entrySet().stream().peek(m -> {
                    if ("法律法规名称".equals(m.getKey().trim())) {
                        matterSetbaseEntity.setPolicyName(m.getValue().toString());
                    }
                    if ("法规类型".equals(m.getKey().trim())) {
                        matterSetbaseEntity.setPolicyType(m.getValue().toString());
                    }

                    if ("条例".equals(m.getKey().trim())) {
                        matterSetbaseEntity.setPolicyitem(m.getValue().toString());
                    }

                    if ("条款内容".equals(m.getKey().trim())) {
                        matterSetbaseEntity.setContent(m.getValue().toString());
                    }

                }).count();
            }
            setbaseEntityArrayList.add(matterSetbaseEntity);
        }
        matterSetbaseService.save(setbaseEntityArrayList);

    }

    public void createZjfwJson(String zjfwJson, MatterEntity matterEntity) {
        if (ObjectUtils.isEmpty(zjfwJson) || "{}".equals(zjfwJson)) {
            return;
        }
        JSONObject root = null;
        try {
            root = JSON.parseObject(zjfwJson);
        } catch (Exception e) {
            zjfwJson = zjfwJson.replaceAll("\\\\", "\\\\\\\\");
            try {
                root = JSON.parseObject(zjfwJson);
            } catch (Exception e1) {
                log.error("error json:" + zjfwJson);
                return;
            }
        }

        if (root.isEmpty()) {
            log.info("root is null");
            return;
        }
        JSONArray jsonArray = null;
        try {
            jsonArray = root.getJSONArray("中介服务");
        } catch (Exception e) {
            //log.error("error 中介服务:"+zjfwJson);
        }

        if (ObjectUtils.isEmpty(jsonArray)) {
            // log.info("中介服务 is null");
            return;
        }
        List<MatterIntermediaryEntity> intermediaryEntityArrayList = new ArrayList<>();
        for (int i = 0; i < jsonArray.size(); i++) {
            MatterIntermediaryEntity matterIntermediaryEntity = new MatterIntermediaryEntity();
            matterIntermediaryEntity.initAttrValue();
            matterIntermediaryEntity.setMatterId(matterEntity.getId());
            matterIntermediaryEntity.setMatterName(matterEntity.getMatterName());
            matterIntermediaryEntity.setSource(SourceEnum.政务网.getValue());
            matterIntermediaryEntity.setCreateTime(new Date());
            matterIntermediaryEntity.setCreateUser("system");
            matterIntermediaryEntity.setCreateUserId(1L);

            jsonArray.getJSONObject(i).entrySet().stream().forEach(m -> {
                matterIntermediaryEntity.setTitle(m.getKey().trim());
                matterIntermediaryEntity.setRemark(m.getValue().toString());

                if ("中介服务实施机构及资质资格要求".equals(m.getKey().trim())) {
                    matterIntermediaryEntity.setIntermediaryRequest(m.getValue().toString());
                }

                if ("服务时限".equals(m.getKey().trim())) {
                    matterIntermediaryEntity.setIntermediaryRequestTime(m.getValue().toString());
                }
              /*  if ("中介服务事项名称".equals(m.getKey().trim())) {
                    matterIntermediaryEntity.setTitle(m.getValue().toString());
                }
              */
            });
            intermediaryEntityArrayList.add(matterIntermediaryEntity);
        }
        matterIntermediaryService.save(intermediaryEntityArrayList);
    }

    @Override
    public Map<String, String> getBaseInfoMap(Context context) {
        Map<String, String> baseInfoMap = new HashMap<>();
        List<MattersDetailEntity> mattersDetailEntities = mattersDetailService.find(new MattersDetailQuery());
        for (MattersDetailEntity itemDetail : mattersDetailEntities) {
            buildBaseJson(itemDetail.getBasicinfoJbxx(), baseInfoMap);
        }
        return baseInfoMap;
    }

    @Override
    public Map<String, String> getSqclInfoMap(Context context) {
        Map<String, String> sqclInfoMap = new HashMap<>();
        List<MattersDetailEntity> mattersDetailEntities = mattersDetailService.find(new MattersDetailQuery());
        for (MattersDetailEntity itemDetail : mattersDetailEntities) {
            buildSqclJson(itemDetail.getMaterialsSqcl(), sqclInfoMap);
        }
        return sqclInfoMap;
    }


    public void buildBaseJson(String json, Map<String, String> model) {
        //将基本信息转成map集合返回
        JSONObject root = JSON.parseObject(json);
        JSONArray jsonArray = root.getJSONArray("基本信息");
        if (!ObjectUtils.isEmpty(jsonArray)) {
            for (int i = 0; i < jsonArray.size(); i++) {
                JSONArray subArray = jsonArray.getJSONArray(i);
                if (!ObjectUtils.isEmpty(subArray)) {
                    for (int j = 0; j < subArray.size(); j++) {
                        subArray.getJSONObject(j).entrySet().stream()
                                .filter(f -> f.getKey().length() < 12)
                                .filter(f -> f.getKey().indexOf("许可证") == -1)
                                .filter(f -> f.getKey().indexOf("《") == -1)
                                .filter(f -> f.getKey().indexOf("x") == -1)
                                .filter(f -> f.getKey().indexOf("X") == -1)
                                .filter(f -> f.getKey().indexOf("*") == -1)
                                .filter(f -> f.getKey().indexOf("关于") == -1)
                                .filter(f -> f.getKey().indexOf(".") == -1)
                                .filter(f -> DataUtil.converStr2Int(f.getKey(), 0) == 0)
                                .filter(f -> !"下载".equals(f.getValue().toString().trim()))
                                .filter(f -> !"查看".equals(f.getValue().toString().trim()))
                                .peek(m -> {
                                    // model.put(m.getKey(), m.getValue().toString());
                                    model.put(m.getKey(), "");
                                }).count();
                    }
                }
            }

        }
    }


    public void buildSqclJson(String sqclJson, Map<String, String> model) {
        //将基本信息转成map集合返回
        JSONObject root = null;
        try {
            root = JSON.parseObject(sqclJson);
        } catch (Exception e) {
            sqclJson = sqclJson.replaceAll("\\\\", "\\\\\\\\");
            try {
                root = JSON.parseObject(sqclJson);
            } catch (Exception e1) {
                log.error("error json:" + sqclJson);
                return;
            }
        }

        if (root.isEmpty()) {
            log.info("root is null");
            return;
        }
        JSONArray jsonArray = null;
        try {
            jsonArray = root.getJSONArray("申请材料");
        } catch (Exception e) {
            log.error("error 无申请材料");
        }

        if (ObjectUtils.isEmpty(jsonArray)) {
            //  log.info("申请材料 is null");
            return;
        }
        if (!ObjectUtils.isEmpty(jsonArray)) {
            for (int i = 0; i < jsonArray.size(); i++) {
                try {
                    JSONObject subObject = jsonArray.getJSONObject(i);
                    if (!ObjectUtils.isEmpty(subObject)) {
                        JSONArray subsArray = subObject.getJSONArray("材料详情");
                        if (!ObjectUtils.isEmpty(subsArray)) {
                            for (int j = 0; j < subsArray.size(); j++) {
                                subsArray.getJSONObject(j).entrySet().stream()
                                        .peek(m -> {
                                            if ("要求提供材料的依据".equals(m.getKey())
                                                    || "备注".equals(m.getKey())
                                                    || "来源渠道说明".equals(m.getKey())
                                                    || "材料名称".equals(m.getKey())
                                                    || "受理标准".equals(m.getKey())
                                                    || "填报须知".equals(m.getKey())
                                                    || "来源渠道".equals(m.getKey())) {
                                                model.put(m.getKey(), "");
                                            } else {
                                                if (model.containsKey(m.getKey())) {
                                                    String value = model.get(m.getKey());
                                                    Set<String> valueSet = Arrays.asList(value.split(",")).stream().collect(Collectors.toSet());
                                                    if (!valueSet.contains(m.getValue().toString())) {
                                                        valueSet.add(m.getValue().toString());
                                                    }
                                                    model.put(m.getKey(), valueSet.stream().collect(Collectors.joining(",")));
                                                } else {
                                                    model.put(m.getKey(), m.getValue().toString());
                                                }
                                            }
                                            //log.info(String.format("model:%s", JSON.toJSONString(model)));
                                            // model.put(m.getKey(), m.getValue().toString());
                                            //model.put(m.getKey(), "");
                                        }).count();
                            }
                        }
                    }
                } catch (Exception e) {
                    log.error("异常:" + sqclJson);
                }
            }

        }
    }


    @Override
    public Rest<String> buildMatterDetail(MatterEntity matterEntity, Context context) {
        //根据事项url 获取事项详细信息，构建相关详细信息
        // String html = null;
        //Boolean bool = false;
        int reCount = 0;
        while (reCount < MAX_RETRY_COUNT) {
            try {
                if (ObjectUtils.isEmpty(matterEntity.getUrl())) {
                    return Rest.fail("事项url不能为空!");
                }
                Document dom = Jsoup.connect(matterEntity.getUrl())
                        .ignoreContentType(true)
                        .ignoreHttpErrors(true)
                        .timeout(60 * 1000).get();
                // html = Jsoup.connect(matterEntity.getUrl()).get().body().html();
                // html = HttpUtil.get(matterEntity.getUrl());
                //System.out.println(html);
                // log.info("build detail:" + matterEntity.getMatterNo());
                //设置基本属性值
                Setting baseInfoSetting = interceptorConfig.getBaseInfoSetting();
                Setting sqclInfoSetting = interceptorConfig.getSqclInfoSetting();


                //根据本地与远端事项版本觉得是否更新或者新增
                Integer matterEditionLocal = DataUtil.converStr2Int(matterEntity.getMatterEdition(), 0);
                Map<String, String> baseInfoMap = MatterDetailHtmlParseUtil.getbaseInfoMapByHtml(dom);
                Integer matterEditionRemote = DataUtil.converStr2Int(baseInfoMap.getOrDefault("事项版本", "0"), 0);

                //由于附件连接有效性，强制更新材料属性与附件地址
                saveDatumInfo(matterEntity, context, dom, sqclInfoSetting);

              /*  if (matterEditionLocal >= matterEditionRemote) {
                    return Rest.fail("本地事项版本大于等于远端，不需要更新！");
                }*/

                // log.info("更新详细事项==>{},id===>{},localVersion===>{},newVersion==>{}", matterEntity.getMatterName(),matterEntity.getId(),matterEntity.getMatterEdition(), matterEditionRemote);
                //更新部门信息
                DeptEntity extCache = deptService.selectOne(new DeptQuery().deptNumber(matterEntity.getDeptCode()));
                // DeptEntity extCache = deptService.getExtCache(matterEntity.getDeptCode());
                matterEntity.setDeptName(extCache == null ? "" : extCache.getName());
                //构建基础信息参数
                savebaseInfo(matterEntity, baseInfoMap, baseInfoSetting);

                //更新受理条件
                saveSltjInfo(matterEntity, context, dom);
                //更新办理流程
                saveBllcInfo(matterEntity, context, dom);
                //更新收费标准
                saveSfbzInfo(matterEntity, context, dom);
                //设定依据
                saveSdyjInfo(matterEntity, context, dom);
                //中介服务
                saveZjfwInfo(matterEntity, context, dom);
                //常见问题
                saveCjwtInfo(matterEntity, context, dom);
                //更新事项更新标志
                matterEntity.setHaveGetMatterInfo("true");

                matterEntity.setMatterEdition(matterEditionRemote + "");

                return Rest.ok("更新详细成功！");

            } catch (SocketTimeoutException readTimeoutException) {
                log.info("请求超时，休眠后重试");
                reCount++;
                try {
                    Thread.sleep(2000);
                } catch (InterruptedException e) {
                }
            } catch (ConnectException connectException) {
                log.info("链接拒绝，休眠后重试");
                reCount++;
                try {
                    Thread.sleep(2000);
                } catch (InterruptedException e) {
                }
            } catch (IOException ioException) {
                log.info("Invalid Http response，休眠后重试");
                reCount++;
                try {
                    Thread.sleep(2000);
                } catch (InterruptedException e) {
                }
            } catch (Exception e) {
                log.error(String.format("获取详细异常! id:%s,matterCode:%s,url:%s", matterEntity.getId(), matterEntity.getMatterNo(), matterEntity.getUrl()), e);
                return Rest.fail("更新事项失败！" + e.getMessage());
            }
        }
        return Rest.fail("更新事项失败！");

    }


    private void saveCjwtInfo(MatterEntity matterEntity, Context context, Document dom) {
        matterQuestionService.deleteByMatterId(matterEntity.getId(), context);
        List<Map<String, Object>> cjwtMapList = MatterDetailHtmlParseUtil.getCjwtMapByHtml(dom);
        List<MatterQuestionEntity> questionEntityList = new ArrayList<>();
        for (Map<String, Object> cjwtMap : cjwtMapList) {
            List<MatterQuestionEntity> collect = cjwtMap.entrySet().stream().map(question -> {
                MatterQuestionEntity questionEntity = new MatterQuestionEntity();
                questionEntity.initAttrValue();
                questionEntity.setQuestion(question.getKey());
                questionEntity.setAnswer(question.getValue().toString());
                questionEntity.setMatterId(matterEntity.getId());
                questionEntity.setMatterCode(matterEntity.getMatterNo());
                questionEntity.setMatterName(matterEntity.getMatterName());
                questionEntity.setSource(SourceEnum.政务网.getValue());
                questionEntity.setCreateTime(new Date());
                questionEntity.setCreateUser("system");
                return questionEntity;
            }).collect(Collectors.toList());
            questionEntityList.addAll(collect);
        }
        matterQuestionService.save(questionEntityList);
    }

    private void saveZjfwInfo(MatterEntity matterEntity, Context context, Document dom) {
        matterIntermediaryService.deleteByMatterId(matterEntity.getId(), context);
        Map<String, String> zjfwMap = MatterDetailHtmlParseUtil.getZjfwMapByHtml(dom);
        List<MatterIntermediaryEntity> intermediaryEntityArrayList = new ArrayList<>();
        zjfwMap.entrySet().stream().forEach(m -> {
            MatterIntermediaryEntity matterIntermediaryEntity = new MatterIntermediaryEntity();
            matterIntermediaryEntity.initAttrValue();
            matterIntermediaryEntity.setMatterId(matterEntity.getId());
            matterIntermediaryEntity.setMatterCode(matterEntity.getMatterNo());
            matterIntermediaryEntity.setMatterName(matterEntity.getMatterName());
            matterIntermediaryEntity.setSource(SourceEnum.政务网.getValue());
            matterIntermediaryEntity.setCreateTime(new Date());
            matterIntermediaryEntity.setCreateUser("system");
            matterIntermediaryEntity.setCreateUserId(1L);
            matterIntermediaryEntity.setTitle(m.getKey().trim());
            matterIntermediaryEntity.setRemark(m.getValue());
            intermediaryEntityArrayList.add(matterIntermediaryEntity);
        });
        matterIntermediaryService.save(intermediaryEntityArrayList);
    }

    private void saveSdyjInfo(MatterEntity matterEntity, Context context, Document dom) {
        matterSetbaseService.deleteByMatterId(matterEntity.getId(), context);
        List<MatterSetbaseEntity> setbaseEntityArrayList = new ArrayList<>();
        List<Map<String, Object>> sdyjMapList = MatterDetailHtmlParseUtil.getSdyjMapByHtml(dom);
        for (Map<String, Object> sdyjMap : sdyjMapList) {
            MatterSetbaseEntity matterSetbaseEntity = new MatterSetbaseEntity();
            matterSetbaseEntity.setMatterId(matterEntity.getId());
            matterSetbaseEntity.setMatterCode(matterEntity.getMatterNo());
            matterSetbaseEntity.setMatterName(matterEntity.getMatterName());
            matterSetbaseEntity.setSource(SourceEnum.政务网.getValue());
            matterSetbaseEntity.setCreateTime(new Date());
            matterSetbaseEntity.setCreateUser("system");
            matterSetbaseEntity.setCreateUserId(1L);
            sdyjMap.entrySet().stream().forEach(m -> {
                if ("法律法规名称".equals(m.getKey().trim())) {
                    matterSetbaseEntity.setPolicyName(m.getValue().toString());
                }
                if ("法规类型".equals(m.getKey().trim())) {
                    matterSetbaseEntity.setPolicyType(m.getValue().toString());
                }

                if ("条例".equals(m.getKey().trim())) {
                    matterSetbaseEntity.setPolicyitem(m.getValue().toString());
                }

                if ("条款内容".equals(m.getKey().trim())) {
                    matterSetbaseEntity.setContent(m.getValue().toString());
                }

            });
            setbaseEntityArrayList.add(matterSetbaseEntity);
        }
        matterSetbaseService.save(setbaseEntityArrayList);
    }

    private void saveSfbzInfo(MatterEntity matterEntity, Context context, Document dom) {
        Map<String, String> sfbzMap = MatterDetailHtmlParseUtil.getSfbzMapByHtml(dom);
        matterChargesService.deleteByMatterId(matterEntity.getId(), context);
        MatterChargesEntity matterChargesEntity = new MatterChargesEntity();
        matterChargesEntity.initAttrValue();
        matterChargesEntity.setContent(sfbzMap.getOrDefault("sfbz", ""));
        matterChargesEntity.setMatterId(matterEntity.getId());
        matterChargesEntity.setMatterCode(matterEntity.getMatterNo());
        matterChargesEntity.setMatterName(matterEntity.getMatterName());
        matterChargesEntity.setSource(SourceEnum.政务网.getValue());
        matterChargesEntity.setCreateTime(new Date());
        matterChargesEntity.setCreateUser("system");
        matterChargesEntity.setCreateUserId(1L);
        matterChargesService.save(matterChargesEntity);
    }

    private void savebaseInfo(MatterEntity matterEntity, Map<String, String> baseInfoMap, Setting baseInfoSetting) {
        //Map<String, String> baseInfoMap = MatterDetailHtmlParseUtil.getbaseInfoMapByHtml(dom);
        for (Map.Entry<String, String> m : baseInfoMap.entrySet()) {
            String value = baseInfoSetting.getOrDefault(m.getKey(), "");
            if (!ObjectUtils.isEmpty(value)) {
                if (!ObjectUtils.isEmpty(m.getValue()) && !"无".equals(m.getValue())) {
                    try {
                        ReflectUtil.setFieldValue(matterEntity, value, m.getValue());
                    } catch (Exception e) {
                        log.error(String.format("设置值异常！key:%s,value:%s,orginKey:%s", value, m.getValue(), m.getKey()));
                    }
                }
            }
        }
    }

    private void saveBllcInfo(MatterEntity matterEntity, Context context, Document dom) {
        List<Map<String, Object>> bllcMapList = MatterDetailHtmlParseUtil.getbllcMapByHtml(dom);

        List<MatterFlowlimitEntity> flowlimitEntityArrayList = new ArrayList<>();
        for (Map<String, Object> bllcMap : bllcMapList) {
            MatterFlowlimitEntity matterFlowlimitEntity = new MatterFlowlimitEntity();
            matterFlowlimitEntity.initAttrValue();
            matterFlowlimitEntity.setMatterId(matterEntity.getId());
            matterFlowlimitEntity.setMatterCode(matterEntity.getMatterNo());
            matterFlowlimitEntity.setMatterName(matterEntity.getMatterName());
            matterFlowlimitEntity.setSource(SourceEnum.政务网.getValue());
            matterFlowlimitEntity.setCreateTime(new Date());
            matterFlowlimitEntity.setCreateUser("system");
            matterFlowlimitEntity.setCreateUserId(1L);

            bllcMap.entrySet().stream().forEach(m -> {
                if ("办理流程".equals(m.getKey().trim())) {
                    matterFlowlimitEntity.setFlowName(m.getValue().toString());
                }
                if ("办理时长".equals(m.getKey().trim())) {
                    matterFlowlimitEntity.setFlowTime(m.getValue().toString());
                }
                if ("转外时限".equals(m.getKey().trim())) {
                    matterFlowlimitEntity.setFlowLimit(m.getValue().toString());
                }
                if ("办理流程说明".equals(m.getKey().trim())) {
                    matterFlowlimitEntity.setFlowDesc(m.getValue().toString());
                }
            });
            flowlimitEntityArrayList.add(matterFlowlimitEntity);
        }

        if (!ObjectUtils.isEmpty(flowlimitEntityArrayList)) {
            matterFlowlimitService.deleteByMatterId(matterEntity.getId(), context);
            matterFlowlimitService.save(flowlimitEntityArrayList);
        }

    }

    private void saveSltjInfo(MatterEntity matterEntity, Context context, Document dom) {
        matterAcceptService.deleteByMatterId(matterEntity.getId(), context);
        Map<String, String> sltjMap = MatterDetailHtmlParseUtil.getSltjMapByHtml(dom);
        MatterAcceptEntity matterAcceptEntity = new MatterAcceptEntity();
        matterAcceptEntity.initAttrValue();
        matterAcceptEntity.setContent(sltjMap.getOrDefault("sltj", ""));
        matterAcceptEntity.setMatterId(matterEntity.getId());
        matterAcceptEntity.setMatterCode(matterEntity.getMatterNo());
        matterAcceptEntity.setMatterName(matterEntity.getMatterName());
        matterAcceptEntity.setSource(SourceEnum.政务网.getValue());
        matterAcceptEntity.setCreateTime(new Date());
        matterAcceptEntity.setCreateUser("system");
        matterAcceptEntity.setCreateUserId(1L);
        matterAcceptService.save(matterAcceptEntity);
    }

    private void saveDatumInfo(MatterEntity matterEntity, Context context, Document dom, Setting sqclInfoSetting) {
        Long[] datumIds = matterDatumService.find(new MatterDatumQuery().matterId(matterEntity.getId()).source(SourceEnum.政务网.getValue())).stream().map(item -> item.getId()).toArray(Long[]::new);
        if (!ObjectUtils.isEmpty(datumIds)) {
            matterDatumService.remove(datumIds, context);
            Long[] datumFileIds = matterDatumFileService.find(new MatterDatumFileQuery().datumIdList(Arrays.asList(datumIds))).stream().map(item -> item.getId()).toArray(Long[]::new);
            if (!ObjectUtils.isEmpty(datumFileIds)) {
                matterDatumService.remove(datumFileIds, context);
            }
        }

        List<Map<String, Object>> sqclList = MatterDetailHtmlParseUtil.getsqclInfoMapByHtml(dom);
        for (Map<String, Object> sqclMap : sqclList) {
            Map<String, String> baseinfoMap = (HashMap<String, String>) sqclMap.get("baseinfo");
            List<MatterDatumFileEntity> blankList = (List<MatterDatumFileEntity>) sqclMap.get("blankList");
            List<MatterDatumFileEntity> sampleList = (List<MatterDatumFileEntity>) sqclMap.get("sampleList");

            MatterDatumEntity datumEntity = new MatterDatumEntity();
            datumEntity.initAttrValue();
            datumEntity.setMatterId(matterEntity.getId());
            datumEntity.setMatterCode(matterEntity.getMatterNo());
            datumEntity.setMatterName(matterEntity.getMatterName());
            datumEntity.setSource(SourceEnum.政务网.getValue());
            datumEntity.setCreateTime(new Date());
            datumEntity.setCreateUser("system");
            datumEntity.setCreateUserId(1L);
            //材料详情
            baseinfoMap.entrySet().stream().forEach(m -> {
                String value = sqclInfoSetting.getOrDefault(m.getKey(), "");
                if (!ObjectUtils.isEmpty(value)) {
                    try {
                        ReflectUtil.setFieldValue(datumEntity, value, m.getValue());
                    } catch (Exception e) {
                        log.error(String.format("设置值异常！key:%s,value:%s", value, m.getValue()));
                    }
                }
            });
            matterDatumService.save(datumEntity);

            //示例样表
            if (!ObjectUtils.isEmpty(sampleList)) {
                sampleList.forEach(fileEntity -> {
                    fileEntity.setDatumId(datumEntity.getId());
                    fileEntity.setMatterCode(matterEntity.getMatterNo());
                    fileEntity.setMaterialName(datumEntity.getMaterialName());
                });
            }
            matterDatumFileService.save(sampleList);

            //空白样表
            if (!ObjectUtils.isEmpty(blankList)) {
                blankList.forEach(fileEntity -> {
                    fileEntity.setDatumId(datumEntity.getId());
                    fileEntity.setMatterCode(matterEntity.getMatterNo());
                    fileEntity.setMaterialName(datumEntity.getMaterialName());
                });
            }
            matterDatumFileService.save(blankList);
        }
    }




    @Override
    public Rest<Void> genMatterMarkdown(MatterQuery matterQuery, Context context) {
        List<String> strList = new ArrayList<>();

        BaseAreaQuery baseAreaQuery = new BaseAreaQuery();
        baseAreaQuery.setAreaCode(matterQuery.getAreaCode());
        BaseAreaEntity baseAreaEntity = baseAreaService.selectOne(baseAreaQuery);

        List<MatterEntity> mattersList = matterExtService.getWebMatterListByAreaCode(matterQuery.getAreaCode(), context).getData();

        //List<MatterEntity> mattersList = this.find(matterQuery);
        int total = 0;
        int page = 1;
        for (MatterEntity matterEntity : mattersList) {
            if (ObjectUtils.isEmpty(matterEntity.getUrl())) {
                continue;
            }
            StringBuilder sb = new StringBuilder();
            int reCount = 0;
            while (reCount < MAX_RETRY_COUNT) {
                try {
                    Document dom = Jsoup.connect(matterEntity.getUrl())
                            .ignoreContentType(true)
                            .ignoreHttpErrors(true)
                            .timeout(60 * 1000).get();

                    //基本属性
                    sb.append("# " + matterEntity.getMatterName() + "\n");
                    sb.append("\n");
                    sb.append("## 基本信息\n");
                    Map<String, String> baseInfoMap = MatterDetailHtmlParseUtil.getbaseInfoMapByHtml(dom);

                    sb.append(String.format("| **名称** \t| %s \t|", "", ""));
                    sb.append("\n");
                    sb.append("|-------:\t|---\t|");
                    sb.append("\n");

                    for (Map.Entry<String, String> entry : baseInfoMap.entrySet()) {
                        sb.append(String.format("| **%s** \t| %s \t|", entry.getKey(), entry.getValue()));
                        sb.append("\n");
                    }
                    sb.append("\n");
                    //更新材料属性
                    sb.append("## 申请材料\n");
                    /**
                     * |  	| **材料名称** 	| **材料必要性** 	| **原件和复印件** 	| **纸质/电子** 	| **纸质材料份数** 	| **要求提供材料的依据** 	|
                     * |---	|:---:	|---	|---	|---	|---	|---	|
                     * | 1 	| 参保人银行账户信息 	| 123 	| 123 	| 123 	| 123 	| 1131 	|
                     */
                    List<Map<String, Object>> sqclList = MatterDetailHtmlParseUtil.getsqclInfoMapByHtml(dom);

                    if (!ObjectUtils.isEmpty(sqclList)) {
                        Map<String, Object> map = sqclList.get(0);
                        Map<String, String> baseinfoHeadMap = (HashMap<String, String>) map.get("baseinfo");

                        sb.append("|  \t");
                        for (Map.Entry<String, String> entry : baseinfoHeadMap.entrySet()) {
                            String key = entry.getKey();
                            if ("要求提供材料的依据".equals(entry.getKey())) continue;

                            sb.append(String.format("| **%s** \t", key));
                        }
                        sb.append("|\n");

                        sb.append("|:---:\t");
                        for (Map.Entry<String, String> entry : baseinfoHeadMap.entrySet()) {
                            if ("要求提供材料的依据".equals(entry.getKey())) continue;
                            sb.append("|:---:\t");
                        }
                        sb.append("|\n");
                        int count = 1;
                        for (Map<String, Object> sqclMap : sqclList) {
                            sb.append(String.format("| %d \t", count));

                            count++;
                            Map<String, String> baseinfoMap = (HashMap<String, String>) sqclMap.get("baseinfo");
                            for (Map.Entry<String, String> entry : baseinfoMap.entrySet()) {
                                if ("要求提供材料的依据".equals(entry.getKey())) continue;

                                sb.append(String.format("| %s \t", entry.getValue()));


                                /*          sb.append(entry.getKey()).append("  ").append(entry.getValue());
                                            sb.append("\n");
                                 */
                            }
                            sb.append("|\n");
                        }
                    }

                    //更新受理条件
                    sb.append("\n");
                    sb.append("## 受理条件\n");
                    Map<String, String> sltjMap = MatterDetailHtmlParseUtil.getSltjMapByHtml(dom);
                    String sltjStr = sltjMap.getOrDefault("sltj", "  ");
                    sb.append(sltjStr);
                    //更新办理流程
                    sb.append("\n");
                    sb.append("## 办理流程\n");
                    List<Map<String, Object>> bllcMapList = MatterDetailHtmlParseUtil.getbllcMapByHtml(dom);
                    for (Map<String, Object> bllcMap : bllcMapList) {
                        for (Map.Entry<String, Object> entry : bllcMap.entrySet()) {
                            if ("sort".equals(entry.getKey())) {
                                sb.append("流程").append(":").append(entry.getValue());
                            } else {
                                sb.append(entry.getKey()).append("  ").append(entry.getValue());
                            }
                            sb.append("\n");
                        }
                    }

                    //更新收费标准
                    sb.append("\n");
                    sb.append("## 收费标准\n");
                    Map<String, String> sfbzMap = MatterDetailHtmlParseUtil.getSfbzMapByHtml(dom);

                    String sfbzStr = sfbzMap.getOrDefault("sfbz", "  ");
                    sb.append(sfbzStr);

                    //设定依据
                    sb.append("\n");
                    sb.append("## 设定依据\n");
                    List<Map<String, Object>> sdyjMapList = MatterDetailHtmlParseUtil.getSdyjMapByHtml(dom);
                    StringBuilder sb2 = new StringBuilder();
                    for (Map<String, Object> bllcMap : sdyjMapList) {
                        for (Map.Entry<String, Object> entry : bllcMap.entrySet()) {
                            if ("sort".equals(entry.getKey())) continue;
                            sb2.append(entry.getKey()).append("  ").append(entry.getValue());
                            sb2.append("\n");
                        }
                        sb2.append("\n");
                    }
                    sb.append(sb2);
                    //中介服务
                    sb.append("\n");
                    sb.append("## 中介服务\n");
                    Map<String, String> zjfwMap = MatterDetailHtmlParseUtil.getZjfwMapByHtml(dom);

                    if (zjfwMap.size() > 0) {
                        for (Map.Entry<String, String> entry : zjfwMap.entrySet()) {
                            sb.append(entry.getKey()).append("  ").append(entry.getValue());
                        }
                    } else {
                        sb.append("无\n");
                    }


                    //常见问题
                    sb.append("\n");
                    sb.append("## 常见问题\n");
                    List<Map<String, Object>> cjwtMapList = MatterDetailHtmlParseUtil.getCjwtMapByHtml(dom);
                    for (Map<String, Object> bllcMap : cjwtMapList) {
                        for (Map.Entry<String, Object> entry : bllcMap.entrySet()) {
                            sb.append(entry.getKey()).append("  ").append(entry.getValue());
                            sb.append("\n\n");
                        }
                    }
                    strList.add(sb.toString());
                    log.info("事项名称:{}", matterEntity.getMatterName());
                    break;
                } catch (Exception e) {
                    log.error("异常", e);
                    reCount++;
                    try {
                        Thread.sleep(2000);
                    } catch (InterruptedException ie) {
                    }
                }
            }
            total++;
            if (total > 200) {
                total = 0;
                String fileName = "gov" + page + ".md";
                if (!ObjectUtils.isEmpty(baseAreaEntity)) {
                    fileName = baseAreaEntity.getName() + page + ".md";
                }

                String genFilePath = matterQuery.getGenFilePath();
                String path = "";
                if (!ObjectUtils.isEmpty(genFilePath)) {
                    path = genFilePath + fileName;
                } else {
                    path = "E:\\md\\" + fileName;
                }

                FileUtil.writeUtf8Lines(strList, path);
                page++;
                strList = new ArrayList<>();
            }

            // break;
        }

/*        String fileName = "gov-1.md";

        if (!ObjectUtils.isEmpty(baseAreaEntity)) {
            fileName = baseAreaEntity.getName() + ".md";
        }

        String path = "E:\\" + fileName;
        FileUtil.writeUtf8Lines(strList, path);*/

        return Rest.ok();
    }


    @Override
    public Rest<Void> genMatterCsv(MatterQuery matterQuery, Context context) {
        BaseAreaQuery baseAreaQuery = new BaseAreaQuery();
        baseAreaQuery.setAreaCode(matterQuery.getAreaCode());
        BaseAreaEntity baseAreaEntity = baseAreaService.selectOne(baseAreaQuery);

        List<String> strList = new ArrayList<>();
        String path = matterQuery.getGenFilePath() + baseAreaEntity.getName() + ".csv";
        List<MatterEntity> mattersList = this.find(matterQuery);
        for (MatterEntity matterEntity : mattersList) {
            if (ObjectUtils.isEmpty(matterEntity.getUrl())) {
                continue;
            }
            StringBuilder sb = new StringBuilder();
            int reCount = 0;
            while (reCount < MAX_RETRY_COUNT) {
                try {
                    Document dom = Jsoup.connect(matterEntity.getUrl())
                            .ignoreContentType(true)
                            .ignoreHttpErrors(true)
                            .timeout(60 * 1000).get();
                    //  sb.append(matterEntity.getMatterName()).append("\n");

                    //基本属性
                    sb.append(matterEntity.getMatterName()).append("-基本信息,");
                    sb.append("事项名称:").append(matterEntity.getMatterName());
                    Map<String, String> baseInfoMap = MatterDetailHtmlParseUtil.getbaseInfoMapByHtml(dom);
                    StringBuilder sb1 = new StringBuilder();

                    for (Map.Entry<String, String> entry : baseInfoMap.entrySet()) {
                        if (sb1.length() > 280) {
                            sb.append(sb1);
                            sb.append("\n");
                            sb1 = new StringBuilder();
                            sb.append(matterEntity.getMatterName()).append("-基本信息,");
                            sb.append("事项名称:").append(matterEntity.getMatterName());
                        }
                        sb1.append(entry.getKey()).append(":").append(StrUtil.replaceChars(entry.getValue(), ",", "，")).append(";");
                    }
                    sb.append(sb1);

                    sb.append("\n");
                    //更新材料属性
                    sb.append(matterEntity.getMatterName()).append("-申请材料,");
                    sb.append("事项名称:").append(matterEntity.getMatterName()).append("  ");
                    List<Map<String, Object>> sqclList = MatterDetailHtmlParseUtil.getsqclInfoMapByHtml(dom);
                    for (Map<String, Object> sqclMap : sqclList) {
                        Map<String, String> baseinfoMap = (HashMap<String, String>) sqclMap.get("baseinfo");
                        for (Map.Entry<String, String> entry : baseinfoMap.entrySet()) {
                            if (!"材料名称".equals(entry.getKey())) {
                                continue;
                            }
                            sb.append(entry.getKey()).append(":").append(StrUtil.replaceChars(entry.getValue(), ",", "，")).append(";");
                        }
      /*                  List<MatterDatumFileEntity> datumFileEntities = (List<MatterDatumFileEntity>) sqclMap.get("blankList");
                        for (MatterDatumFileEntity datumFileEntity : datumFileEntities) {
                            sb.append("材料名称").append("  ").append(datumFileEntity.getFileName());
                            sb.append("材料地址").append("  ").append(datumFileEntity.getFileUrl());
                        }*/
                    }
                    //更新受理条件
                    sb.append("\n");
                    sb.append(matterEntity.getMatterName()).append("-受理条件,");
                    sb.append("事项名称:").append(matterEntity.getMatterName());
                    Map<String, String> sltjMap = MatterDetailHtmlParseUtil.getSltjMapByHtml(dom);
                    String sltjStr = sltjMap.getOrDefault("sltj", "  ");
                    sb.append(sltjStr);
                    //更新办理流程
                    sb.append("\n");
                    sb.append(matterEntity.getMatterName()).append("-办理流程,");
                    sb.append("事项名称:").append(matterEntity.getMatterName());
                    List<Map<String, Object>> bllcMapList = MatterDetailHtmlParseUtil.getbllcMapByHtml(dom);
                    for (Map<String, Object> bllcMap : bllcMapList) {
                        for (Map.Entry<String, Object> entry : bllcMap.entrySet()) {
                            sb.append(entry.getKey()).append(":").append(StrUtil.replaceChars(entry.getValue().toString(), ",", "，")).append(";");
                        }
                    }

                    //更新收费标准
                    sb.append("\n");
                    sb.append(matterEntity.getMatterName()).append("-收费标准,");
                    sb.append("事项名称:").append(matterEntity.getMatterName());
                    Map<String, String> sfbzMap = MatterDetailHtmlParseUtil.getSfbzMapByHtml(dom);

                    String sfbzStr = sfbzMap.getOrDefault("sfbz", "  ");
                    sb.append(StrUtil.replaceChars(sfbzStr, ",", "，"));
                    sb.append("\n");

                    //设定依据
                /*    sb.append(matterEntity.getMatterName()).append("-设定依据,");
                    List<Map<String, Object>> sdyjMapList = MatterDetailHtmlParseUtil.getSdyjMapByHtml(dom);
                    StringBuilder sb2 = new StringBuilder();
                    for (Map<String, Object> bllcMap : sdyjMapList) {
                        for (Map.Entry<String, Object> entry : bllcMap.entrySet()) {
                            if(sb2.length()>360){
                                sb.append(sb2);
                                sb.append("\n");
                                sb2 = new StringBuilder();
                                sb.append(matterEntity.getMatterName()).append("-设定依据,");
                            }
                            sb2.append(entry.getKey()).append("  ").append(entry.getValue());
                        }
                    }
                    sb.append(sb2);*/
                    //中介服务
                    sb.append("\n");
                    sb.append(matterEntity.getMatterName()).append("-中介服务,");
                    sb.append("事项名称:").append(matterEntity.getMatterName());
                    Map<String, String> zjfwMap = MatterDetailHtmlParseUtil.getZjfwMapByHtml(dom);
                    for (Map.Entry<String, String> entry : zjfwMap.entrySet()) {
                        sb.append(entry.getKey()).append(":").append(StrUtil.replaceChars(entry.getValue(), ",", "，")).append(";");
                    }
                    //常见问题
                    sb.append("\n");
                    sb.append(matterEntity.getMatterName()).append("-常见问题,");
                    sb.append("事项名称:").append(matterEntity.getMatterName());
                    StringBuilder sb3 = new StringBuilder();
                    List<Map<String, Object>> cjwtMapList = MatterDetailHtmlParseUtil.getCjwtMapByHtml(dom);
                    for (Map<String, Object> bllcMap : cjwtMapList) {
                        for (Map.Entry<String, Object> entry : bllcMap.entrySet()) {
                            if (sb3.length() > 280) {
                                sb.append(sb3);
                                sb.append("\n");
                                sb3 = new StringBuilder();
                                sb.append(matterEntity.getMatterName()).append("-常见问题,");
                                sb.append("事项名称:").append(matterEntity.getMatterName());
                            }
                            sb3.append(entry.getKey()).append(":").append(StrUtil.replaceChars(entry.getValue().toString(), ",", "，")).append(";");

                        }
                    }
                    sb.append(sb3);
                    strList.add(sb.toString());
                    log.info("事项名称:{}", matterEntity.getMatterName());
                    break;
                } catch (Exception e) {
                    log.error("异常", e);
                    reCount++;
                    try {
                        Thread.sleep(2000);
                    } catch (InterruptedException ie) {
                    }
                }
            }
        }

        //todo 实时爬取后组合
        FileUtil.writeUtf8Lines(strList, path);
/*
        String path = "E:\\temp.csv";
        FileUtil.writeUtf8Lines(strList, path);
*/

        return Rest.ok();
    }

    @Override
    public Rest<Void> genMatterTXT(MatterQuery matterQuery, Context context) {

        BaseAreaQuery baseAreaQuery = new BaseAreaQuery();
        baseAreaQuery.setAreaCode(matterQuery.getAreaCode());
        BaseAreaEntity baseAreaEntity = baseAreaService.selectOne(baseAreaQuery);

        String path = matterQuery.getGenFilePath() + baseAreaEntity.getName() + ".txt";
        FileUtil.writeUtf8String("", path);
        List<MatterEntity> mattersList = matterExtService.getWebMatterListByAreaCode(matterQuery.getAreaCode(), context).getData();
        for (MatterEntity matterEntity : mattersList) {
            if (ObjectUtils.isEmpty(matterEntity.getUrl())) {
                continue;
            }
            StringBuilder sb = new StringBuilder();
            int reCount = 0;
            while (reCount < MAX_RETRY_COUNT) {
                try {
                    Document dom = Jsoup.connect(matterEntity.getUrl())
                            .ignoreContentType(true)
                            .ignoreHttpErrors(true)
                            .timeout(60 * 1000).get();

                    //new 事项之间用*** 分割  内容之间用  \n分割
                    sb.append("事项名称:").append(matterEntity.getMatterName());
                    //更新受理条件
                    sb.append("\n");
                    sb.append("受理条件:");
                    Map<String, String> sltjMap = MatterDetailHtmlParseUtil.getSltjMapByHtml(dom);
                    String sltjStr = sltjMap.getOrDefault("sltj", "  ");
                    sb.append(sltjStr);

                    sb.append("\n");
                    //材料属性
                    sb.append("申请材料:");
                    List<Map<String, Object>> sqclList = MatterDetailHtmlParseUtil.getsqclInfoMapByHtml(dom);
                    for (Map<String, Object> sqclMap : sqclList) {
                        Map<String, String> baseinfoMap = (HashMap<String, String>) sqclMap.get("baseinfo");
                        for (Map.Entry<String, String> entry : baseinfoMap.entrySet()) {
                            if (!"材料名称".equals(entry.getKey())) {
                                continue;
                            }
                            sb.append(StrUtil.replaceChars(entry.getValue(), ",", "，")).append(";");
                        }
                    }
                    sb.append("\n");
                    //承诺办结时限 办理地点 办理时间 咨询电话
                    HashSet<String> whiteBaseSet = new HashSet<>(Arrays.asList("承诺办结时限", "办理地点", "办理时间", "咨询方式"));
                    Map<String, String> baseInfoMap = MatterDetailHtmlParseUtil.getbaseInfoMapByHtml(dom);
                    for (Map.Entry<String, String> entry : baseInfoMap.entrySet()) {
                        if (whiteBaseSet.contains(entry.getKey())) {
                            sb.append(entry.getKey()).append(":").append(StrUtil.replaceChars(entry.getValue(), ",", "，")).append("\n");
                        }
                    }
                    //更新办理流程
                    sb.append("办理流程:");
                    List<Map<String, Object>> bllcMapList = MatterDetailHtmlParseUtil.getbllcMapByHtml(dom);

                    for (int i = 0; i < bllcMapList.size(); i++) {
                        Map<String, Object> bllcMap = bllcMapList.get(i);
                        String flow = (String) bllcMap.get("办理流程");
                        if (!ObjectUtils.isEmpty(flow)) {

                            if (i < bllcMapList.size() - 1) {
                                sb.append(StrUtil.replaceChars(flow, ",", "，")).append("==>");
                            } else {
                                sb.append(StrUtil.replaceChars(flow, ",", "，"));
                            }
                        }
                    }

                    sb.append("\n***\n");
                    FileUtil.appendUtf8String(sb.toString(), path);
                    //   strList.add(sb.toString());
                    //log.info("事项名称:{}", matterEntity.getMatterName());
                    break;
                } catch (Exception e) {
                    log.error("异常", e);
                    reCount++;
                    try {
                        Thread.sleep(2000);
                    } catch (InterruptedException ie) {
                    }
                }
            }

        }


        //  FileUtil.writeUtf8Lines(strList, path);
        return Rest.ok();
    }


    public Rest<String> doImportCustomData(List<MatterCustomImportEntity> list, Context context) {

        //内部先去重复
        list = list.stream().distinct().collect(Collectors.toList());
        int SucessCount = 0;
        int failCount = 0;
        int updateCount = 0;
        for (MatterCustomImportEntity matterCustomImportEntity : list) {
            if (ObjectUtils.isEmpty(matterCustomImportEntity.getAreaCode())) {
                log.info("事项名称:{},区域编码为空", matterCustomImportEntity.getMatterName());
                failCount++;
                continue;
            }

            //通过判断事项区域和名称，判断是否存在 存在则不添加
            MatterEntity matterEntity = this.selectOne(new MatterQuery().areaCode(matterCustomImportEntity.getAreaCode()).matterName(matterCustomImportEntity.getMatterName()));
            if (ObjectUtils.isEmpty(matterEntity)) {
                matterEntity = new MatterEntity();
                matterEntity.initAttrValue();
                BeanUtils.copyProperties(matterCustomImportEntity, matterEntity, BeanUtil.getNullPropertyNames(matterCustomImportEntity));
                matterEntity.setMatterNo(RandomUtil.randomString(12));
                matterEntity.setTid("/file/fileupload/flow.png");

                matterEntity.setCreateTime(new Date());
                matterEntity.setCreateUserId(1L);
                this.save(matterEntity);


                SiteEntity siteEntity = siteService.selectOne(new SiteQuery().areaCode(matterEntity.getAreaCode()));
                if (ObjectUtils.isEmpty(siteEntity)) {
                    //创建一个本级站点
                    siteEntity = new SiteEntity();
                    siteEntity.initAttrValue();
                    siteEntity.setAreaCode(matterEntity.getAreaCode());
                    siteEntity.setAreaName(matterEntity.getAreaName());
                    siteEntity.setSiteName(matterEntity.getAreaName());

                    siteEntity.setCreateTime(new Date());
                    siteEntity.setCreateUserId(1L);
                    siteService.save(siteEntity);
                }

                //保存区域站点部门
                DeptEntity deptEntity = deptService.selectOne(new DeptQuery().siteId(siteEntity.getId()).name(matterEntity.getDeptName()));
                if (ObjectUtils.isEmpty(deptEntity)) {
                    deptEntity = new DeptEntity();
                    deptEntity.initAttrValue();
                    deptEntity.setName(matterEntity.getDeptName());
                    deptEntity.setDeptNumber(RandomUtil.randomString(12));
                    deptEntity.setSiteId(siteEntity.getId());
                    deptEntity.setCreateTime(new Date());
                    deptEntity.setCreateUserId(1L);
                    deptService.save(deptEntity);

                }
                matterEntity.setDeptCode(deptEntity.getDeptNumber());
                this.update(matterEntity);
                //todo 固定新增办事流程
                ArrayList<MatterFlowlimitEntity> matterFlowlimitEntities = new ArrayList<>();

                MatterFlowlimitEntity matterFlowlimitEntity = new MatterFlowlimitEntity();
                matterFlowlimitEntity.setMatterId(matterEntity.getId());
                matterFlowlimitEntity.setMatterName(matterEntity.getMatterName());
                matterFlowlimitEntity.setMatterCode(matterEntity.getMatterNo());
                matterFlowlimitEntity.setFlowName("1. 申请准备。");
                matterFlowlimitEntity.setFlowDesc("（1）访问对应的网上办理渠道，完成用户注册登录；\n" +
                        "（2）找到企业登记一件事办理事项，了解办事指南，准备申请材料；将申请材料处理为电子版。");

                matterFlowlimitEntity.setCreateTime(new Date());
                matterFlowlimitEntity.setCreateUserId(1L);
                matterFlowlimitEntities.add(matterFlowlimitEntity);

                matterFlowlimitEntity = new MatterFlowlimitEntity();
                matterFlowlimitEntity.setMatterId(matterEntity.getId());
                matterFlowlimitEntity.setMatterName(matterEntity.getMatterName());
                matterFlowlimitEntity.setMatterCode(matterEntity.getMatterNo());
                matterFlowlimitEntity.setFlowName("2. 在线申请。");
                matterFlowlimitEntity.setFlowDesc("（1）自行在线申请，上传申请材料（清晰可见）。如需补正材料，按系统的补正流程进行。\n" +
                        "（2）向政务大厅申请帮办代办服务。");

                matterFlowlimitEntity.setCreateTime(new Date());
                matterFlowlimitEntity.setCreateUserId(1L);
                matterFlowlimitEntities.add(matterFlowlimitEntity);

                matterFlowlimitEntity = new MatterFlowlimitEntity();
                matterFlowlimitEntity.setMatterId(matterEntity.getId());
                matterFlowlimitEntity.setMatterName(matterEntity.getMatterName());
                matterFlowlimitEntity.setMatterCode(matterEntity.getMatterNo());
                matterFlowlimitEntity.setFlowName("3. 领取结果。");
                matterFlowlimitEntity.setFlowDesc("（1）电子化结果。选择对应申请的网上办理渠道领取。\n" +
                        "（2）纸质档结果。可选择免费快递邮寄或窗口领取。");

                matterFlowlimitEntity.setCreateTime(new Date());
                matterFlowlimitEntity.setCreateUserId(1L);
                matterFlowlimitEntities.add(matterFlowlimitEntity);

                matterFlowlimitService.save(matterFlowlimitEntities);

                //新增渠道
                MatterChannelEntity matterChannelEntity = new MatterChannelEntity();
                matterChannelEntity.initAttrValue();
                matterChannelEntity.setMatterId(matterEntity.getId());
                matterChannelEntity.setMatterCode(matterEntity.getMatterNo());
                matterChannelEntity.setMatterName(matterEntity.getMatterName());
                matterChannelEntity.setName(matterCustomImportEntity.getWebName());
                matterChannelEntity.setUrl(matterCustomImportEntity.getUrl());

                matterChannelEntity.setCreateTime(new Date());
                matterChannelEntity.setCreateUserId(1L);
                matterChannelService.save(matterChannelEntity);


                SucessCount++;
            } else {
                log.info("事项已存在,更新事项：" + matterEntity.getMatterName());
                BeanUtils.copyProperties(matterCustomImportEntity, matterEntity, new String[]{"id", "areaCode", "matterName"});

                matterEntity.setUpdateTime(new Date());
                matterEntity.setUpdateUserId(1L);
                this.dao.update(matterEntity);

                //todo 部门业务逻辑一致
                SiteEntity siteEntity = siteService.selectOne(new SiteQuery().areaCode(matterEntity.getAreaCode()));
                if (ObjectUtils.isEmpty(siteEntity)) {
                    //创建一个本级站点
                    siteEntity = new SiteEntity();
                    siteEntity.initAttrValue();
                    siteEntity.setAreaCode(matterEntity.getAreaCode());
                    siteEntity.setAreaName(matterEntity.getAreaName());
                    siteEntity.setSiteName(matterEntity.getAreaName());

                    siteEntity.setCreateTime(new Date());
                    siteEntity.setCreateUserId(1L);
                    siteService.save(siteEntity);
                }

                //保存区域站点部门
                DeptEntity deptEntity = deptService.selectOne(new DeptQuery().siteId(siteEntity.getId()).name(matterEntity.getDeptName()));
                if (ObjectUtils.isEmpty(deptEntity)) {
                    deptEntity = new DeptEntity();
                    deptEntity.initAttrValue();
                    deptEntity.setName(matterEntity.getDeptName());
                    deptEntity.setDeptNumber(RandomUtil.randomString(12));
                    deptEntity.setSiteId(siteEntity.getId());
                    deptEntity.setCreateTime(new Date());
                    deptEntity.setCreateUserId(1L);
                    deptService.save(deptEntity);

                }
                matterEntity.setDeptCode(deptEntity.getDeptNumber());
                this.update(matterEntity);





                MatterChannelQuery channelQuery = new MatterChannelQuery();
                channelQuery.setMatterId(matterEntity.getId());
                channelQuery.setAppType(0);
                List<MatterChannelEntity> matterChannelEntities = matterChannelService.find(channelQuery);
                if (!ObjectUtils.isEmpty(matterChannelEntities)) {
                    MatterChannelEntity matterChannelEntity = matterChannelEntities.get(0);
                    matterChannelEntity.setName(matterCustomImportEntity.getWebName());
                    matterChannelEntity.setUrl(matterCustomImportEntity.getUrl());
                    matterChannelEntity.setUpdateTime(new Date());
                    matterChannelEntity.setUpdateUserId(1L);
                    matterChannelService.update(matterChannelEntity);
                } else {
                    //新增渠道
                    MatterChannelEntity matterChannelEntity = new MatterChannelEntity();
                    matterChannelEntity.initAttrValue();
                    matterChannelEntity.setMatterId(matterEntity.getId());
                    matterChannelEntity.setMatterCode(matterEntity.getMatterNo());
                    matterChannelEntity.setMatterName(matterEntity.getMatterName());
                    matterChannelEntity.setName(matterCustomImportEntity.getWebName());
                    matterChannelEntity.setUrl(matterCustomImportEntity.getUrl());

                    matterChannelEntity.setCreateTime(new Date());
                    matterChannelEntity.setCreateUserId(1L);
                    matterChannelService.save(matterChannelEntity);
                }

                updateCount++;
                // failCount++;
            }
        }


        return Rest.ok("成功：" + SucessCount + " 更新：" + updateCount + " 失败：" + failCount);
    }

    /**
     * @param query
     * @return
     */
    @Override
    public Rest<byte[]> genQrCode(MatterQuery query) throws IOException {

        List<MatterEntity> matterEntities = new ArrayList<>();
        MatterQuery matterQuery = new MatterQuery();
        if (!ObjectUtils.isEmpty(query.getIdList())) {
            //针对指定ids的 生成二维码 并压缩包
            matterQuery.setIdList(query.getIdList());
        }
        if (!ObjectUtils.isEmpty(query.getAreaCode())) {
            //针对指定ids的 生成二维码 并压缩包
            matterQuery.setAreaCode(query.getAreaCode());
        }

        matterEntities = this.find(matterQuery);
        ByteArrayOutputStream byteArrayOutputStream = new ByteArrayOutputStream();
        try (ZipOutputStream zipOutputStream = new ZipOutputStream(byteArrayOutputStream)) {
            for (MatterEntity matterEntity : matterEntities) {
                String qrCode = QR_CODE + "areaCode=" + matterEntity.getAreaCode() + "&matterId=" + matterEntity.getId();
                String qrFileName = matterEntity.getAreaName() + "." + matterEntity.getMatterName()+".png";
                // 生成二维码
                BufferedImage qrCodeImage = QrCodeUtil.generate(qrCode, 300, 300);

                // 将二维码写入 ByteArrayOutputStream
                ByteArrayOutputStream imgOutputStream = new ByteArrayOutputStream();
                ImageIO.write(qrCodeImage, "png", imgOutputStream);
                // 将图片流转换为字节数组
                byte[] qrBytes = imgOutputStream.toByteArray();

                ZipEntry zipEntry = new ZipEntry(qrFileName);
                zipOutputStream.putNextEntry(zipEntry);
                zipOutputStream.write(qrBytes);
                zipOutputStream.closeEntry();
            }
        }catch (Exception e){
            return Rest.fail();
        }
        byte[] byteArray = byteArrayOutputStream.toByteArray();
     //   FileUtil.writeBytes(byteArray, "E:\\1.zip");
        return Rest.ok(byteArray);
    }


    public static void main(String[] args) throws IOException {


/*        String sss = "实施主体性质:法定机关  是否进驻政务大厅:是  村居社区名称:无  权力来源:法定本级行使  村居社区代码:无  法定时限办结说明:自受理之日起，在法定期限内办结  行使层级:县级  到办事现场次数:0  网办类型:全程网办  实施主体:邛崃市司法局  是否网办:是  监督投诉方式:投诉电话：028-12345,028-88794281;  事项类型:公共服务  行使方式:依申请行使  承诺时限办结说明:自受理之日起，在承诺期限内办结  是否支持全省就近取件:否  是否支持预约办理:否  事项版本:16  联办机构:无  办件类型:即办件  通办范围:全省  是否支持上门收取申请材料:无  是否支持自助终端办理:否";

        String s = StrUtil.replaceChars(sss, ",", "，");
        System.out.println(s);*/

/*
        ArrayList<MatterEntity> matterEntities = new ArrayList<>();

        MatterEntity matterEntity = new MatterEntity();
        matterEntity.initAttrValue();
        matterEntities.add(matterEntity);

        String path = "E:\\1.txt";

        FileUtil.writeUtf8Lines(matterEntities, path);

*/


     /*   StringBuilder sb = new StringBuilder();

        sb.append("问:小区经业主大会续聘（或选聘）物业服务企业的，物业服务合同备案对召开会议时间是否有要求？  答:有。根据《四川省业主大会和业主委员会指导规则》相关要求，业主委员会应当自提议确定之日起，将业主大会会议拟表决事项在物业管理区域内显著位置公示并告知相关的居民（村民）委员会，公示期不少于15日。\n" +
                "住宅专项维修资金交存-基本信息,是否支持全省就近办理  否是否支持物流快递  否业务办理项编码  11510821008501563R451201700100301认证等级需求  实名认证名称  操作承诺办结时限  1个工作日基本编码  512017001003计划生效日期  无法定办结时限  1个工作日实施主体编码  11510821008501563R网办地址  点击访问服务对象  自然人咨询方式  咨询电话：0825-5428572;事项状态  在用办理时间  星期一至星期五 上午：08:30-12:00 下午：14:00-17:30 备注：法定节假日除外;实施主体性质  受委托组织是否进驻政务大厅  是村居社区名称  无权力来源  上级下放村居社区代码  无法定时限办结说明  自受理之日起，在法定期限内办结行使层级  县级到办事现场次数  0住宅专项维修资金交存-基本信息,网办类型  全程网办实施主体  蓬溪县住房和城乡建设局是否网办  是监督投诉方式  投诉电话：0825-12345;事项类型  公共服务行使方式  依申请行使承诺时限办结说明  自受理之日起，在承诺期限内办结是否支持全省就近取件  否是否支持预约办理  否事项版本  19联办机构  无办件类型  即办件通办范围  无是否支持上门收取申请材料  否是否支持自助终端办理  否数量限制  无办理形式  窗口办理、网上办理乡镇街道名称  无是否支持网上支付  否委托部门  蓬溪县住房保障和房屋管理局乡镇街道代码  无是否收费  否网上办理深度  互联网咨询、互联网收件、互联网预审、互联网受理、互联网办理、互联网办理结果信息反馈实施编码  11510821008501563R4512017001003特别程序  必须到现场原因  无住宅专项维修资金交存-基本信息,办理地点  四川省-遂宁市-蓬溪县-蜀北中路街道-108号,详细地址：蓬溪县住房保障和房屋管理局三楼办公室;计划取消日期  无\n" +
                "住宅专项维修资金交存-申请材料,材料名称  公有住房买卖合同材料名称  身份证明材料材料名称  商品房买卖合同\n" +
                "住宅专项维修资金交存-受理条件,四川省行政区域内商品住宅、售后公有住房住宅专项维修资金（以下称维修资金）的交存、使用、管理和监督，拟申请住宅专项维修资金交存。\n" +
                "住宅专项维修资金交存-办理流程,转外时限  0个工作日sort  1办理流程  受理办理流程说明  1.申请：申请人到政务服务中心综合 窗口提交申请材料；2.受理：窗口工作人员收到申请材料当场或者规定工作日内作出受理或不予受理决定。材料不齐全或不符合法定形式的，办理机关应在规定工作日内一次性告知申请人需要补正的全部内容，对不属于受理范围的，出具不予受理通知书；3.审查：办理人员对申请人提交的材料进行审查，提出初步审查意见；4.决定：办理机关负责人依据审查意见签署审批结果。对予批准办理的，由办理机关向申请人核发办理结果，不予批准的，由办理机关书面通知申请人并说明理由，并告知申请人对结果有异议的，可依法申请行政复议或者提起行政诉讼；5.制证：制证部门完成制证；6.颁发和送达：申请人凭个人身份证明（有效的身份证、临时居住证、户口簿等）和受理通知书到申办窗口领取办理结果。办理时长  0个工作日转外时限  0个工作日sort  2办理流程  办结办理时长  0个工作日\n" +
                "住宅专项维修资金交存-收费标准,该事项无收费标准！\n" +
                "住宅专项维修资金交存-设定依据,法规类型  部门规章条例  第六条条款内容  下列物业的业主应当按照本办法的规定交存住宅专项维修资金： （一）住宅，但一个业主所有且与其他物业不具有共用部位、共用设施设备的除外； （二）住宅小区内的非住宅或者住宅小区外与单幢住宅结构相连的非住宅。 前款所列物业属于出售公有住房的，售房单位应当按照本办法的规定交存住宅专项维修资金。法律法规名称  住宅专项维修资金管理办法sort  1法规类型  地方法规条例  第六十九条条款内容  住宅物业和住宅区内的非住宅物业出售时，物业出售人和买受人应当按照国家和省的规定交存专项维修资金。法律法规名称  四川省物业管理条例sort  2");
        System.out.println(sb.length());*/


     /*   String qrCode = QR_CODE ;
        String qrFileName = "1.zip";
        // 生成二维码
        BufferedImage qrCodeImage = QrCodeUtil.generate(qrCode, 300, 300);


        ByteArrayOutputStream zipOutputStream = new ByteArrayOutputStream();
        ZipOutputStream zipOut = new ZipOutputStream(zipOutputStream);
        // 将二维码写入 ByteArrayOutputStream

        ByteArrayOutputStream imgOutputStream = new ByteArrayOutputStream();
        ImageIO.write(qrCodeImage, "png", imgOutputStream);
        // 将图片流转换为字节数组
        byte[] qrBytes = imgOutputStream.toByteArray();
        ZipEntry zipEntry = new ZipEntry(qrFileName + ".png");
        zipOut.putNextEntry(zipEntry);

      //  IOUtils.write(qrBytes, zipOut);
          zipOut.write(qrBytes);
        zipOut.flush();
        zipOut.closeEntry();

        System.out.println(zipOutputStream.toByteArray().length);

        FileUtil.writeBytes(zipOutputStream.toByteArray(),"E:\\1.zip");
        FileUtil.writeBytes(qrBytes,"E:\\1.png");*/

        String qrCode = QR_CODE;
        BufferedImage qrCodeImage = QrCodeUtil.generate(qrCode, 300, 300);
        ByteArrayOutputStream imgOutputStream = new ByteArrayOutputStream();
        ImageIO.write(qrCodeImage, "png", imgOutputStream);

        //byte[] data = "Hello ZIP".getBytes();
        byte[] data = imgOutputStream.toByteArray();
        ByteArrayOutputStream byteArrayOutputStream = new ByteArrayOutputStream();
        try (ZipOutputStream zipOutputStream = new ZipOutputStream(byteArrayOutputStream)) {
            ZipEntry zipEntry = new ZipEntry("1.png");
            zipOutputStream.putNextEntry(zipEntry);
            zipOutputStream.write(data);
            zipOutputStream.closeEntry();
        }
        FileUtil.writeBytes(byteArrayOutputStream.toByteArray(), "E:\\1.zip");


    }


}