package com.mortals.xhx.module.matter.web;

import cn.hutool.core.codec.Base64;
import cn.hutool.extra.qrcode.QrCodeUtil;
import cn.hutool.setting.Setting;
import com.alibaba.fastjson.JSONObject;
import com.mortals.framework.annotation.UnAuth;
import com.mortals.framework.common.Rest;
import com.mortals.framework.exception.AppException;
import com.mortals.framework.model.Context;
import com.mortals.framework.model.PageInfo;
import com.mortals.framework.model.Result;
import com.mortals.framework.service.ICacheService;
import com.mortals.framework.util.DataUtil;
import com.mortals.framework.utils.ReflectUtils;
import com.mortals.framework.utils.poi.ExcelUtil;
import com.mortals.framework.web.BaseCRUDJsonBodyMappingController;
import com.mortals.xhx.base.framework.config.InterceptorConfig;
import com.mortals.xhx.base.system.param.service.ParamService;
import com.mortals.xhx.common.code.JointInfoShowEnum;
import com.mortals.xhx.common.code.YesNoEnum;
import com.mortals.xhx.module.base.model.BaseAreaEntity;
import com.mortals.xhx.module.base.model.BaseAreaQuery;
import com.mortals.xhx.module.base.service.BaseAreaService;
import com.mortals.xhx.module.matter.model.*;
import com.mortals.xhx.module.matter.model.vo.MatterCustomImportEntity;
import com.mortals.xhx.module.matter.service.*;
import com.mortals.xhx.module.site.model.SiteThemeQuery;
import com.mortals.xhx.module.site.service.SiteThemeService;
import org.apache.commons.lang3.StringUtils;
import org.apache.poi.ss.formula.functions.T;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.util.ObjectUtils;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

import javax.imageio.ImageIO;
import java.awt.image.BufferedImage;
import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

import static com.mortals.framework.ap.SysConstains.MESSAGE_INFO;
import static com.mortals.framework.ap.SysConstains.PAGEINFO_KEY;
import static com.mortals.xhx.common.key.Constant.QR_CODE;

/**
 * 基础事项
 *
 * @author zxfei
 * @date 2022-01-20
 */
@RestController
@RequestMapping("matter")
public class MatterController extends BaseCRUDJsonBodyMappingController<MatterService, MatterEntity, Long> {

    @Autowired
    private ParamService paramService;
    @Autowired
    private InterceptorConfig interceptorConfig;
    @Autowired
    private SiteThemeService siteThemeService;
    @Autowired
    private MatterService matterService;
    @Autowired
    private BaseAreaService baseAreaService;

    @Autowired
    private MatterFlowlimitService matterFlowlimitService;
    @Autowired
    private MatterAcceptService matterAcceptService;
    @Autowired
    private MatterSetbaseService matterSetbaseService;
    @Autowired
    private MatterQuestionService matterQuestionService;
    @Autowired
    private MatterIntermediaryService matterIntermediaryService;
    @Autowired
    private MatterDatumService matterDatumService;
    @Autowired
    private MatterDatumFileService matterDatumFileService;
    @Autowired
    private MatterChargesService matterChargesService;
    @Autowired
    private MatterChannelService matterChannelService;
    @Autowired
    private ICacheService cacheService;

    public MatterController() {
        super.setModuleDesc("基础事项");
    }

    /**
     * @param query
     * @param model
     * @param context
     * @throws AppException
     */
    @Override
    protected void doListBefore(MatterEntity query, Map<String, Object> model, Context context) throws AppException {
        if (ObjectUtils.isEmpty(query.getEventTypeShowNotList())) {
            ArrayList<String> notList = new ArrayList<>();
            notList.add("行政处罚");
            query.setEventTypeShowNotList(notList);
        }

        super.doListBefore(query, model, context);
    }

    @Override
    protected void init(Map<String, Object> model, Context context) {
        this.addDict(model, "appoveObjectShow", paramService.getParamBySecondOrganize("Matter", "appoveObjectShow"));
        this.addDict(model, "operatScopeShow", paramService.getParamBySecondOrganize("Matter", "operatScopeShow"));
        this.addDict(model, "appoveTimeLimitShow", paramService.getParamBySecondOrganize("Matter", "appoveTimeLimitShow"));
        this.addDict(model, "handleType", paramService.getParamBySecondOrganize("Matter", "handleType"));
        this.addDict(model, "isChargesShow", paramService.getParamBySecondOrganize("Matter", "isChargesShow"));
        this.addDict(model, "certificationLevelsShow", paramService.getParamBySecondOrganize("Matter", "certificationLevelsShow"));
        this.addDict(model, "windowToTheSceneNum", paramService.getParamBySecondOrganize("Matter", "windowToTheSceneNum"));
        this.addDict(model, "isOnlineSubscribeShow", paramService.getParamBySecondOrganize("Matter", "isOnlineSubscribeShow"));
        this.addDict(model, "isExpressTakeShow", paramService.getParamBySecondOrganize("Matter", "isExpressTakeShow"));
        this.addDict(model, "isProvinceAcquisitionShow", paramService.getParamBySecondOrganize("Matter", "isProvinceAcquisitionShow"));
        this.addDict(model, "isApplyProvinceShow", paramService.getParamBySecondOrganize("Matter", "isApplyProvinceShow"));
        this.addDict(model, "onlineType", paramService.getParamBySecondOrganize("Matter", "onlineType"));
        this.addDict(model, "onlineToTheSceneNum", paramService.getParamBySecondOrganize("Matter", "onlineToTheSceneNum"));
        this.addDict(model, "onlineOperatDeep", paramService.getParamBySecondOrganize("Matter", "onlineOperatDeep"));
        this.addDict(model, "isExpressTakeOnlineShow", paramService.getParamBySecondOrganize("Matter", "isExpressTakeOnlineShow"));
        this.addDict(model, "isDoorTakeShow", paramService.getParamBySecondOrganize("Matter", "isDoorTakeShow"));
        this.addDict(model, "eventTypeShow", paramService.getParamBySecondOrganize("Matter", "eventTypeShow"));
        this.addDict(model, "performHierarchyShow", paramService.getParamBySecondOrganize("Matter", "performHierarchyShow"));
        this.addDict(model, "powerSourceShow", paramService.getParamBySecondOrganize("Matter", "powerSourceShow"));
        this.addDict(model, "performDeptTypeShow", paramService.getParamBySecondOrganize("Matter", "performDeptTypeShow"));
        this.addDict(model, "goveServiceCenterShow", paramService.getParamBySecondOrganize("Matter", "goveServiceCenterShow"));
        this.addDict(model, "isConvenientCenterShow", paramService.getParamBySecondOrganize("Matter", "isConvenientCenterShow"));
        this.addDict(model, "terminalHandle", paramService.getParamBySecondOrganize("Matter", "terminalHandle"));
        this.addDict(model, "isOnline", paramService.getParamBySecondOrganize("Matter", "isOnline"));
        this.addDict(model, "isOnlinePayShow", paramService.getParamBySecondOrganize("Matter", "isOnlinePayShow"));
        this.addDict(model, "entrustmentDepartmen", paramService.getParamBySecondOrganize("Matter", "entrustmentDepartmen"));
        this.addDict(model, "matterStatus", paramService.getParamBySecondOrganize("Matter", "matterStatus"));
        this.addDict(model, "type", paramService.getParamBySecondOrganize("Matter", "type"));
        this.addDict(model, "mattertype", paramService.getParamBySecondOrganize("Matter", "mattertype"));
        this.addDict(model, "eventType", paramService.getParamBySecondOrganize("Matter", "eventType"));
        this.addDict(model, "parson", paramService.getParamBySecondOrganize("Matter", "parson"));
        this.addDict(model, "lengal", paramService.getParamBySecondOrganize("Matter", "lengal"));
        this.addDict(model, "source", paramService.getParamBySecondOrganize("Matter", "source"));
        this.addDict(model, "jointInfoShow", JointInfoShowEnum.getEnumMap());

        Setting baseInfoSetting = interceptorConfig.getBaseInfoSetting();
        Map<String, String> baseInfoMap = new HashMap<>();
        baseInfoSetting.entrySet().stream().forEach(item -> {
            if (!ObjectUtils.isEmpty(item.getValue())) {
                baseInfoMap.put(item.getValue(), item.getKey());
            }
        });
        this.addDict(model, "baseInfo", baseInfoMap);

        Setting sqclInfoSetting = interceptorConfig.getSqclInfoSetting();
        Map<String, String> sqclInfoMap = new HashMap<>();
        sqclInfoSetting.entrySet().stream().forEach(item -> {
            if (!ObjectUtils.isEmpty(item.getValue())) {
                sqclInfoMap.put(item.getValue(), item.getKey());
            }
        });
        this.addDict(model, "sqclInfo", sqclInfoMap);
        //theme
        Map<String, String> themeMap = siteThemeService.find(new SiteThemeQuery()).stream().collect(Collectors.toMap(x -> x.getThemeCode(), y -> y.getThemeName(), (o, n) -> n));
        this.addDict(model, "theme", themeMap);
        super.init(model, context);
    }


    /**
     * 添加事项到站点
     */
    @PostMapping(value = "addMatterToSite")
    public String addBusinessToSite(@RequestBody Map<String, Object> map) {
        JSONObject jsonObject = new JSONObject();
        Map<String, Object> model = new HashMap<String, Object>();
        String matterIds = (String) map.get("matterIds");
        Long siteId = DataUtil.converStr2Long(map.get("siteId").toString(), 0L);

        try {
            Rest<String> rest = this.service.addMatterToSite(matterIds, siteId, getContext());
            jsonObject.put(KEY_RESULT_MSG, rest.getMsg());
            jsonObject.put(KEY_RESULT_DATA, model);
            jsonObject.put(KEY_RESULT_CODE, VALUE_RESULT_SUCCESS);
            if (!ObjectUtils.isEmpty(getContext()) && !ObjectUtils.isEmpty(getContext().getUser())) {
                this.recordSysLog(this.request, rest.getMsg());
            }
        } catch (Exception e) {
            log.error("获取异常", e);
            jsonObject.put(KEY_RESULT_CODE, VALUE_RESULT_FAILURE);
            jsonObject.put(KEY_RESULT_MSG, super.convertException(e));
        }
        return jsonObject.toJSONString();
    }


    /**
     * @param id
     * @param model
     * @param entity
     * @param context
     * @return
     * @throws AppException
     */
    @Override
    protected int infoAfter(Long id, Map<String, Object> model, MatterEntity entity, Context context) throws AppException {

        if (!ObjectUtils.isEmpty(entity)) {
            List<MatterAcceptEntity> matterAcceptEntities = matterAcceptService.find(new MatterAcceptQuery().matterId(entity.getId()));
            entity.setMatterAcceptList(matterAcceptEntities);

            List<MatterDatumEntity> matterDatumEntities = matterDatumService.find(new MatterDatumQuery().matterId(entity.getId()));
            entity.setMatterDatumList(matterDatumEntities);

            List<MatterChannelEntity> matterChannelEntities = matterChannelService.find(new MatterChannelQuery().matterId(entity.getId()));
            entity.setMatterChannelList(matterChannelEntities);

            List<MatterChargesEntity> matterChargesEntities = matterChargesService.find(new MatterChargesQuery().matterId(entity.getId()));
            entity.setMatterChargesList(matterChargesEntities);

            List<MatterFlowlimitEntity> matterFlowlimitEntities = matterFlowlimitService.find(new MatterFlowlimitQuery().matterId(entity.getId()));
            entity.setMatterFlowlimitList(matterFlowlimitEntities);

            List<MatterIntermediaryEntity> matterIntermediaryEntities = matterIntermediaryService.find(new MatterIntermediaryQuery().matterId(entity.getId()));
            entity.setMatterIntermediaryList(matterIntermediaryEntities);

            List<MatterQuestionEntity> matterQuestionEntities = matterQuestionService.find(new MatterQuestionQuery().matterId(entity.getId()));
            entity.setMatterQuestionList(matterQuestionEntities);

            List<MatterSetbaseEntity> matterSetbaseEntities = matterSetbaseService.find(new MatterSetbaseQuery().matterId(entity.getId()));
            entity.setMatterSetbaseList(matterSetbaseEntities);
        }

        //生成二维码图片
        String qrCode = QR_CODE + "areaCode=" + entity.getAreaCode() + "&matterId=" + entity.getId();
        // 生成二维码
        BufferedImage qrCodeImage = QrCodeUtil.generate(qrCode, 300, 300);

        // 将二维码写入 ByteArrayOutputStream
        ByteArrayOutputStream imgOutputStream = new ByteArrayOutputStream();
        try {
            ImageIO.write(qrCodeImage, "png", imgOutputStream);
            String qrCodeBase64Str = Base64.encode(imgOutputStream.toByteArray());
            entity.setQrCode(qrCodeBase64Str);

        } catch (IOException e) {
            throw new AppException(e.getMessage());
        }

        return super.infoAfter(id, model, entity, context);
    }

    @PostMapping(value = "sublist")
    @UnAuth
    public Rest<Object> sublist(@RequestBody MatterEntity query) {
        Rest<Object> ret = new Rest<>();
        Map<String, Object> model = new HashMap<>();
        Context context = this.getContext();
        String busiDesc = "查询差集" + this.getModuleDesc();
        int code = VALUE_RESULT_SUCCESS;
        try {
            PageInfo pageInfo = buildPageInfo(query);
            Result<MatterEntity> result = this.getService().findSubList(query, pageInfo, context);
            model.put(KEY_RESULT_DATA, result.getList());
            model.put(PAGEINFO_KEY, result.getPageInfo());
            parsePageInfo(model, result.getPageInfo());
            model.put(MESSAGE_INFO, busiDesc + "成功");
            if (!ObjectUtils.isEmpty(getContext()) && !ObjectUtils.isEmpty(getContext().getUser())) {
                recordSysLog(request, busiDesc + " 【成功】");
            }
        } catch (Exception e) {
            code = VALUE_RESULT_FAILURE;
            this.doException(request, busiDesc, model, e);
        }
        this.init(model, context);
        ret.setCode(code);
        ret.setData(model);
        ret.setMsg(model.get(MESSAGE_INFO) == null ? "" : model.remove(MESSAGE_INFO).toString());
        return ret;
    }





    @PostMapping(value = "genMatterMarkdown")
    @UnAuth
    public Rest<Object> genMatterMarkdown(@RequestBody MatterQuery query) {
        Rest<Object> ret = new Rest<>();
        Map<String, Object> model = new HashMap<>();
        Context context = this.getContext();
        String busiDesc = "生成文本" + this.getModuleDesc();
        int code = VALUE_RESULT_SUCCESS;
        try {

            if (!ObjectUtils.isEmpty(query.getIsSub()) && YesNoEnum.YES.getValue() == query.getIsSub()) {
                BaseAreaEntity baseAreaEntity = new BaseAreaEntity();
                baseAreaEntity.setAreaCode(query.getAreaCode());
                List<String> subAreaCodeList = baseAreaService.getSubAreaByRootName(baseAreaEntity, context).getData();

                for (String subAreaCode : subAreaCodeList) {
                    MatterQuery matterEntity = new MatterQuery();
                    matterEntity.setAreaCode(subAreaCode);
                    matterEntity.setGenFilePath(query.getGenFilePath());
                    this.service.genMatterMarkdown(matterEntity, getContext());
                }
            } else {
                this.service.genMatterMarkdown(query, getContext());
            }
            model.put(MESSAGE_INFO, busiDesc + "成功");

        } catch (Exception e) {
            code = VALUE_RESULT_FAILURE;
            this.doException(request, busiDesc, model, e);
        }
        ret.setCode(code);
        ret.setData(model);
        ret.setMsg(model.get(MESSAGE_INFO) == null ? "" : model.remove(MESSAGE_INFO).toString());
        return ret;
    }


    @GetMapping(value = "genMatterCsv")
    @UnAuth
    public String genMatterCsv(@RequestParam(value = "areaCode", defaultValue = "") String areaCode
            , @RequestParam(value = "genFilePath", defaultValue = "/home/txt/") String genFilePath) {
        Rest<Object> ret = new Rest<>();
        Map<String, Object> model = new HashMap<>();
        Context context = this.getContext();
        String busiDesc = "生成文本" + this.getModuleDesc();
        int code = VALUE_RESULT_SUCCESS;
        try {
            if (ObjectUtils.isEmpty(areaCode)) {
                throw new AppException("区域编码不能为空！");
            }
            MatterQuery query = new MatterQuery();
            query.setAreaCode(areaCode);
            query.setGenFilePath(genFilePath);
            BaseAreaQuery baseAreaQuery = new BaseAreaQuery();
            baseAreaQuery.setAreaCode(areaCode);
            BaseAreaEntity baseAreaEntity = baseAreaService.selectOne(baseAreaQuery);

            if (ObjectUtils.isEmpty(baseAreaEntity)) {
                throw new AppException("区域编码不存在！" + areaCode);
            }
            boolean setnx = cacheService.setnx("genMatterCsv:" + areaCode, "1", 60 * 20);
            if (!setnx) {
                return baseAreaEntity.getName() + ".csv 正在生成中，请稍后查看文件！";
            }
            new Thread(() -> {
                this.service.genMatterCsv(query, getContext());
                cacheService.del("genMatterCsv:" + areaCode);
            }).start();
            return "异步生成文件中，请稍后查看文件！，文件访问地址: http://192.168.0.98:11071" + genFilePath;


/*
            this.service.genMatterCsv(query, getContext());
            model.put(MESSAGE_INFO, busiDesc + "成功");*/

        } catch (Exception e) {
            code = VALUE_RESULT_FAILURE;
            this.doException(request, busiDesc, model, e);
        }
        ret.setCode(code);
        ret.setData(model);
        ret.setMsg(model.get(MESSAGE_INFO) == null ? "" : model.remove(MESSAGE_INFO).toString());
        return ret.toString();
    }

    @GetMapping(value = "genMatterTxt")
    @UnAuth
    public String genMatterTxt(@RequestParam(value = "areaCode", defaultValue = "") String areaCode
            , @RequestParam(value = "genFilePath", defaultValue = "/home/txt/") String genFilePath) {
        Rest<Object> ret = new Rest<>();
        Map<String, Object> model = new HashMap<>();
        String busiDesc = "生成文本" + this.getModuleDesc();
        int code = VALUE_RESULT_SUCCESS;
        try {
            if (ObjectUtils.isEmpty(areaCode)) {
                throw new AppException("区域编码不能为空！");
            }
            MatterQuery query = new MatterQuery();
            query.setAreaCode(areaCode);
            query.setGenFilePath(genFilePath);
            BaseAreaQuery baseAreaQuery = new BaseAreaQuery();
            baseAreaQuery.setAreaCode(areaCode);
            BaseAreaEntity baseAreaEntity = baseAreaService.selectOne(baseAreaQuery);

            if (ObjectUtils.isEmpty(baseAreaEntity)) {
                throw new AppException("区域编码不存在！" + areaCode);
            }
            boolean setnx = cacheService.setnx("genMatterTxt:" + areaCode, "1", 60 * 20);
            if (!setnx) {
                return baseAreaEntity.getName() + ".txt 正在生成中，请稍后查看文件！";
            }
            new Thread(() -> {
                this.service.genMatterTXT(query, getContext());
                cacheService.del("genMatterTxt:" + areaCode);
            }).start();
            return "异步生成文件中，请稍后查看文件！，文件访问地址: http://192.168.0.98:11071" + genFilePath;

        } catch (Exception e) {
            code = VALUE_RESULT_FAILURE;
            this.doException(request, busiDesc, model, e);
        }
        ret.setCode(code);
        ret.setData(model);
        ret.setMsg(model.get(MESSAGE_INFO) == null ? "" : model.remove(MESSAGE_INFO).toString());
        return ret.toString();
    }


    @PostMapping("/importDataCustom")
    @UnAuth
    public String importDataCustom(MultipartFile file, boolean updateSupport) {
        JSONObject ret = new JSONObject();
        Map<String, Object> model = new HashMap<>();
        Map<String, Object> excelConfData = new HashMap<>();
        excelConfData.put("headereRow", 0);
        excelConfData.put("dataRow", 1);
        excelConfData.put("sheetName", StringUtils.EMPTY);
        Context context = getContext();
        String busiDesc = "导入" + getModuleDesc();
        int code = VALUE_RESULT_SUCCESS;
        try {
            ExcelUtil<MatterCustomImportEntity, Long> util = new ExcelUtil<>(MatterCustomImportEntity.class);
            List<MatterCustomImportEntity> list = util.importExcel((String) excelConfData.get("sheetName"), file.getInputStream(), (Integer) excelConfData.get("headereRow"), (Integer) excelConfData.get("dataRow"));
            Rest<String> rest = this.service.doImportCustomData(list, context);
            //  doImportDataBefore(list, updateSupport, context);
            // String message = this.service.importList(list, updateSupport, getContext());
            // doImportExcelAfter(file, list, getContext());
            model.put(MESSAGE_INFO, rest.getMsg());
            recordSysLog(request, busiDesc + " 【成功】");
        } catch (Exception e) {
            code = VALUE_RESULT_FAILURE;
            doException(request, busiDesc, model, e);
        }
        ret.put(KEY_RESULT_CODE, code);
        ret.put(KEY_RESULT_MSG, model.remove(MESSAGE_INFO));
        return ret.toJSONString();
    }


    /**
     * 生成二维码
     *
     * @author: zxfei
     * @date: 2025/3/29 9:50
     */
    @PostMapping(value = "genQrCode")
    @UnAuth
    public Rest<Object> genQrCode(@RequestBody MatterQuery query) {
        Rest<Object> ret = new Rest<>();
        Map<String, Object> model = new HashMap<>();
        Context context = this.getContext();
        String busiDesc = "生成二维码" + this.getModuleDesc();
        int code = VALUE_RESULT_SUCCESS;
        String fileName = "_qrcodes.zip";
        try {
            if (ObjectUtils.isEmpty(query.getAreaCode()) && ObjectUtils.isEmpty(query.getIdList())) {
                throw new AppException("区域编码不能为空！");
            }
            if (!ObjectUtils.isEmpty(query.getAreaCode())) {
                BaseAreaEntity baseAreaEntity = baseAreaService.selectOne(new BaseAreaQuery().areaCode(query.getAreaCode()));
                if (ObjectUtils.isEmpty(baseAreaEntity)) {
                    throw new AppException("区域编码不存在！" + query.getAreaCode());
                }
                fileName = baseAreaEntity.getName() + fileName;
            }

            Rest<byte[]> rest = this.service.genQrCode(query);

            this.responseStream(response, rest.getData(), fileName);
        } catch (Exception e) {
            code = VALUE_RESULT_FAILURE;
            this.doException(request, busiDesc, model, e);
        }
        ret.setCode(code);
        ret.setData(model);
        ret.setMsg(model.get(MESSAGE_INFO) == null ? "" : model.remove(MESSAGE_INFO).toString());
        return ret;

    }

}