package com.mortals.xhx.module.ph.service.impl;
import com.mortals.framework.model.PageInfo;
import com.mortals.xhx.module.app.model.AppInfoFieldEntity;
import com.mortals.xhx.module.app.model.AppInfoFieldQuery;
import org.springframework.beans.BeanUtils;
import org.springframework.stereotype.Service;
import com.mortals.framework.service.impl.AbstractCRUDServiceImpl;
import com.mortals.framework.exception.AppException;
import com.mortals.framework.model.Context;
import com.mortals.xhx.module.ph.dao.PhQueueDao;
import com.mortals.xhx.module.ph.model.PhQueueEntity;
import com.mortals.xhx.module.ph.service.PhQueueService;
import org.springframework.beans.factory.annotation.Autowired;
import com.mortals.xhx.module.workman.model.WorkmanEntity;
import com.mortals.xhx.module.workman.model.WorkmanQuery;
import com.mortals.xhx.module.workman.service.WorkmanService;
import org.springframework.util.ObjectUtils;
import java.util.Date;
import java.util.Arrays;
import java.util.List;
import java.util.Map;
import java.util.function.Function;
import java.util.stream.Collectors;

import lombok.extern.slf4j.Slf4j;
/**
 * PhQueueService
 * 排号汇总 service实现
 *
 * @author zxfei
 * @date 2024-07-18
 */
@Service("phQueueService")
@Slf4j
public class PhQueueServiceImpl extends AbstractCRUDServiceImpl<PhQueueDao, PhQueueEntity, Long> implements PhQueueService {
    @Autowired
    private WorkmanService workmanService;


    @Override
    protected void findAfter(PhQueueEntity params,PageInfo pageInfo, Context context, List<PhQueueEntity> list) throws AppException {
        List<Long> idList = list.stream().map(i -> i.getId()).collect(Collectors.toList());
        WorkmanQuery workmanQuery = new WorkmanQuery();
        workmanQuery.setCreateUserIdList(idList);
        Map<Long, WorkmanEntity> workmanMap = workmanService.find(workmanQuery).stream().collect(Collectors.toMap(WorkmanEntity::getCreateUserId, Function.identity(), (n1, n2) -> n1));
        list.forEach(item -> item.setWorkmanEntity(workmanMap.get(item.getId())));
        super.findAfter(params,pageInfo, context, list);
    }

    /**
     * @param entity
     * @param context
     * @param list
     * @throws AppException
     */
    @Override
    protected void findAfter(PhQueueEntity entity, Context context, List<PhQueueEntity> list) throws AppException {
        super.findAfter(entity, context, list);
    }

    @Override
    protected void saveAfter(PhQueueEntity entity, Context context) throws AppException {
        if(!ObjectUtils.isEmpty(entity.getWorkmanEntity())){
            WorkmanEntity workmanEntity = new WorkmanEntity();
            workmanEntity.initAttrValue();
            WorkmanEntity subEntity=entity.getWorkmanEntity();
            BeanUtils.copyProperties(subEntity,workmanEntity);
            workmanEntity.setCreateUserId(entity.getId());
            workmanEntity.setCreateUserId(this.getContextUserId(context));
            workmanEntity.setCreateTime(new Date());
            workmanService.save(workmanEntity,context);
        }
        super.saveAfter(entity, context);
    }

    @Override
    protected void updateAfter(PhQueueEntity entity, Context context) throws AppException {
        if(!ObjectUtils.isEmpty(entity.getWorkmanEntity())){
            WorkmanEntity subEntity=entity.getWorkmanEntity();
            subEntity.setUpdateUserId(this.getContextUserId(context));
            subEntity.setUpdateTime(new Date());
            workmanService.update(subEntity,context);
        }
        super.updateAfter(entity, context);
    }
    @Override
    protected void removeAfter(Long[] ids, Context context, int result) throws AppException {
        List<WorkmanEntity> workmanlist = workmanService.find(new WorkmanQuery().createUserIdList(Arrays.asList(ids)));
        workmanService.removeList(workmanlist,context);
        super.removeAfter(ids, context, result);
    }
}