package com.mortals.xhx.module.site.service;

import com.mortals.framework.common.Rest;
import com.mortals.framework.exception.AppException;
import com.mortals.framework.model.Context;
import com.mortals.framework.service.ICRUDCacheService;
import com.mortals.xhx.module.matter.model.MatterEntity;
import com.mortals.xhx.module.site.dao.SiteDao;
import com.mortals.xhx.module.site.model.SiteEntity;
import com.mortals.xhx.module.site.model.SiteQuery;
import com.mortals.xhx.module.site.model.SiteTreeSelect;
import com.mortals.xhx.module.site.model.vo.SiteAreaVo;

import java.util.List;
import java.util.Map;

/**
 * SiteService
 * <p>
 * 站点 service接口
 *
 * @author zxfei
 * @date 2022-01-12
 */
public interface SiteService extends ICRUDCacheService<SiteEntity, Long> {


    SiteDao getDao();

    /**
     * 区域站点树
     *
     * @param context
     * @return
     */
    List<SiteTreeSelect> siteTree(Context context);

    /**
     * 根据站点列表获取区域站点树
     *
     * @param context
     * @return
     */
    List<SiteTreeSelect> getSiteTree(Context context);

    List<SiteEntity> getFlatSitesByAreaCode(String areaCode, Context context);

    List<SiteEntity> getFlatSitesBySiteId(Long siteId, Context context);

    Integer countSitesByArea(SiteQuery query, Context context);

    Rest<List<SiteAreaVo>> getAreaSitesBySite(SiteQuery query, Context context);

    Rest<List<SiteEntity>> getAreaSitesByAreaLevel(SiteQuery query, Context context);

    void setSiteTree(List<SiteTreeSelect> list, Context context);

    void removeSiteTree(List<SiteTreeSelect> list, Context context);

    List<SiteEntity> getListAllInfoByQuery(SiteQuery query, Context context) throws AppException;

    void changeUrlPath(SiteEntity siteEntity);

    Rest<List<MatterEntity>> getMatterAllListByGOV(Map<String, String> params, Integer pageNum, Context context);

    Rest<String> syncMatterBySiteId(SiteEntity siteEntity, Context context);

    void deleteBysiteIdAndSource(Long siteId, Integer source, Context context);

    void updateAllSiteTree(Context context);

    void updateSiteInfo(SiteEntity siteEntity, Context context);


}