package com.mortals.xhx.module.site.service.impl;

import com.alibaba.fastjson.JSON;
import com.mortals.framework.exception.AppException;
import com.mortals.framework.model.Context;
import com.mortals.framework.model.PageInfo;
import com.mortals.framework.model.Result;
import com.mortals.framework.service.impl.AbstractCRUDCacheServiceImpl;
import com.mortals.xhx.common.code.IsBusinessEnum;
import com.mortals.xhx.module.business.model.BusinessEntity;
import com.mortals.xhx.module.business.model.BusinessMatterEntity;
import com.mortals.xhx.module.business.model.BusinessMatterQuery;
import com.mortals.xhx.module.business.model.BusinessQuery;
import com.mortals.xhx.module.business.service.BusinessMatterService;
import com.mortals.xhx.module.business.service.BusinessService;
import com.mortals.xhx.module.site.dao.SiteBusinessDao;
import com.mortals.xhx.module.site.model.SiteBusinessEntity;
import com.mortals.xhx.module.site.model.SiteBusinessQuery;
import com.mortals.xhx.module.site.model.SiteBusinessTreeSelect;
import com.mortals.xhx.module.site.service.SiteBusinessService;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.annotation.Lazy;
import org.springframework.stereotype.Service;
import org.springframework.util.ObjectUtils;

import java.util.*;
import java.util.stream.Collectors;

/**
 * SiteBusinessService
 * 站点事项 service实现
 *
 * @author zxfei
 * @date 2022-01-13
 */
@Service("siteBusinessService")
@Slf4j
public class SiteBusinessServiceImpl extends AbstractCRUDCacheServiceImpl<SiteBusinessDao, SiteBusinessEntity, Long> implements SiteBusinessService {

    @Autowired
    @Lazy
    private BusinessService businessService;

    @Autowired
    @Lazy
    private BusinessMatterService businessMatterService;

    /**
     * @param params
     * @param pageInfo
     * @param context
     * @return
     * @throws AppException
     */
    @Override
    protected SiteBusinessEntity findBefore(SiteBusinessEntity params, PageInfo pageInfo, Context context) throws AppException {
        if (!ObjectUtils.isEmpty(params.getBusinessIdNotList())) {
            Set<Long> notIdSet = params.getBusinessIdNotList().stream().collect(Collectors.toSet());
            //排除掉了父级，但是子集存在，需要吧父id排除到notlist外
            SiteBusinessQuery siteBusinessQuery = new SiteBusinessQuery();
          //  siteBusinessQuery.setBusinessIdNotList(params.getBusinessIdNotList());
            siteBusinessQuery.setIsBusiness(IsBusinessEnum.一级业务.getValue());
            List<SiteBusinessEntity> rootSiteBusiness = this.find(siteBusinessQuery).stream().collect(Collectors.toList());
           //遍历所有一级业务
            for (SiteBusinessEntity root : rootSiteBusiness) {
                SiteBusinessQuery query = new SiteBusinessQuery();
                query.setParentId(root.getBusinessId());
                //根据一级业务找到对应的所有二级业务
                List<SiteBusinessEntity> childSiteBusiness = this.find(query, context);
                //查询所有二级业务。如果二级业务不全部存在，则把一级业务排除掉
                //如果二级业务全部存在，则把一级业务排除掉 排除【[498 500]   子（493 494 498） 500-根
                Boolean bool = false;
                for (SiteBusinessEntity child : childSiteBusiness) {
                    if (!notIdSet.contains(child.getBusinessId())) {
                        //存在非排除的业务id的孩子id 存在与二级节点中。父节点排除在外
                        bool = true;
                        break;
                    }
                }
                if (bool) {
                    //排除掉父id
                    notIdSet.remove(root.getBusinessId());
                }
            }
            params.setBusinessIdNotList(notIdSet.stream().collect(Collectors.toList()));
        }


        return super.findBefore(params, pageInfo, context);
    }

    @Override
    protected void findAfter(SiteBusinessEntity params, PageInfo pageInfo, Context context, List<SiteBusinessEntity> list) throws AppException {
        if (!ObjectUtils.isEmpty(params.getBusinessIdNotList())) {
            //排除掉已经存在的ids
            log.info("BusinessIdNotList():{}", JSON.toJSONString(params.getBusinessIdNotList()));
            Iterator<SiteBusinessEntity> iterator = list.iterator();
            while (iterator.hasNext()) {
                SiteBusinessQuery siteBusinessQuery = new SiteBusinessQuery();
                SiteBusinessEntity item = iterator.next();
                siteBusinessQuery.siteId(item.getSiteId());
                siteBusinessQuery.setParentId(item.getBusinessId());
                int count = this.count(siteBusinessQuery, context);
                if (count > 0) {
                    siteBusinessQuery.setBusinessIdNotList(params.getBusinessIdNotList());
                    List<SiteBusinessEntity> childs = this.find(siteBusinessQuery);
                    if (ObjectUtils.isEmpty(childs)) {
                        //子节点已经全部选中，删除父节点
                        iterator.remove();
                        pageInfo.setTotalResult(pageInfo.getTotalResult() - 1);
                    } else {
                        childs.stream().forEach(item1 -> {
                            buildChildBusiness(item1);
                        });
                        item.setChildren(childs);

                    }
                }
                buildChildBusiness(item);
            }
        } else {
            list.stream().peek(item -> {
                SiteBusinessQuery siteBusinessQuery = new SiteBusinessQuery();
                List<SiteBusinessEntity> childs = this.find(siteBusinessQuery.siteId(item.getSiteId()).parentId(item.getBusinessId()));
                childs.stream().forEach(item1 -> {
                    buildChildBusiness(item1);
                });
                item.setChildren(childs);
                buildChildBusiness(item);
            }).count();
            super.findAfter(params, pageInfo, context, list);
        }


    }

    private void buildChildBusiness(SiteBusinessEntity item) {
        if (!ObjectUtils.isEmpty(item.getBusinessId())) {
            BusinessEntity businessEntity = businessService.get(item.getBusinessId());
            if (!ObjectUtils.isEmpty(businessEntity)) {
                item.setFlownum(businessEntity.getFlownum());
                item.setCanorder(businessEntity.getCanorder());
                item.setCantake(businessEntity.getCantake());
                item.setDatashow(businessEntity.getDatashow());
                item.setEnglishname(businessEntity.getEnglishname());
                item.setBusinessType(businessEntity.getBusinessType());
                item.setRemark(businessEntity.getRemark());
                //判断业务是否含有子业务
                if (businessEntity.getIsBusiness() == IsBusinessEnum.一级业务.getValue()) {
                    //添加二级业务列表
                    List<BusinessEntity> businessEntities = businessService.find(new BusinessQuery().parentId(businessEntity.getId()));
                    if (!ObjectUtils.isEmpty(businessEntities)) {
                        item.setBusinessChildren(businessEntities);
                    }
                }
            }
        }
    }

    @Override
    protected void updateAfter(SiteBusinessEntity entity, Context context) throws AppException {
        //更新事项业务表中字段
        List<BusinessMatterEntity> businessMatterEntities = businessMatterService.find(new BusinessMatterQuery().siteBusinessId(entity.getId()))
                .stream()
                .filter(item -> !item.getSiteBusinessName().equals(entity.getBusinessName()))
                .peek(item -> item.setSiteBusinessName(entity.getBusinessName()))
                .collect(Collectors.toList());
        if (!ObjectUtils.isEmpty(businessMatterEntities)) {
            businessMatterService.update(businessMatterEntities, context);
        }
        super.updateAfter(entity, context);
    }

    @Override
    protected void removeBefore(Long[] ids, Context context) throws AppException {
        List<SiteBusinessEntity> childs = Arrays.asList(ids).stream()
                .flatMap(item -> {
                    SiteBusinessEntity siteBusinessEntity = this.get(item);
                    return this.find(new SiteBusinessQuery().parentId(siteBusinessEntity.getId())).stream();
                }).collect(Collectors.toList());
        log.info("delete childs list：{}",JSON.toJSONString(childs));
        Long[] childIds = childs.stream().map(item->item.getId()).toArray(Long[]::new);
        this.getDao().delete(childIds);
        Long[] bussinessIds = childs.stream().map(item->item.getBusinessId()).toArray(Long[]::new);
        log.info("childBussinessIds:{}",JSON.toJSONString(bussinessIds));
        if(!ObjectUtils.isEmpty(bussinessIds)){
            businessService.remove(bussinessIds,context);
        }

        Long[] mainIds = this.get(ids, context).stream().map(item -> item.getBusinessId()).toArray(Long[]::new);
        if(!ObjectUtils.isEmpty(mainIds)){
            //删除
            businessService.remove(mainIds,context);
        }
    }


    @Override
    public boolean hasChildBySiteBusinessId(Long siteBusinessId) {
        List<SiteBusinessEntity> list = this.find(new SiteBusinessQuery().parentId(siteBusinessId));
        return list.size() > 0 ? true : false;
    }

    @Override
    public List<SiteBusinessEntity> selectSiteBusinessList(SiteBusinessEntity siteBusiness) {
        return this.find(new SiteBusinessQuery());
    }

    @Override
    public List<SiteBusinessTreeSelect> buildSiteBusinessTreeSelect(List<SiteBusinessEntity> list) {
        List<SiteBusinessEntity> returnList = new ArrayList<>();
        List<Long> tempList = list.stream().map(SiteBusinessEntity::getId).collect(Collectors.toList());

        for (Iterator<SiteBusinessEntity> iterator = list.iterator(); iterator.hasNext(); ) {
            SiteBusinessEntity siteBusinessEntity = iterator.next();
            // 如果是顶级节点, 遍历该父节点的所有子节点
            if (!tempList.contains(siteBusinessEntity.getParentId())) {
                recursionFn(list, siteBusinessEntity);
                returnList.add(siteBusinessEntity);
            }
        }
        if (returnList.isEmpty()) {
            returnList = list;
        }

        return returnList.stream().map(SiteBusinessTreeSelect::new).collect(Collectors.toList());
    }

    @Override
    public Result<SiteBusinessEntity> flatList(SiteBusinessEntity query, PageInfo pageInfo, Context context) {
        Result<SiteBusinessEntity> result = this.dao.getList(query, pageInfo);
        result.getList().stream().forEach(item -> {
            if (!ObjectUtils.isEmpty(item.getBusinessId())) {
                BusinessEntity businessEntity = businessService.get(item.getBusinessId());
                if (!ObjectUtils.isEmpty(businessEntity)) {
                    BeanUtils.copyProperties(businessEntity, item, "id");
                }
            }
        });

        return result;
    }


    /**
     * 递归列表
     */
    private void recursionFn(List<SiteBusinessEntity> list, SiteBusinessEntity t) {
        // 得到子节点列表
        List<SiteBusinessEntity> childList = getChildList(list, t);
        t.setChildren(childList);
        for (SiteBusinessEntity tChild : childList) {
            if (hasChild(list, tChild)) {
                recursionFn(list, tChild);
            }
        }
    }

    /**
     * 判断是否有子节点
     */
    private boolean hasChild(List<SiteBusinessEntity> list, SiteBusinessEntity t) {
        return getChildList(list, t).size() > 0 ? true : false;
    }

    /**
     * 得到子节点列表
     */
    private List<SiteBusinessEntity> getChildList(List<SiteBusinessEntity> list, SiteBusinessEntity t) {
        return list.stream().map(item -> {
            if (!ObjectUtils.isEmpty(item.getParentId()) && item.getParentId() == t.getId()) {
                return item;
            }
            return null;
        }).filter(f -> f != null).collect(Collectors.toList());
    }

}