package com.mortals.xhx.module.site.service.impl;

import com.mortals.framework.common.Rest;
import com.mortals.xhx.module.business.model.BusinessEntity;
import com.mortals.xhx.module.business.model.BusinessQuery;
import com.mortals.xhx.module.business.service.BusinessService;
import com.mortals.xhx.module.site.model.SiteBusinessQuery;
import com.mortals.xhx.module.site.service.SiteBusinessService;
import com.mortals.xhx.module.window.model.WindowBusinessQuery;
import com.mortals.xhx.module.window.model.WindowHallEntity;
import com.mortals.xhx.module.window.model.WindowHallQuery;
import com.mortals.xhx.module.window.service.WindowBusinessService;
import com.mortals.xhx.module.window.service.WindowHallService;
import org.checkerframework.checker.units.qual.A;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import com.mortals.framework.service.impl.AbstractCRUDServiceImpl;
import com.mortals.framework.exception.AppException;
import com.mortals.framework.model.Context;
import com.mortals.xhx.module.site.dao.SiteHallDao;
import com.mortals.xhx.module.site.model.SiteHallEntity;
import com.mortals.xhx.module.site.service.SiteHallService;
import org.springframework.util.ObjectUtils;

import java.util.List;
import java.util.stream.Collectors;

/**
 * SiteHallService
 * 站点大厅信息 service实现
 *
 * @author zxfei
 * @date 2023-04-25
 */
@Service("siteHallService")
public class SiteHallServiceImpl extends AbstractCRUDServiceImpl<SiteHallDao, SiteHallEntity, Long> implements SiteHallService {

    @Autowired
    private WindowHallService windowHallService;
    @Autowired
    private WindowBusinessService windowBusinessService;
    @Autowired
    private SiteBusinessService siteBusinessService;
    @Autowired
    private BusinessService businessService;


    /**
     * @param entity
     * @param context
     * @throws AppException
     */
    @Override
    protected void updateAfter(SiteHallEntity entity, Context context) throws AppException {
        super.updateAfter(entity, context);
        WindowHallEntity condition = new WindowHallEntity();
        condition.setHallId(entity.getId());
        WindowHallEntity windowHallEntity = new WindowHallEntity();
        windowHallEntity.setHallName(entity.getHallName());
        windowHallService.getDao().update(windowHallEntity, condition);
    }

    /**
     * @param siteHall
     * @param context
     * @return
     */
    @Override
    public Rest<List<BusinessEntity>> getBusinessByHall(SiteHallEntity siteHall, Context context) {
        if (ObjectUtils.isEmpty(siteHall.getId())) {
            return Rest.fail("大厅ID不能为空！");
        }
        List<Long> windowIds = windowHallService.find(new WindowHallQuery().hallId(siteHall.getId())).parallelStream().map(i -> i.getWindowId()).distinct().collect(Collectors.toList());
        if (ObjectUtils.isEmpty(windowIds)) {
            return Rest.fail("窗口list不能为空！");
        }
        WindowBusinessQuery windowBusinessQuery = new WindowBusinessQuery();
        windowBusinessQuery.setWindowIdList(windowIds);
        List<Long> siteBusinessIdList = windowBusinessService.find(windowBusinessQuery).parallelStream().map(i -> i.getSiteBusinessId()).distinct().collect(Collectors.toList());
        if (ObjectUtils.isEmpty(siteBusinessIdList)) {
            return Rest.fail("站点业务list不能为空！");
        }
        SiteBusinessQuery siteBusinessQuery = new SiteBusinessQuery();
        siteBusinessQuery.setBusinessIdList(siteBusinessIdList);
        List<Long> businessIdList = siteBusinessService.find(siteBusinessQuery).parallelStream().map(i -> i.getBusinessId()).distinct().collect(Collectors.toList());
        if (ObjectUtils.isEmpty(businessIdList)) {
            return Rest.fail("业务list不能为空！");
        }
        BusinessQuery businessQuery = new BusinessQuery();
        businessQuery.setIdList(businessIdList);
        List<BusinessEntity> businessEntities = businessService.find(businessQuery);
        return Rest.ok(businessEntities);
    }
}