package com.mortals.xhx.module.site.service.impl;

import com.mortals.framework.common.Rest;
import com.mortals.framework.exception.AppException;
import com.mortals.framework.model.Context;
import com.mortals.framework.model.PageInfo;
import com.mortals.xhx.common.code.SourceEnum;
import com.mortals.xhx.module.matter.model.MatterEntity;
import com.mortals.xhx.module.matter.model.MatterQuery;
import com.mortals.xhx.module.matter.service.MatterService;
import com.mortals.xhx.module.site.model.*;
import com.mortals.xhx.module.site.service.SiteService;
import com.mortals.xhx.module.site.service.SiteThemeMatterService;
import com.mortals.xhx.module.site.service.SiteThemeService;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import com.mortals.framework.service.impl.AbstractCRUDServiceImpl;
import com.mortals.xhx.module.site.dao.SiteMatterDao;
import com.mortals.xhx.module.site.service.SiteMatterService;
import org.springframework.util.ObjectUtils;

import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.stream.Collectors;

/**
 * SiteMatterService
 * 站点事项 service实现
 *
 * @author zxfei
 * @date 2022-01-12
 */
@Service("siteMatterService")
@Slf4j
public class SiteMatterServiceImpl extends AbstractCRUDServiceImpl<SiteMatterDao, SiteMatterEntity, Long> implements SiteMatterService {

    @Autowired
    private MatterService matterService;
    @Autowired
    private SiteThemeMatterService siteThemeMatterService;
    @Autowired
    private SiteService siteService;

    @Override
    protected void findAfter(SiteMatterEntity params, PageInfo pageInfo, Context context, List<SiteMatterEntity> list) throws AppException {

        if (!ObjectUtils.isEmpty(list)) {
            List<Long> matterIdlist = list.stream().map(item -> item.getMatterId()).collect(Collectors.toList());
            Map<Long, MatterEntity> matterEntityMap = matterService.find(new MatterQuery().idList(matterIdlist)).parallelStream().collect(Collectors.toMap(x -> x.getId(), y -> y, (o, n) -> n));
            list.forEach(item -> {
                if (!ObjectUtils.isEmpty(item.getMatterId())) {
                    MatterEntity matterEntity = matterEntityMap.get(item.getMatterId());
                    if (!ObjectUtils.isEmpty(matterEntity)) {
                        item.setBelongDept(matterEntity.getBelongDept());
                        item.setWindowToTheSceneNum(matterEntity.getWindowToTheSceneNum());
                        item.setOnlineToTheSceneNum(matterEntity.getOnlineToTheSceneNum());
                        item.setDeptCode(matterEntity.getDeptCode());
                        item.setAreaCode(matterEntity.getAreaCode());
                    }
                }
            });
        }
        super.findAfter(params, pageInfo, context, list);
    }

    /**
     * @param entity
     * @param context
     * @throws AppException
     */
    @Override
    protected void updateBefore(SiteMatterEntity entity, Context context) throws AppException {
        super.updateBefore(entity, context);
        SiteMatterEntity beforeSiteMatterEntity = this.get(entity.getId(), context);
        if (!ObjectUtils.isEmpty(beforeSiteMatterEntity.getDeptId()) && !beforeSiteMatterEntity.getDeptId().equals(entity.getDeptId())) {
            //更新事项中的部门编号
            MatterEntity matterEntity = matterService.get(entity.getMatterId(), context);
            if (!ObjectUtils.isEmpty(matterEntity)) {
                matterEntity.setDeptCode(entity.getDeptCode());
                matterEntity.setDeptName(entity.getDeptName());
                matterService.update(matterEntity, context);
            }
        }

        SiteThemeMatterEntity siteThemeMatterEntity = siteThemeMatterService.selectOne(new SiteThemeMatterQuery().matterId(entity.getMatterId()));
        if (!ObjectUtils.isEmpty(siteThemeMatterEntity)) {
            siteThemeMatterEntity.setHot(entity.getHot());
            siteThemeMatterEntity.setDisplay(entity.getDisplay());
        }

    }

    /**
     * @param entity
     * @param context
     * @throws AppException
     */
    @Override
    protected void saveBefore(SiteMatterEntity entity, Context context) throws AppException {
        super.saveBefore(entity, context);
        SiteThemeMatterEntity siteThemeMatterEntity = siteThemeMatterService.selectOne(new SiteThemeMatterQuery().matterId(entity.getMatterId()));
        if (!ObjectUtils.isEmpty(siteThemeMatterEntity)) {
            siteThemeMatterEntity.setHot(entity.getHot());
            siteThemeMatterEntity.setDisplay(entity.getDisplay());
        }
    }

    @Override
    public void deleteBysiteIdAndSource(Long siteId, Integer source, Context context) {
        Map<String, Object> condition = new HashMap<>();
        condition.put("siteId", siteId);
        condition.put("source", source);
        this.dao.delete(condition);
    }

    /**
     * @param siteEntity
     * @param context
     * @return
     */
    @Override
    public Rest<String> doSyncSiteMatter(SiteEntity siteEntity, Context context) {

        List<MatterEntity> matterList = matterService.getDao().getMatterListByAreaCode(new MatterQuery().areaCode(siteEntity.getAreaCode()).source(SourceEnum.政务网.getValue()));
        //查询站点事项相关
        SiteEntity site = siteService.selectOne(new SiteQuery().areaCode(siteEntity.getAreaCode()));
        if (!ObjectUtils.isEmpty(site)) {
            log.info("同步站点事项到站点.....");
            SiteMatterQuery siteMatterQuery = new SiteMatterQuery();
            siteMatterQuery.setSiteId(site.getId());
            siteMatterQuery.setSource(SourceEnum.政务网.getValue());
            List<SiteMatterEntity> localList = this.getDao().getList(siteMatterQuery);
            log.info("本地站点事项数量.....{}",localList.size());
            //找到差集 然后添加
            Set<Long> existSiteMatterIdSet = localList.parallelStream().map(i -> i.getMatterId()).collect(Collectors.toSet());
            List<MatterEntity> subList = matterList.parallelStream().filter(f -> !existSiteMatterIdSet.contains(f.getId())).collect(Collectors.toList());
            if(!ObjectUtils.isEmpty(subList)){
                //需要新增的站点事项
                log.info("需要新增的站点事项数量.....{}",subList.size());
                List<SiteMatterEntity> siteMatterList = subList
                        .parallelStream()
                        .map(item ->matterService.switchMatterToSiteMatter(item, site, null).getData())
                        .filter(f -> f != null)
                        .collect(Collectors.toList());
                this.save(siteMatterList);
            }
/*

            //判断是否已经有站点事项，如果有则不添加
            Boolean bool = checkSiteMatter(site);
            if (!bool) {
                //添加站点事项 批量添加
                List<SiteMatterEntity> siteMatterList = matterList.stream().map(item -> {
                    return matterService.switchMatterToSiteMatter(item, site, null).getData();
                }).filter(f -> f != null).collect(Collectors.toList());
                this.save(siteMatterList);
            }
*/
            log.info("同步站点事项到站点完成.....");
        }

        return Rest.ok();
    }




}