package com.mortals.xhx.module.site.service.impl;

import cn.hutool.core.collection.ListUtil;
import cn.hutool.core.util.StrUtil;
import cn.hutool.core.util.URLUtil;
import cn.hutool.core.util.ZipUtil;
import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.serializer.SerializerFeature;
import com.mortals.framework.ap.GlobalSysInfo;
import com.mortals.framework.common.Rest;
import com.mortals.framework.exception.AppException;
import com.mortals.framework.model.Context;
import com.mortals.framework.model.PageInfo;
import com.mortals.framework.model.Result;
import com.mortals.framework.service.impl.AbstractCRUDCacheServiceImpl;
import com.mortals.framework.util.DataUtil;
import com.mortals.framework.util.HttpUtil;
import com.mortals.framework.util.StringUtils;
import com.mortals.framework.util.ThreadPool;
import com.mortals.xhx.base.system.user.model.UserEntity;
import com.mortals.xhx.base.system.user.service.UserService;
import com.mortals.xhx.common.code.AreaLevelDxTypeEnum;
import com.mortals.xhx.common.code.AreaLevelEnum;
import com.mortals.xhx.common.code.SourceEnum;
import com.mortals.xhx.common.code.YesNoEnum;
import com.mortals.xhx.common.key.Constant;
import com.mortals.xhx.common.pdu.RespData;
import com.mortals.xhx.common.pdu.device.DevicePdu;
import com.mortals.xhx.common.pdu.user.UserPdu;
import com.mortals.xhx.common.utils.AreaMatchCodeUtil;
import com.mortals.xhx.common.utils.MatterHtmlParseUtil;
import com.mortals.xhx.common.utils.SyncTreeSiteThread;
import com.mortals.xhx.feign.device.IDeviceFeign;
import com.mortals.xhx.feign.user.IUserFeign;
import com.mortals.xhx.module.app.model.AppCategoryEntity;
import com.mortals.xhx.module.app.model.AppCategoryQuery;
import com.mortals.xhx.module.app.model.AppEntity;
import com.mortals.xhx.module.app.model.AppQuery;
import com.mortals.xhx.module.app.service.AppCategoryService;
import com.mortals.xhx.module.app.service.AppService;
import com.mortals.xhx.module.area.model.AreaEntity;
import com.mortals.xhx.module.area.model.AreaQuery;
import com.mortals.xhx.module.area.service.AreaService;
import com.mortals.xhx.module.dept.model.DeptEntity;
import com.mortals.xhx.module.dept.model.DeptQuery;
import com.mortals.xhx.module.dept.service.DeptService;
import com.mortals.xhx.module.matter.model.MatterEntity;
import com.mortals.xhx.module.matter.model.MatterQuery;
import com.mortals.xhx.module.matter.service.MatterService;
import com.mortals.xhx.module.matters.service.MattersService;
import com.mortals.xhx.module.model.model.ModelEntity;
import com.mortals.xhx.module.model.model.ModelQuery;
import com.mortals.xhx.module.model.service.ModelService;
import com.mortals.xhx.module.site.dao.SiteDao;
import com.mortals.xhx.module.site.model.*;
import com.mortals.xhx.module.site.model.vo.SiteAreaVo;
import com.mortals.xhx.module.site.service.*;
import com.mortals.xhx.module.window.model.WindowEntity;
import com.mortals.xhx.module.window.model.WindowQuery;
import com.mortals.xhx.module.window.service.WindowService;
import com.mortals.xhx.module.workman.model.WorkmanEntity;
import com.mortals.xhx.module.workman.model.WorkmanQuery;
import com.mortals.xhx.module.workman.service.WorkmanService;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.collections4.CollectionUtils;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.checkerframework.checker.units.qual.A;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.annotation.Lazy;
import org.springframework.stereotype.Service;
import org.springframework.util.ObjectUtils;

import java.util.*;
import java.util.stream.Collectors;

import static com.mortals.framework.util.HttpUtil.HEADER_CONTENT_TYPE;
import static com.mortals.xhx.common.key.Constant.*;

/**
 * SiteService
 * 站点 service实现
 *
 * @author zxfei
 * @date 2022-01-12
 */
@Service("siteService")
@Slf4j
public class SiteServiceImpl extends AbstractCRUDCacheServiceImpl<SiteDao, SiteEntity, Long> implements SiteService {
    // private List<SiteTreeSelect> siteTreeList;
    /**
     * 根据用户id 暂存对应站点树 默认0为全站点树
     */
    private Map<Long, List<SiteTreeSelect>> siteTreeMap = new HashMap<>();
    @Autowired
    @Lazy
    private AreaService areaService;
    @Autowired
    @Lazy
    private ModelService modelService;
    @Autowired
    @Lazy
    private IUserFeign userFeign;
    @Autowired
    @Lazy
    private UserService userService;
    @Autowired
    private DeptService deptService;

    @Autowired
    private MatterService matterService;
    @Autowired
    private WindowService windowService;
    @Autowired
    private SiteBusinessService siteBusinessService;
    @Autowired
    private SiteThemeMatterService siteThemeMatterService;
    @Autowired
    private SiteMatterService siteMatterService;
    @Autowired
    private WorkmanService workmanService;
    @Autowired
    private IDeviceFeign deviceFeign;

    @Autowired
    private AppService appService;
    @Autowired
    private AppCategoryService appCategoryService;


    private volatile Boolean refresh = false;

    @Override
    protected String getExtKey(SiteEntity data) {
        return data.getSiteCode();
    }

    @Override
    protected void updateBefore(SiteEntity entity, Context context) throws AppException {
        //校验下前后名称是否一直 如果不一致 同步修改所以关联
        SiteEntity oldSite = this.get(entity.getId(), context);
        if (!oldSite.getSiteName().equals(entity.getSiteName())) {
            //todo
            //更新关联站点名称
            updateAssociateSiteName(oldSite, entity, context);
            refresh = true;
        }

        if (!ObjectUtils.isEmpty(oldSite.getEnglishName())&&!oldSite.getEnglishName().equals(entity.getEnglishName())) {
            refresh = true;
        }

        super.updateBefore(entity, context);
    }


    private void updateAssociateSiteName(SiteEntity oldSite, SiteEntity newSite, Context context) {
        //更新窗口站点名称
        WindowEntity windowEntity = new WindowEntity();
        windowEntity.setSiteName(newSite.getSiteName());
        WindowEntity condition = new WindowEntity();
        condition.setSiteName(oldSite.getSiteName());
        windowService.updateBatch(windowEntity, condition, context);

        //更新站点业务名称
        SiteBusinessEntity siteBusinessEntity = new SiteBusinessEntity();
        siteBusinessEntity.setSiteName(newSite.getSiteName());
        SiteBusinessEntity siteBusinessCondition = new SiteBusinessEntity();
        siteBusinessCondition.setSiteName(oldSite.getSiteName());
        siteBusinessService.updateBatch(siteBusinessEntity, siteBusinessCondition, context);
        //更新站点事项名称
        SiteMatterEntity siteMatterEntity = new SiteMatterEntity();
        siteMatterEntity.setSiteName(newSite.getSiteName());
        SiteMatterEntity siteMatterCondition = new SiteMatterEntity();
        siteMatterCondition.setSiteName(oldSite.getSiteName());
        siteMatterService.updateBatch(siteMatterEntity, siteMatterCondition, context);

        //更新站点主题事项名称
        SiteThemeMatterEntity siteThemeMatterEntity = new SiteThemeMatterEntity();
        siteThemeMatterEntity.setSiteName(newSite.getSiteName());
        SiteThemeMatterEntity siteThemeMatterCondition = new SiteThemeMatterEntity();
        siteThemeMatterCondition.setSiteName(oldSite.getSiteName());
        siteThemeMatterService.updateBatch(siteThemeMatterEntity, siteThemeMatterCondition, context);

        //更新工作人员站点

        WorkmanEntity workmanEntity = new WorkmanEntity();
        workmanEntity.setSiteName(newSite.getSiteName());
        WorkmanEntity workmanEntityCondition = new WorkmanEntity();
        workmanEntityCondition.setSiteName(oldSite.getSiteName());
        workmanService.updateBatch(workmanEntity, workmanEntityCondition, context);


    }

    @Override
    protected void validData(SiteEntity entity, Context context) throws AppException {
        if (ObjectUtils.isEmpty(entity.getAreaCode())) {
            throw new AppException("区域编码不能为空！");
        }
        AreaEntity areaEntity = areaService.getCache(entity.getAreaCode());
        if (ObjectUtils.isEmpty(areaEntity)) {
            throw new AppException(String.format("区域不存在！区域编码：%s", entity.getAreaCode()));
        }
        super.validData(entity, context);
    }

    @Override
    protected void saveBefore(SiteEntity entity, Context context) throws AppException {
        List<SiteEntity> siteEntities = this.find(new SiteQuery().areaCode(entity.getAreaCode()));
        entity.setSiteCode(entity.getAreaCode() + Constant.SITE_SPLIT_MARK + StringUtils.lpad(siteEntities.size() + 1, Constant.SITE_CODE_NUM));
        super.saveBefore(entity, context);
    }


    @Override
    protected void saveAfter(SiteEntity entity, Context context) throws AppException {
        super.saveAfter(entity, context);
        userService.refreshUser();
      //  this.updateAllSiteTree(context);
        //通知php系统更新
        String phpUrl = GlobalSysInfo.getParamValue(PARAM_SERVER_PHP_HTTP_URL, "http://127.0.0.1:11078/zwfw_api");
        HashMap<String, String> paramsMap = new HashMap<>();
        paramsMap.put("siteid", entity.getId().toString());

        phpUrl += "/api/site/childSiteSet";
        String resp = null;
        try {
            resp = HttpUtil.doGet(phpUrl, paramsMap);
            log.info("thirdPartyUrl =>resp:{}", resp);
        } catch (Exception e) {
            log.error("异常:", e);
        }


    }


    @Override
    public List<SiteTreeSelect> siteTree(Context context) {
        Map<String, AreaEntity> areaMap = new HashMap<>();
        SiteQuery siteQuery = new SiteQuery();
        //如果站点为空 或者用户为空 返回空数组
        if (context.getUser() == null) {
            return new ArrayList<>();
        }
        List<SiteEntity> siteList = new ArrayList<>();

        if (context.getUser().getSiteIds() != null) {
            Set<String> siteSet = Arrays.stream(context.getUser().getSiteIds().split(",")).filter(f -> !f.equals("")).collect(Collectors.toSet());
            if (!ObjectUtils.isEmpty(siteSet)) {
                List<Long> siteIdList = siteSet.stream().map(Long::parseLong).collect(Collectors.toList());
                if (!ObjectUtils.isEmpty(siteIdList)) {
                    siteQuery.idList(siteIdList);
                }
            }
            //查詢指定的站點ids
            // log.info(String.format("siteQuery==>%s", JSON.toJSONString(siteQuery)));
            siteList = this.find(siteQuery);
        }
        //如果是管理员 默认全部站点
        if (context.getUser().isAdmin() || context.getUser().getId() == 0L) {
            log.info("user is admin ! id:{}", context.getUser().getId());
            siteList = this.find(new SiteQuery());
        }

        if (ObjectUtils.isEmpty(siteList)) {
            return new ArrayList<>();
        }

        //turn to sitemap
        Map<String, SiteEntity> siteMap = siteList.parallelStream().collect(Collectors.toMap(x -> x.getSiteCode(), y -> y, (o, n) -> n));
        //遍历过滤站点树
        for (SiteEntity siteEntity : siteList) {
            AreaEntity areaEntity = areaService.getExtCache(siteEntity.getAreaCode());
            //根据区域添加父节点
            if (!ObjectUtils.isEmpty(areaEntity)
                    && !ObjectUtils.isEmpty(areaEntity.getPid())) {
                //递归查找父节点并添加
                areaMap.put(areaEntity.getAreaCode(), areaEntity);
                //递归
                recursionFn(areaMap, areaEntity);
            }
        }
        //log.info("rebuildAreaMap==>{}", JSON.toJSONString(areaMap));
        //areaMap 为当前站点相关区域树结构 已经剔除不必要的分支结构
        return buildSiteTreeSelect(areaMap, siteMap);
    }

    public List<SiteTreeSelect> buildSiteTreeSelect(Map<String, AreaEntity> areaMap, Map<String, SiteEntity> siteMap) {
        List<AreaEntity> areaList = areaMap.values().stream().collect(Collectors.toList());
        List<AreaEntity> returnList = new ArrayList<>();
        //区域id
        List<String> tempList = areaList.stream().map(AreaEntity::getIid).collect(Collectors.toList());
        for (AreaEntity areaEntity : areaList) {
            //查询当前区域列表中最顶级的节点 并构建树
            if (!tempList.contains(areaEntity.getPid())) {
                recursion(areaList, areaEntity);
                returnList.add(areaEntity);
            }
        }
        //区域树
        if (returnList.isEmpty()) {
            returnList = areaList;
        }

        //构建站点树
        List<SiteTreeSelect> collect = returnList.parallelStream().map(item ->
                new SiteTreeSelect(item, siteMap)
        ).collect(Collectors.toList());
        //this.siteTreeList = collect;
        return collect;

    }

    private void recursion(List<AreaEntity> list, AreaEntity entity) {
        // 得到子节点列表
        List<AreaEntity> childList = getChildList(list, entity);
        entity.setChildren(childList);
        for (AreaEntity tChild : childList) {
            if (hasChild(list, tChild)) {
                recursion(list, tChild);
            }
        }
    }

    /**
     * 得到子节点列表
     */
    private List<AreaEntity> getChildList(List<AreaEntity> list, AreaEntity entity) {
        return list.stream()
                .filter(item -> item.getPid().equals(entity.getIid()))
                .collect(Collectors.toList());
    }

    /**
     * 判断是否有子节点
     */
    private boolean hasChild(List<AreaEntity> list, AreaEntity t) {
        return getChildList(list, t).size() > 0 ? true : false;
    }


    /**
     * 递归列表
     */
    private void recursionFn(Map<String, AreaEntity> areaMap, AreaEntity t) {
        // 得到父节点并加入
        AreaEntity areaEntity = areaService.selectOne(new AreaQuery().iid(t.getPid()));
        if (!ObjectUtils.isEmpty(areaEntity)) {
            areaMap.put(areaEntity.getAreaCode(), areaEntity);
            recursionFn(areaMap, areaEntity);
        }
    }


    @Override
    protected void updateAfter(SiteEntity entity, Context context) throws AppException {
        super.updateAfter(entity, context);
        //刷新站点树
        if (refresh) {
            //通知设备更新站点信息
            DevicePdu devicePdu = new DevicePdu();
            devicePdu.setSiteId(entity.getId());
            Rest<Void> rest1 = deviceFeign.refreshMessage(devicePdu);
            log.info("调用刷新结果：{}", JSON.toJSONString(rest1));
            refresh = false;
            //刷新用户站点树
           // cacheService.del(USER_SITE_TREE);
           // this.updateAllSiteTree(context);
            userService.refreshUser();
        }

        //通知php系统更新
        String phpUrl = GlobalSysInfo.getParamValue(PARAM_SERVER_PHP_HTTP_URL, "http://127.0.0.1:11078/zwfw_api");
        HashMap<String, String> paramsMap = new HashMap<>();
        paramsMap.put("siteid", entity.getId().toString());

        phpUrl += "/api/site/updateSite";
        String resp = null;
        try {
            resp = HttpUtil.doGet(phpUrl, paramsMap);
            log.info("thirdPartyUrl updateSite =>resp:{}", resp);
        } catch (Exception e) {
            log.error("异常:", e);
        }

    }


    @Override
    protected void removeAfter(Long[] ids, Context context, int result) throws AppException {

        Rest<String> rest = userFeign.synchSiteAuth();
        if (rest.getCode().equals(YesNoEnum.YES.getValue())) {
            UserPdu userPdu = new UserPdu();
            userPdu.setPage(1);
            userPdu.setSize(-1);
            Rest<RespData<List<UserPdu>>> userRest = userFeign.list(userPdu);
            if(YesNoEnum.YES.getValue()==userRest.getCode()){
                if(!ObjectUtils.isEmpty(userRest.getData())
                        &&!ObjectUtils.isEmpty(userRest.getData().getData())) {
                    userService.updateUserList(userRest.getData().getData());
                }
            }

        }
        super.removeAfter(ids, context, result);
    }

    @Override
    public List<SiteTreeSelect> getSiteTree(Context context) {
        // List<SiteTreeSelect> siteTreeSelects = siteTreeMap.get(context.getUser().getId());
        List<SiteTreeSelect> collect = getSiteTreeSelects(context.getUser().getId().toString());
        // log.info("siteTree:{}", JSON.toJSONString(collect));
        if (!ObjectUtils.isEmpty(collect)) {
            return collect;
        } else {
            //反全站点树  如果没有全站点树，则临时构建 admin 为全站点树
            return getSiteTreeSelects("1");
        }
    }

    private List<SiteTreeSelect> getSiteTreeSelects(String userId) {
        String siteTreeSelectStr = cacheService.hget(USER_SITE_TREE, userId, String.class);
        //log.info("userId:{},siteTreeSelectStr:{}", userId, siteTreeSelectStr);
        //反序列化树对象
        if (ObjectUtils.isEmpty(siteTreeSelectStr)) {
            return new ArrayList<>();
        }
        JSONArray jsonArray = JSON.parseArray(siteTreeSelectStr);
        if (ObjectUtils.isEmpty(jsonArray)) {
            return new ArrayList<>();
        }
        List<SiteTreeSelect> collect = jsonArray.stream().map(item -> {
            SiteTreeSelect siteTreeSelect = JSON.parseObject(item.toString(), SiteTreeSelect.class);
            return siteTreeSelect;
        }).collect(Collectors.toList());
        return collect;
    }

    @Override
    public List<SiteEntity> getFlatSitesByAreaCode(String areaCode, Context context) {
        if (ObjectUtils.isEmpty(areaCode)) {
            throw new AppException("区域编码不能为空！");
        }
        AreaEntity areaEntity = areaService.getExtCache(areaCode);
        if (ObjectUtils.isEmpty(areaEntity)) {
            throw new AppException(String.format("区域不存在！区域编码：%s", areaCode));
        }
        String matchCode = areaCode.replaceAll("(0)+$", "");
        matchCode = AreaMatchCodeUtil.getMatchCode(matchCode, areaEntity.getAreaLevel());
        SiteQuery siteQuery = new SiteQuery();
        siteQuery.setAreaCode(matchCode + "%");
        List<SiteEntity> siteEntities = this.find(siteQuery);

        PageInfo pageInfo = new PageInfo();
        pageInfo.setCurrPage(1);
        pageInfo.setPrePageResult(10);
        return siteEntities;
    }

    @Override
    public List<SiteEntity> getFlatSitesBySiteId(Long siteId, Context context) {
        List<SiteEntity> siteList = new ArrayList<>();
        if (ObjectUtils.isEmpty(siteId)) {
            throw new AppException("站点Id不能为空！");
        }
        SiteEntity siteCache = this.get(siteId);
        if (ObjectUtils.isEmpty(siteCache)) {
            throw new AppException("站点不存在，siteId:" + siteId);
        }
        //siteList.add(siteCache);
        AreaEntity areaEntity = areaService.getExtCache(siteCache.getAreaCode());
        if (ObjectUtils.isEmpty(areaEntity)) {
            areaEntity = areaService.selectOne(new AreaQuery().areaCode(siteCache.getAreaCode()));
            if (ObjectUtils.isEmpty(areaEntity)) {
                throw new AppException(String.format("区域不存在！区域编码：%s", siteCache.getAreaCode()));
            }
        }

        String matchCode = siteCache.getAreaCode().replaceAll("(0)+$", "");
        matchCode = AreaMatchCodeUtil.getMatchCode(matchCode, areaEntity.getAreaLevel());

        SiteQuery siteQuery = new SiteQuery();
        siteQuery.setAreaCode(matchCode + "%");
        List<SiteEntity> siteEntities = this.find(siteQuery);

        if (!ObjectUtils.isEmpty(siteEntities)) {
            siteList.addAll(siteEntities);
        }
        return siteList;
    }

    /**
     * @param query
     * @param context
     * @return
     */
    @Override
    public Integer countSitesByArea(SiteQuery query, Context context) {
        List<SiteEntity> siteList = new ArrayList<>();

        if (ObjectUtils.isEmpty(query.getAreaCode())) {
            throw new AppException("区域编码不能为空！");
        }
        AreaEntity areaEntity = areaService.getExtCache(query.getAreaCode());
        if (ObjectUtils.isEmpty(areaEntity)) {
            throw new AppException(String.format("区域不存在！区域编码：%s", query.getAreaCode()));
        }
        String matchCode = query.getAreaCode().replaceAll("(0)+$", "");
        matchCode = AreaMatchCodeUtil.getMatchCode(matchCode, areaEntity.getAreaLevel());
        SiteQuery siteQuery = new SiteQuery();
        siteQuery.setAreaCode(matchCode + "%");
        List<SiteEntity> siteEntities = this.find(siteQuery);

        if (!ObjectUtils.isEmpty(siteEntities)) {
            siteList.addAll(siteEntities);
        }
        return siteList.size();
    }

    @Override
    public Rest<List<SiteAreaVo>> getAreaSitesBySite(SiteQuery query, Context context) {
        List<SiteAreaVo> list = new ArrayList<>();
        SiteAreaVo siteAreaVo = new SiteAreaVo();
        //获取当前节点的第一层子节点
        if (ObjectUtils.isEmpty(query.getAreaName())) {
            throw new AppException("区域名称查询参数不能为空！");
        }
        //SiteEntity siteCache = this.getCache(query.getId().toString());
        AreaEntity areaEntity = areaService.selectOne(new AreaQuery().name(query.getAreaName()), context);
        if (ObjectUtils.isEmpty(areaEntity)) {
            throw new AppException("区域名称不存在！");
        }
        if ("False".equalsIgnoreCase(areaEntity.getHaveSonArea())) {
            throw new AppException("当前站点无子区域！");
        }
        siteAreaVo.setAreaCode(areaEntity.getAreaCode());
        siteAreaVo.setAreaName(AreaLevelEnum.getByValue(areaEntity.getAreaLevel()).getValue() == AreaLevelEnum.地市州.getValue() ? "市本级" : areaEntity.getName());
        SiteQuery siteQuery = new SiteQuery();
        siteQuery.setAreaCode(areaEntity.getAreaCode());
        if (!ObjectUtils.isEmpty(query.getSiteName())) {
            siteQuery.setSiteName(query.getSiteName());
        }
        List<SiteEntity> siteEntityList = this.find(siteQuery);
        //去除部分敏感信息；

        for (SiteEntity siteEntity : siteEntityList) {
            siteEntity.setLeadingOfficial("****");
            siteEntity.setLeadingOfficialTelephone("****");
            siteEntity.setSiteIp("****");
        }

        siteAreaVo.setSiteList(siteEntityList);
        list.add(siteAreaVo);
        //两层结构{xxx:[{sitename:xxx},{}...],bbb:[{sitename:xxx},{}...]}
        AreaQuery areaQuery = new AreaQuery();
        areaQuery.setPid(areaEntity.getIid());
        List<AreaEntity> childList = areaService.find(areaQuery, context);
        for (AreaEntity child : childList) {
            siteAreaVo = new SiteAreaVo();
            siteAreaVo.setAreaCode(child.getAreaCode());
            siteAreaVo.setAreaName(child.getName());
            String matchCode = child.getAreaCode().replaceAll("(0)+$", "");
            //不足位数，补零  5115 020 120 01  市 区 街道 社区
            matchCode = AreaMatchCodeUtil.getMatchCode(matchCode, child.getAreaLevel());

            siteQuery = new SiteQuery();
            siteQuery.setAreaCode(matchCode + "%");
            if (!ObjectUtils.isEmpty(query.getSiteName())) {
                siteQuery.setSiteName(query.getSiteName());
            }
            List<SiteEntity> siteEntities = this.find(siteQuery);
            if (!ObjectUtils.isEmpty(siteEntities)) {
                siteAreaVo.setSiteList(siteEntities);
                //map.put(child.getName(), siteEntities);
            } else {
                //没有站点
                siteAreaVo.setSiteList(Collections.EMPTY_LIST);
            }
            if (!ObjectUtils.isEmpty(siteAreaVo.getSiteList())) {
                list.add(siteAreaVo);
            }
        }
        return Rest.ok(list);
    }

    /**
     * @param query
     * @param context
     * @return
     */
    @Override
    public Rest<List<SiteEntity>> getAreaSitesByAreaLevel(SiteQuery query, Context context) {
        if (!ObjectUtils.isEmpty(query.getSiteName())) {
            List<SiteEntity> siteEntities = this.find(new SiteQuery().siteName(query.getSiteName()));
            if (!ObjectUtils.isEmpty(siteEntities)) {
                List<Integer> areaLevelList = siteEntities.stream().map(siteEntity -> {
                    AreaEntity areaCache = areaService.getExtCache(siteEntity.getAreaCode());
                    if (!ObjectUtils.isEmpty(areaCache)) {
                        return areaCache.getAreaLevel();
                    }
                    return null;
                }).filter(f -> f != null).collect(Collectors.toList());
                query.setAreaLevelList(ObjectUtils.isEmpty(areaLevelList) ? Arrays.asList(1) : areaLevelList);
            } else {
                return Rest.ok(new ArrayList<>());
            }
        }
        //获取所有层级的区域
        List<AreaEntity> areaEntities = areaService.find(new AreaQuery().areaLevelList(query.getAreaLevelList()));

        //如果针对某个区域查询 过滤到其它区域
        if (!ObjectUtils.isEmpty(query.getSiteName())) {
            List<SiteEntity> siteEntities = this.find(new SiteQuery().siteName(query.getSiteName()));
            if (!ObjectUtils.isEmpty(siteEntities)) {
                Set<String> areaSet = siteEntities.stream().map(siteEntity -> {
                    AreaEntity areaCache = areaService.getExtCache(siteEntity.getAreaCode());
                    if (!ObjectUtils.isEmpty(areaCache)) {
                        return areaCache.getName();
                    }
                    return null;
                }).filter(f -> f != null).collect(Collectors.toSet());
                areaEntities = areaEntities.stream().filter(f -> areaSet.contains(f.getName())).collect(Collectors.toList());
            }
        }

        List<SiteEntity> list = new ArrayList<>();
        for (AreaEntity areaEntity : areaEntities) {
            List<SiteEntity> siteEntityList = this.find(new SiteQuery().areaCode(areaEntity.getAreaCode()));
            if (!ObjectUtils.isEmpty(siteEntityList)) {
                list.addAll(siteEntityList);
            }
        }

        list.stream().forEach(item -> {
            AreaEntity areaEntity = areaService.getCache(item.getAreaCode());
            if (!ObjectUtils.isEmpty(areaEntity)) {
                item.setAreaLevel(areaEntity.getAreaLevel());
            } else {
                item.setAreaLevel(0);
            }
        });

        return Rest.ok(list);
    }

    private boolean getSite(String rootAreaCode, String areaCode, List<SiteEntity> siteList, SiteEntity siteEntity) {
        // AreaEntity areaTemp = areaService.getExtCache(siteEntity.getAreaCode());
        if (rootAreaCode.equals(areaCode)) {
            siteList.add(siteEntity);
            //log.info("add site:{}", siteEntity.getSiteCode());
            return true;
        } else {
            AreaEntity areaTemp = areaService.getExtCache(areaCode);
            if (!ObjectUtils.isEmpty(areaTemp) && !ObjectUtils.isEmpty(areaTemp.getPid())) {
                //根据子节点查找父节点
                AreaEntity areaEntity = areaService.selectOne(new AreaQuery().iid(areaTemp.getPid()));
                if (!ObjectUtils.isEmpty(areaEntity)) {
                    return getSite(rootAreaCode, areaEntity.getAreaCode(), siteList, siteEntity);
                }
            }
            return false;
        }
    }

    @Override
    public void setSiteTree(List<SiteTreeSelect> list, Context context) {
        // siteTreeMap.put(context.getUser().getId(), list);
        //存放到redis中去
        cacheService.hset(USER_SITE_TREE, context.getUser().getId().toString(), JSON.toJSONString(list));
    }

    @Override
    public void removeSiteTree(List<SiteTreeSelect> list, Context context) {
        siteTreeMap.remove(context.getUser().getId());
        cacheService.hdel(USER_SITE_TREE, context.getUser().getId().toString());

    }

    @Override
    public List<SiteEntity> getListAllInfoByQuery(SiteQuery query, Context context) throws AppException {
        List<SiteEntity> list = this.find(query, context);
        if (CollectionUtils.isNotEmpty(list)) {
            list.stream().forEach(item -> {
                ModelQuery modelQuery = new ModelQuery();
                List<String> modelIds = StringUtils.converStr2List(item.getModelIds());
                List<Long> ids = modelIds.stream().map(s -> DataUtil.converStr2Long(s.trim(), 0)).collect(Collectors.toList());
                if (ids.isEmpty()) {
                    ids.add(0l);
                }
                modelQuery.setIdList(ids);
                List<ModelEntity> modelEntityList = modelService.find(modelQuery);
                item.setModelList(modelEntityList);
            });
        }

        return list;
    }


    @Override
    protected void findAfter(SiteEntity params, PageInfo pageInfo, Context context, List<SiteEntity> list) throws AppException {
        list.stream().forEach(site -> changeUrlPath(site));
        super.findAfter(params, pageInfo, context, list);
    }

    @Override
    public void changeUrlPath(SiteEntity siteEntity) {
        if (!ObjectUtils.isEmpty(siteEntity) && !ObjectUtils.isEmpty(siteEntity.getLogoPath())) {
            String domain = GlobalSysInfo.getParamValue(PARAM_SERVER_HTTP_URL, "http://192.168.0.98:11091");
            String fullPath = URLUtil.completeUrl(domain, siteEntity.getLogoPath());
            siteEntity.setLogoFullPath(fullPath);
        }
    }


    @Override
    public Rest<List<MatterEntity>> getMatterAllListByGOV(Map<String, String> params, Integer pageNum, Context context) {
        //分页获取所有事项列表
        List<MatterEntity> allList = new ArrayList<>();
        String url = GlobalSysInfo.getParamValue(Constant.GOV_MATTER_PAGELIST_URL, "http://www.sczwfw.gov.cn/jiq/interface/item/tags");
        for (int i = 1; i <= pageNum; i++) {
            params.put("pageno", String.valueOf(i));
            Rest<List<MatterEntity>> rest = MatterHtmlParseUtil.getMatterList(params, url);
            if (rest.getCode() == YesNoEnum.YES.getValue() && !ObjectUtils.isEmpty(rest.getData())) {
                allList.addAll(rest.getData());
            }
        }
        return Rest.ok(allList);
    }

    private List<MatterEntity> getMatters(HashMap<String, String> params, Context context) {
        String url = GlobalSysInfo.getParamValue(Constant.GOV_MATTER_PAGELIST_URL, "http://www.sczwfw.gov.cn/jiq/interface/item/tags");
        Rest<Map<String, Integer>> restStat = MatterHtmlParseUtil.statSiteMatterCount(params, url);
        if (restStat.getCode() == YesNoEnum.YES.getValue()) {
            Integer pageNum = restStat.getData().getOrDefault("pageNum", 0);
            Integer total = restStat.getData().getOrDefault("total", 0);
            //获取事项全列表
            Rest<List<MatterEntity>> matterAllRest = this.getMatterAllListByGOV(params, pageNum, context);
            if (matterAllRest.getCode() == YesNoEnum.YES.getValue()) {
                return matterAllRest.getData();
            }
        }
        return new ArrayList<MatterEntity>();
    }


    @Override
    public Rest<String> syncMatterBySiteId(SiteEntity siteEntity, Context context) {
        //根据站点中区域编码 查询当前是否已经存在事项
        //根据站点区域编码查询政务网事项列表
        AreaEntity areaEntity = areaService.getCache(siteEntity.getAreaCode());
        //String dxType = AreaLevelDxTypeEnum.getByValue(areaEntity == null ? 2 : areaEntity.getAreaLevel()).getDesc();
        // String dxType="2";
        //String dxType = "3";
        log.info("抓取事项开始");
        List<MatterEntity> govMatterList = new ArrayList<>();
        List<DeptEntity> deptEntities = deptService.find(new DeptQuery().siteId(siteEntity.getId()).source(SourceEnum.政务网.getValue()));
        for (DeptEntity deptEntity : deptEntities) {
            HashMap<String, String> params = new HashMap<>();
            params.put("areaCode", areaEntity.getAreaCode());
            params.put("dxType", "6");
            params.put("deptCode", deptEntity.getDeptNumber());
            params.put("searchtext", "");
            params.put("taskType", "");
            List<MatterEntity> deptMatterList = this.getMatters(params, context);

            if (!ObjectUtils.isEmpty(deptMatterList)) {
                govMatterList.addAll(deptMatterList);
            }
        }
        log.info("抓取事项结束");
        Set<String> matterNoSet = matterService.getDao().getMatterListByAreaCode(new MatterQuery().areaCode(siteEntity.getAreaCode())).parallelStream().map(i -> i.getMatterNo()).collect(Collectors.toSet());

        log.info("计算差集");
        // List<MatterEntity> subList = this.subList(govMatterList, localMatterList);
        List<MatterEntity> subList = this.subListSet(govMatterList, matterNoSet);

        log.info("抓取事项总数：{}需要添加事项数量：{}", govMatterList.size(), subList.size());
        subList = subList.stream().distinct().collect(Collectors.toList());
        log.info("需要添加事项过滤后数量：{}", subList.size());
        //差集进行插入并更新详细数据
        if (!ObjectUtils.isEmpty(subList)) {
            for (MatterEntity matterEntity : subList) {
                DeptEntity deptCache =deptService.selectOne(new DeptQuery().deptNumber(matterEntity.getDeptCode()));
               // DeptEntity deptCache = deptService.getExtCache(matterEntity.getDeptCode());
                matterEntity.setDeptName(deptCache == null ? "" : deptCache.getName());
                matterEntity.setCreateTime(new Date());
                matterService.save(matterEntity, context);
            }
        }
        return Rest.ok("同步事项条数成功！");
    }

    @Override
    public void deleteBysiteIdAndSource(Long siteId, Integer source, Context context) {
        Map<String, Object> condition = new HashMap<>();
        condition.put("siteId", siteId);
        condition.put("source", source);
        this.dao.delete(condition);
    }

    /**
     * @param context
     */
    @Override
    public void updateAllSiteTree(Context context) {
        UserEntity userEntity = new UserEntity();
        userEntity.initAttrValue();
        userEntity.setId(0L);
        Context contextTemp = new Context();
        contextTemp.setUser(userEntity);
        SyncTreeSiteThread syncTreeSiteThread = new SyncTreeSiteThread(contextTemp);
        ThreadPool.getInstance().execute(syncTreeSiteThread);

        userEntity = new UserEntity();
        userEntity.initAttrValue();
        userEntity.setId(1L);
        contextTemp = new Context();
        contextTemp.setUser(userEntity);
        syncTreeSiteThread = new SyncTreeSiteThread(contextTemp);
        ThreadPool.getInstance().execute(syncTreeSiteThread);
    }

    /**
     * 外部添加站点，更新站点相关信息
     *
     * @param siteEntity
     * @param context
     */
    @Override
    public void updateSiteInfo(SiteEntity siteEntity, Context context) {

        List<AppEntity> updateAppList = appService.find(new AppQuery().siteId(siteEntity.getId())).stream().map(item -> {
                   item.setSiteId(siteEntity.getId());
                   item.setSiteName(siteEntity.getSiteName());
            return item;
        }).collect(Collectors.toList());
        if(!ObjectUtils.isEmpty(updateAppList)){
            appService.update(updateAppList,context);
        }

        List<AppCategoryEntity> updateAppCategoryList = appCategoryService.find(new AppCategoryQuery().siteId(siteEntity.getId())).stream().map(item -> {
            item.setSiteId(siteEntity.getId());
            item.setSiteName(siteEntity.getSiteName());
            return item;
        }).collect(Collectors.toList());
        if(!ObjectUtils.isEmpty(updateAppCategoryList)){
            appCategoryService.update(updateAppCategoryList,context);
        }

        List<WindowEntity> updateWindowList = windowService.find(new WindowQuery().siteId(siteEntity.getId())).stream().map(item -> {
            item.setSiteId(siteEntity.getId());
            item.setSiteName(siteEntity.getSiteName());
            return item;
        }).collect(Collectors.toList());
        if(!ObjectUtils.isEmpty(updateWindowList)){
            windowService.update(updateWindowList,context);
        }


    }


    public List<MatterEntity> subList(List<MatterEntity> firstList, List<MatterEntity> secondList) {
        Set<String> secondSet = secondList.parallelStream().map(e -> e.getMatterNo()).collect(Collectors.toSet());
        return firstList.parallelStream().filter(item -> !secondSet.contains(item.getMatterNo())).collect(Collectors.toList());
    }

    public List<MatterEntity> subListSet(List<MatterEntity> firstList, Set<String> secondSet) {
        // Set<String> secondSet = secondList.parallelStream().map(e -> e.getMatterNo()).collect(Collectors.toSet());
        return firstList.parallelStream().filter(item -> !secondSet.contains(item.getMatterNo())).collect(Collectors.toList());
    }


    public static void main(String[] args) {

        String matchCode = "511530000000".replaceAll("(0)+$", "");
        System.out.println(matchCode);

        matchCode = StrUtil.padAfter(matchCode, 7, "0");

        System.out.println(matchCode);

    }


}