package com.mortals.xhx.module.site.service.impl;

import com.mortals.framework.ap.GlobalSysInfo;
import com.mortals.framework.common.Rest;
import com.mortals.xhx.ManagerBaseApplication;
import com.mortals.xhx.common.code.SourceEnum;
import com.mortals.xhx.common.code.YesNoEnum;
import com.mortals.xhx.common.key.Constant;
import com.mortals.xhx.common.utils.BeanUtil;
import com.mortals.xhx.common.utils.MatterTypeHtmlParseUtil;
import com.mortals.xhx.module.matter.model.MatterEntity;
import com.mortals.xhx.module.site.model.*;
import com.mortals.xhx.module.site.service.SiteMatterService;
import com.mortals.xhx.module.site.service.SiteService;
import com.mortals.xhx.module.site.service.SiteThemeService;
import lombok.extern.slf4j.Slf4j;
import org.checkerframework.checker.units.qual.A;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import com.mortals.framework.service.impl.AbstractCRUDServiceImpl;
import com.mortals.framework.exception.AppException;
import com.mortals.framework.model.Context;
import com.mortals.xhx.module.site.dao.SiteThemeMatterDao;
import com.mortals.xhx.module.site.service.SiteThemeMatterService;
import org.springframework.util.ObjectUtils;

import java.util.*;
import java.util.stream.Collectors;

/**
 * SiteThemeMatterService
 * 站点主题事项 service实现
 *
 * @author zxfei
 * @date 2022-11-23
 */
@Service("siteThemeMatterService")
@Slf4j
public class SiteThemeMatterServiceImpl extends AbstractCRUDServiceImpl<SiteThemeMatterDao, SiteThemeMatterEntity, Long> implements SiteThemeMatterService {

    @Autowired
    private SiteThemeService siteThemeService;
    @Autowired
    private SiteMatterService siteMatterService;
    @Autowired
    private SiteService siteService;


    @Override
    public Rest<String> syncThemeMatterBySiteId(Long siteId, String userType, Context context) {
        String url = GlobalSysInfo.getParamValue(Constant.GOV_MATTER_PAGELIST_URL, "http://www.sczwfw.gov.cn/jiq/interface/item/tags");
        SiteEntity siteEntity = siteService.get(siteId, context);
        if (ObjectUtils.isEmpty(siteEntity)) {
            return Rest.fail("站点不存在！");
        }
        //查询主题列表 分个人服务和法人服务
        HashMap<String, String> params = new HashMap<>();
        params.put("dxType", userType);
        params.put("areaCode", siteEntity.getAreaCode());
        params.put("deptCode", "");
        params.put("searchtext", "");
        params.put("taskType", "");
        params.put("type", "1");
        params.put("theme", "");

        //获取主题数量
        Rest<Map<String, Integer>> themeMatterRest = MatterTypeHtmlParseUtil.getThemeMatterPage(params, url);
        Integer totalTheme = themeMatterRest.getData().get("total");
        Integer sum = 0;

        //个人服务或法人服务
        List<SiteThemeEntity> siteThemeEntities = siteThemeService.find(new SiteThemeQuery().siteId(siteId).userType(userType));
        for (SiteThemeEntity item : siteThemeEntities) {
            params.put("theme", item.getThemeCode());
            List<MatterEntity> allList = new ArrayList<>();
            Rest<Map<String, Integer>> themeMatterPageRest = MatterTypeHtmlParseUtil.getThemeMatterPage(params, url);
            Integer pageNum = themeMatterPageRest.getData().get("pageNum");
            Integer total = themeMatterPageRest.getData().get("total");
            if (pageNum > 0 && total > 0) {
                int count = 0;
                for (int i = 1; i <= pageNum; i++) {
                    params.put("pageno", String.valueOf(i));
                    Rest<List<MatterEntity>> restList = null;
                    if ("2".equals(userType)) {
                        restList = MatterTypeHtmlParseUtil.getGrThemeMatterList(params, url);
                    } else if ("3".equals(userType)) {
                        restList = MatterTypeHtmlParseUtil.getFrThemeMatterList(params, url);
                    }

                    if (!ObjectUtils.isEmpty(restList) && restList.getCode() == YesNoEnum.YES.getValue()) {
                        allList.addAll(restList.getData());
                        count += restList.getData().size();
                    }
                }
                if (count != total) {
                    log.info(String.format("服务获取数量异常,count:%s,total:%s,theme:%s", count, total, item.getThemeName()));
                }
            }
            sum += allList.size();
            if (!ObjectUtils.isEmpty(allList)) {
                //当前主题列表 查找站点事项id列表，保存站点主题事项
                List<String> matterCodeList = allList.stream().map(m -> m.getMatterNo()).collect(Collectors.toList());
                SiteMatterQuery siteMatterQuery = new SiteMatterQuery();
                siteMatterQuery.setSiteId(siteId);
                siteMatterQuery.setMatterCodeList(matterCodeList);
                List<SiteMatterEntity> siteMatterEntities = siteMatterService.find(siteMatterQuery, context);
                log.info("siteId:{},抓取数量:{},本地数量:{}", siteId, allList.size(), siteMatterEntities.size());
                if (!ObjectUtils.isEmpty(siteMatterEntities) ) {
                    List<SiteThemeMatterEntity> collect = siteMatterEntities.stream().map(siteMatterEntity -> {
                        SiteThemeMatterEntity siteThemeMatterEntity = new SiteThemeMatterEntity();
                        siteThemeMatterEntity.initAttrValue();
                        BeanUtils.copyProperties(siteMatterEntity, siteThemeMatterEntity, BeanUtil.getNullPropertyNames(siteMatterEntity));
                        siteThemeMatterEntity.setThemeCode(item.getThemeCode());
                        siteThemeMatterEntity.setThemeName(item.getThemeName());
                        siteThemeMatterEntity.setUserType(item.getUserType());
                        siteThemeMatterEntity.setSiteId(siteEntity.getId());
                        siteThemeMatterEntity.setSiteName(siteEntity.getSiteName());
                        siteThemeMatterEntity.setCreateTime(new Date());
                        return siteThemeMatterEntity;
                    }).collect(Collectors.toList());
                    if (!ObjectUtils.isEmpty(collect)) {
                        log.info("保存数量：" + collect.size());
                        this.save(collect, context);
                    }
                }
            }
        }

        if (sum != totalTheme) {
            log.info(String.format("服务获取主题事项重数量异常,sum:%s,totalTheme:%s,userType:%s", sum, totalTheme, userType));
        }

        return Rest.ok("主题更新完成");
    }

    @Override
    public Rest<String> syncTownThemeMatterBySiteId(SiteEntity siteEntity, String dxType, Context context) {
        String url = GlobalSysInfo.getParamValue(Constant.GOV_MATTER_PAGELIST_URL, "http://www.sczwfw.gov.cn/jiq/interface/item/tags");

        HashMap<String, String> params = new HashMap<>();
        params.put("dxType", dxType);
        params.put("areaCode", siteEntity.getAreaCode());
        List<SiteThemeEntity> siteThemeEntities = siteThemeService.find(new SiteThemeQuery().siteId(siteEntity.getId()));
        int sum = 0;
        for (SiteThemeEntity item : siteThemeEntities) {
            params.put("theme", item.getThemeCode());
            params.put("userType", item.getUserType());
            List<MatterEntity> allList = new ArrayList<>();

            Rest<Map<String, Integer>> themeMatterPageRest = MatterTypeHtmlParseUtil.getTownThemeMatterPage(params, url);
            Integer pageNum = themeMatterPageRest.getData().get("pageNum");
            Integer total = themeMatterPageRest.getData().get("total");

            if (pageNum > 0 && total > 0) {
                int count = 0;
                for (int i = 1; i <= pageNum; i++) {
                    params.put("pageno", String.valueOf(i));
                    Rest<List<MatterEntity>> restList = MatterTypeHtmlParseUtil.getTownThemeMatterList(params, url);
                    if (restList.getCode() == YesNoEnum.YES.getValue()) {
                        allList.addAll(restList.getData());
                        count += restList.getData().size();
                    }
                }
                if (count != total) {
                    log.info(String.format("获取数量异常,count:%s,total:%s,theme:%s", count, total, item.getThemeName()));
                }
                sum += count;
            }


            if (!ObjectUtils.isEmpty(allList)) {
                //当前主题列表 查找站点事项id列表，保存站点主题事项
                List<String> matterCodeList = allList.stream().map(m -> m.getMatterNo()).collect(Collectors.toList());
                SiteMatterQuery siteMatterQuery = new SiteMatterQuery();
                siteMatterQuery.setSiteId(siteEntity.getId());
                siteMatterQuery.setMatterCodeList(matterCodeList);
                List<SiteMatterEntity> siteMatterEntities = siteMatterService.find(siteMatterQuery, context);
                log.info(String.format("size1:%d,size2:%d", allList.size(), siteMatterEntities.size()));
                if (!ObjectUtils.isEmpty(siteMatterEntities)) {
                    List<SiteThemeMatterEntity> collect = siteMatterEntities.stream().map(siteMatterEntity -> {
                        SiteThemeMatterEntity siteThemeMatterEntity = new SiteThemeMatterEntity();
                        siteThemeMatterEntity.initAttrValue();
                        BeanUtils.copyProperties(siteMatterEntity, siteThemeMatterEntity, BeanUtil.getNullPropertyNames(siteMatterEntity));
                        siteThemeMatterEntity.setThemeCode(item.getThemeCode());
                        siteThemeMatterEntity.setThemeName(item.getThemeName());
                        siteThemeMatterEntity.setUserType(item.getUserType());
                        return siteThemeMatterEntity;
                    }).collect(Collectors.toList());
                    if (!ObjectUtils.isEmpty(collect)) {
                        this.save(collect, context);
                    }
                }
            }
        }
        return Rest.ok("乡镇主题更新完成", "更新条数:" + sum);
    }

    @Override
    public void deleteGovBySiteId(Long siteId, Context context) {
        Map<String, Object> condition = new HashMap<>();
        condition.put("siteId", siteId);
        condition.put("source", SourceEnum.政务网.getValue());
        this.dao.delete(condition);
    }
}