package com.mortals.xhx.module.site.web;

import com.alibaba.fastjson.JSONObject;
import com.mortals.framework.annotation.SlideWindowLimit;
import com.mortals.framework.annotation.UnAuth;
import com.mortals.framework.common.Rest;
import com.mortals.framework.exception.AppException;
import com.mortals.framework.model.Context;
import com.mortals.framework.model.PageInfo;
import com.mortals.framework.model.Result;
import com.mortals.framework.service.ICacheService;
import com.mortals.framework.util.StringUtils;
import com.mortals.framework.util.ThreadPool;
import com.mortals.framework.web.BaseCRUDJsonBodyMappingController;
import com.mortals.xhx.base.system.param.service.ParamService;
import com.mortals.xhx.base.system.user.model.UserEntity;
import com.mortals.xhx.common.code.AreaLevelEnum;
import com.mortals.xhx.common.code.YesNoEnum;
import com.mortals.xhx.common.key.Constant;
import com.mortals.xhx.common.keys.RedisCacheKeys;
import com.mortals.xhx.common.pdu.RespData;
import com.mortals.xhx.common.pdu.user.UserPdu;
import com.mortals.xhx.common.utils.SyncDeptThread;
import com.mortals.xhx.common.utils.SyncGovMatterDetailThread;
import com.mortals.xhx.common.utils.SyncTreeSiteThread;
import com.mortals.xhx.feign.user.IUserFeign;
import com.mortals.xhx.module.area.model.AreaEntity;
import com.mortals.xhx.module.area.model.AreaQuery;
import com.mortals.xhx.module.area.service.AreaService;
import com.mortals.xhx.module.dept.service.DeptService;
import com.mortals.xhx.module.matter.service.MatterService;
import com.mortals.xhx.module.model.model.ModelEntity;
import com.mortals.xhx.module.model.model.ModelQuery;
import com.mortals.xhx.module.model.service.ModelService;
import com.mortals.xhx.module.site.model.SiteEntity;
import com.mortals.xhx.module.site.model.SiteQuery;
import com.mortals.xhx.module.site.model.SiteTreeSelect;
import com.mortals.xhx.module.site.model.vo.SiteAreaVo;
import com.mortals.xhx.module.site.service.SiteMatterService;
import com.mortals.xhx.module.site.service.SiteService;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.collections4.CollectionUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.annotation.Lazy;
import org.springframework.util.ObjectUtils;
import org.springframework.web.bind.annotation.*;

import java.util.*;
import java.util.stream.Collectors;

/**
 * 站点
 *
 * @author zxfei
 * @date 2022-01-20
 */
@RestController
@RequestMapping("site")
@Slf4j

public class SiteController extends BaseCRUDJsonBodyMappingController<SiteService, SiteEntity, Long> {

    @Autowired
    private ParamService paramService;
    @Autowired
    private ModelService modelService;
    @Autowired
    private ICacheService cacheService;
    @Autowired
    private SiteService siteService;
    @Autowired
    private AreaService areaService;
    @Autowired
    @Lazy
    private IUserFeign userFeign;

    public SiteController() {
        super.setModuleDesc("站点");
    }

    @Override
    protected void init(Map<String, Object> model, Context context) {
        this.addDict(model, "haveSonArea", paramService.getParamBySecondOrganize("Site", "haveSonArea"));
        this.addDict(model, "workday1", paramService.getParamBySecondOrganize("Site", "workday1"));
        this.addDict(model, "workday2", paramService.getParamBySecondOrganize("Site", "workday2"));
        this.addDict(model, "workday3", paramService.getParamBySecondOrganize("Site", "workday3"));
        this.addDict(model, "workday4", paramService.getParamBySecondOrganize("Site", "workday4"));
        this.addDict(model, "workday5", paramService.getParamBySecondOrganize("Site", "workday5"));
        this.addDict(model, "workday6", paramService.getParamBySecondOrganize("Site", "workday6"));
        this.addDict(model, "workday7", paramService.getParamBySecondOrganize("Site", "workday7"));
        this.addDict(model, "level", paramService.getParamBySecondOrganize("Site", "level"));
        this.addDict(model, "building", paramService.getParamBySecondOrganize("Site", "building"));
        this.addDict(model, "isSite", paramService.getParamBySecondOrganize("Site", "isSite"));
        this.addDict(model, "status", paramService.getParamBySecondOrganize("Site", "status"));
        this.addDict(model, "modelIds", modelService.find(new ModelQuery()).stream().collect(Collectors.toMap(x -> x.getId().toString(), y -> y.getModelName())));
        super.init(model, context);
    }

    /**
     * 构建站点树
     */
    @GetMapping(value = "siteTree")
    public String siteTree() {
        JSONObject jsonObject = new JSONObject();
        Map<String, Object> model = new HashMap<>();
        String busiDesc = this.getModuleDesc() + "构建站点树";
        Context context = getContext();
        try {
            List<SiteTreeSelect> siteTree = this.service.getSiteTree(context);
            if (ObjectUtils.isEmpty(siteTree)) {
                log.info("为初始化站点树，重新构建！");
                siteTree = this.service.siteTree(context);
                siteService.setSiteTree(siteTree, context);
            }
            model.put("siteTree", siteTree);
            this.init(model, context);
            if (!ObjectUtils.isEmpty(context) && !ObjectUtils.isEmpty(context.getUser())) {
                recordSysLog(request, busiDesc + " 【成功】");
            }
            jsonObject.put(KEY_RESULT_DATA, model);
            jsonObject.put(KEY_RESULT_CODE, VALUE_RESULT_SUCCESS);
        } catch (Exception e) {
            log.error("构建站点树异常", e);
            jsonObject.put(KEY_RESULT_CODE, VALUE_RESULT_FAILURE);
            jsonObject.put(KEY_RESULT_MSG, super.convertException(e));
        }
        return jsonObject.toJSONString();
    }


    /**
     * 获取全站点树
     */
    @GetMapping(value = "getAllSiteTree")
    @UnAuth
    public String getAllSiteTree() {
        JSONObject jsonObject = new JSONObject();
        Map<String, Object> model = new HashMap<>();
        String busiDesc = this.getModuleDesc() + "构建站点树";
        try {
            Context context = new Context();
            UserEntity userEntity = new UserEntity();
            userEntity.setId(0L);
            context.setUser(userEntity);
            List<SiteTreeSelect> siteTree = this.service.getSiteTree(context);
            model.put("siteTree", siteTree);
            this.init(model, getContext());
            if (!ObjectUtils.isEmpty(getContext()) && !ObjectUtils.isEmpty(getContext().getUser())) {
                recordSysLog(request, busiDesc + " 【成功】");
            }
            jsonObject.put(KEY_RESULT_DATA, model);
            jsonObject.put(KEY_RESULT_CODE, VALUE_RESULT_SUCCESS);
        } catch (Exception e) {
            log.error("构建全站点树异常", e);
            jsonObject.put(KEY_RESULT_CODE, VALUE_RESULT_FAILURE);
            jsonObject.put(KEY_RESULT_MSG, super.convertException(e));
        }
        return jsonObject.toJSONString();
    }

    /**
     * 根据区域编码查询站点列表
     */
    @PostMapping(value = "getFlatSitesByAreaCode")
    @UnAuth
    public String getFlatSitesByAreaCode(@RequestBody SiteQuery site) {
        JSONObject jsonObject = new JSONObject();
        String busiDesc = "根据区域编码查询站点列表" + this.getModuleDesc();
        try {
            if (!ObjectUtils.isEmpty(site.getSiteName())) {
                List<SiteEntity> siteEntities = this.service.find(new SiteQuery().siteName(site.getSiteName()));

                if (!ObjectUtils.isEmpty(siteEntities)) {
                    site.setAreaCodeList(siteEntities.stream().map(i -> i.getAreaCode()).collect(Collectors.toList()));
                }
            }
            if (!ObjectUtils.isEmpty(site.getAreaName())) {
                List<AreaEntity> areaEntities = areaService.find(new AreaQuery().name(site.getAreaName()));
                if (!ObjectUtils.isEmpty(areaEntities)) {
                    site.setAreaCodeList(areaEntities.stream().map(i -> i.getAreaCode()).collect(Collectors.toList()));
                }
            }
            if (ObjectUtils.isEmpty(site.getAreaCodeList())) {
                site.setAreaCodeList(Arrays.asList(site.getAreaCode()));
            }
            List<SiteEntity> siteEntityList = site.getAreaCodeList().stream().flatMap(areaCode -> {
                return this.service.getFlatSitesByAreaCode(areaCode, getContext()).stream();
            }).distinct().collect(Collectors.toList());
            //List<SiteEntity> siteEntityList = this.service.getFlatSitesByAreaCode(site.getAreaCode(), getContext());

/*            if (!ObjectUtils.isEmpty(site.getSiteName())) {
                siteEntityList = siteEntityList.stream().filter(item -> site.getSiteName().equals(item.getSiteName())).collect(Collectors.toList());
            }*/
            if (!ObjectUtils.isEmpty(getContext()) && !ObjectUtils.isEmpty(getContext().getUser())) {
                recordSysLog(request, busiDesc + " 【成功】");
            }
            jsonObject.put(KEY_RESULT_DATA, siteEntityList);
            jsonObject.put(KEY_RESULT_CODE, VALUE_RESULT_SUCCESS);
            jsonObject.put(KEY_RESULT_MSG, "查询站点列表成功！");
        } catch (Exception e) {
            log.error("获取异常", e);
            jsonObject.put(KEY_RESULT_CODE, VALUE_RESULT_FAILURE);
            jsonObject.put(KEY_RESULT_MSG, super.convertException(e));
        }
        return jsonObject.toJSONString();
    }


    /**
     * 根据区域id查询站点列表ids
     */
    @PostMapping(value = "getFlatSitesByAreaIds")
    @UnAuth
    public String getFlatSitesByAreaIds(@RequestBody SiteQuery site) {
        JSONObject jsonObject = new JSONObject();
        String busiDesc = "根据区域编码查询站点列表" + this.getModuleDesc();
        try {
            String siteIds = site.getAreaCodeList().stream().flatMap(areaId ->
                    this.service.getFlatSitesByAreaCode(areaId, getContext()).stream()
            ).map(item -> item.getId().toString()).distinct().collect(Collectors.joining(","));
            if (!ObjectUtils.isEmpty(getContext()) && !ObjectUtils.isEmpty(getContext().getUser())) {
                recordSysLog(request, busiDesc + " 【成功】");
            }
            jsonObject.put(KEY_RESULT_DATA, siteIds);
            jsonObject.put(KEY_RESULT_CODE, VALUE_RESULT_SUCCESS);
            jsonObject.put(KEY_RESULT_MSG, "查询站点列表成功！");
        } catch (Exception e) {
            log.error("获取异常", e);
            jsonObject.put(KEY_RESULT_CODE, VALUE_RESULT_FAILURE);
            jsonObject.put(KEY_RESULT_MSG, super.convertException(e));
        }
        return jsonObject.toJSONString();
    }

    /**
     * 根据站点id查询站点列表子集，包含当前站点
     */
    @PostMapping(value = "getFlatSitesBySiteId")
    @UnAuth
    public String getFlatSitesBySiteId(@RequestBody SiteQuery site) {
        JSONObject jsonObject = new JSONObject();
        Map<String, Object> model = new HashMap<>();
        String busiDesc = "根据站点id查询站点列表子集" + this.getModuleDesc();
        try {
            List<SiteEntity> siteEntityList = this.service.getFlatSitesBySiteId(site.getId(), getContext());
            recordSysLog(request, busiDesc + " 【成功】");
            jsonObject.put(KEY_RESULT_DATA, siteEntityList);
            jsonObject.put(KEY_RESULT_CODE, VALUE_RESULT_SUCCESS);
            jsonObject.put(KEY_RESULT_MSG, busiDesc + "成功！");
        } catch (Exception e) {
            log.error("获取异常", e);
            jsonObject.put(KEY_RESULT_CODE, VALUE_RESULT_FAILURE);
            jsonObject.put(KEY_RESULT_MSG, super.convertException(e));
        }
        return jsonObject.toJSONString();
    }

    /**
     * 根据区域编码查询站点分组列表
     */
    @PostMapping(value = "getFlatSitesGroupByAreaCode")
    @UnAuth
    public String getFlatSitesGroupByAreaCode(@RequestBody SiteQuery site) {
        JSONObject jsonObject = new JSONObject();
        String busiDesc = "根据区域编码查询站点列表" + this.getModuleDesc();
        List<SiteEntity> group = new ArrayList<>();
        Map<String, List<SiteEntity>> collect = new HashMap<>();
        try {
            if (ObjectUtils.isEmpty(site.getAreaName()) && ObjectUtils.isEmpty(site.getAreaCode())) {
                List<SiteEntity> siteEntities = this.service.find(new SiteQuery());
                collect = siteEntities.stream().collect(Collectors.groupingBy(SiteEntity::getAreaCode));
            } else {
                if (!ObjectUtils.isEmpty(site.getSiteName())) {
                    List<SiteEntity> siteEntities = this.service.find(new SiteQuery().siteName(site.getSiteName()));

                    if (!ObjectUtils.isEmpty(siteEntities)) {
                        site.setAreaCodeList(siteEntities.stream().map(i -> i.getAreaCode()).collect(Collectors.toList()));
                    }
                }
                if (!ObjectUtils.isEmpty(site.getAreaName())) {
                    List<AreaEntity> areaEntities = areaService.find(new AreaQuery().name(site.getAreaName()));
                    if (!ObjectUtils.isEmpty(areaEntities)) {
                        site.setAreaCodeList(areaEntities.stream().map(i -> i.getAreaCode()).collect(Collectors.toList()));
                    }
                }
                if (ObjectUtils.isEmpty(site.getAreaCodeList())) {
                    site.setAreaCodeList(Arrays.asList(site.getAreaCode()));
                }
                List<SiteEntity> siteEntityList = site.getAreaCodeList().stream()
                        .flatMap(areaCode -> this.service.getFlatSitesByAreaCode(areaCode, getContext()).stream())
                        .distinct()
                        .collect(Collectors.toList());
                collect = siteEntityList.stream().collect(Collectors.groupingBy(x -> x.getAreaCode()));
            }

            group = collect.entrySet().stream().map(item -> {
                AreaEntity areaEntity = areaService.getCache(item.getKey());
                SiteEntity siteEntity = new SiteEntity();
                siteEntity.setAreaName(areaEntity.getName());
                siteEntity.setAreaCode(areaEntity.getAreaCode());
                siteEntity.setSubList(item.getValue());
                siteEntity.setTotal(item.getValue().size());
                return siteEntity;
            }).sorted(Comparator.comparing(SiteEntity::getAreaCode)).collect(Collectors.toList());

            jsonObject.put(KEY_RESULT_DATA, group);
            jsonObject.put(KEY_RESULT_CODE, VALUE_RESULT_SUCCESS);
            jsonObject.put(KEY_RESULT_MSG, "查询站点分组列表成功！");

            if (!ObjectUtils.isEmpty(getContext()) && !ObjectUtils.isEmpty(getContext().getUser())) {
                recordSysLog(request, busiDesc + " 【成功】");
            }
        } catch (Exception e) {
            log.error("获取异常", e);
            jsonObject.put(KEY_RESULT_CODE, VALUE_RESULT_FAILURE);
            jsonObject.put(KEY_RESULT_MSG, super.convertException(e));
        }
        return jsonObject.toJSONString();
    }


    /**
     * 根据区域获取子站点数量
     */
    @PostMapping(value = "countSitesByArea")
    @UnAuth
    public Rest<Integer> countSitesByArea(@RequestBody SiteQuery site) {
        String busiDesc = "根据站点id查询站点列表子集" + this.getModuleDesc();
        try {
            Integer count = this.service.countSitesByArea(site, getContext());
            recordSysLog(request, busiDesc + " 【成功】");
            return Rest.ok(count);
        } catch (Exception e) {
            log.error("获取异常", e);
            return Rest.fail(e.getMessage());
        }
    }

    /**
     * 根据站点id查询站点列表子集，包含当前站点,二层结构
     * {rootname:}
     */
    @PostMapping(value = "getAreaSitesBySite")
    @UnAuth
    public String getAreaSitesBySite(@RequestBody SiteQuery site) {
        JSONObject jsonObject = new JSONObject();
        String busiDesc = "根据站点查询区域站点列表子集" + this.getModuleDesc();
        try {
            Rest<List<SiteAreaVo>> rest = this.service.getAreaSitesBySite(site, getContext());
            recordSysLog(request, busiDesc + " 【成功】");
            jsonObject.put(KEY_RESULT_DATA, rest.getData());
            jsonObject.put(KEY_RESULT_CODE, VALUE_RESULT_SUCCESS);
            jsonObject.put(KEY_RESULT_MSG, busiDesc + "成功！");
        } catch (Exception e) {
            log.error("获取异常", e);
            jsonObject.put(KEY_RESULT_CODE, VALUE_RESULT_FAILURE);
            jsonObject.put(KEY_RESULT_MSG, super.convertException(e));
        }
        return jsonObject.toJSONString();
    }


    /**
     * 根据区域等级查询当前区域所有站点
     */
    @PostMapping(value = "getAreaSitesByAreaLevel")
    @UnAuth
    public String getAreaSitesByAreaLevel(@RequestBody SiteQuery site) {
        JSONObject jsonObject = new JSONObject();
        String busiDesc = "根据区域等级查询当前区域所有站点" + this.getModuleDesc();
        try {
            Rest<List<SiteEntity>> rest = this.service.getAreaSitesByAreaLevel(site, getContext());
            recordSysLog(request, busiDesc + " 【成功】");
            jsonObject.put(KEY_RESULT_DATA, rest.getData());
            jsonObject.put(KEY_RESULT_CODE, VALUE_RESULT_SUCCESS);
            jsonObject.put(KEY_RESULT_MSG, busiDesc + "成功！");
        } catch (Exception e) {
            log.error("获取异常", e);
            jsonObject.put(KEY_RESULT_CODE, VALUE_RESULT_FAILURE);
            jsonObject.put(KEY_RESULT_MSG, super.convertException(e));
        }
        return jsonObject.toJSONString();
    }


    /**
     * 根据区域等级查询当前区域所有站点
     */
    @PostMapping(value = "getSitesGroupByAreaLevel")
    @UnAuth
    public String getSitesGroupByAreaLevel(@RequestBody SiteQuery site) {
        JSONObject jsonObject = new JSONObject();
        String busiDesc = "根据区域等级查询当前区域所有站点" + this.getModuleDesc();
        Map<Integer, List<SiteEntity>> collect = new HashMap<>();
        List<SiteEntity> groupList = new ArrayList<>();

        try {
            if (ObjectUtils.isEmpty(site.getAreaLevel())) {
                //全部
                for (int i = 1; i <= 5; i++) {
                    site.setAreaLevel(i);
                    Rest<List<SiteEntity>> rest = this.service.getAreaSitesByAreaLevel(site, getContext());
                    // String areaName = AreaLevelEnum.getByValue(i).getDesc();
                    collect.put(i, rest.getData());
                }
            } else {
                Rest<List<SiteEntity>> rest = this.service.getAreaSitesByAreaLevel(site, getContext());
                if (YesNoEnum.YES.getValue() == rest.getCode()) {
                    collect = rest.getData().stream().collect(Collectors.groupingBy(x -> x.getAreaLevel()));
                    jsonObject.put(KEY_RESULT_DATA, collect);
                }
            }

            //排序统计
            for (int i = 1; i <= 5; i++) {
                List<SiteEntity> siteEntities = collect.get(i);
                SiteEntity siteEntity = new SiteEntity();
                siteEntity.setAreaName(AreaLevelEnum.getByValue(i).getDesc());
                if (!ObjectUtils.isEmpty(siteEntities)) {
                    siteEntity.setTotal(siteEntities.size());
                    siteEntity.setSubList(siteEntities);
                }
                groupList.add(siteEntity);
            }
            recordSysLog(request, busiDesc + " 【成功】");
            jsonObject.put(KEY_RESULT_DATA, groupList);
            jsonObject.put(KEY_RESULT_CODE, VALUE_RESULT_SUCCESS);
            jsonObject.put(KEY_RESULT_MSG, busiDesc + "成功！");
        } catch (Exception e) {
            log.error("获取异常", e);
            jsonObject.put(KEY_RESULT_CODE, VALUE_RESULT_FAILURE);
            jsonObject.put(KEY_RESULT_MSG, super.convertException(e));
        }
        return jsonObject.toJSONString();
    }


    @Override
    protected int infoAfter(Long id, Map<String, Object> model, SiteEntity entity, Context context) throws AppException {
        this.service.changeUrlPath(entity);
        return super.infoAfter(id, model, entity, context);
    }

    /**
     * 条件查询，无分页信息
     *
     * @param query
     * @return
     */
    @PostMapping({"list/noPage"})
    @UnAuth
    public Rest<Object> getList(@RequestBody SiteQuery query) {
        Rest<Object> ret = new Rest();
        Map<String, Object> model = new HashMap();
        Context context = this.getContext();
        String busiDesc = "查询" + this.getModuleDesc();
        int code = 1;
        try {
            if (CollectionUtils.isNotEmpty(query.getAreaCodeList())) {
                List<SiteEntity> siteEntityList = query.getAreaCodeList().parallelStream().flatMap(areaId ->
                        this.service.getFlatSitesByAreaCode(areaId, getContext()).stream()
                ).distinct().collect(Collectors.toList());
                model.put("data", siteEntityList);
            } else {
                List<SiteEntity> result = this.getService().find(query, context);
                model.put("data", result);
            }
            model.put("message_info", busiDesc + "成功");
            this.recordSysLog(this.request, busiDesc + " 【成功】");
        } catch (Exception var9) {
            code = -1;
            this.doException(this.request, busiDesc, model, var9);
        }
        //this.init(model, context);
        ret.setCode(code);
        ret.setData(model);
        //ret.setDict(model.remove("dict"));
        ret.setMsg(model.get("message_info") == null ? "" : model.remove("message_info").toString());
        return ret;
    }

    /**
     * 包含站点下面的模块以及模块下面的数据统计
     *
     * @param query
     * @return
     */
    @PostMapping({"list/allInfo"})
    @UnAuth
    public Rest<Object> allListInfo(@RequestBody SiteQuery query) {
        Rest<Object> ret = new Rest();
        Map<String, Object> model = new HashMap();
        Context context = this.getContext();
        String busiDesc = "查询" + this.getModuleDesc();

        int code = 1;
        try {
            List<SiteEntity> result = this.getService().getListAllInfoByQuery(query, context);
            model.put("data", result);
            model.put("message_info", busiDesc + "成功");
            this.recordSysLog(this.request, busiDesc + " 【成功】");
        } catch (Exception var9) {
            code = -1;
            this.doException(this.request, busiDesc, model, var9);
        }
        this.init(model, context);
        ret.setCode(code);
        ret.setData(model);
        ret.setDict(model.remove("dict"));
        ret.setMsg(model.get("message_info") == null ? "" : model.remove("message_info").toString());
        return ret;
    }


    /**
     * 同步站点事项数据
     */
    @PostMapping(value = "syncGovMatterBySiteId")
    @UnAuth
    public String syncGovMatterBySiteId(@RequestBody SiteQuery site) {
        JSONObject jsonObject = new JSONObject();
        String busiDesc = "同步站点事项数据" + this.getModuleDesc();
        try {
            SiteEntity siteEntity = this.service.get(site.getId(), getContext());
            //启动
            if (ObjectUtils.isEmpty(siteEntity)) {
                throw new AppException("当前站点为空！");
            }
            boolean bool = cacheService.setnx(RedisCacheKeys.getSyncMatterLockKey() + siteEntity.getAreaCode(), siteEntity.getAreaCode(), 60L);
            if (!bool) {
                throw new AppException("当前正在同步事项数据中，请勿重复提交！");
            }
            ThreadPool.getInstance().execute(new SyncGovMatterDetailThread(siteEntity, getContext()));
            recordSysLog(request, busiDesc + " 【成功】");
            jsonObject.put(KEY_RESULT_CODE, VALUE_RESULT_SUCCESS);
            jsonObject.put(KEY_RESULT_MSG, "同步站点事项数据命令下发成功！");
        } catch (Exception e) {
            log.error("获取异常", e);
            jsonObject.put(KEY_RESULT_CODE, VALUE_RESULT_FAILURE);
            jsonObject.put(KEY_RESULT_MSG, super.convertException(e));
        }
        return jsonObject.toJSONString();
    }


    /**
     * 外部站点添加
     */
    @PostMapping(value = "api/add")
    @UnAuth
    public String apiAdd(@RequestBody SiteEntity siteEntity) {
        JSONObject jsonObject = new JSONObject();
        String busiDesc = "外部站点添加" + this.getModuleDesc();
        try {
            String areaCode = siteEntity.getAreaCode();
            String siteName = siteEntity.getSiteName();
            if (ObjectUtils.isEmpty(areaCode)) {
                throw new AppException("区域编码不能为空！");
            }
            SiteEntity one = this.service.selectOne(new SiteQuery().areaCode(areaCode).siteName(siteName), getContext());
            if (!ObjectUtils.isEmpty(one)) throw new AppException("当前区域下站点名称已存在！");

            AreaEntity areaEntity = areaService.selectOne(new AreaQuery().areaCode(areaCode), getContext());
            siteEntity.initAttrValue();
            //查询是否有站点了，如果有站点，这强制更新为站点 id=1的数据
            SiteEntity site = this.service.get(1L);

            if (!ObjectUtils.isEmpty(site)) {
                siteEntity.setId(site.getId());
            }

            siteEntity.setAreaCode(areaCode);
            siteEntity.setSiteName(siteName);
            if (!ObjectUtils.isEmpty(areaEntity)) {
                siteEntity.setAreaID(areaEntity.getIid());
                siteEntity.setAreaName(areaEntity.getName());
                siteEntity.setAreaLevel(areaEntity.getAreaLevel());
            }

            if (!ObjectUtils.isEmpty(siteEntity.getId())) {
                List<SiteEntity> siteEntities = this.service.find(new SiteQuery().areaCode(siteEntity.getAreaCode()));
                siteEntity.setSiteCode(siteEntity.getAreaCode() + Constant.SITE_SPLIT_MARK + StringUtils.lpad(siteEntities.size() + 1, Constant.SITE_CODE_NUM));
                this.service.getDao().update(siteEntity);

                //todo 更新用户站点树

/*                Rest<String> rest = userFeign.synchSiteAuth();
                if (rest.getCode().equals(YesNoEnum.YES.getValue())) {
                    UserPdu userPdu = new UserPdu();
                    userPdu.setPage(1);
                    userPdu.setSize(-1);
                    Rest<RespData<List<UserPdu>>> list = userFeign.list(userPdu);
                    this.service.updateUserList(list.getData().getData());

               // ThreadPool.getInstance().execute(new SyncTreeSiteThread(this,context));

                this.service.updateAllSiteTree(null);*/
            } else {
                this.service.save(siteEntity, getContext());
            }

            //todo 更新相关表中的站点名称，站点编码，站点ID
            this.service.updateSiteInfo(siteEntity, getContext());

            recordSysLog(request, busiDesc + " 【成功】");
            jsonObject.put(KEY_RESULT_CODE, VALUE_RESULT_SUCCESS);
            jsonObject.put(KEY_RESULT_DATA, siteEntity);
            jsonObject.put(KEY_RESULT_MSG, "外部站点添加成功！");
        } catch (Exception e) {
            log.error("外部站点添加", e);
            jsonObject.put(KEY_RESULT_CODE, VALUE_RESULT_FAILURE);
            jsonObject.put(KEY_RESULT_MSG, super.convertException(e));
        }
        return jsonObject.toJSONString();
    }


}