package com.mortals.xhx.module.site.web;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.TypeReference;
import com.mortals.framework.annotation.UnAuth;
import com.mortals.framework.common.Rest;
import com.mortals.framework.model.Result;
import com.mortals.framework.service.ICacheService;
import com.mortals.framework.web.BaseCRUDJsonBodyMappingController;
import com.mortals.xhx.base.system.param.service.ParamService;
import com.mortals.xhx.common.code.YesNoEnum;
import com.mortals.xhx.module.business.model.BusinessEntity;
import com.mortals.xhx.module.site.model.*;
import com.mortals.xhx.module.window.model.WindowBusinessEntity;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.util.ObjectUtils;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;
import org.springframework.web.bind.annotation.*;
import com.mortals.framework.model.Context;

import com.mortals.xhx.module.site.service.SiteHallService;

import java.util.HashMap;
import java.util.List;
import java.util.Map;

import static com.mortals.xhx.common.key.RedisKey.*;

/**
 * 站点大厅信息
 *
 * @author zxfei
 * @date 2023-04-25
 */
@RestController
@RequestMapping("site/hall")
public class SiteHallController extends BaseCRUDJsonBodyMappingController<SiteHallService, SiteHallEntity, Long> {

    @Autowired
    private ParamService paramService;
    @Autowired
    private ICacheService cacheService;

    public SiteHallController() {
        super.setModuleDesc("站点大厅信息");
    }

    @Override
    protected void init(Map<String, Object> model, Context context) {
        this.addDict(model, "floor", paramService.getParamBySecondOrganize("SiteHall", "floor"));
        this.addDict(model, "build", paramService.getParamBySecondOrganize("SiteHall", "build"));
        super.init(model, context);
    }


    /**
     * 根据大厅查询所属业务
     */
    @PostMapping(value = "getBusinessByHall")
    @UnAuth
    public Rest<Object> getBusinessByHall(@RequestBody SiteHallEntity siteHall) {
        Rest<Object> ret = new Rest();
        Map<String, Object> model = new HashMap();
        String busiDesc = "根据大厅查询所属业务";
        int code = VALUE_RESULT_SUCCESS;
        try {
            if (ObjectUtils.isEmpty(siteHall.getId())) {
                return Rest.fail("大厅ID不能为空！");
            }
            Rest<List<BusinessEntity>> rest = null;
            String cacheResult = cacheService.get(KEY_SEARCH_HALL_BUSINESS_CACHE + siteHall.getId(), String.class);
            if (!ObjectUtils.isEmpty(cacheResult)) {
                rest = JSON.parseObject(cacheResult, new TypeReference<Rest<List<BusinessEntity>>>() {
                });
            } else {
                //查询数据库
                rest = this.service.getBusinessByHall(siteHall, getContext());
                if(rest.getCode()== YesNoEnum.YES.getValue()){
                    cacheService.set(KEY_SEARCH_HALL_BUSINESS_CACHE + siteHall.getId(), JSON.toJSONString(rest), KEY_SEARCH_TIMEOUTT_CACHE);
                }
            }

            if (!ObjectUtils.isEmpty(getContext()) && !ObjectUtils.isEmpty(getContext().getUser())) {
                recordSysLog(request, busiDesc + " 【成功】");
            }
            model.put(KEY_RESULT_DATA, rest.getData());
            model.put("message_info", busiDesc + "成功");
        } catch (Exception e) {
            log.error("获取异常", e);
            code = VALUE_RESULT_FAILURE;
            this.doException(this.request, busiDesc, model, e);
        }
        ret.setCode(code);
        ret.setData(model);
        ret.setMsg(model.get("message_info") == null ? "" : model.remove("message_info").toString());
        return ret;
    }


}