package com.mortals.xhx.module.skin.service.impl;

import cn.hutool.core.io.FileUtil;
import cn.hutool.core.lang.Validator;
import cn.hutool.core.net.URLDecoder;
import cn.hutool.core.net.url.UrlBuilder;
import cn.hutool.core.util.CharsetUtil;
import cn.hutool.core.util.ReUtil;
import cn.hutool.core.util.StrUtil;
import cn.hutool.core.util.ZipUtil;
import com.mortals.framework.ap.GlobalSysInfo;
import com.mortals.framework.common.Rest;
import com.mortals.framework.exception.AppException;
import com.mortals.framework.model.Context;
import com.mortals.framework.model.PageInfo;
import com.mortals.framework.service.impl.AbstractCRUDServiceImpl;
import com.mortals.framework.util.ThreadPool;
import com.mortals.xhx.base.system.param.service.ParamService;
import com.mortals.xhx.base.system.upload.service.UploadService;
import com.mortals.xhx.common.code.FiletypeEnum;
import com.mortals.xhx.common.key.Constant;
import com.mortals.xhx.common.utils.BeanUtil;
import com.mortals.xhx.common.utils.FreeMarkerUtils;
import com.mortals.xhx.common.utils.SendSkinRefreshTask;
import com.mortals.xhx.feign.skin.ISkinFillFeign;
import com.mortals.xhx.feign.skin.ISkinSampleFeign;
import com.mortals.xhx.module.app.model.AppEntity;
import com.mortals.xhx.module.app.model.AppQuery;
import com.mortals.xhx.module.app.model.AppVersionEntity;
import com.mortals.xhx.module.app.model.AppVersionQuery;
import com.mortals.xhx.module.app.service.AppService;
import com.mortals.xhx.module.app.service.AppVersionService;
import com.mortals.xhx.module.model.model.ModelEntity;
import com.mortals.xhx.module.model.model.ModelQuery;
import com.mortals.xhx.module.model.service.ModelService;
import com.mortals.xhx.module.product.model.ProductEntity;
import com.mortals.xhx.module.product.service.ProductService;
import com.mortals.xhx.module.skin.dao.SkinBaseDao;
import com.mortals.xhx.module.skin.model.SkinBaseEntity;
import com.mortals.xhx.module.skin.model.SkinBaseQuery;
import com.mortals.xhx.module.skin.model.SkinFieldEntity;
import com.mortals.xhx.module.skin.model.SkinFieldQuery;
import com.mortals.xhx.module.skin.service.SkinBaseService;
import com.mortals.xhx.module.skin.service.SkinFieldService;
import freemarker.template.Template;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.fileupload.FileItem;
import org.apache.commons.fileupload.FileItemFactory;
import org.apache.commons.fileupload.disk.DiskFileItemFactory;
import org.apache.commons.io.IOUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.MediaType;
import org.springframework.stereotype.Service;
import org.springframework.util.ObjectUtils;
import org.springframework.web.multipart.MultipartFile;
import org.springframework.web.multipart.commons.CommonsMultipartFile;

import java.io.*;
import java.nio.file.CopyOption;
import java.nio.file.StandardCopyOption;
import java.util.Arrays;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;
import java.util.zip.ZipEntry;
import java.util.zip.ZipFile;
import java.util.zip.ZipOutputStream;


/**
 * SkinBaseService
 * 系统基础皮肤 service实现
 *
 * @author zxfei
 * @date 2022-05-24
 */
@Service("skinBaseService")
@Slf4j
public class SkinBaseServiceImpl extends AbstractCRUDServiceImpl<SkinBaseDao, SkinBaseEntity, Long> implements SkinBaseService {

    @Autowired
    private SkinFieldService skinFieldService;
    @Autowired
    private UploadService uploadService;
    @Autowired
    private ProductService productService;
    @Autowired
    private ParamService paramService;
    @Autowired
    private ISkinSampleFeign skinSampleFeign;
    @Autowired
    private ISkinFillFeign skinFillFeign;
    @Autowired
    private AppService appService;
    @Autowired
    private ModelService modelService;
    @Autowired
    private AppVersionService appVersionService;

    @Override
    protected void validData(SkinBaseEntity entity, Context context) throws AppException {
        SkinBaseQuery skinBaseQuery = new SkinBaseQuery();
        skinBaseQuery.setName(entity.getName());
        SkinBaseEntity skinBaseEntity = this.selectOne(skinBaseQuery);
        if (!ObjectUtils.isEmpty(skinBaseEntity) && entity.newEntity()) {
            throw new AppException("皮肤名称已存在");
        }
        super.validData(entity, context);
    }

    @Override
    protected void findAfter(SkinBaseEntity entity, PageInfo pageInfo, Context context, List<SkinBaseEntity> list) throws AppException {

        Map<String, String> imageResolutionMap = paramService.getParamBySecondOrganize("SkinBase", "imageResolution");

        String domainUrl = "";
        if (!ObjectUtils.isEmpty(entity.getServerName()) && Validator.isIpv4(entity.getServerName())) {
            domainUrl = String.format("http://%s:%d", entity.getServerName(), entity.getServerPort() > 0 ? entity.getServerPort() : 11078);
        } else {
            boolean isDomain = ReUtil.isMatch("^[a-zA-Z0-9][-a-zA-Z0-9]{0,62}(\\.[a-zA-Z0-9][-a-zA-Z0-9]{0,62})*\\.[a-zA-Z]{2,}$", entity.getServerName());
            if (isDomain) {
                domainUrl = String.format("https://%s", entity.getServerName());
            } else {
                domainUrl = "";
            }

        }

        for (SkinBaseEntity item : list) {
            List<SkinFieldEntity> skinFieldEntities = skinFieldService.find(new SkinFieldQuery().skinId(item.getId()));
            //  String domain = GlobalSysInfo.getParamValue(Constant.DOMAIN, "http://192.168.0.98:11071/");
            for (SkinFieldEntity field : skinFieldEntities) {
                if ("2".equals(field.getFieldType())) {
                    String fieldUrl = String.format("%s%s", domainUrl, StrUtil.prependIfMissing(field.getFieldValue(), "/", "/"));
                    //UrlBuilder builder = UrlBuilder.ofHttp(domainUrl, CharsetUtil.CHARSET_UTF_8).addPath(field.getFieldValue());
                    field.setFieldValueUrl(fieldUrl);
                }
            }

            ProductEntity productEntity = productService.get(item.getProductId());
            item.setProductCode(productEntity == null ? "" : productEntity.getProductCode());
            item.setImageResolutionValue(imageResolutionMap.get(entity.getImageResolution() == null ? "1" : entity.getImageResolution()));
            item.setSkinFieldList(skinFieldEntities);
            if (!ObjectUtils.isEmpty(item.getPreviewImagePath())) {
                // UrlBuilder builder = UrlBuilder.ofHttp(domainUrl, CharsetUtil.CHARSET_UTF_8).addPath(item.getPreviewImagePath());
                String fieldUrl = String.format("%s%s", domainUrl, StrUtil.prependIfMissing(item.getPreviewImagePath(), "/", "/"));
                // item.setPreviewImagePath(builder.build());
                item.setPreviewImagePath(fieldUrl);
                //log.info("cssUrl:" + URLDecoder.decode(item.getPreviewImagePath(), CharsetUtil.CHARSET_UTF_8));
            }
            if (!ObjectUtils.isEmpty(item.getCssFilePath())) {
                //UrlBuilder builder = UrlBuilder.ofHttp(domainUrl, CharsetUtil.CHARSET_UTF_8).addPath(item.getCssFilePath());
                //item.setCssFilePath(builder.build());
                String fieldUrl = String.format("%s%s", domainUrl, StrUtil.prependIfMissing(item.getCssFilePath(), "/", "/"));
                item.setCssFilePath(fieldUrl);
                //log.info("cssUrl:" + URLDecoder.decode(item.getCssFilePath(), CharsetUtil.CHARSET_UTF_8));
            }
        }
        super.findAfter(entity, pageInfo, context, list);
    }

    @Override
    protected void saveAfter(SkinBaseEntity entity, Context context) throws AppException {
        //删除关联后新增
        saveSkinFields(entity, context);
        super.saveAfter(entity, context);
    }

    @Override
    protected void updateAfter(SkinBaseEntity entity, Context context) throws AppException {
        saveSkinFields(entity, context);

        SendSkinRefreshTask sendSkinRefreshTask = new SendSkinRefreshTask();
        ThreadPool.getInstance().execute(sendSkinRefreshTask);
    }

    private void saveSkinFields(SkinBaseEntity entity, Context context) {
        Long[] ids = skinFieldService.find(new SkinFieldQuery().skinId(entity.getId())).stream().map(item -> item.getId()).toArray(Long[]::new);
        if (!ObjectUtils.isEmpty(ids)) {
            skinFieldService.remove(ids, context);
        }
        if (!ObjectUtils.isEmpty(entity.getSkinFieldList())) {
            entity.getSkinFieldList().stream().forEach(item -> {
                item.setSkinId(entity.getId());
            });
            skinFieldService.save(entity.getSkinFieldList(), context);
        }

        SendSkinRefreshTask sendSkinRefreshTask = new SendSkinRefreshTask();
        ThreadPool.getInstance().execute(sendSkinRefreshTask);
    }

    @Override
    protected void updateBefore(SkinBaseEntity entity, Context context) throws AppException {
        genTemplateCss(entity, context);
        super.updateBefore(entity, context);
    }

    @Override
    protected void saveBefore(SkinBaseEntity entity, Context context) throws AppException {
        //todo 合成实现css模板文件
        genTemplateCss(entity, context);
        super.saveBefore(entity, context);
    }


    @Override
    protected void removeAfter(Long[] ids, Context context, int result) throws AppException {
        super.removeAfter(ids, context, result);
        SkinFieldQuery skinFieldQuery = new SkinFieldQuery();
        skinFieldQuery.setSkinIdList(Arrays.asList(ids));
        Long[] skinFieldIds = skinFieldService.find(skinFieldQuery, context).stream().map(item -> item.getId()).toArray(Long[]::new);
        if (!ObjectUtils.isEmpty(skinFieldIds)) {
            skinFieldService.remove(skinFieldIds, context);
        }
        SendSkinRefreshTask sendSkinRefreshTask = new SendSkinRefreshTask();
        ThreadPool.getInstance().execute(sendSkinRefreshTask);

    }


    private void genTemplateCss(SkinBaseEntity entity, Context context) {
        //加载模板，合成模板数据
        Map data = new HashMap();
        List<SkinFieldEntity> collect = entity.getSkinFieldList().stream().map(field -> {
            SkinFieldEntity skinFieldEntity = new SkinFieldEntity();
            BeanUtils.copyProperties(field, skinFieldEntity, BeanUtil.getNullPropertyNames(field));

            if ("2".equals(skinFieldEntity.getFieldType())) {
                String result = StrUtil.prependIfMissing(skinFieldEntity.getFieldValue(), "/", "/");
                skinFieldEntity.setFieldValue(result);
            }
            // updateUrl(skinFieldEntity);
            return skinFieldEntity;
        }).collect(Collectors.toList());

        data.put("columns", collect);
        String temp = "skin.css.ftl";
        try {
            FreeMarkerUtils.initFreeMarker(temp);
            StringWriter sw = new StringWriter();
            Template template = FreeMarkerUtils.getTemplate(temp);
            template.process(data, sw);

            InputStream inputStream = new ByteArrayInputStream(sw.toString().getBytes(Constant.UTF8));
            MultipartFile file = getMultipartFile(inputStream, "file.css");
            String filePath = uploadService.saveFileUpload(file, "/file/fileupload/skin", context.getUser());
            entity.setCssFilePath(filePath);

        } catch (Exception e) {
            log.error("css失败：", e);

        }

    }

    /**
     * 获取封装得MultipartFile
     *
     * @param inputStream inputStream
     * @param fileName    fileName
     * @return MultipartFile
     */
    public MultipartFile getMultipartFile(InputStream inputStream, String fileName) {
        FileItem fileItem = createFileItem(inputStream, fileName);
        //CommonsMultipartFile是feign对multipartFile的封装，但是要FileItem类对象
        return new CommonsMultipartFile(fileItem);
    }


    /**
     * FileItem类对象创建
     *
     * @param inputStream inputStream
     * @param fileName    fileName
     * @return FileItem
     */
    public FileItem createFileItem(InputStream inputStream, String fileName) {
        FileItemFactory factory = new DiskFileItemFactory(16, null);
        String textFieldName = "file";
        FileItem item = factory.createItem(textFieldName, MediaType.MULTIPART_FORM_DATA_VALUE, true, fileName);
        int bytesRead = 0;
        byte[] buffer = new byte[10 * 1024 * 1024];
        OutputStream os = null;
        //使用输出流输出输入流的字节
        try {
            os = item.getOutputStream();
            while ((bytesRead = inputStream.read(buffer, 0, 8192)) != -1) {
                os.write(buffer, 0, bytesRead);
            }
            inputStream.close();
        } catch (IOException e) {
            log.error("Stream copy exception", e);
            throw new IllegalArgumentException("文件上传失败");
        } finally {
            if (os != null) {
                try {
                    os.close();
                } catch (IOException e) {
                    log.error("Stream close exception", e);

                }
            }
            if (inputStream != null) {
                try {
                    inputStream.close();
                } catch (IOException e) {
                    log.error("Stream close exception", e);
                }
            }
        }

        return item;
    }


    /**
     * 临时方法 归类下载
     *
     * @param context
     * @return
     */
    @Override
    public Rest<byte[]> compressSkinZip(SkinBaseEntity query, Context context) {
        ByteArrayOutputStream outputStream = new ByteArrayOutputStream();
        ZipOutputStream zip = new ZipOutputStream(outputStream);
        //查询所有皮肤
        List<SkinBaseEntity> skinBaseEntities = this.find(new SkinBaseEntity());
        for (SkinBaseEntity skinBaseEntity : skinBaseEntities) {
            String cssFilePath = skinBaseEntity.getCssFilePath();
            String filePath = uploadService.getFilePath(cssFilePath);
            File file = new File(filePath);
            if (file.exists()) {
                try {
                    //读取css文件 提取图片地址
                    List<String> cssLines = FileUtil.readUtf8Lines(file);
                    for (String cssLine : cssLines) {
                        String cssFileUrl = StrUtil.subBetween(cssLine, "url(", ")");
                        if (!ObjectUtils.isEmpty(cssFileUrl)) {

                            String cssFileUrlPath = uploadService.getFilePath(cssFileUrl);

                            File cssFile = new File(cssFileUrlPath);

                            log.info("cssFileUrl:{} ,存在:{}", cssFileUrl, cssFile.exists());
                            if (cssFile.exists()) {
                                zip.putNextEntry(new ZipEntry(StrUtil.removePrefix(cssFileUrlPath, "/")));
                                IOUtils.write(FileUtil.readBytes(cssFile), zip);
                                zip.flush();
                                zip.closeEntry();
                            }
                        }
                    }
                    zip.putNextEntry(new ZipEntry(StrUtil.removePrefix(filePath, "/")));
                    IOUtils.write(FileUtil.readBytes(file), zip);
                    zip.flush();
                    zip.closeEntry();
                } catch (Exception e) {
                    // log.error("异常", e.getMessage());
                }
            }

            String previewImagePath = skinBaseEntity.getPreviewImagePath();
            filePath = uploadService.getFilePath(previewImagePath);
            file = new File(filePath);
            if (file.exists()) {
                try {
                    zip.putNextEntry(new ZipEntry(StrUtil.removePrefix(filePath, "/")));
                    IOUtils.write(FileUtil.readBytes(file), zip);
                    zip.flush();
                    zip.closeEntry();
                } catch (Exception e) {

                    log.error("异常", e.getMessage());
                }
            }
        }
        //查询所有皮肤skinFiled
        SkinFieldEntity skinFieldQuery = new SkinFieldEntity();
        skinFieldQuery.setFieldType("2");
        List<SkinFieldEntity> skinFieldEntities = skinFieldService.find(skinFieldQuery);
        for (SkinFieldEntity skinFieldEntity : skinFieldEntities) {
            String fieldValue = skinFieldEntity.getFieldValue();
            String filePath = uploadService.getFilePath(fieldValue);
            File file = new File(filePath);
            if (file.exists()) {
                try {
                    zip.putNextEntry(new ZipEntry(StrUtil.removePrefix(filePath, "/")));
                    IOUtils.write(FileUtil.readBytes(file), zip);
                    zip.flush();
                    zip.closeEntry();
                } catch (Exception e) {
                    log.error("异常", e.getMessage());
                }
            }
        }

        //查询所有app 图标
        List<AppEntity> appEntityList = appService.find(new AppQuery());
        for (AppEntity appEntity : appEntityList) {
            String fieldValue = appEntity.getFilePath();
            String filePath = uploadService.getFilePath(fieldValue);
            File file = new File(filePath);
            if (file.exists()) {
                try {
                    zip.putNextEntry(new ZipEntry(StrUtil.removePrefix(filePath, "/")));
                    IOUtils.write(FileUtil.readBytes(file), zip);
                    zip.flush();
                    zip.closeEntry();
                } catch (Exception e) {
                    log.error("异常", e.getMessage());
                }
            }
            String appIconPath = appEntity.getAppIconPath();
            filePath = uploadService.getFilePath(appIconPath);
            file = new File(filePath);
            if (file.exists()) {
                try {
                    zip.putNextEntry(new ZipEntry(StrUtil.removePrefix(filePath, "/")));
                    IOUtils.write(FileUtil.readBytes(file), zip);
                    zip.flush();
                    zip.closeEntry();
                } catch (Exception e) {
                    log.error("异常", e.getMessage());
                }
            }

        }

        //查询所有model 图标
        List<ModelEntity> modelEntities = modelService.find(new ModelQuery());
        for (ModelEntity modelEntity : modelEntities) {
            String fieldValue = modelEntity.getModelIcon();
            String filePath = uploadService.getFilePath(fieldValue);
            File file = new File(filePath);
            if (file.exists()) {
                try {
                    zip.putNextEntry(new ZipEntry(StrUtil.removePrefix(filePath, "/")));
                    IOUtils.write(FileUtil.readBytes(file), zip);
                    zip.flush();
                    zip.closeEntry();
                } catch (Exception e) {
                    log.error("异常", e.getMessage());
                }
            }
        }

        //应用版本
        List<Long> appIdList = appService.find(new AppQuery().siteId(1L)).stream().map(i -> i.getId()).collect(Collectors.toList());
        AppVersionQuery appVersionQuery = new AppVersionQuery();
        if (!ObjectUtils.isEmpty(appIdList)) {
            appVersionQuery.setAppIdList(appIdList);
        }
        appVersionQuery.setUsed(1);
        List<AppVersionEntity> appVersionEntities = appVersionService.find(appVersionQuery);
        for (AppVersionEntity appVersionEntity : appVersionEntities) {
            String filePath = appVersionEntity.getFilePath();
            filePath = uploadService.getFilePath(filePath);
            File file = new File(filePath);
            if (file.exists()) {
                try {
                    zip.putNextEntry(new ZipEntry(StrUtil.removePrefix(filePath, "/")));
                    IOUtils.write(FileUtil.readBytes(file), zip);
                    zip.flush();
                    zip.closeEntry();
                } catch (Exception e) {
                    log.error("异常", e.getMessage());
                }
            }
        }


        IOUtils.closeQuietly(zip);
        return Rest.ok(outputStream.toByteArray());
    }

    /**
     * 刷新所有资源路径，补全/
     *
     * @param context
     * @return
     */
    @Override
    public Rest<Void> updateResourcePath(SkinBaseEntity query, Context context) {
        List<SkinBaseEntity> skinBaseEntities = this.find(new SkinBaseEntity());
        for (SkinBaseEntity skinBaseEntity : skinBaseEntities) {
            if (!ObjectUtils.isEmpty(skinBaseEntity.getCssFilePath())) {
                skinBaseEntity.setCssFilePath(StrUtil.prependIfMissing(skinBaseEntity.getCssFilePath(), "/"));
            }
            if (!ObjectUtils.isEmpty(skinBaseEntity.getPreviewImagePath())) {
                skinBaseEntity.setPreviewImagePath(StrUtil.prependIfMissing(skinBaseEntity.getPreviewImagePath(), "/"));
            }

            if (!ObjectUtils.isEmpty(query.getPrependPath())) {

            }


        }
        this.getDao().updateBatch(skinBaseEntities);

        SkinFieldEntity skinFieldQuery = new SkinFieldEntity();
        skinFieldQuery.setFieldType("2");
        List<SkinFieldEntity> skinFieldEntities = skinFieldService.find(skinFieldQuery);
        for (SkinFieldEntity skinFieldEntity : skinFieldEntities) {
            if (!ObjectUtils.isEmpty(skinFieldEntity.getFieldValue())) {
                skinFieldEntity.setFieldValue(StrUtil.prependIfMissing(skinFieldEntity.getFieldValue(), "/"));
            }
        }

        skinFieldService.update(skinFieldEntities, context);
        List<AppEntity> appEntityList = appService.find(new AppQuery());
        for (AppEntity appEntity : appEntityList) {
            if (!ObjectUtils.isEmpty(appEntity.getAppIconPath())) {
                appEntity.setAppIconPath(StrUtil.prependIfMissing(appEntity.getAppIconPath(), "/"));
            }
        }
        appService.update(appEntityList, context);

        List<ModelEntity> modelEntities = modelService.find(new ModelQuery());
        for (ModelEntity modelEntity : modelEntities) {
            if (!ObjectUtils.isEmpty(modelEntity.getModelIcon())) {
                modelEntity.setModelIcon(StrUtil.prependIfMissing(modelEntity.getModelIcon(), "/"));
            }
        }
        modelService.update(modelEntities, context);
        return Rest.ok();
    }


    public static void main(String[] args) {

/*        String str = "/file/upload/12332.png";

        String result = StrUtil.prependIfMissing(str, "/", "/");


        System.out.println(result);*/
        String str = "file/upload/12332.png";


        String s1 = StrUtil.subAfter(str, "/", false);
        System.out.println(s1);


        String s = StrUtil.replaceFirst(str, "/file/upload/", "/file/upload/abc/");
        System.out.println(s);


        File appendFile = new File("F:\\license\\license.key");
        File appendFile1 = new File("F:\\license\\license.pub");

        File zipFile = new File("F:\\license\\license.zip");
        //CopyOption copyOption = new CopyOption();

        ZipUtil.append(zipFile.toPath(), appendFile.toPath(), StandardCopyOption.REPLACE_EXISTING);
        ZipUtil.append(zipFile.toPath(), appendFile1.toPath(), StandardCopyOption.REPLACE_EXISTING);


        //  ZipUtil.zip("F:\\1.zip");
/*        File file = new File("F:\\1.zip");
        ZipFile zipFile = ZipUtil.toZipFile(file, null);*/

        //zipFile.

    }
}