package com.mortals.xhx.module.skin.service.impl;

import com.mortals.framework.exception.AppException;
import com.mortals.framework.model.Context;
import com.mortals.xhx.common.utils.BeanUtil;
import com.mortals.xhx.module.skin.model.SkinFieldQuery;
import org.springframework.beans.BeanUtils;
import org.springframework.stereotype.Service;
import com.mortals.framework.service.impl.AbstractCRUDServiceImpl;
import com.mortals.xhx.module.skin.dao.SkinFieldDao;
import com.mortals.xhx.module.skin.model.SkinFieldEntity;
import com.mortals.xhx.module.skin.service.SkinFieldService;
import org.springframework.util.ObjectUtils;

import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.stream.Collectors;

/**
 * SkinFieldService
 * 皮肤属性字段配置 service实现
 *
 * @author zxfei
 * @date 2022-05-24
 */
@Service("skinFieldService")
public class SkinFieldServiceImpl extends AbstractCRUDServiceImpl<SkinFieldDao, SkinFieldEntity, Long> implements SkinFieldService {

    /**
     * @param entity
     * @param context
     * @throws AppException
     */
    @Override
    protected void updateAfter(SkinFieldEntity entity, Context context) throws AppException {
        super.updateAfter(entity, context);
        if (0L == entity.getSkinId()) {
            //更新模板属性  所有引用了当前模板的属性
            SkinFieldQuery skinFieldQuery = new SkinFieldQuery();
            skinFieldQuery.setTemplateId(entity.getTemplateId());
            skinFieldQuery.setFieldCode(entity.getFieldCode());
            List<SkinFieldEntity> updateList = this.find(skinFieldQuery, context).stream()
                    .filter(f -> f.getSkinId() != 0L)
                    .map(item -> {
                        item.setFieldName(entity.getFieldName());
                        item.setFieldType(entity.getFieldType());
                        item.setFieldOrderNo(entity.getFieldOrderNo());
                        item.setEnabled(entity.getEnabled());
                        return item;
                    })
                    .collect(Collectors.toList());
            if (!ObjectUtils.isEmpty(updateList)) {
                this.update(updateList, context);
            }


        }
    }


    /**
     * @param entity
     * @param context
     * @throws AppException
     */
    @Override
    protected void saveAfter(SkinFieldEntity entity, Context context) throws AppException {
        super.saveAfter(entity, context);
        if (0L == entity.getSkinId()) {
            //新增模板属性
            SkinFieldQuery skinFieldQuery = new SkinFieldQuery();
            skinFieldQuery.setTemplateId(entity.getTemplateId());
            //skinFieldQuery.setFieldCode(entity.getFieldCode());
            //重复提交
            Map<Long, Set<String>> collect = this.find(skinFieldQuery, context).stream().collect(Collectors.groupingBy(x -> x.getSkinId(), Collectors.mapping(y -> y.getFieldCode(), Collectors.toSet())));
            for (Map.Entry<Long, Set<String>> entry : collect.entrySet()) {
                Long key = entry.getKey();
                Set<String> value = entry.getValue();
                if (key == 0L) continue;
                if (value.contains(entity.getFieldCode())) continue;
                //新增属性
                SkinFieldEntity target = new SkinFieldEntity();
                BeanUtils.copyProperties(entity, target, BeanUtil.getNullPropertyNames(entity));
                target.setId(null);
                target.setSkinId(key);
                this.save(target, context);
            }
        }

    }


}