package com.mortals.xhx.module.skin.web;

import cn.hutool.core.lang.Validator;
import cn.hutool.core.net.url.UrlBuilder;
import cn.hutool.core.util.CharsetUtil;
import cn.hutool.core.util.ReUtil;
import cn.hutool.core.util.StrUtil;
import com.mortals.framework.annotation.UnAuth;
import com.mortals.framework.ap.GlobalSysInfo;
import com.mortals.framework.common.Rest;
import com.mortals.framework.exception.AppException;
import com.mortals.framework.model.PageInfo;
import com.mortals.framework.model.Result;
import com.mortals.framework.util.DataUtil;
import com.mortals.framework.web.BaseCRUDJsonBodyMappingController;
import com.mortals.xhx.base.system.param.service.ParamService;
import com.mortals.xhx.common.key.Constant;
import com.mortals.xhx.module.product.model.ProductEntity;
import com.mortals.xhx.module.product.model.ProductQuery;
import com.mortals.xhx.module.product.service.ProductService;
import com.mortals.xhx.module.site.model.SiteBusinessEntity;
import com.mortals.xhx.module.skin.model.SkinFieldEntity;
import com.mortals.xhx.module.skin.model.SkinFieldQuery;
import com.mortals.xhx.module.skin.service.SkinFieldService;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.io.IOUtils;
import org.apache.commons.lang3.ObjectUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;
import org.springframework.web.bind.annotation.*;
import com.mortals.framework.model.Context;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import com.mortals.framework.web.BaseCRUDJsonBodyMappingController;
import com.mortals.xhx.module.skin.model.SkinBaseEntity;
import com.mortals.xhx.module.skin.model.SkinBaseQuery;
import com.mortals.xhx.module.skin.service.SkinBaseService;
import org.apache.commons.lang3.ArrayUtils;
import com.mortals.framework.util.StringUtils;

import java.io.IOException;
import java.net.URLDecoder;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

import com.alibaba.fastjson.JSONObject;

import java.util.Arrays;

import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

import static com.mortals.framework.ap.SysConstains.*;

/**
 * 系统基础皮肤
 *
 * @author zxfei
 * @date 2022-05-24
 */
@RestController
@RequestMapping("skin/base")
@Slf4j
public class SkinBaseController extends BaseCRUDJsonBodyMappingController<SkinBaseService, SkinBaseEntity, Long> {

    @Autowired
    private ParamService paramService;
    @Autowired
    private ProductService productService;
    @Autowired
    private SkinFieldService skinFieldService;

    public SkinBaseController() {
        super.setModuleDesc("系统基础皮肤");
    }


    @Override
    protected void doListBefore(SkinBaseEntity query, Map<String, Object> model, Context context) throws AppException {
        if (!ObjectUtils.isEmpty(query.getProductCode())) {
            ProductEntity productEntity = productService.selectOne(new ProductQuery().productCode(query.getProductCode()));
            if (!ObjectUtils.isEmpty(productEntity)) {
                query.setProductId(productEntity.getId());
                query.setProductCode(null);
            }
        }
        String serverName = request.getHeader("server-name");
        Integer serverPort = DataUtil.converStr2Int(request.getHeader("server-port"), 0);
        // log.info("【应用请求】【请求体】-->serverName{} ,port:{}", serverName, serverPort);
        query.setServerName(serverName);
        query.setServerPort(serverPort);

        super.doListBefore(query, model, context);
    }

    @Override
    protected void init(Map<String, Object> model, Context context) {
        this.addDict(model, "imageResolution", paramService.getParamBySecondOrganize("SkinBase", "imageResolution"));
        super.init(model, context);
    }


    @Override
    protected int viewAfter(Long id, Map<String, Object> model, SkinBaseEntity entity, Context context) throws AppException {
        List<SkinFieldEntity> skinFieldEntities = skinFieldService.find(new SkinFieldQuery().skinId(entity.getId()));

        String serverName = request.getHeader("server-name");
        Integer serverPort = DataUtil.converStr2Int(request.getHeader("server-port"), 0);

        String domainUrl = "";
        if (!ObjectUtils.isEmpty(serverName) && Validator.isIpv4(serverName)) {
            domainUrl = String.format("http://%s:%d", serverName, serverPort > 0 ? serverPort : 11078);
        } else {
            boolean isDomain = ReUtil.isMatch("^[a-zA-Z0-9][-a-zA-Z0-9]{0,62}(\\.[a-zA-Z0-9][-a-zA-Z0-9]{0,62})*\\.[a-zA-Z]{2,}$", entity.getServerName());
            if (isDomain) {
                domainUrl = String.format("https://%s", serverName);
            } else {
                serverName = "";
            }

        }
        for (SkinFieldEntity field : skinFieldEntities) {
            if ("2".equals(field.getFieldType())) {
                String fieldUrl = String.format("%s/%s", domainUrl, field.getFieldValue());
                field.setFieldValueUrl(fieldUrl);
            }

        }
        entity.setSkinFieldList(skinFieldEntities);
        return super.viewAfter(id, model, entity, context);
    }


    /**
     * 获取皮肤所有相关资源文件并压缩打包成zip
     */
    @GetMapping(value = "zip")
    @UnAuth
    public void zip() {
        JSONObject jsonObject = new JSONObject();
        try {
            SkinBaseEntity query = new SkinBaseEntity();
            Rest<byte[]> rest = this.service.compressSkinZip(query, getContext());

            genCode(response, rest.getData());

            jsonObject.put(KEY_RESULT_CODE, VALUE_RESULT_SUCCESS);
            jsonObject.put(KEY_RESULT_MSG, "压缩文件成功！");
        } catch (Exception e) {
            log.error("获取异常", e);
            jsonObject.put(KEY_RESULT_CODE, VALUE_RESULT_FAILURE);
            jsonObject.put(KEY_RESULT_MSG, super.convertException(e));
        }
    }


    @GetMapping(value = "updateSkinPath")
    @UnAuth
    public void updateSkinPath() {
        JSONObject jsonObject = new JSONObject();
        try {
            SkinBaseEntity query = new SkinBaseEntity();
           this.service.updateResourcePath(query, getContext());

            jsonObject.put(KEY_RESULT_CODE, VALUE_RESULT_SUCCESS);
            jsonObject.put(KEY_RESULT_MSG, "更新成功！");
        } catch (Exception e) {
            log.error("获取异常", e);
            jsonObject.put(KEY_RESULT_CODE, VALUE_RESULT_FAILURE);
            jsonObject.put(KEY_RESULT_MSG, super.convertException(e));
        }
    }


    /**
     * 生成zip文件
     */
    private void genCode(HttpServletResponse response, byte[] data) throws IOException {
        response.reset();
        response.addHeader("Access-Control-Allow-Origin", "*");
        response.addHeader("Access-Control-Expose-Headers", "Content-Disposition");
        response.setHeader("Content-Disposition", "attachment; filename=\"skin.zip\"");
        response.addHeader("Content-Length", "" + data.length);
        response.setContentType("application/octet-stream; charset=UTF-8");
        IOUtils.write(data, response.getOutputStream());
    }


}