package com.mortals.xhx.module.window.service.impl;

import cn.hutool.core.collection.ListUtil;
import cn.hutool.core.util.StrUtil;
import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.TypeReference;
import com.alibaba.fastjson.serializer.SerializerFeature;
import com.mortals.framework.ap.GlobalSysInfo;
import com.mortals.framework.exception.AppException;
import com.mortals.framework.model.Context;
import com.mortals.framework.model.PageInfo;
import com.mortals.framework.model.Result;
import com.mortals.framework.service.ICacheService;
import com.mortals.framework.util.HttpUtil;
import com.mortals.xhx.common.code.YesNoEnum;
import com.mortals.xhx.common.utils.BeanUtil;
import com.mortals.xhx.module.business.model.BusinessEntity;
import com.mortals.xhx.module.business.model.BusinessQuery;
import com.mortals.xhx.module.site.model.SiteBusinessEntity;
import com.mortals.xhx.module.site.model.SiteBusinessQuery;
import com.mortals.xhx.module.site.model.SiteEntity;
import com.mortals.xhx.module.window.model.*;
import com.mortals.xhx.module.window.service.WindowHallService;
import com.mortals.xhx.module.window.service.WindowService;
import lombok.extern.slf4j.Slf4j;
import org.apache.poi.ss.formula.functions.T;
import org.checkerframework.checker.units.qual.A;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import com.mortals.framework.service.impl.AbstractCRUDServiceImpl;
import com.mortals.xhx.module.window.dao.WindowBusinessDao;
import com.mortals.xhx.module.window.service.WindowBusinessService;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.ObjectUtils;

import java.util.*;
import java.util.function.Function;
import java.util.stream.Collectors;

import static com.mortals.framework.util.HttpUtil.HEADER_CONTENT_TYPE;
import static com.mortals.xhx.common.key.Constant.PARAM_SERVER_PHP_HTTP_URL;
import static com.mortals.xhx.common.key.RedisKey.*;

/**
 * WindowBusinessService
 * 窗口业务 service实现
 *
 * @author zxfei
 * @date 2022-01-12
 */
@Service("windowBusinessService")
@Slf4j
public class WindowBusinessServiceImpl extends AbstractCRUDServiceImpl<WindowBusinessDao, WindowBusinessEntity, Long> implements WindowBusinessService {
    @Autowired
    private WindowService windowService;
    @Autowired
    private WindowHallService windowHallService;
    @Autowired
    private ICacheService cacheService;


    /**
     * @param entity
     * @param pageInfo
     * @param context
     * @return
     * @throws AppException
     */
    @Override
    public Result<WindowBusinessEntity> find(WindowBusinessEntity entity, PageInfo pageInfo, Context context) throws AppException {
        WindowBusinessEntity query = this.findBefore(entity, pageInfo, context);
        //判断查询结果是否用缓存
        if (!ObjectUtils.isEmpty(query) && !ObjectUtils.isEmpty(query.getSearchCache()) && !ObjectUtils.isEmpty(query.getSearchCache() == YesNoEnum.YES.getValue())) {
            //根据查询条件 获取缓存，如果缓存有则直接取缓存，否则读数据库后 再写入缓存
            return getWindowBusinessEntityCacheResult(entity, pageInfo, context, query);
        } else {
            Result<WindowBusinessEntity> result = this.dao.getList(query, pageInfo);
            this.findAfter(entity, pageInfo, context, result.getList());
            return result;
        }
    }

    private Result<WindowBusinessEntity> getWindowBusinessEntityCacheResult(WindowBusinessEntity entity, PageInfo pageInfo, Context context, WindowBusinessEntity query) {
        Result<WindowBusinessEntity> result = null;
        if (!ObjectUtils.isEmpty(query.getSiteBusinessId())) {
            Long siteBusinessId = query.getSiteBusinessId();
            String cacheResult = cacheService.get(KEY_SEARCH_SITEBUSINESSID_CACHE + siteBusinessId, String.class);
            if (!ObjectUtils.isEmpty(cacheResult)) {
                result = JSON.parseObject(cacheResult, new TypeReference<Result<WindowBusinessEntity>>() {
                });
            } else {
                //查询数据库
                result = this.dao.getList(query, pageInfo);
                this.findAfter(entity, pageInfo, context, result.getList());
                if (!ObjectUtils.isEmpty(result.getList())) {
                    cacheService.set(KEY_SEARCH_SITEBUSINESSID_CACHE + siteBusinessId, JSON.toJSONString(result), KEY_SEARCH_TIMEOUTT_CACHE);
                }
            }
        } else if (!ObjectUtils.isEmpty(query.getWindowId())) {
            Long windowId = query.getWindowId();
            String cacheResult = cacheService.get(KEY_SEARCH_WINDOWID_CACHE + windowId, String.class);
            if (!ObjectUtils.isEmpty(cacheResult)) {
                result = JSON.parseObject(cacheResult, new TypeReference<Result<WindowBusinessEntity>>() {
                });
            } else {
                //查询数据库
                result = this.dao.getList(query, pageInfo);
                this.findAfter(entity, pageInfo, context, result.getList());
                //无查询结果的不做保存
                if (!ObjectUtils.isEmpty(result.getList())) {
                    cacheService.set(KEY_SEARCH_WINDOWID_CACHE + windowId, JSON.toJSONString(result), KEY_SEARCH_TIMEOUTT_CACHE);
                }
            }
        } else if (!ObjectUtils.isEmpty(query.getWindowIdList())) {
            String windowIdlist = query.getWindowIdList().stream().map(String::valueOf).collect(Collectors.joining("#"));
            String cacheResult = cacheService.get(KEY_SEARCH_WINDOWID_LIST_CACHE + windowIdlist, String.class);
            if (!ObjectUtils.isEmpty(cacheResult)) {
                result = JSON.parseObject(cacheResult, new TypeReference<Result<WindowBusinessEntity>>() {
                });
            } else {
                //查询数据库
                result = this.dao.getList(query, pageInfo);
                this.findAfter(entity, pageInfo, context, result.getList());
                if (!ObjectUtils.isEmpty(result.getList())) {
                    cacheService.set(KEY_SEARCH_WINDOWID_LIST_CACHE + windowIdlist, JSON.toJSONString(result), KEY_SEARCH_TIMEOUTT_CACHE);
                }
            }
        } else {
            result = this.dao.getList(query, pageInfo);
            this.findAfter(entity, pageInfo, context, result.getList());
        }
        return result;
    }

    @Override
    protected void findAfter(WindowBusinessEntity entity, PageInfo pageInfo, Context context, List<WindowBusinessEntity> list) throws AppException {
        Map<Long, WindowHallEntity> windowHallEntityMap = windowHallService.getCacheList().stream().collect(Collectors.toMap(x -> x.getWindowId(), y -> y, (o, n) -> n));
        Iterator iterator = list.iterator();
        while (iterator.hasNext()) {
            WindowBusinessEntity item = (WindowBusinessEntity) iterator.next();
            WindowEntity windowEntity = windowService.getCache(item.getWindowId().toString());
            if (!ObjectUtils.isEmpty(item.getWindowId()) && !ObjectUtils.isEmpty(windowEntity)) {
                item.setDeptId(windowEntity.getDeptId());
                item.setDeptName(windowEntity.getDeptName());
                item.setFromnum(windowEntity.getFromnum());
                WindowHallEntity windowHallEntity = windowHallEntityMap.get(item.getWindowId());
                if (!ObjectUtils.isEmpty(windowHallEntity)) {
                    item.setHallId(windowHallEntity.getHallId());
                    item.setHallName(windowHallEntity.getHallName());
                } else {
                    //log.info("windowhallEntity is null windowId:{}",item.getWindowId());
                }
            } else {
                iterator.remove();
            }
        }
        super.findAfter(entity, pageInfo, context, list);
    }


    /**
     * @param ids
     * @param context
     * @throws AppException
     */
    @Override
    protected void removeBefore(Long[] ids, Context context) throws AppException {
        Arrays.asList(ids).forEach(id -> {
            WindowBusinessEntity windowBusinessEntity = this.get(id, context);
            //更新站点业务缓存 SiteBusinessId
            updateWindowBusinessCacheResult(windowBusinessEntity, context);
            // pushChangeMsg(windowBusinessEntity);
        });
        super.removeBefore(ids, context);
    }


    /**
     * @param entity
     * @param context
     * @throws AppException
     */
    @Override
    protected void saveAfter(WindowBusinessEntity entity, Context context) throws AppException {
        //通知php更新
        //更新站点业务缓存 SiteBusinessId
        updateWindowBusinessCacheResult(entity, context);
        super.saveAfter(entity, context);
    }


    /*    */

    /**
     * @param entity
     * @param context
     * @throws AppException
     */
    @Override
    protected void updateAfter(WindowBusinessEntity entity, Context context) throws AppException {
        //pushChangeMsg(entity);

        //更新站点业务缓存 SiteBusinessId
        updateWindowBusinessCacheResult(entity, context);

        super.updateAfter(entity, context);
    }

    private void updateWindowBusinessCacheResult(WindowBusinessEntity entity, Context context) {
        Long siteBusinessId = entity.getSiteBusinessId();
        String cacheResult = cacheService.get(KEY_SEARCH_SITEBUSINESSID_CACHE + siteBusinessId, String.class);

        //更新查询结果
        //查询数据库
        PageInfo pageInfo = new PageInfo();
        pageInfo.setPrePageResult(-1);
        Result<WindowBusinessEntity> result = this.dao.getList(new WindowBusinessQuery().siteBusinessId(siteBusinessId), pageInfo);
        this.findAfter(entity, pageInfo, context, result.getList());
        if (!ObjectUtils.isEmpty(result.getList())) {
            cacheService.set(KEY_SEARCH_SITEBUSINESSID_CACHE + siteBusinessId, JSON.toJSONString(result), KEY_SEARCH_TIMEOUTT_CACHE);
        }


        Long windowId = entity.getWindowId();
        cacheService.get(KEY_SEARCH_WINDOWID_CACHE + windowId, String.class);

        //查询数据库
        pageInfo = new PageInfo();
        pageInfo.setPrePageResult(-1);
        result = this.dao.getList(new WindowBusinessQuery().windowId(windowId), pageInfo);
        this.findAfter(entity, pageInfo, context, result.getList());
        //无查询结果的不做保存
        if (!ObjectUtils.isEmpty(result.getList())) {
            cacheService.set(KEY_SEARCH_WINDOWID_CACHE + windowId, JSON.toJSONString(result), KEY_SEARCH_TIMEOUTT_CACHE);
        }

        //窗口列表更新
        Set<String> scanKeys = cacheService.scan(KEY_SEARCH_WINDOWID_LIST_CACHE);
        for (String scanKey : scanKeys) {
            //log.info("scanKey:{}", scanKey);
            String winIds = StrUtil.removePrefixIgnoreCase(scanKey, KEY_SEARCH_WINDOWID_LIST_CACHE);
            List<String> split = StrUtil.split(winIds, "#");
            List<Long> windowIdlist = split.stream().map(m -> {
                try {
                    return Long.parseLong(m);
                } catch (Exception e) {
                    return -1L;
                }
            }).collect(Collectors.toList());

            if (windowIdlist.contains(windowId)) {
                //更新窗口列表缓存
                WindowBusinessQuery windowBusinessQuery = new WindowBusinessQuery();
                windowBusinessQuery.setWindowIdList(windowIdlist);
                //查询数据库
                pageInfo = new PageInfo();
                pageInfo.setPrePageResult(-1);
                result = this.dao.getList(windowBusinessQuery, pageInfo);

                this.findAfter(entity, pageInfo, context, result.getList());
                if (!ObjectUtils.isEmpty(result.getList())) {
                    String windowIdlistStr = windowIdlist.stream().map(String::valueOf).collect(Collectors.joining("#"));
                    cacheService.set(KEY_SEARCH_WINDOWID_LIST_CACHE + windowIdlistStr, JSON.toJSONString(result), KEY_SEARCH_TIMEOUTT_CACHE);
                } else {

                }

            }

        }

    }


    private void pushChangeMsg(WindowBusinessEntity entity) {
        String phpUrl = GlobalSysInfo.getParamValue(PARAM_SERVER_PHP_HTTP_URL, "http://127.0.0.1:11078/zwfw_api");
        phpUrl += "/api/window/winNameChange";
        HashMap<String, Object> paramsMap = new HashMap<>();
        paramsMap.put("windowid", entity.getWindowId());
        paramsMap.put("typeinfo", 1);
        String resp = null;
        try {
            Map<String, String> header = new HashMap<>();
            header.put(HEADER_CONTENT_TYPE, "application/json");
            log.info("\n thirdPartyUrl=>{} \n reqbody=>{} ", phpUrl, JSON.toJSONString(paramsMap));
            resp = HttpUtil.doPost(phpUrl, header, JSON.toJSONString(paramsMap, SerializerFeature.WriteMapNullValue));
            log.info("thirdPartyUrl =>resp:{}", resp);
        } catch (Exception e) {
            log.error("异常:", e);
        }
    }
}