package com.mortals.xhx.module.window.service.impl;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.serializer.SerializerFeature;
import com.mortals.framework.ap.GlobalSysInfo;
import com.mortals.framework.exception.AppException;
import com.mortals.framework.model.Context;
import com.mortals.framework.util.HttpUtil;
import com.mortals.xhx.module.dept.model.DeptEntity;
import com.mortals.xhx.module.dept.service.DeptService;
import com.mortals.xhx.module.matter.model.MatterEntity;
import com.mortals.xhx.module.matter.service.MatterService;
import com.mortals.xhx.module.window.model.WindowMatterQuery;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import com.mortals.framework.service.impl.AbstractCRUDServiceImpl;
import com.mortals.xhx.module.window.dao.WindowMatterDao;
import com.mortals.xhx.module.window.model.WindowMatterEntity;
import com.mortals.xhx.module.window.service.WindowMatterService;
import org.springframework.util.ObjectUtils;

import java.util.Arrays;
import java.util.HashMap;
import java.util.Map;

import static com.mortals.framework.util.HttpUtil.HEADER_CONTENT_TYPE;
import static com.mortals.xhx.common.key.Constant.PARAM_SERVER_PHP_HTTP_URL;

/**
 * WindowMatterService
 * 窗口事项 service实现
 *
 * @author zxfei
 * @date 2022-01-12
 */
@Service("windowMatterService")
@Slf4j
public class WindowMatterServiceImpl extends AbstractCRUDServiceImpl<WindowMatterDao, WindowMatterEntity, Long> implements WindowMatterService {

    @Autowired
    private MatterService matterService;
    @Autowired
    private DeptService deptService;


    /**
     * @param entity
     * @param context
     * @throws AppException
     */
    @Override
    protected void saveBefore(WindowMatterEntity entity, Context context) throws AppException {
        super.saveBefore(entity, context);
        if (!ObjectUtils.isEmpty(entity.getDeptId())) {
            DeptEntity cache = deptService.getCache(entity.getDeptId().toString());
            if (ObjectUtils.isEmpty(entity.getDeptCode())) {
                entity.setDeptCode(cache == null ? "" : cache.getDeptNumber());
            }
        }
    }

    /**
     * @param entity
     * @param context
     * @throws AppException
     */
    @Override
    protected void saveAfter(WindowMatterEntity entity, Context context) throws AppException {
        pushChangeMsg(entity);
        super.saveAfter(entity, context);
    }

    /**
     * @param id
     * @param context
     * @throws AppException
     */
    @Override
    protected void removeBefore(Long id, Context context) throws AppException {
        super.removeBefore(id, context);
        WindowMatterEntity windowMatterEntity = this.get(id, context);
        pushChangeMsg(windowMatterEntity);
    }

    /**
     * @param ids
     * @param context
     * @throws AppException
     */
    @Override
    protected void removeBefore(Long[] ids, Context context) throws AppException {
        super.removeBefore(ids, context);
        Arrays.asList(ids).forEach(id -> {
            WindowMatterEntity windowMatterEntity = this.get(id, context);
            pushChangeMsg(windowMatterEntity);
        });
    }

    /**
     * @param entity
     * @param context
     * @return
     * @throws AppException
     */
    @Override
    public WindowMatterEntity save(WindowMatterEntity entity, Context context) throws AppException {
        this.saveBefore(entity, context);
        //校验是否存在
        WindowMatterQuery windowMatterQuery = new WindowMatterQuery();
        windowMatterQuery.setWindowId(entity.getWindowId());
        windowMatterQuery.setSiteMatterId(entity.getSiteMatterId());
        int count = this.count(windowMatterQuery, context);
        if (count == 0) {
            this.dao.insert(entity);
            this.saveAfter(entity, context);
        }
        return entity;

    }


    /**
     * @param entity
     * @param context
     * @throws AppException
     */
    @Override
    protected void updateBefore(WindowMatterEntity entity, Context context) throws AppException {
        super.updateBefore(entity, context);
        WindowMatterEntity beforeWindowMatterEntity = this.get(entity.getId(), context);
        if (!ObjectUtils.isEmpty(beforeWindowMatterEntity.getDeptCode()) && !beforeWindowMatterEntity.getDeptCode().equals(entity.getDeptCode())) {
            //更新
            MatterEntity matterEntity = matterService.get(entity.getSiteMatterId());
            if (!ObjectUtils.isEmpty(matterEntity)) {
                matterEntity.setDeptCode(entity.getDeptCode());
                matterEntity.setDeptName(entity.getDeptName());
                matterService.update(matterEntity, context);
            }
        }
    }

    /**
     * @param entity
     * @param context
     * @throws AppException
     */
    @Override
    protected void updateAfter(WindowMatterEntity entity, Context context) throws AppException {
        super.updateAfter(entity, context);
        pushChangeMsg(entity);
    }

    private void pushChangeMsg(WindowMatterEntity entity) {
        String phpUrl = GlobalSysInfo.getParamValue(PARAM_SERVER_PHP_HTTP_URL, "http://127.0.0.1:11078/zwfw_api");
        phpUrl += "/api/window/winNameChange";
        HashMap<String, Object> paramsMap = new HashMap<>();
        paramsMap.put("windowid", entity.getWindowId());
        paramsMap.put("typeinfo", 0);
        String resp = null;
        try {
            Map<String, String> header = new HashMap<>();
            header.put(HEADER_CONTENT_TYPE, "application/json");
            log.info("\n thirdPartyUrl=>{} \n reqbody=>{} ", phpUrl, JSON.toJSONString(paramsMap));
            resp = HttpUtil.doPost(phpUrl, header, JSON.toJSONString(paramsMap, SerializerFeature.WriteMapNullValue));
            log.info("thirdPartyUrl =>resp:{}", resp);
        } catch (Exception e) {
            log.error("异常:", e);
        }
    }
}