package com.mortals.xhx.module.window.web;

import com.alibaba.fastjson.JSONObject;
import com.mortals.framework.annotation.UnAuth;
import com.mortals.framework.common.Rest;
import com.mortals.framework.exception.AppException;
import com.mortals.framework.model.Context;
import com.mortals.framework.model.PageInfo;
import com.mortals.framework.model.Result;
import com.mortals.framework.service.ICacheService;
import com.mortals.framework.util.DataUtil;
import com.mortals.framework.web.BaseCRUDJsonBodyMappingController;
import com.mortals.xhx.base.system.param.service.ParamService;
import com.mortals.xhx.module.dept.model.DeptEntity;
import com.mortals.xhx.module.dept.model.DeptQuery;
import com.mortals.xhx.module.dept.service.DeptService;
import com.mortals.xhx.module.site.model.SiteEntity;
import com.mortals.xhx.module.site.service.SiteService;
import com.mortals.xhx.module.window.model.WindowEntity;
import com.mortals.xhx.module.window.service.WindowService;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.util.ObjectUtils;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import java.util.*;

import static com.mortals.framework.ap.SysConstains.MESSAGE_INFO;
import static com.mortals.framework.ap.SysConstains.PAGEINFO_KEY;
import static com.mortals.xhx.common.key.RedisKey.KEY_SEARCH_HALL_BUSINESS_CACHE;

/**
 * 站点部门窗口
 *
 * @author zxfei
 * @date 2022-01-20
 */
@RestController
@RequestMapping("window")
@Slf4j
public class WindowController extends BaseCRUDJsonBodyMappingController<WindowService, WindowEntity, Long> {

    @Autowired
    private ParamService paramService;
    @Autowired
    private SiteService siteService;
    @Autowired
    private DeptService deptService;
    @Autowired
    private ICacheService cacheService;


    public WindowController() {
        super.setModuleDesc("站点部门窗口");
    }

    /**
     *
     */
    @Override
    @UnAuth
    public void downloadTemplate() {
        super.downloadTemplate();
    }

    @Override
    protected void init(Map<String, Object> model, Context context) {
        this.addDict(model, "hongqi", paramService.getParamBySecondOrganize("Window", "hongqi"));
        this.addDict(model, "building", paramService.getParamBySecondOrganize("Window", "building"));
        this.addDict(model, "level", paramService.getParamBySecondOrganize("Window", "level"));
        super.init(model, context);
    }

    /**
     * 添加业务到窗口
     */
    @PostMapping(value = "addBusinessToWindow")
    public String addBusinessToSite(@RequestBody Map<String, Object> map) {
        JSONObject jsonObject = new JSONObject();
        Map<String, Object> model = new HashMap<>();
        String businessIds = (String) map.get("businessIds");
        Long windowId = DataUtil.converObj2Long(map.get("windowId"));
        try {
            this.service.addBusinessToWindow(businessIds, windowId, getContext());
            //更新完窗口业务，删除缓存列表
            Set<String> scanSet = cacheService.scan(KEY_SEARCH_HALL_BUSINESS_CACHE);
            for (String delKey : scanSet) {
                cacheService.del(delKey);
            }
            jsonObject.put(KEY_RESULT_DATA, model);
            jsonObject.put(KEY_RESULT_CODE, VALUE_RESULT_SUCCESS);
            jsonObject.put(KEY_RESULT_MSG, "添加业务到窗口成功！");
        } catch (Exception e) {
            log.error("获取异常", e);
            jsonObject.put(KEY_RESULT_CODE, VALUE_RESULT_FAILURE);
            jsonObject.put(KEY_RESULT_MSG, super.convertException(e));
        }
        return jsonObject.toJSONString();
    }


    @PostMapping(value = "subHalllist")
    @UnAuth
    public Rest<Object> subHalllist(@RequestBody WindowEntity query) {
        Rest<Object> ret = new Rest<>();
        Map<String, Object> model = new HashMap<>();
        Context context = this.getContext();
        String busiDesc = "查询差集" + this.getModuleDesc();
        int code = VALUE_RESULT_SUCCESS;
        try {
            PageInfo pageInfo = buildPageInfo(query);
            Result<WindowEntity> result = this.getService().findSubHallList(query, pageInfo, context);
            model.put(KEY_RESULT_DATA, result.getList());
            model.put(PAGEINFO_KEY, result.getPageInfo());
            parsePageInfo(model, result.getPageInfo());
            model.put(MESSAGE_INFO, busiDesc + "成功");
            if (!ObjectUtils.isEmpty(getContext()) && !ObjectUtils.isEmpty(getContext().getUser())) {
                recordSysLog(request, busiDesc + " 【成功】");
            }
        } catch (Exception e) {
            code = VALUE_RESULT_FAILURE;
            this.doException(request, busiDesc, model, e);
        }
        this.init(model, context);
        ret.setCode(code);
        ret.setData(model);
        ret.setMsg(model.get(MESSAGE_INFO) == null ? "" : model.remove(MESSAGE_INFO).toString());
        return ret;
    }


    @PostMapping(value = "getHallByDept")
    @UnAuth
    public Rest<Object> getHallByDept(@RequestBody WindowEntity query) {
        Rest<Object> ret = new Rest<>();
        Map<String, Object> model = new HashMap<>();
        Context context = this.getContext();
        String busiDesc = "根据部门查询所属大厅！";
        int code = VALUE_RESULT_SUCCESS;
        try {
            Result<WindowEntity> result = this.getService().getHallByDept(query, context);
            model.put(KEY_RESULT_DATA, result.getList());
            model.put(PAGEINFO_KEY, result.getPageInfo());
            parsePageInfo(model, result.getPageInfo());
            model.put(MESSAGE_INFO, busiDesc + "成功");
            if (!ObjectUtils.isEmpty(getContext()) && !ObjectUtils.isEmpty(getContext().getUser())) {
                recordSysLog(request, busiDesc + " 【成功】");
            }
        } catch (Exception e) {
            code = VALUE_RESULT_FAILURE;
            this.doException(request, busiDesc, model, e);
        }
        this.init(model, context);
        ret.setCode(code);
        ret.setData(model);
        ret.setMsg(model.get(MESSAGE_INFO) == null ? "" : model.remove(MESSAGE_INFO).toString());
        return ret;
    }


    /**
     * @param list
     * @param updateSupport
     * @param context
     * @throws AppException
     */
    @Override
    public void doImportDataBefore(List<WindowEntity> list, boolean updateSupport, Context context) throws AppException {
        String siteId = request.getParameter("siteId");
        String deptId = request.getParameter("deptId");
        SiteEntity siteEntity = null;
        DeptEntity deptEntity = null;
        if (!ObjectUtils.isEmpty(siteId)) {
            siteEntity = siteService.get(Long.parseLong(siteId));
        }

        if (!ObjectUtils.isEmpty(deptId)) {
            deptEntity = deptService.get(Long.parseLong(deptId));
        }
        for (WindowEntity windowEntity : list) {
            if (!ObjectUtils.isEmpty(siteEntity)) {
                windowEntity.setSiteId(siteEntity.getId());
                windowEntity.setSiteName(siteEntity.getSiteName());
            }
            if (!ObjectUtils.isEmpty(deptEntity)) {
                windowEntity.setDeptId(deptEntity.getId());
                windowEntity.setDeptName(deptEntity.getName());
            } else {
                //判断是否填入了部门名称
                if (!ObjectUtils.isEmpty(windowEntity.getDeptName())) {
                    //根据部门名称与站点 更新窗口
                    DeptQuery deptQuery = new DeptQuery();
                    deptQuery.setName(windowEntity.getDeptName());
                    deptQuery.setSiteId(DataUtil.converStr2Long(siteId, 0L));
                    deptEntity = deptService.selectOne(deptQuery);
                    if (!ObjectUtils.isEmpty(deptEntity)) {
                        windowEntity.setDeptId(deptEntity.getId());
                        windowEntity.setDeptName(deptEntity.getName());
                    }
                }
            }
        }
        //siteId为空 或deptId为空  则删除 不导入
        Iterator<WindowEntity> iterator = list.iterator();
        while (iterator.hasNext()) {
            WindowEntity next = iterator.next();
            if (ObjectUtils.isEmpty(next.getSiteId()) || ObjectUtils.isEmpty(next.getDeptId())) {
                iterator.remove();
            }
        }
    }
}