package com.mortals.xhx.module.window.web;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.serializer.SerializerFeature;
import com.mortals.framework.ap.GlobalSysInfo;
import com.mortals.framework.exception.AppException;
import com.mortals.framework.model.Context;
import com.mortals.framework.model.OrderCol;
import com.mortals.framework.util.HttpUtil;
import com.mortals.framework.web.BaseCRUDJsonBodyMappingController;
import com.mortals.xhx.base.system.param.service.ParamService;
import com.mortals.xhx.module.window.model.WindowMatterEntity;
import com.mortals.xhx.module.window.service.WindowMatterService;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.Map;

import static com.mortals.framework.util.HttpUtil.HEADER_CONTENT_TYPE;
import static com.mortals.xhx.common.key.Constant.PARAM_SERVER_PHP_HTTP_URL;

/**
 * 窗口事项
 *
 * @author zxfei
 * @date 2022-01-20
 */
@RestController
@RequestMapping("window/matter")
@Slf4j
public class WindowMatterController extends BaseCRUDJsonBodyMappingController<WindowMatterService, WindowMatterEntity, Long> {

    @Autowired
    private ParamService paramService;

    public WindowMatterController() {
        super.setModuleDesc("窗口事项");
    }



    @Override
    protected void doListBefore(WindowMatterEntity query, Map<String, Object> model, Context context) throws AppException {
        query.setOrderColList(new ArrayList<OrderCol>() {
            {
                add(new OrderCol("a.sort", OrderCol.ASCENDING));
            }
        });
        super.doListBefore(query, model, context);
    }


    /**
     * @param entity
     * @param model
     * @param context
     * @return
     * @throws AppException
     */
    @Override
    protected int saveAfter(WindowMatterEntity entity, Map<String, Object> model, Context context) throws AppException {
        String phpUrl = GlobalSysInfo.getParamValue(PARAM_SERVER_PHP_HTTP_URL, "http://127.0.0.1:11078/zwfw_api");
        HashMap<String, Object> paramsMap = new HashMap<>();
        paramsMap.put("windowid", entity.getWindowId());
        String resp = null;
        try {
            Map<String, String> header = new HashMap<>();
            header.put(HEADER_CONTENT_TYPE, "application/json");
            log.info("\n thirdPartyUrl=>{} \n reqbody=>{} ", phpUrl, JSON.toJSONString(paramsMap));
            resp = HttpUtil.doPost(phpUrl, header, JSON.toJSONString(paramsMap, SerializerFeature.WriteMapNullValue));
            log.info("thirdPartyUrl =>resp:{}", resp);
        } catch (Exception e) {
            log.error("异常:", e);
        }
        return super.saveAfter(entity, model, context);
    }


    /**
     * @param ids
     * @param context
     * @throws AppException
     */
    @Override
    protected void deleteBefore(Long[] ids, Context context) throws AppException {
        super.deleteBefore(ids, context);
    }
}