package com.mortals.xhx.module.workman.web;

import cn.hutool.Hutool;
import cn.hutool.core.codec.Base64;
import cn.hutool.core.text.PasswdStrength;
import cn.hutool.core.util.StrUtil;
import cn.hutool.extra.pinyin.PinyinUtil;
import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.mortals.framework.annotation.UnAuth;
import com.mortals.framework.common.Rest;
import com.mortals.framework.exception.AppException;
import com.mortals.framework.model.Context;
import com.mortals.framework.util.FileUtil;
import com.mortals.framework.web.BaseCRUDJsonBodyMappingController;
import com.mortals.xhx.base.system.param.service.ParamService;
import com.mortals.xhx.base.system.upload.service.UploadService;
import com.mortals.xhx.base.system.user.model.UserEntity;
import com.mortals.xhx.common.code.OnlineEnum;
import com.mortals.xhx.common.code.YesNoEnum;
import com.mortals.xhx.module.dept.model.DeptEntity;
import com.mortals.xhx.module.dept.model.DeptQuery;
import com.mortals.xhx.module.dept.service.DeptService;
import com.mortals.xhx.module.site.model.SiteEntity;
import com.mortals.xhx.module.site.service.SiteService;
import com.mortals.xhx.module.window.model.WindowEntity;
import com.mortals.xhx.module.window.service.WindowService;
import com.mortals.xhx.module.workman.model.WorkmanEntity;
import com.mortals.xhx.module.workman.service.WorkmanService;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.util.ObjectUtils;
import org.springframework.web.bind.annotation.*;

import java.io.IOException;
import java.util.Date;
import java.util.List;
import java.util.Map;

/**
 * 工作人员
 *
 * @author zxfei
 * @date 2022-01-20
 */
@RestController
@RequestMapping("workman")
@Slf4j
public class WorkmanController extends BaseCRUDJsonBodyMappingController<WorkmanService, WorkmanEntity, Long> {

    @Autowired
    private ParamService paramService;
    @Autowired
    private SiteService siteService;
    @Autowired
    private DeptService deptService;
    @Autowired
    private WindowService windowService;
    @Autowired
    private UploadService uploadService;


    public WorkmanController() {
        super.setModuleDesc("工作人员");
    }


    @Override
    protected void init(Map<String, Object> model, Context context) {
        this.addDict(model, "politicalstatus", paramService.getParamBySecondOrganize("Workman", "politicalstatus"));
        this.addDict(model, "dangyuan", paramService.getParamBySecondOrganize("Workman", "dangyuan"));
        this.addDict(model, "starlevel", paramService.getParamBySecondOrganize("Workman", "starlevel"));
        this.addDict(model, "online", OnlineEnum.getEnumMap());

        super.init(model, context);
    }


    /**
     * @param query
     * @param model
     * @param context
     * @throws AppException
     */
    @Override
    protected void doListBefore(WorkmanEntity query, Map<String, Object> model, Context context) throws AppException {
        if (!ObjectUtils.isEmpty(query)) {
            query.setDeleted(YesNoEnum.NO.getValue());
        }
        super.doListBefore(query, model, context);
    }

    /**
     *
     */
    @Override
    @UnAuth
    public void downloadTemplate() {
        super.downloadTemplate();
    }

    /**
     * 工作人员登录
     */
    @PostMapping(value = "doLogin")
    @UnAuth
    public String doLogin(@RequestBody WorkmanEntity query) {
        log.info("doLogin==>{}", JSON.toJSONString(query));
        JSONObject ret = new JSONObject();
        try {
            String ip = super.getRequestIP(request);
            WorkmanEntity workmanEntity = this.service.doLogin(query.getLoginName(), query.getLoginPwd(), ip);
            workmanEntity.setLastLoginAddress(ip);
            workmanEntity.setLastLoginTime(new Date());
            if (query.getWindowId() != null) {
                workmanEntity.setWindowId(query.getWindowId());
                WindowEntity windowEntity = windowService.get(query.getWindowId());
                if (windowEntity != null) {
                    workmanEntity.setWindowName(windowEntity.getName());
                }
            }
            this.service.getDao().update(workmanEntity);
            if (query.getSiteId() != workmanEntity.getSiteId()) {
                throw new AppException("当前工作人员不在当前站点下！");
            }

            ret.put(KEY_RESULT_CODE, VALUE_RESULT_SUCCESS);
            ret.put(KEY_RESULT_MSG, "工作人员登录成功！");
            ret.put(KEY_RESULT_DATA, workmanEntity);
            recordSysLog(request, getCurUser(), "工作人员登录成功!");
            return ret.toJSONString();
        } catch (Exception e) {
            log.error("工作人员登录错误", e);
            ret.put(KEY_RESULT_CODE, VALUE_RESULT_FAILURE);
            ret.put(KEY_RESULT_MSG, super.convertException(e));
            return ret.toJSONString();
        }
    }

    @PostMapping(value = "doImportPic")
    public String doImportPic(@RequestBody WorkmanEntity query) {
        JSONObject ret = new JSONObject();
        try {
            log.info("filePath:" + query.getFilePath());
            this.service.doImportPic(query.getFilePath());
            ret.put(KEY_RESULT_CODE, VALUE_RESULT_SUCCESS);
            recordSysLog(request, getCurUser(), "解析工作人员图片成功!");
            return ret.toJSONString();
        } catch (Exception e) {
            log.error("解析工作人员错误", e);
            ret.put(KEY_RESULT_CODE, VALUE_RESULT_FAILURE);
            ret.put(KEY_RESULT_MSG, super.convertException(e));
            return ret.toJSONString();
        }
    }


    @RequestMapping(value = "change/password", method = RequestMethod.POST)
    @UnAuth
    public String changePassword(@RequestBody WorkmanEntity query) {
        JSONObject ret = new JSONObject();
        try {
            //校验密码复杂度 todo
/*            if (!checkPwdComplexity(query.getNewPwd())) {
                PasswdStrength.check(query.getNewPwd())
            }*/
            boolean bool = service.updateUserPwd(query.getLoginName(), query.getNewPwd());
            if (bool) {
                ret.put(KEY_RESULT_CODE, VALUE_RESULT_SUCCESS);
                ret.put(KEY_RESULT_MSG, "密码修改成功！");
            } else {
                throw new AppException("账户密码修改失败！");
            }
            recordSysLog(request, getCurUser(), "工作人员更新密码成功! workman:" + query.getLoginName());
        } catch (Exception e) {
            ret.put(KEY_RESULT_CODE, VALUE_RESULT_FAILURE);
            ret.put(KEY_RESULT_MSG, super.convertException(e));
        }
        return ret.toJSONString();
    }


    @RequestMapping(value = "person/change/password", method = RequestMethod.POST)
    @UnAuth
    public String personChangePassword(@RequestBody WorkmanEntity query) {
        JSONObject ret = new JSONObject();
        try {
            boolean bool = service.updateUserPwd(query.getLoginName(), query.getOldPwd(), query.getNewPwd());
            if (bool) {
                ret.put(KEY_RESULT_CODE, VALUE_RESULT_SUCCESS);
                ret.put(KEY_RESULT_MSG, "密码修改成功！");
            } else {
                throw new AppException("账户密码修改失败！");
            }

            UserEntity clientUser = new UserEntity();
            clientUser.setLoginName("终端机");
            clientUser.setRealName("终端机");
            clientUser.setId(-1L);

            recordSysLog(request, clientUser, "工作人员更新密码成功! workman:" + query.getLoginName());

        } catch (Exception e) {
            ret.put(KEY_RESULT_CODE, VALUE_RESULT_FAILURE);
            ret.put(KEY_RESULT_MSG, super.convertException(e));
        }
        return ret.toJSONString();
    }


    /**
     * @param list
     * @param updateSupport
     * @param context
     * @throws AppException
     */
    @Override
    public void doImportDataBefore(List<WorkmanEntity> list, boolean updateSupport, Context context) throws AppException {
        String siteId = request.getParameter("siteId");
        String deptId = request.getParameter("deptId");
        String windowId = request.getParameter("windowId");

        log.info("siteId:{},deptId:{},windowId:{},", siteId, deptId, windowId);
        SiteEntity siteEntity = null;
        DeptEntity deptEntity = null;
        WindowEntity windowEntity = null;
        if (!ObjectUtils.isEmpty(siteId)) {
            siteEntity = siteService.get(Long.parseLong(siteId));
        }


        if (!ObjectUtils.isEmpty(deptId)) {
            deptEntity = deptService.get(Long.parseLong(deptId));
        }
        if (!ObjectUtils.isEmpty(windowId)) {
            windowEntity = windowService.get(Long.parseLong(windowId));
        }

        for (WorkmanEntity workmanEntity : list) {
            if (!ObjectUtils.isEmpty(siteEntity)) {
                workmanEntity.setSiteId(siteEntity.getId());
                workmanEntity.setSiteName(siteEntity.getSiteName());
            }

            if (ObjectUtils.isEmpty(workmanEntity.getLoginName())) {
                String loginName = PinyinUtil.getPinyin(StrUtil.cleanBlank(workmanEntity.getName()), "");
                workmanEntity.setLoginName(loginName);
            }

            log.info("loginPwd:" + workmanEntity.getLoginPwd());
            if (ObjectUtils.isEmpty(workmanEntity.getLoginPwd()) || "".equals(workmanEntity.getLoginPwd().trim())) {
                workmanEntity.setLoginPwd("123");
            }
            if (!ObjectUtils.isEmpty(workmanEntity.getDeptName())) {
                DeptEntity dept = deptService.selectOne(new DeptQuery().name(workmanEntity.getDeptName()).siteId(siteEntity.getId()));
                if (!ObjectUtils.isEmpty(dept)) {
                    workmanEntity.setDeptId(dept.getId());
                    workmanEntity.setDeptName(dept.getName());
                }
            }


            if (!ObjectUtils.isEmpty(deptEntity)) {
                workmanEntity.setDeptId(deptEntity.getId());
                workmanEntity.setDeptName(deptEntity.getName());
            }


            if (!ObjectUtils.isEmpty(windowEntity)) {
                workmanEntity.setWindowId(windowEntity.getId());
                workmanEntity.setWindowName(windowEntity.getName());
            }

            if (!ObjectUtils.isEmpty(workmanEntity.getPicObj())) {
                String extension = workmanEntity.getPicObj().suggestFileExtension();
                String newName = "/file/fileupload/" + new Date().getTime() + "." + extension;
                String filePath = uploadService.getFilePath(newName);
                try {
                    boolean bool = FileUtil.write(filePath, workmanEntity.getPicObj().getData(), true, true);
                    if (bool) {
                        workmanEntity.setPhotoPath(newName);
                        workmanEntity.setPicObj(null);
                    }
                } catch (IOException e) {
                    log.error("写入证照异常", e);
                }
            }
        }
        super.doImportDataBefore(list, updateSupport, context);
    }


    public static void main(String[] args) {

        System.out.println(StrUtil.cleanBlank(PinyinUtil.getPinyin("张 三", "")));
//
//        System.out.println(Base64.encode(new JSONObject().toJSONString()));
//        System.out.println( com.mortals.framework.util.Base64.encode(new JSONObject().toJSONString().getBytes()));
//
//
        System.out.println(Base64.encode("[]"));

    }

}