package com.mortals.xhx.feign.bill.smartoffice;

import com.mortals.framework.common.Rest;
import com.mortals.xhx.feign.IFeign;
import feign.hystrix.FallbackFactory;
import lombok.extern.slf4j.Slf4j;
import org.springframework.cloud.openfeign.FeignClient;
import org.springframework.stereotype.Component;
import org.springframework.web.bind.annotation.PostMapping;

/**
 * 服务系统 Feign接口
 *
 * @author zxfei
 * @date 2022-10-26
 */
@FeignClient(name = "bill-manager", path = "/bill", fallbackFactory = BillFeignFallbackFactory.class)
public interface IBillFeign extends IFeign {

    /**
     * 用户刷新通知
     *
     * @param
     * @return
     */
    @PostMapping(value = "/user/refreshUser")
    Rest<Void> refreshUser();

}


@Slf4j
@Component
class BillFeignFallbackFactory implements FallbackFactory<IBillFeign> {
    @Override
    public IBillFeign create(Throwable t) {
        return new IBillFeign() {
            /**
             * @return
             */
            @Override
            public Rest<Void> refreshUser() {
                return Rest.fail("暂时无法通知设备，请稍后再试!");
            }
        };
    }
}








