package com.mortals.xhx.feign.product;

import com.mortals.framework.common.Rest;
import com.mortals.xhx.common.pdu.RespData;
import com.mortals.xhx.common.pdu.product.ProductPdu;
import com.mortals.xhx.feign.IFeign;
import feign.hystrix.FallbackFactory;
import lombok.extern.slf4j.Slf4j;
import org.springframework.cloud.openfeign.FeignClient;
import org.springframework.stereotype.Component;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;

import java.util.List;

@FeignClient(name = "device-manager", path = "/m", fallbackFactory = ProductFeignFallbackFactory.class)
public interface IProductFeign extends IFeign {

    /**
     * 查看产品列表
     *
     * @param productPdu
     * @return
     */
    @PostMapping(value = "/product/interlist")
    Rest<RespData<List<ProductPdu>>> list(@RequestBody ProductPdu productPdu);
}

@Slf4j
@Component
class ProductFeignFallbackFactory implements FallbackFactory<IProductFeign> {

    @Override
    public IProductFeign create(Throwable throwable) {

        return new IProductFeign(){

            @Override
            public Rest<RespData<List<ProductPdu>>> list(ProductPdu productPdu) {
                return Rest.fail("暂时无法获取产品列表，请稍后再试!");
            }
        };
    }
}
