package com.mortals.xhx.feign.skin;

import com.mortals.framework.common.Rest;
import com.mortals.xhx.feign.IFeign;
import feign.hystrix.FallbackFactory;
import lombok.extern.slf4j.Slf4j;
import org.springframework.cloud.openfeign.FeignClient;
import org.springframework.stereotype.Component;
import org.springframework.web.bind.annotation.*;

/**
 * 样表皮肤 Feign接口
 * @author zxfei
 * @date 2022-10-26
 */
@FeignClient(name = "sample-form-manager", path = "/sampleform", fallbackFactory = SkinFillFeignFallbackFactory.class)
public interface ISkinSampleFeign extends IFeign {


    /**
     * 皮肤变更通知
     *
     * @param
     * @return
     */
    @PostMapping(value = "/skin/refreshSkin")
    Rest<Void> refreshSkin();



    /**
     * 用户刷新通知
     *
     * @param
     * @return
     */
    @PostMapping(value = "/user/refreshUser")
    Rest<Void> refreshUser();

}


@Slf4j
@Component
class SkinSampleFeignFallbackFactory implements FallbackFactory<ISkinFillFeign> {
    @Override
    public ISkinFillFeign create(Throwable t) {
        return new ISkinFillFeign() {

            /**
             * @param
             * @return
             */
            @Override
            public Rest<Void> refreshSkin() {
                return Rest.fail("暂时无法通知设备，请稍后再试!");
            }

            /**
             * @return
             */
            @Override
            public Rest<Void> refreshUser() {
                return Rest.fail("暂时无法通知设备，请稍后再试!");
            }
        };
    }
}








